USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetDeviceDiskDriveBase' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetDeviceDiskDriveBase]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 24.01.2019
-- Last change: 
-- Copyright:   (c) Raynet GmbH
-- Description:	Get from all Disk Drives the Information
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================
CREATE FUNCTION [dbo].[f_GetDeviceDiskDriveBase]()
  RETURNS TABLE
AS
RETURN
  SELECT 
      s.ComputerID,
      s.Class,
      s.HardwareObjectID         AS CIMDiskDriveID,
      s.HardwareName             AS DiskDriveID, 
      ph.Value                   AS DiskNumber,
      CASE 
        WHEN mo.Value LIKE '%Samsung%'                THEN 'Samsung'
        WHEN mo.Value LIKE 'ST[0-9][0-9][0-9][0-9]%'  THEN 'Samsung'
        WHEN mo.Value LIKE '%WD[0-9][0-9][0-9][0-9]%' THEN 'Western Digital'
        WHEN mo.Value LIKE '%IBM%'                    THEN 'IBM'
	    WHEN mo.Value LIKE '%Microsoft%'              THEN 'Microsoft'
        WHEN mo.Value LIKE '%LSI%'                    THEN 'LSI'
        WHEN mo.Value LIKE '%SanDisk%'                THEN 'SanDisk'
        WHEN mo.Value LIKE '%VMware%'                 THEN 'VMware'
        ELSE ma.Value
      END                        AS Manufacturer,
      mo.Value                   AS Model,
      sr.Value                   AS SerialNumber,
      it.Value                   AS Interface,
      dbo.CIMBigInteger(s.Value) AS TotalCapacity
    FROM dbo.f_HardwareProperties('CIM_DiskDrive') s
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('SerialNumber') sr
        ON s.HardwareObjectID = sr.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Manufacturer') ma
        ON s.HardwareObjectID = ma.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Model') mo
        ON s.HardwareObjectID = mo.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('InterfaceType') it
        ON s.HardwareObjectID = it.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('PhysicalIndex') ph
        ON s.HardwareObjectID = ph.HardwareObjectID
    WHERE s.Property = 'Size'

