USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetDeviceLogicalDisk' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetDeviceLogicalDisk]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 24.01.2019
-- Last change: 
-- Copyright:   (c) Raynet GmbH
-- Description:	Get all Partitions incl Link to f_GetDeviceDiskDrive
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================


CREATE FUNCTION [dbo].[f_GetDeviceLogicalDisk]()
  RETURNS TABLE
AS
RETURN
  WITH DiskDriveBase AS (SELECT ComputerID, DiskDriveID, DiskNumber FROM dbo.f_GetDeviceDiskDriveBase())
  SELECT 
      ld.ComputerID, 
      ld.HardwareObjectID                                            AS CIMLogicalDiskID,
      CAST(NULL AS BIGINT)                                           AS CIMDiskDriveID,
      d.DiskDriveID,
      d.DiskNumber,
      ROW_NUMBER() OVER(PARTITION BY ld.ComputerID, d.DiskNumber
                        ORDER BY ld.HardwareObjectID ASC)            AS PartitionNumber,
      ld.HardwareName                                                AS InterfaceName, 
      ld.HardwareName                                                AS DriveLetter, 
      ld.MediaType,
      ld.Size,
      ld.FreeSpace,
      ld.FileSystem, 
      ld.VolumeName,
      ld.VolumeSerialNumber
    FROM (SELECT
              l.ComputerID,
              l.HardwareObjectID,
              l.HardwareName,
              dbo.CIMBigInteger(s.Value) AS Size,
              dbo.CIMBigInteger(f.Value) AS FreeSpace,
              fs.Value                   AS FileSystem,
              dbo.CIMInteger(dt.Value)   AS DriveType,
              CASE
                WHEN mt.Value = '11' THEN 'Removable Media'
                WHEN mt.Value = '12' THEN 'Fixed Disk'
              END                        AS MediaType,
              v.Value                    AS VolumeName,
              vs.Value                   AS VolumeSerialNumber
            FROM (SELECT DISTINCT
                      ComputerID,
                      HardwareObjectID,
                      HardwareName
                    FROM dbo.f_HardwareProperties('CIM_LogicalDisk')) l
              LEFT JOIN dbo.f_HardwareValuesByPropertyName('Size') s
                ON l.HardwareObjectID = s.HardwareObjectID
              LEFT JOIN dbo.f_HardwareValuesByPropertyName('FreeSpace') f
                ON l.HardwareObjectID = f.HardwareObjectID
              LEFT JOIN dbo.f_HardwareValuesByPropertyName('FileSystem') fs
                ON l.HardwareObjectID = fs.HardwareObjectID
              LEFT JOIN dbo.f_HardwareValuesByPropertyName('DriveType') dt
                ON l.HardwareObjectID = dt.HardwareObjectID
              LEFT JOIN dbo.f_HardwareValuesByPropertyName('VolumeName') v
                ON l.HardwareObjectID = v.HardwareObjectID
              LEFT JOIN dbo.f_HardwareValuesByPropertyName('VolumeSerialNumber') vs
                ON l.HardwareObjectID = vs.HardwareObjectID
              LEFT JOIN dbo.f_HardwareValuesByPropertyName('MediaType') mt
                ON l.HardwareObjectID = mt.HardwareObjectID) ld
      LEFT JOIN (SELECT
                     dda.ComputerID,
                     REPLACE(RIGHT(LEFT(dda.Value, LEN(dda.Value)-1), CHARINDEX('""', REVERSE(dda.Value), 2)-2), '\\', '\') AS Antecedent,
                     RIGHT(LEFT(ldd.Value, LEN(ldd.Value)-1), CHARINDEX('""', REVERSE(ldd.Value), 2)-2) AS Dependent
                   FROM dbo.f_HardwareProperties('Win32_DiskDriveToDiskPartition') dda
                     LEFT JOIN dbo.f_HardwareValuesByPropertyName('Dependent') ddd
                       ON dda.HardwareObjectID = ddd.HardwareObjectID
                     INNER JOIN dbo.f_HardwareProperties('Win32_LogicalDiskToPartition') lda
                       ON lda.Property = 'Antecedent'
                         AND dda.ComputerID = lda.ComputerID
                         AND ddd.Value = lda.Value
                     LEFT JOIN dbo.f_HardwareValuesByPropertyName('Dependent') ldd
                       ON lda.HardwareObjectID = ldd.HardwareObjectID
                   WHERE dda.Property = 'Antecedent') lDisk
        ON ld.ComputerID = lDisk.ComputerID
          AND ld.HardwareName = lDisk.Dependent
      LEFT JOIN DiskDriveBase d
        ON ld.ComputerID = d.ComputerID
          AND lDisk.Antecedent = d.DiskDriveID
    WHERE ld.DriveType = 3
  UNION SELECT 
      ld.ComputerID,
      CAST(NULL AS BIGINT)                                           AS CIMLogicalDiskID,
      ld.CIMDiskDriveID,
      ld.DiskDriveID,
      COALESCE(d.DiskNumber, ld.DiskNumber)                          AS DiskNumber,
      ROW_NUMBER() OVER(PARTITION BY ld.ComputerID, ld.DiskDriveID
                        ORDER BY ld.CIMDISKDriveID ASC)              AS PartitionNumber,
      ld.InterfaceName,
      NULL                                                           AS DriveLetter,
      NULL                                                           AS MediaType,
      ld.Size,
      ld.FreeSpace,
      NULL                                                           AS FileSystem, 
      NULL                                                           AS VolumeName,
      NULL                                                           AS VolumeSerialNumber 
    FROM (SELECT
              s.ComputerID, 
              s.HardwareObjectID                                             AS CIMDiskDriveID,
              LEFT(s.HardwareName, PATINDEX('%[a-z][0-9]%', s.HardwareName)) AS DiskDriveID, 
              ASCII(SUBSTRING(s.HardwareName, PATINDEX('%[a-z][0-9]%', s.HardwareName),1)) -97
                                                                             AS DiskNumber,
              s.HardwareName                                                 AS InterfaceName, 
              dbo.CIMBigInteger(s.Value)                                     AS Size, 
              dbo.CIMBigInteger(f.Value)                                     AS FreeSpace
            FROM dbo.f_HardwareProperties('CIM_DiskDrive') s
              LEFT JOIN dbo.f_HardwareValuesByPropertyName('FreeSpace') f
                ON s.HardwareObjectID = f.HardwareObjectID
            WHERE s.Property = 'Size' AND s.Class = 'MGS_DiskDrive'
              AND s.HardwareName LIKE '%[a-z][0-9]%') ld
      LEFT JOIN DiskDriveBase d
        ON ld.ComputerID = d.ComputerID
          AND d.DiskDriveID = ld.DiskDriveID
GO

