USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetDeviceMemoryModul' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetDeviceMemoryModul]
GO
-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 16.01.2019
-- Last change: 
-- Copyright:   (c) Raynet GmbH
-- Description:	Get All Memory Modules
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_GetDeviceMemoryModul]()
  RETURNS TABLE
AS
RETURN
  SELECT
      sz.ComputerID,
      IIF(ml.BankLocator IS NULL, 
          CAST(ROW_NUMBER() OVER (PARTITION BY sz.ComputerID 
                                  ORDER BY sz.ComputerID, sz.HardwareObjectID ASC) AS NVARCHAR(256)), 
          ml.BankLocator)                                         AS BankLocator,
      mm.Value                                                    AS Manufacturer,
	  sz.Size,
      dbo.CIMInteger(IIF(msp.Value = 'Unknown', NULL, msp.Value)) AS Speed,
      mt.Value                                                    AS Type,
	  mtd.TypeDetail,
      msn.SerialNumber,
      mpt.Value                                                   AS PartNumber
    FROM (SELECT
              COALESCE(mc.ComputerID,       ms.ComputerID)        AS ComputerID,
              COALESCE(mc.HardwareObjectID, ms.HardwareObjectID)  AS HardwareObjectID,
              COALESCE(mc.HardwareName,     ms.HardwareName)      AS HardwareName,
              CASE
                WHEN mc.Property = 'Capacity' THEN dbo.CIMBigInteger(mc.Value)
                WHEN CHARINDEX('MB', ms.Value) > 0 THEN dbo.CIMBigInteger(LEFT(ms.Value , CHARINDEX('MB', ms.Value) -2)) *1024 *1024
                WHEN CHARINDEX('KB', ms.Value) > 0 THEN dbo.CIMBigInteger(LEFT(ms.Value , CHARINDEX('MB', ms.Value) -2)) *1024
                WHEN CHARINDEX('GB', ms.Value) > 0 THEN dbo.CIMBigInteger(LEFT(ms.Value , CHARINDEX('MB', ms.Value) -2))
                WHEN ms.Value LIKE '%[^0-9]%' THEN dbo.CIMBigInteger(NULL)
                ELSE dbo.CIMBigInteger(ms.Value)
              END                                                 AS [Size]
		    FROM (SELECT
                      ComputerID, 
                      HardwareObjectID,
                      HardwareName,
                      Property,
                      Value
                    FROM dbo.f_HardwareProperties('CIM_PhysicalMemory') 
                    WHERE Property = 'Capacity') mc
              FULL OUTER JOIN (SELECT
                                   ComputerID, 
                                   HardwareObjectID, 
                                   HardwareName,
                                   Property, 
                                   Value
                                 FROM dbo.f_HardwareProperties('CIM_PhysicalMemory') 
                                 WHERE Property = 'Size') ms
                ON mc.ComputerID = ms.ComputerID) sz
      LEFT JOIN (SELECT
                     HardwareObjectID,
                     MAX(Value)          AS BankLocator
                   FROM dbo.f_HardwareProperties('CIM_PhysicalMemory')
                   WHERE Property IN ('BankLocator', 'Bank Locator', 'Locator')
                   GROUP BY ComputerID, HardwareObjectID) ml
        ON sz.HardwareObjectID = ml.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Manufacturer') mm
        ON sz.HardwareObjectID = mm.HardwareObjectID
      LEFT JOIN (SELECT
                     HardwareObjectID,
                     MAX(Value)          AS SerialNumber
                   FROM dbo.f_HardwareProperties('CIM_PhysicalMemory')
                   WHERE Property IN ('SerialNumber', 'Serial Number')
                   GROUP BY ComputerID, HardwareObjectID) msn
        ON sz.HardwareObjectID = msn.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Speed') msp
        ON sz.HardwareObjectID = msp.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Type') mt
        ON sz.HardwareObjectID = mt.HardwareObjectID
      LEFT JOIN (SELECT
                     HardwareObjectID,
                     MAX(Value)          AS TypeDetail
                   FROM dbo.f_HardwareProperties('CIM_PhysicalMemory')
                   WHERE Property IN ('TypeDetail', 'Type Detail')
                   GROUP BY ComputerID, HardwareObjectID) mtd
        ON sz.HardwareObjectID = mtd.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('PartType') mpt
        ON sz.HardwareObjectID = mpt.HardwareObjectID
GO

