USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetDeviceVideoController' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetDeviceVideoController]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 25.01.2019
-- Last change: 
-- Copyright:   (c) Raynet GmbH
-- Description:	Get All Graphicscard
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_GetDeviceVideoController]()
  RETURNS TABLE
AS
RETURN
  SELECT
      vc.ComputerID,
      vc.HardwareObjectID         AS CIMPCVideoControllerID,
      CASE
        WHEN vc.Value LIKE '%Microsoft%' THEN 'Microsoft'
        WHEN vc.Value LIKE '%VMware%'    THEN 'VMware'
        WHEN vc.Value LIKE '%Intel%'     THEN 'Intel'
        WHEN vc.Value LIKE '%AMD%'       THEN 'AMD'
        WHEN vc.Value LIKE '%NVidia%'    THEN 'NVidia'
        WHEN vc.Value LIKE '%ATI%'       THEN 'ATI'
        WHEN vc.Value LIKE '%Matrox%'    THEN 'Matrox'
        WHEN vc.Value LIKE '%Asus%'      THEN 'Asus'
        WHEN vc.Value LIKE '%Gainward%'  THEN 'Gainward'
        WHEN vc.Value LIKE '%GigaByte%'  THEN 'GigaByte'
        WHEN vc.Value LIKE '%3dfx%'      THEN '3dfx'
        WHEN vc.Value LIKE '%S3%'        THEN 'S3'
        WHEN vc.Value LIKE '%SIS%'       THEN 'SIS'
        WHEN vc.Value LIKE '%3dfx%'      THEN '3dfx'
        WHEN vp.Value LIKE '%ATI%'       THEN 'ATI'
        WHEN vp.Value LIKE '%RAGE XL%'   THEN 'ATI'
        WHEN vp.Value LIKE '%RADEON% '   THEN 'ATI'
        WHEN vc.Value LIKE 'GXT[46]500%' THEN 'IBM'
        WHEN vc.Value LIKE '%ASPEED%'    THEN 'ASPEED Technology'
        WHEN vc.Value LIKE '%Novell%'    THEN 'Novell'
        WHEN vc.Value LIKE '%VNC%'       THEN 'VNC'
      END                         AS Manufacturer,
      vc.Value                    AS Model,
      vp.Value                    AS VideoProcessor,
      ar.Value                    AS AdapterRAM,
      dv.Value                    AS DriverVersion,
      dbo.f_SQLDateTime(dd.Value) AS DriverDate,
      CASE
        WHEN vc.Value LIKE '%Remote%'               THEN 1
        WHEN vc.Value = 'Mirage Driver'             THEN 1
        WHEN vc.Value LIKE '%NVC%'                  THEN 1
        WHEN vc.Value LIKE '%mv video hook driver%' THEN 1
        ELSE 0
      END                         AS ISRemoteGraphicDriver
    FROM dbo.f_HardwareProperties('CIM_PCVideoController') vc
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('AdapterRAM') ar
        ON vc.HardwareObjectID = ar.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('VideoProcessor') vp
        ON vc.HardwareObjectID = vp.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('DriverDate') dd
        ON vc.HardwareObjectID = dd.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('DriverVersion') dv
        ON vc.HardwareObjectID = dv.HardwareObjectID
    WHERE vc.Property = 'Name'
GO

