USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetDeviceInfo' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetDeviceInfo]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 21.01.2019
-- Last change: 
-- Copyright:   (c) Raynet GmbH
-- Description:	Get Device Info
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_GetDeviceInfo] ()
  RETURNS TABLE
AS
RETURN
  SELECT
      m.ComputerID        AS ComputerID,
      m.Manufacturer      AS Manufacturer,
      m.Model             AS Model,
      s.SerialNumber      AS SerialNumber,
      IIF(LEN(uuid.Value) NOT IN (36,38)
            OR uuid.Value LIKE '%[^-0-9a-fA-F]%', NULL, uuid.Value)
                          AS UUID,
      ch.TypeID           AS ChassisTypeID,
      ch.Type             AS ChassisType,
      bV.Value            AS BiosVersion,
      m.Virtual           AS Virtual
    FROM dbo.f_GetDeviceModel() m
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('UUID') uuid
        ON m.CIMProductID = uuid.HardwareObjectID 
      LEFT JOIN dbo.f_GetDeviceSerialNumber() s
        ON m.ComputerID = s.ComputerID 
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Version') bV
        ON s.CIMBIOSElementID = bV.HardwareObjectID
      LEFT JOIN dbo.f_GetDeviceChassis(0) ch
        ON m.ComputerID = ch.ComputerID
GO


