USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetDeviceOSLastLoginUser' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetDeviceOSLastLoginUser]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 15.01.2019
-- Last change: 
-- Copyright:   (c) Raynet GmbH
-- Description:	Get all Last OS Login User
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================


CREATE FUNCTION [dbo].[f_GetDeviceOSLastLoginUser] ()
  RETURNS TABLE
AS
RETURN
  WITH UserInv AS (
    SELECT
        ir.ComputerID,
        IIF(o.IsDomain = 1, d.FlatName, NULL)  AS UserDomain,
        u.UserCN                               AS UserName,
        ir.LastInventoryDate
      FROM dbo.f_GetLastInventoryDate(2) ir
        LEFT JOIN dbo.[User] u WITH (NOLOCK)
          ON ir.UserID = u.UserID 
        LEFT JOIN dbo.[Organization] o WITH (NOLOCK)
          ON u.UserOUID = o.OrganizationID
        LEFT JOIN dbo.[Domain] d WITH (NOLOCK)
          ON o.DomainID = d.OrganizationID
    UNION SELECT
        u.ComputerID,
        CASE
           WHEN CHARINDEX('\', u.Value) > 0 THEN LEFT(u.Value, CHARINDEX('\', u.Value) -1)
           WHEN CHARINDEX('@', u.Value) > 0 THEN RIGHT(u.Value, CHARINDEX('@', REVERSE(u.Value)) -1)
         END   AS UserDomain,
         CASE
           WHEN CHARINDEX('\', u.Value) > 0 THEN RIGHT(u.Value, CHARINDEX('\', REVERSE(u.Value)) -1)
           WHEN CHARINDEX('@', u.Value) > 0 THEN LEFT(u.Value, CHARINDEX('@', u.Value) -1)
           ELSE u.Value
         END   AS UserName,
        ir.LastInventoryDate
       FROM dbo.f_HardwareProperties('CIM_UnitaryComputerSystem') u
         LEFT JOIN dbo.f_GetLastInventoryDate(1) ir
           ON u.ComputerID = ir.ComputerID
       WHERE u.Property = 'UserName')
  SELECT DISTINCT
      ui.ComputerID,
	  ui.UserDomain,
	  ui.UserName,
	  ui.LastInventoryDate
    FROM UserInv ui WITH (NOLOCK) 
    WHERE ui.LastInventoryDate IN (SELECT TOP 1 LastInventoryDate
                                     FROM UserInv uiTmp WITH (NOLOCK) 
                                     WHERE uiTmp.ComputerID = ui.ComputerID
                                     ORDER BY uiTmp.LastInventoryDate DESC)
GO

