USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetDeviceOSInfo' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetDeviceOSInfo]

GO
-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 22.01.2019
-- Last change: 
-- Copyright:   (c) Raynet GmbH
-- Description:	Get Base Operating System Information from All Systems
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================


CREATE FUNCTION [dbo].[f_GetDeviceOSInfo] ()
  RETURNS TABLE
AS

RETURN
  SELECT
      osName.ComputerID                                     AS [ComputerID],
--    opCSName.Value                                        AS [ComputerName],
      CASE
        WHEN osName.OSName      LIKE '%Server%'  THEN 'Server'
        WHEN osName.OSName      LIKE '%Windows%' THEN 'Client'
        WHEN osName.OSName      LIKE '%MAC%'     THEN 'Client'
        WHEN osName.OSName      LIKE '%Desktop%' THEN 'Client'
        WHEN osName.ProductName LIKE '%Server%'  THEN 'Server'
        WHEN osName.ProductName LIKE '%Desktop%' THEN 'Client'
        ELSE 'Server'
      END                                                   AS [SystemType],
      CASE
        WHEN osName.HardwareName LIKE '%Windows%' THEN 'Windows'
        ELSE osName.HardwareName
      END                                                   AS [OSType],
      CASE
        WHEN osMan.Value  !=   'Unknown'   THEN osMan.Value 
        WHEN osName.OSName LIKE '%HP-UX%'   THEN 'Hewlett-Packard'
        WHEN osName.OSName LIKE '%Mac OS%'  THEN 'Apple Inc.'
        WHEN osName.OSName LIKE '%AIX%'     THEN 'IBM'
        WHEN osName.OSName LIKE '%SunOS%'   THEN 'Sun Microsystems'
        WHEN osName.OSName LIKE '%RedHat%'  THEN 'Red Hat'
        WHEN osName.OSName LIKE '%Red Hat%' THEN 'Red Hat'
        WHEN osName.OSName LIKE '%Oracle%'  THEN 'Oracle'
        WHEN osName.OSName LIKE '%CentOS%'  THEN 'CentOS'
        WHEN osName.OSName LIKE '%SuSE%'    THEN 'Novell'
        WHEN osName.OSName LIKE '%Ubuntu%'  THEN 'Ubuntu'
        WHEN osName.OSName LIKE '%Debian%'  THEN 'Debian'
        WHEN osName.OSName LIKE '%ESX%'     THEN 'VMware'
        ELSE 'Unknown'
      END                                                   AS [Manufacturer],
      osName.ProductName                                    AS [ProductName],
      osVersion.Value                                       AS [Version],
      CASE
        WHEN osName.OSName LIKE '%2000%'       THEN '2000'
        WHEN osName.OSName LIKE '%2003 R2%'
          OR osName.OSName LIKE '%2003R2%'     THEN '2003 R2'
        WHEN osName.OSName LIKE '%2003%'       THEN '2003'
        WHEN osName.OSName LIKE '%2008 R2%'
          OR osName.OSName LIKE '%2008R2%'     THEN '2008 R2'
        WHEN osName.OSName LIKE '%2008%'       THEN '2008'
        WHEN osName.OSName LIKE '%2012 R2%'
          OR osName.OSName LIKE '%2012R2%'     THEN '2012 R2'
        WHEN osName.OSName LIKE '%2012%'       THEN '2012'
        WHEN osName.OSName LIKE '%2016%'       THEN '2016'
        WHEN osName.OSName LIKE '%NT%  '       THEN 'NT'
        ELSE osVersion.Value
      END                                                   AS [Release],
      opKernelV.Value                                       AS [KernelVersion],
      CASE
        WHEN osName.OSName LIKE '%Enterprise%'                    THEN 'Enterprise'
        WHEN osName.OSName LIKE '%Datacenter%'                    THEN 'Datacenter'
        WHEN osName.OSName LIKE '%Advanced%'                      THEN 'Advanced'
        WHEN osName.OSName LIKE '%Standard%'                      THEN 'Standard'
        WHEN osName.OSName LIKE 'Microsoft Windows 2000 Server'   THEN ''
        WHEN osName.OSName LIKE '%Small Business%'                THEN 'Small Business'
        WHEN osName.OSName LIKE '%Home%'                          THEN 'Home'
        WHEN osName.OSName LIKE '%Embedded Systems%'              THEN 'Embedded Systems'
        WHEN osName.OSName LIKE '%Compute Cluster%'               THEN 'Compute Cluster' 
        WHEN osName.OSName LIKE '%Storage%'                       THEN 'Storage' 
        WHEN osName.OSName LIKE '%Essential Business%'            THEN 'Essential Business' 
        WHEN osName.OSName LIKE '%Essentials%'                    THEN 'Essentials' 
        WHEN osName.OSName LIKE '%Itanium%'                       THEN 'Itanium' 
        WHEN osName.OSName LIKE '%Hyper-V%'                       THEN 'Hyper-V' 
        WHEN osName.OSName LIKE '%MultiPoint Premium%'            THEN 'MultiPoint Premium' 
        WHEN osName.OSName LIKE '%Foundation%'                    THEN 'Foundation' 
        WHEN osName.OSName LIKE '%Web%'                           THEN 'Web' 
        WHEN osName.OSName LIKE '%HPC%'                           THEN 'HPC' 
        WHEN osName.ProductName LIKE '%Enterprise%'               THEN 'Enterprise'
        WHEN osName.ProductName LIKE '%Standard%'                 THEN 'Standard'
        ELSE 'Unknown'
      END                                                   AS [Edition],
      opServicePack.Value                                   AS [ServicePack],
      opArchitecture.Value                                  AS [Architecture],
      opLang.Language,
      opLicKey.Value                                        AS [LicenseKey],
      dbo.f_SQLDateTime(opInstallDate.Value)                AS [InstallDate],
      dbo.f_SQLDateTime(opLastBoot.Value)                   AS [LastBoot],
      ISNULL(lLogin.UserDomain + '\', '') +lLogin.UserName  AS [LastLogin]
    FROM (SELECT DISTINCT
              os.ComputerID, 
              MAX(os.HardwareObjectID)            AS HardwareObjectID, 
              MAX(HardwareName)                   AS HardwareName, 
              MAX(osN.Value)                      AS OSName,
              MAX(COALESCE(osC.Value, osN.Value)) AS ProductName
            FROM dbo.f_HardwareProperties('CIM_OperatingSystem') os
              LEFT JOIN dbo.f_HardwareValuesByPropertyName('Name') osN
                ON os.HardwareObjectID = osN.HardwareObjectID
              LEFT JOIN dbo.f_HardwareValuesByPropertyName('Caption') osC
                ON os.HardwareObjectID = osC.HardwareObjectID
            WHERE os.Property IN ('Caption', 'Name') AND os.Value IS NOT NULL
            GROUP BY os.ComputerID) osName
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Manufacturer') osMan
        ON osName.HardwareObjectID = osMan.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Version') osVersion
        ON osName.HardwareObjectID = osVersion.HardwareObjectID
--    LEFT JOIN dbo.f_HardwareValuesByPropertyName('CSNAME') opCSName
--      ON osName.HardwareObjectID = opCSName.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('ServicePackMajorVersion') opServicePack
        ON osName.HardwareObjectID = opServicePack.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('OSArchitecture') opArchitecture
        ON osName.HardwareObjectID = opArchitecture.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('KernelRelease') opKernelV
        ON osName.HardwareObjectID = opKernelV.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('InstallDate') opInstallDate
        ON osName.HardwareObjectID = opInstallDate.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('LastBootUpTime') opLastBoot
        ON osName.HardwareObjectID = opLastBoot.HardwareObjectID
      LEFT JOIN dbo.f_HardwareProperties('CIM_OperatingSystem') opLicKey
        ON osName.ComputerID = opLicKey.ComputerID
          AND opLicKey.Property = 'LicenseKey' AND opLicKey.Value IS NOT NULL
      LEFT JOIN dbo.f_GetDeviceOSLastLoginUser() lLogin
        ON osName.ComputerID = lLogin.ComputerID
      LEFT JOIN dbo.f_GetDeviceOSLanguage() opLang
        ON osName.ComputerID = opLang.ComputerID

GO
