USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetVirtualVMwareGuestInfo' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetVirtualVMwareGuestInfo]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 22.01.2019
-- Last change: 
-- Copyright:   (c) Raynet GmbH
-- Description:	Get Guest Device Info from VMware Side
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_GetVirtualVMwareGuestInfo] ()
  RETURNS TABLE
AS
RETURN
  SELECT
      e.ComputerID                        AS ESXHostID,
      e.HardwareObjectID                  AS VMWAREVMComputerSystemID,
      e.Value                             AS ElementName,
      h.Value                             AS HostName,
      ip.Value                            AS IPAddress,
      CAST(m.Value AS BIGINT) *1024 *1024 AS Memory,
      CAST(cp.Value AS INT)               AS CPUs,
      os.Value                            AS GuestOS,
      ps.Value                            AS PowerState,
      a.Value                             AS Annotation
    FROM f_HardwareProperties('VMWARE_VMComputerSystem') e
      LEFT JOIN f_HardwareValuesByPropertyName('HostName') h
        ON e.HardwareObjectID = h.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('IPAddress') ip
        ON e.HardwareObjectID = ip.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('memorySizeMB') m
        ON e.HardwareObjectID = m.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('numCpu') cp
        ON e.HardwareObjectID = cp.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('guestFullName') os
        ON e.HardwareObjectID = os.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('annotation') a
        ON e.HardwareObjectID = a.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('PowerState') ps
        ON e.HardwareObjectID = ps.HardwareObjectID
    WHERE e.Property = 'ElementName'
GO
