USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetVirtualVMwareLinkVCenter' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetVirtualVMwareLinkVCenter]
GO
-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 18.01.2019
-- Last change: 
-- Copyright:   (c) Raynet GmbH
-- Description:	Get vCenter Link info
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================


CREATE FUNCTION [dbo].[f_GetVirtualVMwareLinkVCenter] ()
  RETURNS TABLE
AS
RETURN
   SELECT
      vCenterFQDN.ComputerID                     AS vCenterID,
	  vCenterFQDN.ComputerCN                     AS vCenterCN,
      COALESCE(vCenterFQDN.FQDN, vCenter.Server) AS vCenterFQDN,
      vCenter.ServerURL                          AS vCenterURLAddress,
      vCenter.UUID                               AS vCenterUUID
    FROM (SELECT DISTINCT
              vC.Value                               AS ServerURL,
              TRY_CAST(vU.Value AS UNIQUEIDENTIFIER) AS UUID,
             REPLACE(REPLACE(REPLACE(REPLACE(vC.Value, '/sdk', ''), 'https://', ''), 'http://', ''), '/', '')
                       AS [Server]
            FROM dbo.f_HardwareProperties('VMWARE_InfrastructureSDK') vC
              LEFT JOIN dbo.f_HardwareValuesByPropertyName('Server') vU
                ON vC.HardwareObjectID = vU.HardwareObjectID
            WHERE vC.Property = 'ServerURL') vCenter
      LEFT JOIN dbo.f_GetDeviceNetworkInterface() nic
        ON vCenter.Server = nic.IPAddress
      LEFT JOIN dbo.f_GetDeviceFQDN(0) fqdn
        ON vCenter.Server = fqdn.FQDN
          OR vCenter.Server = fqdn.ComputerCN
      LEFT JOIN dbo.f_GetDeviceFQDN(0) vCenterFQDN
        ON COALESCE(fqdn.ComputerID, nic.ComputerID) = vCenterFQDN.ComputerID
GO


