USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_GetVirtualVMwareLinkGuest' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_GetVirtualVMwareLinkGuest]
GO
-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 22.01.2019
-- Last change: 04.04.2019
-- Copyright:   (c) Raynet GmbH
-- Description:	Get ESX Guest Link
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================


CREATE PROCEDURE [dbo].[sp_GetVirtualVMwareLinkGuest]
AS
BEGIN
  DECLARE @bShowTable   BIT    = 1

  IF OBJECT_ID('tempdb..#sp_GetVirtualVMwareLinkGuest') IS NOT NULL
  BEGIN
    TRUNCATE TABLE #sp_GetVirtualVMwareLinkGuest
    SET @bShowTable = 0
  END
  ELSE
    CREATE TABLE #sp_GetVirtualVMwareLinkGuest (
      ESXHostID                BIGINT                                    NOT NULL,
      GuestID                  BIGINT                                    NULL,
      GuestUUID                UNIQUEIDENTIFIER                          NULL,
      VMWAREVMComputerSystemID BIGINT                                    NOT NULL,
      vCenterURLAddress        NVARCHAR(256)    COLLATE database_default NULL,
      vCenterUUID              UNIQUEIDENTIFIER                          NULL)

  IF OBJECT_ID('tempdb..#f_GetDeviceFQDN_VMLink') IS NOT NULL
    DROP TABLE #f_GetDeviceFQDN_VMLink

  CREATE TABLE #f_GetDeviceFQDN_VMLink (
    ComputerID               BIGINT                                      NOT NULL,
	ComputerCN               NVARCHAR(256)      COLLATE database_default NOT NULL,
    FQDN                     NVARCHAR(256)      COLLATE database_default NOT NULL)

  CREATE CLUSTERED INDEX IDX_C_#f_GetDeviceFQDN_VMLink_ComputerID_fqdn ON #f_GetDeviceFQDN_VMLink(ComputerID, FQDN)

  INSERT INTO #f_GetDeviceFQDN_VMLink
  SELECT * FROM dbo.f_GetDeviceFQDN(0)

  IF OBJECT_ID('tempdb..#f_GetDeviceVirtualVMwareUUID_VMLink') IS NOT NULL
    DROP TABLE #f_GetDeviceVirtualVMwareUUID_VMLink

  CREATE TABLE #f_GetDeviceVirtualVMwareUUID_VMLink (
    GuestID                  BIGINT                                      NOT NULL,
	CIMBIOSElementID         BIGINT                                          NULL,
    UUID                     UNIQUEIDENTIFIER                            NOT NULL)

  CREATE CLUSTERED INDEX IDX_C_#f_GetDeviceVirtualVMwareUUID_VMLink_GuestID_UUID ON #f_GetDeviceVirtualVMwareUUID_VMLink(GuestID, UUID)

  INSERT INTO #f_GetDeviceVirtualVMwareUUID_VMLink
  SELECT * FROM f_GetDeviceVirtualVMwareUUID()
    WHERE UUID IS NOT NULL

  IF OBJECT_ID('tempdb..#f_GetVirtualVMwareGuestUUID_VMLink') IS NOT NULL
    DROP TABLE #f_GetVirtualVMwareGuestUUID_VMLink

  CREATE TABLE #f_GetVirtualVMwareGuestUUID_VMLink (
    ESXHostID                BIGINT                                      NOT NULL,
	VMWAREVMComputerSystemID BIGINT                                      NOT NULL,
    UUID                     UNIQUEIDENTIFIER                            NOT NULL)

  CREATE CLUSTERED INDEX IDX_C_#f_GetVirtualVMwareGuestUUID_VMLink_ESXHostID_UUID ON #f_GetVirtualVMwareGuestUUID_VMLink(ESXHostID, UUID)

  INSERT INTO #f_GetVirtualVMwareGuestUUID_VMLink
  SELECT * FROM f_GetVirtualVMwareGuestUUID()
    WHERE UUID IS NOT NULL

  INSERT INTO #sp_GetVirtualVMwareLinkGuest (ESXHostID, GuestID, VMWAREVMComputerSystemID)
  SELECT
      esx.ESXHostID,
      guest.GuestID,
      esx.VMWAREVMComputerSystemID 
    FROM #f_GetVirtualVMwareGuestUUID_VMLink esx
      LEFT JOIN #f_GetDeviceVirtualVMwareUUID_VMLink guest
        ON esx.UUID = guest.UUID

  INSERT INTO #sp_GetVirtualVMwareLinkGuest (ESXHostID, GuestID, VMWAREVMComputerSystemID)
  SELECT
      esx.ComputerID        AS ESXHostID,
      fqdn.ComputerID       AS GuestID,
      esx.HardwareObjectID  AS VMWAREVMComputerSystemID 
    FROM dbo.f_HardwareProperties('VMWARE_VMComputerSystem') esx
      LEFT JOIN #f_GetDeviceFQDN_VMLink fqdn
        ON esx.Value = fqdn.FQDN
    WHERE esx.Property = 'HostName' AND fqdn.ComputerID <> esx.ComputerID

  IF OBJECT_ID('tempdb..#f_GetDeviceNetworkInterface_VMLink') IS NOT NULL
    DROP TABLE #f_GetDeviceNetworkInterface_VMLink

  CREATE TABLE #f_GetDeviceNetworkInterface_VMLink (
    GuestID                  BIGINT                                      NOT NULL,
    GuestCN                  NVARCHAR(256)      COLLATE database_default NULL,
    IPAddress                NVARCHAR(124)      COLLATE database_default NULL)

  CREATE CLUSTERED INDEX IDX_C_#f_GetDeviceNetworkInterface_VMLink_GuestID_GuestCN_IPAddress ON #f_GetDeviceNetworkInterface_VMLink(GuestID, GuestCN, IPAddress)

  INSERT INTO #f_GetDeviceNetworkInterface_VMLink(GuestID, GuestCN, IPAddress)
  SELECT
      c.ComputerID,
      c.ComputerCN,
      LEFT(nic.IPAddress,124) AS IPAddress
    FROM dbo.f_GetDeviceNetworkInterface() nic
      LEFT JOIN #f_GetDeviceFQDN_VMLink c
        ON nic.ComputerID = c.ComputerID
    WHERE nic.IPEnabled = 1
      AND nic.ComputerID NOT IN (SELECT GuestID FROM #sp_GetVirtualVMwareLinkGuest)

  IF OBJECT_ID('tempdb..#tmp_VMComputerSystem_VMLink') IS NOT NULL
    DROP TABLE #tmp_VMComputerSystem_VMLink

  CREATE TABLE #tmp_VMComputerSystem_VMLink (
    ESXHostID                BIGINT                                      NOT NULL,
    VMWAREVMComputerSystemID BIGINT                                      NULL,
    Elementname              NVARCHAR(256)      COLLATE database_default NULL,
    IPAddress                NVARCHAR(124)      COLLATE database_default NULL)

  CREATE CLUSTERED INDEX IDX_C_#tmp_VMComputerSystem_VMLink_ESXHostID_Elementname_IPAddress ON #tmp_VMComputerSystem_VMLink(ESXHostID, Elementname, IPAddress)

  INSERT INTO #tmp_VMComputerSystem_VMLink(ESXHostID, VMWAREVMComputerSystemID, Elementname, IPAddress)
  SELECT
      esx.ComputerID        AS ESXHostID,
      esx.HardwareObjectID  AS VMWAREVMComputerSystemID,
      esx.Value             AS Elementname,
      LEFT(vmIP.Value,124)  AS IPAddress
    FROM dbo.f_HardwareProperties('VMWARE_VMComputerSystem') esx
      INNER JOIN dbo.f_HardwareValuesByPropertyName('IPAddress') vmIP
        ON vmIP.HardwareObjectID = esx.HardwareObjectID
    WHERE esx.Property = 'Elementname'
      AND esx.HardwareObjectID NOT IN (SELECT VMWAREVMComputerSystemID FROM #sp_GetVirtualVMwareLinkGuest)

  INSERT INTO #sp_GetVirtualVMwareLinkGuest (ESXHostID, GuestID, VMWAREVMComputerSystemID)
  SELECT
      esx.ESXHostID,
      nic.GuestID,
      esx.VMWAREVMComputerSystemID 
    FROM #tmp_VMComputerSystem_VMLink esx
      INNER JOIN #f_GetDeviceNetworkInterface_VMLink nic
        ON esx.Elementname = nic.GuestCN
          AND esx.IPAddress = nic.IPAddress
    WHERE esx.ESXHostID <> nic.GuestID

  UPDATE rel
      SET vCenterURLAddress = vC.Value,
          vCenterUUID       = TRY_CAST(vU.Value AS UNIQUEIDENTIFIER)
    FROM #sp_GetVirtualVMwareLinkGuest rel
      INNER JOIN dbo.f_HardwareProperties('VMWARE_InfrastructureSDK') vC
        ON vC.Property = 'ServerURL'
          AND rel.ESXHostID = vc.ComputerID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Server') vU
        ON vC.HardwareObjectID = vU.HardwareObjectID

  UPDATE rel
      SET GuestUUID = gU.UUID
    FROM #sp_GetVirtualVMwareLinkGuest rel
      INNER JOIN #f_GetVirtualVMwareGuestUUID_VMLink gU
        ON rel.VMWAREVMComputerSystemID = gU.VMWAREVMComputerSystemID

  IF @bShowTable = 1
  BEGIN
    SELECT DISTINCT * FROM #sp_GetVirtualVMwareLinkGuest
    DROP TABLE #sp_GetVirtualVMwareLinkGuest
  END

  DROP TABLE #f_GetDeviceFQDN_VMLink
  DROP TABLE #f_GetDeviceVirtualVMwareUUID_VMLink
  DROP TABLE #f_GetVirtualVMwareGuestUUID_VMLink
  DROP TABLE #f_GetDeviceNetworkInterface_VMLink
  DROP TABLE #tmp_VMComputerSystem_VMLink
END
GO

