USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_GetVirtualHyperVLinkGuest' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_GetVirtualHyperVLinkGuest]
GO
-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 30.01.2019
-- Last change: 05.04.2019
-- Copyright:   (c) Raynet GmbH
-- Description:	Get HyperV Guest Link
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

-- Parameter bWithSerial
--   0 or null GuestFQDN = GuestFQDN (From Host Inventory)
--             HostFQDN  = HostFQDN  (From Guest Inventory)
--             BIOSSerialNumber only if one Serialnumber in Guest Inventies
--   1         Match allways used SerialNumber

CREATE PROCEDURE [dbo].[sp_GetVirtualHyperVLinkGuest] (@bWithSerial BIT = 0)
AS
BEGIN
--  DECLARE @bWithSerial  BIT    = 0
  DECLARE @bDebug       BIT    = 1
  DECLARE @bShowTable   BIT    = 1

  IF OBJECT_ID('tempdb..#sp_GetVirtualHyperVLinkGuest') IS NOT NULL
  BEGIN
    TRUNCATE TABLE #sp_GetVirtualHyperVLinkGuest
    SET @bShowTable = 0
  END
  ELSE
    CREATE TABLE #sp_GetVirtualHyperVLinkGuest (
      HyperVHostID   BIGINT                                    NOT NULL,
      HyperVHostCN   NVARCHAR(256)    COLLATE database_default NULL,
      HyperVHostFQDN NVARCHAR(256)    COLLATE database_default NULL,
      GuestID        BIGINT                                    NULL,
      GuestCN        NVARCHAR(256)    COLLATE database_default NULL,
      GuestFQDN      NVARCHAR(256)    COLLATE database_default NULL,
      PowerState     NVARCHAR(30)     COLLATE database_default NULL)

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert into #f_GetDeviceFQDN_HyperVLink'
  IF OBJECT_ID('tempdb..#f_GetDeviceFQDN_HyperVLink') IS NOT NULL
    DROP TABLE #f_GetDeviceFQDN_HyperVLink

  CREATE TABLE #f_GetDeviceFQDN_HyperVLink (
    ComputerID               BIGINT                                      NOT NULL,
    ComputerCN               NVARCHAR(256)      COLLATE database_default NOT NULL,
    FQDN                     NVARCHAR(256)      COLLATE database_default NULL)

  INSERT INTO #f_GetDeviceFQDN_HyperVLink
  SELECT * FROM dbo.f_GetDeviceFQDN(1)

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert into #f_GetVirtualHyperVGuestInfo_HyperVLink'
  IF OBJECT_ID('tempdb..#f_GetVirtualHyperVGuestInfo_HyperVLink') IS NOT NULL
    DROP TABLE #f_GetVirtualHyperVGuestInfo_HyperVLink

  CREATE TABLE #f_GetVirtualHyperVGuestInfo_HyperVLink (
    HyperVHostID             BIGINT                                      NOT NULL,
    FQDN                     NVARCHAR(256)      COLLATE database_default NULL,
    PowerState               NVARCHAR(30)       COLLATE database_default NULL,
    PowerStateID             INT                                         NOT NULL)

  INSERT INTO #f_GetVirtualHyperVGuestInfo_HyperVLink(HyperVHostID, FQDN, PowerState, PowerStateID)
  SELECT
     g.HyperVHostID,
     g.FQDN,
     g.PowerState,
     g.PowerStateID 
   FROM dbo.f_GetVirtualHyperVGuestInfo() g

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert into #f_GetVirtualHyperVGuestInfo'
  IF OBJECT_ID('tempdb..#f_GetVirtualHyperVGuestInfo') IS NOT NULL
    DROP TABLE #f_GetVirtualHyperVGuestInfo

  CREATE TABLE #f_GetVirtualHyperVGuestInfo (
    HyperVHostID             BIGINT                                      NOT NULL,
    SerialNumber             NVARCHAR(256)      COLLATE database_default NOT NULL,
    PowerStateID             INT                                         NOT NULL)

  CREATE NONCLUSTERED INDEX IDX_C_#f_GetVirtualHyperVGuestInfo_SerialNumber  ON #f_GetVirtualHyperVGuestInfo(SerialNumber)

  INSERT INTO #f_GetVirtualHyperVGuestInfo(HyperVHostID, SerialNumber, PowerStateID)
  SELECT g.HyperVHostID, g.BIOSSerialNumber, g.PowerStateID
    FROM dbo.f_GetVirtualHyperVGuestInfo() g
    WHERE g.BIOSSerialNumber IS NOT NULL

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert into #f_GetDeviceSerialNumber'
  IF OBJECT_ID('tempdb..#f_GetDeviceSerialNumber') IS NOT NULL
    DROP TABLE #f_GetDeviceSerialNumber

  CREATE TABLE #f_GetDeviceSerialNumber (
    GuestID                  BIGINT                                      NOT NULL,
    SerialNumber             NVARCHAR(256)      COLLATE database_default NOT NULL)

  CREATE NONCLUSTERED INDEX IDX_C_#f_GetDeviceSerialNumber_SerialNumber  ON #f_GetDeviceSerialNumber(SerialNumber)

  INSERT INTO #f_GetDeviceSerialNumber(GuestID, SerialNumber)
  SELECT ser.ComputerID AS GuestID, ser.SerialNumber
    FROM dbo.f_GetDeviceSerialNumber() ser
    WHERE ser.ComputerID > 0


  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert into #tmpSerial'
  IF OBJECT_ID('tempdb..#tmpSerial') IS NOT NULL
    DROP TABLE #tmpSerial

  CREATE TABLE #tmpSerial (
    HyperVHostID             BIGINT                                      NOT NULL,
    GuestID                  BIGINT                                      NOT NULL,
    SerialNumber             NVARCHAR(256)      COLLATE database_default NOT NULL,
    PowerStateID             INT                                         NOT NULL)

  INSERT INTO #tmpSerial (HyperVHostID, GuestID, SerialNumber, PowerStateID)
  SELECT g.HyperVHostID, ser.GuestID, ser.SerialNumber, g.PowerStateID
    FROM #f_GetVirtualHyperVGuestInfo g
      INNER JOIN #f_GetDeviceSerialNumber ser
        ON g.SerialNumber = ser.SerialNumber

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert into #tmpSerialON'
  IF OBJECT_ID('tempdb..#tmpSerialON') IS NOT NULL
    DROP TABLE #tmpSerialON

  CREATE TABLE #tmpSerialON (
    HyperVHostID             BIGINT                                      NOT NULL,
    GuestID                  BIGINT                                      NOT NULL,
    SerialNumber             NVARCHAR(256)      COLLATE database_default NOT NULL,
    PowerStateID             INT                                         NOT NULL)

  INSERT INTO #tmpSerialON(HyperVHostID, GuestID, SerialNumber, PowerStateID)
  SELECT HyperVHostID, MAX(GuestID) AS GuestID, SerialNumber, PowerStateID
    FROM #tmpSerial
    WHERE PowerStateID = 2
    GROUP BY HyperVHostID, SerialNumber, PowerStateID
    HAVING COUNT(GuestID) = 1

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert into #tmpSerialOff'
  IF OBJECT_ID('tempdb..#tmpSerialOFF') IS NOT NULL
    DROP TABLE #tmpSerialOFF

  CREATE TABLE #tmpSerialOFF (
    HyperVHostID             BIGINT                                      NOT NULL,
    GuestID                  BIGINT                                      NOT NULL,
    SerialNumber             NVARCHAR(256)      COLLATE database_default NOT NULL,
    PowerStateID             INT                                         NOT NULL)

  INSERT INTO #tmpSerialON(HyperVHostID, GuestID, SerialNumber, PowerStateID)
  SELECT HyperVHostID, MAX(GuestID) AS GuestID, SerialNumber, PowerStateID
    FROM #tmpSerial
    WHERE PowerStateID = 3
      AND SerialNumber NOT IN (SELECT SerialNumber FROM #tmpSerialON)
    GROUP BY HyperVHostID, SerialNumber, PowerStateID
    HAVING COUNT(*) = 1 

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert into #tmpRelationHyperV - Serial'
  IF OBJECT_ID('tempdb..#tmpRelationHyperV') IS NOT NULL
    DROP TABLE #tmpRelationHyperV

  CREATE TABLE #tmpRelationHyperV (
    HyperVHostID             BIGINT                                      NOT NULL,
    GuestID                  BIGINT                                      NOT NULL,
    GuestFQDN                NVARCHAR(256)      COLLATE database_default NULL,
    PowerStateID             INT                                         NULL)

  IF @bWithSerial = 1
    INSERT INTO #tmpRelationHyperV(HyperVHostID, GuestID, GuestFQDN, PowerStateID)
    SELECT HyperVHostId, GuestID, CAST(NULL AS NVARCHAR(255)) AS GuestFQDN, PowerStateID       FROM #tmpSerial
  ELSE
  BEGIN
    INSERT INTO #tmpRelationHyperV(HyperVHostID, GuestID, GuestFQDN, PowerStateID)
    SELECT HyperVHostId, GuestID, CAST(NULL AS NVARCHAR(255)) AS GuestFQDN, PowerStateID       FROM #tmpSerialON

    INSERT INTO #tmpRelationHyperV(HyperVHostID, GuestID, GuestFQDN, PowerStateID)
    SELECT HyperVHostId, GuestID, CAST(NULL AS NVARCHAR(255)) AS GuestFQDN, PowerStateID       FROM #tmpSerialOFF
  END

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert into #tmpRelationHyperV - GuestInfo'
  INSERT INTO #tmpRelationHyperV(HyperVHostID, GuestID, GuestFQDN, PowerStateID)
  SELECT
      h.HyperVHostID,
      f.ComputerID                AS GuestID,
      h.FQDN                      AS GuestFQDN,
      h.PowerStateID
    FROM (SELECT HyperVHostID, FQDN, PowerStateID FROM #f_GetVirtualHyperVGuestInfo_HyperVLink WHERE LEN(FQDN) > 0) h
	         INNER JOIN #f_GetDeviceFQDN_HyperVLink f
	           ON h.FQDN = f.FQDN
	     UNION SELECT
	         h.ComputerID                AS HyperVHostID,
	         g.HyperVGuestID             AS GuestID,
	         CAST(NULL AS NVARCHAR(255)) AS GuestFQDN,
	         hi.PowerStateID
	       FROM dbo.f_GetVirtualHyperVHostInfo() g
	         INNER JOIN #f_GetDeviceFQDN_HyperVLink h
	           ON g.FQDN = h.FQDN
	         LEFT JOIN #f_GetVirtualHyperVGuestInfo_HyperVLink hi
	           ON h.ComputerID = hi.HyperVHostID


  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert into #tmpRelationHyperV - HostInfo'
  INSERT INTO #sp_GetVirtualHyperVLinkGuest(HyperVHostID, HyperVHostCN, HyperVHostFQDN, GuestID, GuestCN, GuestFQDN, PowerState)
  SELECT
      tmp.HyperVHostID,
      MAX(hf.ComputerCN)                    AS HyperVHostCN,
      MAX(hf.fqdn)                          AS HyperVHostFQDN,
	  tmp.GuestID,
      MAX(COALESCE(cf.ComputerCN, CASE
	                                WHEN CHARINDEX('.', tmp.GuestFQDN) > 0 THEN LEFT(tmp.GuestFQDN, CHARINDEX('.', tmp.GuestFQDN) -1)
                                    ELSE tmp.GuestFQDN
                                  END))     AS GuestCN,
      MAX(COALESCE(cf.fqdn, tmp.GuestFQDN)) AS GuestFQDN,	  
      CASE MIN(tmp.PowerStateID)
        WHEN 2 THEN 'Running'
        WHEN 3 THEN 'Off'
      END                              AS PowerState
    FROM #tmpRelationHyperV tmp
      LEFT JOIN #f_GetDeviceFQDN_HyperVLink hf
        ON tmp.HyperVHostID = hf.ComputerID
      LEFT JOIN #f_GetDeviceFQDN_HyperVLink cf
        ON tmp.GuestID = cf.ComputerID
    GROUP BY tmp.HyperVHostID, tmp.GuestID

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Show all'
  IF @bShowTable = 1
  BEGIN
    SELECT DISTINCT * FROM #sp_GetVirtualHyperVLinkGuest
    DROP TABLE #sp_GetVirtualHyperVLinkGuest
  END

  DROP TABLE #f_GetDeviceFQDN_HyperVLink
  DROP TABLE #f_GetVirtualHyperVGuestInfo_HyperVLink
  DROP TABLE #f_GetDeviceSerialNumber
  DROP TABLE #f_GetVirtualHyperVGuestInfo
  DROP TABLE #tmpRelationHyperV
  DROP TABLE #tmpSerial
  DROP TABLE #tmpSerialOFF
  DROP TABLE #tmpSerialON

END
GO
