USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetVirtualZoneLinkGuest' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetVirtualZoneLinkGuest]
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 30.01.2019
-- Last change: 
-- Copyright:   (c) Raynet GmbH
-- Description:	Get Sun/Oracle Solaris Zone Guest Link
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================


CREATE FUNCTION [dbo].[f_GetVirtualZoneLinkGuest] ()
  RETURNS TABLE
AS
RETURN
  WITH guest AS (SELECT
                     t.ComputerID         AS GuestID,
                     t.HardwareObjectID   AS UnitaryComputerSystemID,
                     t.Value              AS VMType,
                     vID.Value            AS VMID,
                     vN.Value             AS VMName,
                     pIN.Value            AS IdentifyingNumber,
                     pH.Value             AS HostID
                   FROM f_HardwareProperties('CIM_UnitaryComputerSystem') t
                     INNER JOIN f_HardwareValuesByPropertyName('VMID') vID
                       ON t.HardwareObjectID = vID.HardwareObjectID
                     INNER JOIN f_HardwareValuesByPropertyName('VMName') vN
                       ON t.HardwareObjectID = vN.HardwareObjectID
                     LEFT JOIN f_HardwareProperties('CIM_Product') pIN
                       ON t.ComputerID = pIN.ComputerID
                         AND pIN.Property = 'IdentifyingNumber'
                     LEFT JOIN f_HardwareValuesByPropertyName('HostID') pH
                       ON pIN.HardwareObjectID = pH.HardwareObjectID
                   WHERE t.Property = 'VMType' AND t.Value = 'Zone'
                     AND vID.Value != 'global'),
       host AS  (SELECT
                     t.ComputerID         AS ZoneHostID,
                     t.HardwareObjectID   AS UnitaryComputerSystemID,
                     t.Value              AS VMType,
                     pIN.Value            AS IdentifyingNumber,
                     pH.Value             AS HostID
                   FROM f_HardwareProperties('CIM_UnitaryComputerSystem') t
                     INNER JOIN f_HardwareValuesByPropertyName('VMID') vID
                       ON t.HardwareObjectID = vID.HardwareObjectID
                     LEFT JOIN f_HardwareProperties('CIM_Product') pIN
                       ON t.ComputerID = pIN.ComputerID
                         AND pIN.Property = 'IdentifyingNumber'
                     LEFT JOIN f_HardwareValuesByPropertyName('HostID') pH
                       ON pIN.HardwareObjectID = pH.HardwareObjectID
                   WHERE t.Property = 'VMType' AND t.Value = 'Zone'
                     AND vID.Value = 'global')
  SELECT
      host.ZoneHostID,
      host.UnitaryComputerSystemID        AS ZoneHostUnitaryComputerSystemID,
      COALESCE(host.HostID, guest.HostID) AS ProductHostID,
      guest.GuestID,
      guest.UnitaryComputerSystemID       AS GuestUnitaryComputerSystemID
    FROM guest
      LEFT JOIN host
        ON guest.HostID = host.HostID
GO

