USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetVirtualZoneHostInfo' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetVirtualZoneHostInfo]
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 30.01.2019
-- Last change: 
-- Copyright:   (c) Raynet GmbH
-- Description:	Get Sun/Oracle Solaris Zone Host Info
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

-- Notiz:
-- If no link can created to the Host, it create
-- a dummy Host, with calculated system information
-- like CPUs, Cores and Memory
CREATE FUNCTION [dbo].[f_GetVirtualZoneHostInfo] ()
  RETURNS TABLE
AS
RETURN
  SELECT
      g.ZoneHostID,
      g.ProductHostID,
      d.Manufacturer            AS Manufacturer,
      d.Model                   AS Model,
      p.Name                    AS CPUName,
	  p.TotalCPUs               AS TotalCPUs,
	  p.TotalCores              AS TotalCores,
      p.TotalLogical            AS TotalLogical,
	  p.ClockSpeed              AS ClockSpeed,
      m.Size                    AS Memory,
      os.ProductName + ' ' + os.Version
                                AS OperatingSystem,
      ir.LastInventoryDate      AS LastInventoryDate
    FROM dbo.f_GetVirtualZoneLinkGuest() g
      LEFT JOIN dbo.f_GetDeviceModel() d
        ON g.ZoneHostID = d.ComputerID
      LEFT JOIN dbo.f_GetDeviceProcessor() p
        ON g.ZoneHostID = p.ComputerID
      LEFT JOIN (SELECT ComputerID, SUM(Size) AS Size FROM dbo.f_GetDeviceMemoryModul() GROUP BY ComputerID) m
        ON g.ZoneHostID = m.ComputerID
      LEFT JOIN dbo.f_GetLastInventoryDate(1) ir
        ON g.ZoneHostID = ir.ComputerID
      LEFT JOIN dbo.f_GetDeviceOSInfo() os
        ON g.ZoneHostID = os.ComputerID
	WHERE g.ZoneHostID > 0
  UNION SELECT
      CAST(NULL AS INT)         AS ZoneHostID,
      g.ProductHostID,
      MAX(d.Manufacturer)       AS Manufacturer,
      MAX(d.Model)              AS Model,
      MAX(p.Name)               AS CPUName,
	  SUM(p.TotalCPUs)          AS TotalCPUs,
	  SUM(p.TotalCores)         AS TotalCores,
      SUM(p.TotalLogical)       AS TotalLogical,
	  MAX(p.ClockSpeed)         AS ClockSpeed,
      SUM(m.Size)               AS Memory,
      MAX(os.ProductName + ' ' + os.Version)
                                AS OperatingSystem,
	  MAX(ir.LastInventoryDate) AS LastInventoryDate
    FROM dbo.f_GetVirtualZoneLinkGuest() g
      LEFT JOIN dbo.f_GetDeviceModel() d
        ON g.GuestID = d.ComputerID
      LEFT JOIN dbo.f_GetDeviceProcessor() p
        ON g.GuestID = p.ComputerID
      LEFT JOIN (SELECT ComputerID, SUM(Size) AS Size FROM dbo.f_GetDeviceMemoryModul() GROUP BY ComputerID) m
        ON g.GuestID = m.ComputerID
      LEFT JOIN dbo.f_GetLastInventoryDate(1) ir
        ON g.GuestID = ir.ComputerID
      LEFT JOIN dbo.f_GetDeviceOSInfo() os
        ON g.GuestID = os.ComputerID
    WHERE g.ZoneHostID IS NULL
    GROUP BY g.ProductHostID

GO

