USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetVirtualLPARLinkGuest' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetVirtualLPARLinkGuest]
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 30.01.2019
-- Last change: 
-- Copyright:   (c) Raynet GmbH
-- Description:	Get AIX LPAR Host Guest Link
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_GetVirtualLPARLinkGuest] ()
  RETURNS TABLE
AS
RETURN
  WITH guest AS (SELECT
                     t.ComputerID         AS GuestID,
                     t.HardwareObjectID   AS UnitaryComputerSystemID,
                     t.Value              AS VMType,
                     vID.Value            AS VMID,
                     vN.Value             AS VMName,
                     pIN.Value            AS IdentifyingNumber,
                     pH.Value             AS HostID
                   FROM f_HardwareProperties('CIM_UnitaryComputerSystem') t
                     INNER JOIN f_HardwareValuesByPropertyName('VMID') vID
                       ON t.HardwareObjectID = vID.HardwareObjectID
                     INNER JOIN f_HardwareValuesByPropertyName('VMName') vN
                       ON t.HardwareObjectID = vN.HardwareObjectID
                     LEFT JOIN f_HardwareProperties('CIM_Product') pIN
                       ON t.ComputerID = pIN.ComputerID
                         AND pIN.Property = 'IdentifyingNumber'
                     LEFT JOIN f_HardwareValuesByPropertyName('HostID') pH
                       ON pIN.HardwareObjectID = pH.HardwareObjectID
                     LEFT JOIN f_HardwareProperties('CIM_BIOSElement') bS
                       ON t.ComputerID = bs.ComputerID
                         AND bS.Property = 'SerialNumber'
                   WHERE t.Property = 'VMType' AND t.Value = 'LPAR'
                     AND (LEFT(pH.Value, 12) = LEFT(bS.Value, 12)
                       OR pH.Value != bS.Value)),
       host AS  (SELECT
                     t.ComputerID         AS LPARHostID,
                     t.HardwareObjectID   AS UnitaryComputerSystemID,
                     t.Value              AS VMType,
                     pIN.Value            AS IdentifyingNumber,
                     pH.Value             AS HostID
                   FROM f_HardwareProperties('CIM_UnitaryComputerSystem') t
                     INNER JOIN f_HardwareValuesByPropertyName('VMID') vID
                       ON t.HardwareObjectID = vID.HardwareObjectID
                     LEFT JOIN f_HardwareProperties('CIM_Product') pIN
                       ON t.ComputerID = pIN.ComputerID
                         AND pIN.Property = 'IdentifyingNumber'
                     LEFT JOIN f_HardwareValuesByPropertyName('HostID') pH
                       ON pIN.HardwareObjectID = pH.HardwareObjectID
                     LEFT JOIN f_HardwareProperties('CIM_BIOSElement') bS
                       ON t.ComputerID = bs.ComputerID
                         AND bS.Property = 'SerialNumber'
                   WHERE t.Property = 'VMType' AND t.Value = 'LPAR'
                     AND (LEFT(pH.Value, 12) != LEFT(bS.Value, 12)
                       OR pH.Value = bS.Value))
  SELECT
      host.LPARHostID,
      host.UnitaryComputerSystemID        AS LPARHostUnitaryComputerSystemID,
      COALESCE(host.HostID, guest.HostID) AS ProductHostID,
      COALESCE(host.IdentifyingNumber, guest.IdentifyingNumber)
                                          AS IdentifyingNumber,
      guest.GuestID,
      guest.UnitaryComputerSystemID       AS GuestUnitaryComputerSystemID
    FROM guest
      LEFT JOIN host
        ON guest.HostID = host.HostID
GO

