USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetVirtualLPARHostInfo' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetVirtualLPARHostInfo]
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 30.01.2019
-- Last change: 
-- Copyright:   (c) Raynet GmbH
-- Description:	Get AIX LPAR Host Info
-- ===============================================
-- Notiz:
-- If no link can created to the Host, it create
-- a dummy Host, with calculated system information
-- like CPUs, Cores and Memory
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_GetVirtualLPARHostInfo] ()
  RETURNS TABLE
AS
RETURN
  SELECT
      g.LPARHostID,
      g.ProductHostID,
      g.IdentifyingNumber,
      d.Manufacturer            AS Manufacturer,
      d.Model                   AS Model,
      p.Name                    AS CPUName,
	  p.TotalCPUs               AS TotalCPUs,
	  p.TotalCores              AS TotalCores,
      p.TotalLogical            AS TotalLogical,
	  p.ClockSpeed              AS ClockSpeed,
      m.Size                    AS Memory,
      os.ProductName + ' ' + os.Version
                                AS OperatingSystem,
      ir.LastInventoryDate      AS LastInventoryDate
    FROM dbo.f_GetVirtualLPARLinkGuest() g
      LEFT JOIN dbo.f_GetDeviceModel() d
        ON g.LPARHostID = d.ComputerID
      LEFT JOIN dbo.f_GetDeviceProcessor() p
        ON g.LPARHostID = p.ComputerID
      LEFT JOIN (SELECT ComputerID, SUM(Size) AS Size FROM dbo.f_GetDeviceMemoryModul() GROUP BY ComputerID) m
        ON g.LPARHostID = m.ComputerID
      LEFT JOIN dbo.f_GetLastInventoryDate(1) ir
        ON g.LPARHostID = ir.ComputerID
      LEFT JOIN dbo.f_GetDeviceOSInfo() os
        ON g.LPARHostID = os.ComputerID
	WHERE g.LPARHostID > 0
  UNION SELECT
      CAST(NULL AS INT)         AS LPARHostID,
      g.ProductHostID,
      MAX(g.IdentifyingNumber)  AS IdentifyingNumber,
      MAX(d.Manufacturer)       AS Manufacturer,
      MAX(d.Model)              AS Model,
      MAX(p.Name)               AS CPUName,
	  COALESCE(MAX(nc.CPUs),  SUM(p.TotalCPUs))                AS TotalCPUs,
	  COALESCE(MAX(nc.COres), SUM(p.TotalCores))               AS TotalCores,
      IIF(MAX(nc.CPUs) IS NOT NULL, NULL, SUM(p.TotalLogical)) AS TotalLogical,
	  MAX(p.ClockSpeed)         AS ClockSpeed,
      SUM(m.Size)               AS Memory,
      MAX(os.ProductName + ' ' + os.Version)
                                AS OperatingSystem,
	  MAX(ir.LastInventoryDate) AS LastInventoryDate
    FROM dbo.f_GetVirtualLPARLinkGuest() g
      LEFT JOIN dbo.f_GetDeviceModel() d
        ON g.GuestID = d.ComputerID
      LEFT JOIN (SELECT
                     nCPUs.ComputerID,
                     dbo.CIMInteger(nCPUs.Value) AS CPUs,
                     dbo.CIMInteger(nCores.Value) AS Cores
                   FROM dbo.f_HardwareProperties('MGS_ComputerSystem') nCPUs
                     LEFT JOIN dbo.f_HardwareValuesByPropertyName('NodeCapacityInCores') nCores
                       ON nCPUs.HardwareObjectID = nCores.HardwareObjectID
                   WHERE nCPUs.Property = 'NodeCapacity') nC
        ON g.GuestID = nC.ComputerID
      LEFT JOIN dbo.f_GetDeviceProcessor() p
        ON g.GuestID = p.ComputerID
      LEFT JOIN (SELECT ComputerID, SUM(Size) AS Size FROM dbo.f_GetDeviceMemoryModul() GROUP BY ComputerID) m
        ON g.GuestID = m.ComputerID
      LEFT JOIN dbo.f_GetLastInventoryDate(1) ir
        ON g.GuestID = ir.ComputerID
      LEFT JOIN dbo.f_GetDeviceOSInfo() os
        ON g.GuestID = os.ComputerID
    WHERE g.LPARHostID IS NULL
    GROUP BY g.ProductHostID

GO

