USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_GetVirtualLPARPools' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_GetVirtualLPARPools]
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 20.09.2019
-- Last change: 
-- Copyright:   (c) Raynet GmbH
-- Description:	Get AIX LPAR Pool Info
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================


CREATE PROCEDURE [dbo].[sp_GetVirtualLPARPools]
AS
BEGIN
  DECLARE @bShowTable   BIT    = 1
  DECLARE @bDebug       BIT    = (SELECT Value FROM ReportingConfiguration WHERE Property = 'Debug')

  IF @bDebug IS NULL SET @bDebug = 0

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Start sp_GetVirtualLPARPools'

  IF OBJECT_ID('tempdb..#sp_GetVirtualLPARPools') IS NOT NULL
    BEGIN
      IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '  - clean table #sp_GetVirtualLPARPools'
      TRUNCATE TABLE #sp_GetVirtualLPARPools
      SET @bShowTable = 0
    END
    ELSE
    BEGIN
      IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '  - create table #sp_GetVirtualLPARPools'
      CREATE TABLE #sp_GetVirtualLPARPools (
        LPARHostID                      BIGINT                                  NULL,
        LPARHostUnitaryComputerSystemID BIGINT                                  NULL,
        ProductHostID                   NVARCHAR(125)  COLLATE database_default NULL,
        IdentifyingNumber               NVARCHAR(125)  COLLATE database_default NULL,
        GuestID                         BIGINT                                  NULL,
        GuestUnitaryComputerSystemID    BIGINT                                  NOT NULL,
        SharedPoolID                    NVARCHAR(125)  COLLATE database_default NULL,
        SharedPoolCapacityInCores       INT                                     NULL,
        Entitlement                     DECIMAL(15,2)                           NULL)
    END
  
  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '  - create table #tmpGetVirtualLPARLinkGuest_Pools'
  IF OBJECT_ID('tempdb..#tmpGetVirtualLPARLinkGuest_Pools') IS NOT NULL
    DROP TABLE #tmpGetVirtualLPARLinkGuest_Pools
  CREATE TABLE #tmpGetVirtualLPARLinkGuest_Pools(
    LPARHostID                      BIGINT                                  NULL,
    LPARHostUnitaryComputerSystemID BIGINT                                  NULL,
    ProductHostID                   NVARCHAR(125)  COLLATE database_default NULL,
    IdentifyingNumber               NVARCHAR(125)  COLLATE database_default NULL,
    GuestID                         BIGINT                                  NULL,
    GuestUnitaryComputerSystemID    BIGINT                                  NOT NULL)
    
  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '  - Get Data from f_GetVirtualLPARLinkGuest()'
  INSERT INTO #tmpGetVirtualLPARLinkGuest_Pools(LPARHostID, LPARHostUnitaryComputerSystemID, ProductHostID, IdentifyingNumber,
                                                GuestID, GuestUnitaryComputerSystemID)
  SELECT
      g.LPARHostID,
      g.LPARHostUnitaryComputerSystemID,
      g.ProductHostID,
      g.IdentifyingNumber,
      g.GuestID,
      g.GuestUnitaryComputerSystemID
    FROM dbo.f_GetVirtualLPARLinkGuest() g
  
  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '  - Get Shared Pool Information'
  INSERT INTO #sp_GetVirtualLPARPools(LPARHostID, LPARHostUnitaryComputerSystemID, ProductHostID, IdentifyingNumber,
                                      GuestID, GuestUnitaryComputerSystemID, SharedPoolID, SharedPoolCapacityInCores, Entitlement)
  SELECT
      p.LPARHostID,
      p.LPARHostUnitaryComputerSystemID,
      p.ProductHostID,
      p.IdentifyingNumber,
      p.GuestID,
      p.GuestUnitaryComputerSystemID,
      vSID.Value                                           AS SharedPoolID,
      CAST(vSCC.Value AS INT)                              AS SharedPoolCapacityInCores,
      CAST(REPLACE(vEnt.Value, ',', '.') AS DECIMAL(15,2)) AS Entitlement
    FROM #tmpGetVirtualLPARLinkGuest_Pools p
      INNER JOIN f_HardwareValuesByPropertyName('SharedPoolID') vSID
        ON p.GuestUnitaryComputerSystemID = vSID.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('SharedPoolCapacityInCores') vSCC
        ON p.GuestUnitaryComputerSystemID = vSCC.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('VMEntitlement') vEnt
        ON p.GuestUnitaryComputerSystemID = vEnt.HardwareObjectID

  IF @bShowTable = 1
  BEGIN
    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '  - Show Pools'
    SELECT * FROM #sp_GetVirtualLPARPools
    DROP TABLE #sp_GetVirtualLPARPools
  END

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '  - Remove temp tables'
  DROP TABLE #tmpGetVirtualLPARLinkGuest_Pools

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- End sp_GetVirtualLPARPools'
END
GO

