USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_GetSoftwareInstallInformation' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_GetSoftwareInstallInformation]
GO
-- ==============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 02.04.2019
-- Copyright:   (c) Raynet GmbH
-- Description: Get Software filtered by Evidence
-- ==============================================
-- Comment:
-- Created on Base f_GetSoftwareInstallInformation for Better Performance (40 fach)
-- ==============================================
-- Parameter iEvidenceGroup
-- - null       All Evidences or sEvidenceFilter
-- - 1          Only Windows ARP Information
-- - 2          Only Windows MSI
-- - 3          Other the Windows ARP or MSI
--
-- Parameter sEvidenceFilter
--   Comma Separated List of Evidences
--   If null then all
--   If iEvidenceGroup is set then this is ignored
--
-- Parameter bNoUpdate
-- - 0 or null  All Packages are shown
-- - 1          Update Packages are filtered out
--
-- Parameter iLastSeenDays
-- - count of days of maximum older of Inventory
-- - If null all
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE PROCEDURE [dbo].[sp_GetSoftwareInstallInformation] 
(
  @iEvidenceGroup  TINYINT        = NULL,
  @sEvidenceFilter NVARCHAR(1000) = NULL,
  @bNoUpdate       BIT            = 0,
  @iLastSeenDays   INT            = NULL
)
AS
BEGIN
--  DECLARE @iEvidenceGroup  TINYINT        = NULL
--  DECLARE @sEvidenceFilter NVARCHAR(1000) = NULL
--  DECLARE @bNoUpdate       BIT            = 0
--  DECLARE @iLastSeenDays   INT            = NULL

  DECLARE @bShowTable   BIT    = 1
  DECLARE @bAllEvidence BIT    = 1
  DECLARE @bDebug       BIT    = 0

  DECLARE @sEvidenceGroup1 NVARCHAR(256) = (SELECT Value FROM ReportingConfiguration WHERE Property = 'SoftwareEvidence1')
  DECLARE @sEvidenceGroup2 NVARCHAR(256) = (SELECT Value FROM ReportingConfiguration WHERE Property = 'SoftwareEvidence2')
  DECLARE @sEvidenceGroup3 NVARCHAR(256) = (SELECT Value FROM ReportingConfiguration WHERE Property = 'SoftwareEvidence3')

  IF @sEvidenceGroup1 IS NULL SET @sEvidenceGroup1 = 'Uninstall,MSI,ManageSoft + MSI,ManageSoft'
  IF @sEvidenceGroup2 IS NULL SET @sEvidenceGroup2 = 'MSI,ManageSoft + MSI'
  IF @sEvidenceGroup3 IS NULL SET @sEvidenceGroup3 = 'DPKG,RPM,OUI,IA,ISMP,LPP,VISDK,SUNPKG,ASPA,BEA'

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Get Evidence Filter'

  IF @iEvidenceGroup IS NOT NULL OR @sEvidenceFilter IS NOT NULL
    SET @bAllEvidence = 0

  IF OBJECT_ID('tempdb..#tmpEvidenceFilter') IS NOT NULL
    DROP TABLE #tmpEvidenceFilter

  CREATE TABLE #tmpEvidenceFilter (
    Evidence VARCHAR(32)   COLLATE database_default NOT NULL)

  INSERT INTO #tmpEvidenceFilter
  SELECT 
      Split.a.value('.', 'VARCHAR(32)') AS Evidence  
    FROM (SELECT
            CAST ('<M>' + 
              REPLACE(CASE
                        WHEN @iEvidenceGroup = 1 THEN @sEvidenceGroup1
                        WHEN @iEvidenceGroup = 2 THEN @sEvidenceGroup2
                        WHEN @iEvidenceGroup = 3 THEN @sEvidenceGroup3
                        WHEN @sEvidenceFilter IS NOT NULL THEN @sEvidenceFilter
                      END, ',', '</M><M>') + '</M>' AS XML) AS Evidence
         ) AS A CROSS APPLY Evidence.nodes ('/M') AS Split(a)

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Get Device List filtered by @iLastSeenDays: ' + ISNULL(CAST(@iLastSeenDays AS NVARCHAR(6)), '(NULL)')
  IF OBJECT_ID('tempdb..#tmpDevices') IS NOT NULL
    DROP TABLE #tmpDevices

  CREATE TABLE #tmpDevices (
    ComputerID BIGINT NOT NULL)

  IF @iLastSeenDays IS NULL
    INSERT INTO #tmpDevices
	SELECT ComputerID FROM Computer WITH (NOLOCK)
  ELSE
    INSERT INTO #tmpDevices
    SELECT 
        ComputerID
      FROM dbo.f_GetLastInventoryDate(1) ir
      WHERE @iLastSeenDays IS NULL OR DATEDIFF(day,  ir.LastInventoryDate, CURRENT_TIMESTAMP) <= @iLastSeenDays

  IF OBJECT_ID('tempdb..#sp_GetSoftwareInstallInformation') IS NOT NULL
  BEGIN
    TRUNCATE TABLE #sp_GetSoftwareInstallInformation
    SET @bShowTable = 0
  END
  ELSE
  BEGIN
    CREATE TABLE #sp_GetSoftwareInstallInformation (
      ComputerID           BIGINT                                 NOT NULL,
      SoftwareOccurrenceID BIGINT                                 NOT NULL,
      SoftwareID           BIGINT                                 NULL,
      SoftwareDetailsID    BIGINT                                 NULL,
      Evidence             VARCHAR(32)   COLLATE database_default NULL,
      Publisher            NVARCHAR(256) COLLATE database_default NULL,
      ProductName          NVARCHAR(256) COLLATE database_default NULL,
      ProductVersion       NVARCHAR(256) COLLATE database_default NULL,
      ProductCode          NVARCHAR(256) COLLATE database_default NULL,
      Summary              NVARCHAR(256) COLLATE database_default NULL,
      InstallationDate     DATETIME                               NULL)

    CREATE NONCLUSTERED INDEX IDX_C_#sp_GetSoftwareInstallInformation_ComputerID           ON #sp_GetSoftwareInstallInformation(ComputerID)
  END

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Get Information from SoftwareOccurence'
  INSERT INTO #sp_GetSoftwareInstallInformation (ComputerID, SoftwareOccurrenceID, SoftwareDetailsID, Evidence, InstallationDate)
  SELECT
      so.ComputerID,
	  so.SoftwareOccurrenceID,
      so.SoftwareDetailsID,
	  so.Evidence,
	  so.InstallationDate
    FROM dbo.SoftwareOccurrence so WITH (NOLOCK)
      INNER JOIN #tmpDevices d
        ON so.ComputerID = d.ComputerID
    WHERE @bAllEvidence = 1 OR so.Evidence IN (SELECT Evidence FROM #tmpEvidenceFilter)
/*      INNER JOIN #tmpEvidenceFilter e
        ON so.Evidence = e.Evidence
          OR @bAllEvidence = 1 */
    
  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Add Softwareinformation from SoftwareDetails'
  UPDATE sii
      SET Publisher      = RTRIM(LTRIM(REPLACE(REPLACE(REPLACE(sd.Publisher, '%', '_'), CHAR(13), ''), CHAR(10), ' '))),
          ProductName    = RTRIM(LTRIM(REPLACE(REPLACE(REPLACE(sd.RawSoftwareName, '%', '_'), CHAR(13), ''), CHAR(10), ' '))),
          ProductVersion = RTRIM(LTRIM(REPLACE(sd.RawVersion, '%', '_')))
    FROM #sp_GetSoftwareInstallInformation sii
      INNER JOIN dbo.SoftwareDetails sd WITH (NOLOCK) 
        ON sd.SoftwareDetailsID = sii.SoftwareDetailsID

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Add ProductCode from Property IdentifyingNumber';
	UPDATE sii
	SET ProductCode    = pin.ProductCode
	FROM #sp_GetSoftwareInstallInformation sii
	INNER JOIN (SELECT 
	SoftwareOccurrenceID,
	Value as ProductCode
	FROM dbo.f_SoftwareValuesByPropertyName('IdentifyingNumber')
	) pin
	ON sii.SoftwareOccurrenceID = pin.SoftwareOccurrenceID

	IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Add ProductCode from Property ProductCode';
		UPDATE sii
		SET ProductCode    = ppc.ProductCode
		FROM #sp_GetSoftwareInstallInformation sii
		INNER JOIN (SELECT 
		SoftwareOccurrenceID,
		Value as ProductCode
		FROM dbo.f_SoftwareValuesByPropertyName('ProductCode')
		) ppc
		ON sii.SoftwareOccurrenceID = ppc.SoftwareOccurrenceID

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Add Softwareinformation from SoftwareVersion'
  UPDATE sii
      SET ProductName    = RTRIM(LTRIM(REPLACE(REPLACE(REPLACE(sv.SoftwareName, '%', '_'), CHAR(13), ''), CHAR(10), ' '))),
	      ProductVersion = RTRIM(LTRIM(REPLACE(sv.Version, '%', '_')))
    FROM #sp_GetSoftwareInstallInformation sii
      INNER JOIN dbo.SoftwareVersion sv
        ON sii.SoftwareID = sv.SoftwareID
          AND LEN(sii.ProductName) = 0

  UPDATE sii
      SET Summary        = RTRIM(LTRIM(REPLACE(REPLACE(REPLACE(sv.Value, '%', '_'), CHAR(13), ''), CHAR(10), ' ')))
    FROM #sp_GetSoftwareInstallInformation sii
	  INNER JOIN dbo.f_SoftwareValuesByPropertyName('Summary') sv
        ON sii.SoftwareOccurrenceID = sv.SoftwareOccurrenceID

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Remove Update Packages'
  IF @bNoUpdate = 1
    DELETE 
      FROM #sp_GetSoftwareInstallInformation
      WHERE ProductName LIKE 'Microsoft Update %' 
        OR ProductName LIKE '%Update for Windows%'
        OR ProductName LIKE 'Microsoft App Update%'
        OR ProductName LIKE '%Update for Microsoft%'

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Remove not needed entries'
  IF OBJECT_ID('tempdb..#tmpSoftwareOccurenceID') IS NOT NULL
    DROP TABLE #tmpSoftwareOccurenceID

  CREATE TABLE #tmpSoftwareOccurenceID (
    SoftwareOccurrenceID BIGINT NOT NULL)

  INSERT INTO #tmpSoftwareOccurenceID
  SELECT
      sii.SoftwareOccurrenceID
    FROM #sp_GetSoftwareInstallInformation sii
      INNER JOIN dbo.f_SoftwareValuesByPropertyName('DisplayName') sp 
        ON sii.SoftwareOccurrenceID = sp.SoftwareOccurrenceID
          AND sii.ProductName = sp.Value

  DELETE
    FROM #sp_GetSoftwareInstallInformation
	WHERE Evidence = 'ManageSoft' AND SoftwareOccurrenceID NOT IN (SELECT SoftwareOccurrenceID FROM #tmpSoftwareOccurenceID) 

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Show Table'
  IF @bShowTable = 1
  BEGIN
    SELECT * FROM #sp_GetSoftwareInstallInformation  sii
    DROP TABLE #sp_GetSoftwareInstallInformation
  END

  DROP TABLE #tmpEvidenceFilter
  DROP TABLE #tmpSoftwareOccurenceID
END
GO
