USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetSoftwareFileUsage' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetSoftwareFileUsage]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 03.05.2019
-- Copyright:   (c) Raynet GmbH
-- Description:	Get All File Usage Information
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_GetSoftwareFileUsage] ()
  RETURNS TABLE
AS
RETURN
  WITH sesMonth AS (SELECT
                        spw.SoftwareUsagePerWeekID,
                        SUM(IIF(DATEDIFF(day,  spw.StartOfWeek, CURRENT_TIMESTAMP) <= 30, spw.Sessions, 0)) SessionsPerMonth
                      FROM dbo.SoftwareUsagePerWeek spw
                      GROUP BY spw.SoftwareUsagePerWeekID)
  SELECT
      sup2.ComputerID,
      sup2.UserID,
      sfTmp.SoftwareFileNameID,
      MAX(sfTmp.Publisher)          AS Publisher,
      MAX(sfTmp.ProductName)        AS ProductName,
      MAX(sfTmp.ProductVersion)     AS ProductVersion,
      MAX(sfTmp.FileDescription)    AS FileDescription,
      MAX(sfTmp.FileLongName)       AS FileLongName,
      MIN(sup2.StartOfWeek)         AS FirstUsage,
      MAX(sup2.StartOfWeek)         AS LastUsage,
      SUM(sm.SessionsPerMonth)      AS SessionPerMonth
    FROM (SELECT DISTINCT
              sup.SoftwareUsagePerWeekID,
              sfu.SoftwareFileNameID,
              sfu.CompanyName                                                  AS Publisher,
              COALESCE(sfu.ProductName, sv.SoftwareName)                       AS ProductName,
              COALESCE(sfu.ProductVersion, sv.Version)                         AS ProductVersion,
              sfu.Description                                                  AS FileDescription,
              sfu.LongName                                                     AS FileLongName
            FROM dbo.SoftwareUsagePerWeek sup
              LEFT JOIN dbo.SoftwareFileUsage sfu
                ON sup.SoftwareFileUsageID = sfu.SoftwareFileUsageID
              LEFT JOIN dbo.SoftwareVersion sv
                ON sup.SoftwareID = sv.SoftwareID
            WHERE sup.Duration > 0) sfTmp
      LEFT JOIN dbo.SoftwareUsagePerWeek sup2
        ON sfTmp.SoftwareUsagePerWeekID = sup2.SoftwareUsagePerWeekID
      LEFT JOIN sesMonth sm
        ON sfTmp.SoftwareUsagePerWeekID = sm.SoftwareUsagePerWeekID
    GROUP BY sup2.ComputerID, sup2.UserID, sfTmp.SoftwareFileNameID
GO

