USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetSoftwareTagFiles' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetSoftwareTagFiles]
GO
-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 15.01.2019
-- Copyright:   (c) Raynet GmbH
-- Description:	Get All Tag File Information
--              like swidtag
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_GetSoftwareTagFiles] ()
  RETURNS TABLE
AS
RETURN
  -- ISO SWID Tag old Format (vor 2009)
  WITH IsoTagOld AS (
    SELECT
        itf.SoftwareIsoTagFileID,
        itfx.x.value('(/*:SoftwareIdentity/*:Entity/@name)[1]', 'nvarchar(256)')   AS Publisher,
        itfx.x.value('(/*:SoftwareIdentity/@name)[1]', 'nvarchar(256)')            AS ProductName,
        itfx.x.value('(/*:SoftwareIdentity/@version)[1]', 'nvarchar(256)')         AS ProductVersion,
	    CAST(NULL AS NVARCHAR(256))                                                AS ProductGuid,
        COALESCE(itfx.x.value('(/*:SoftwareIdentity/@uniqueId)[1]', 'nvarchar(256)'),
                 itfx.x.value('(/*:SoftwareIdentity/@tagId)[1]', 'nvarchar(256)')) AS SoftwareUniqueID,
        itfx.x.value('(/*:SoftwareIdentity/*:Entity/@regid)[1]', 'nvarchar(256)')  AS SwidLicensorID
--      IIF(itf.MD5 = 'NO_MD5', NULL, itf.MD5)                                     AS MD5
      FROM SoftwareIsoTagFile itf WITH (NOLOCK)
        CROSS APPLY (SELECT CAST(CAST(
                                   REPLACE(itf.TagContent, 
                                           'UTF-8', 'ISO-8859-1') AS VARCHAR(MAX)) AS XML) x) itfx
      WHERE itf.TagContent LIKE '%SoftwareIdentity%'),
  -- ISO SWID Tag without software_licensor tag
  IsoTagWithoutLic AS (
    SELECT
        itf.SoftwareIsoTagFileID,
        itfx.x.value('(/*:software_identification_tag/*:software_creator/*:name)[1]', 'nvarchar(256)')  AS Publisher,
        itfx.x.value('(/*:software_identification_tag/*:product_title)[1]', 'nvarchar(256)')            AS ProductName,
        itfx.x.value('(/*:software_identification_tag/*:product_version/*:name)[1]', 'nvarchar(256)')   AS ProductVersion,
	    CAST(NULL AS NVARCHAR(256))                                                                     AS ProductGuid,
        itfx.x.value('(/*:software_identification_tag/*:software_id/*:unique_id)[1]', 'nvarchar(256)')  AS SoftwareUniqueID,
        CAST(NULL AS NVARCHAR(256))                                                                     AS SwidLicensorID
--      IIF(itf.MD5 = 'NO_MD5', NULL, itf.MD5)                                     AS MD5
      FROM SoftwareIsoTagFile itf WITH (NOLOCK)
        CROSS APPLY (SELECT CAST(CAST(
                                   REPLACE(itf.TagContent, 
                                           'UTF-8', 'ISO-8859-1') AS VARCHAR(MAX)) AS XML) x) itfx
      WHERE itf.TagContent LIKE '%<software_identification_tag %'
        AND itf.TagContent NOT LIKE '%<software_licensor%')
  -- ISO SWID Tag new Format
  SELECT
      itf.SoftwareIsoTagFileID,
      COALESCE(ite.Name, itf.OriginalArpPublisher, 
               ito.Publisher, itwl.Publisher)                          AS Publisher,
      COALESCE(itsv.ProductTitle, itf.OriginalArpDisplayName,
               ito.ProductName, itwl.ProductName)                      AS ProductName,
      COALESCE(itsv.ProductVersionName, itf.OriginalArpDisplayVersion, 
               ito.ProductVersion, itwl.ProductVersion)                AS ProductVersion,
      dbo.CIMInteger(itsv.ProductVersionMajor)                         AS ProductVersionMajor,
      dbo.CIMInteger(itsv.ProductVersionMinor)                         AS ProductVersionMinor,
      dbo.CIMInteger(itsv.ProductVersionBuild)                         AS ProductVersionBuild,
      dbo.CIMInteger(itsv.ProductVersionReview)                        AS ProductVersionReview,
      itf.OriginalArpGuid                                              AS ProductGuid,
      COALESCE(itsv.SoftwareUniqueID, 
               ito.SoftwareUniqueID, itwl.SoftwareUniqueID)            AS SoftwareUniqueID,
      COALESCE(ite.RegID, ito.SwidLicensorId)                          AS SwidLicensorID,
      IIF(itf.MD5 = 'NO_MD5', NULL, itf.MD5)                           AS MD5
    FROM dbo.SoftwareIsoTagFile itf WITH (NOLOCK)
      LEFT JOIN dbo.SoftwareIsoTagSoftwareVersion itsv WITH (NOLOCK)
        ON itf.SoftwareIsoTagSoftwareVersionID = itsv.SoftwareIsoTagSoftwareVersionID
      LEFT JOIN dbo.SoftwareIsoTagEntity ite WITH (NOLOCK)
        ON itf.SoftwareCreatorEntityID = ite.SoftwareIsoTagEntityID
      LEFT JOIN IsoTagOld ito
        ON itf.SoftwareIsoTagFileID = ito.SoftwareIsoTagFileID
      LEFT JOIN IsoTagWithoutLic itwl
        ON itf.SoftwareIsoTagFileID = itwl.SoftwareIsoTagFileID

