USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetSoftwareFiles' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetSoftwareFiles]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 15.01.2019
-- Copyright:   (c) Raynet GmbH
-- Description:	Get All FileInformation
-- ===============================================
-- Parameter @iType
-- - 0 or null   All Entries
-- - 1           Only with ProductName
-- - 2           Only with ProductName and with out Tag Files
-- - 3           Only Tag Files
-- - 4           Only Tag Files and only with ProductName
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_GetSoftwareFiles] (@iType TINYINT)
  RETURNS TABLE
AS
RETURN
  SELECT
      tmp.ComputerID,
      tmp.SoftwareFileID,
      tmp.SoftwareFileNameID,
      tmp.SoftwareIsoTagFileID,
      tmp.FilePath,
      tmp.FileName,
      tmp.FileDescription,
      tmp.FileSize,
      tmp.FileMD5,
      tmp.FileDate,
      tmp.Publisher,
      tmp.ProductName,
      tmp.ProductVersion,
      tmp.ProductGuid,
      tmp.SwidUniqueId,
      tmp.SwidLicensorId
    FROM (SELECT
              sf.ComputerID,
              sf.SoftwareFileID,
              sf.SoftwareFileNameID,
              sf.SoftwareIsoTagFileID,
              sfp.Path                                                 AS FilePath,
              sfn.Name                                                 AS [FileName],
              sf.FileDescription                                       AS FileDescription,
              sf.Size                                                  AS FileSize,
              COALESCE(IsoTag.MD5, 
                       IIF(sf.MD5 = 'NO_MD5', NULL, sf.MD5))           AS FileMD5,
              sf.DateTime                                              AS FileDate,
              COALESCE(IsoTAG.Publisher, sf.CompanyName)               AS Publisher,
              COALESCE(IsoTAG.ProductName, sf.FileDescription)         AS ProductName,
              COALESCE(IsoTAG.ProductVersion, sf.FileVersion)          AS ProductVersion,
              IsoTag.ProductGuid                                       AS ProductGuid,
              IsoTAG.SoftwareUniqueID                                  AS SwidUniqueId,
              IsoTAG.SwidLicensorID                                    AS SwidLicensorId
            FROM dbo.SoftwareFile sf WITH (NOLOCK)
              LEFT JOIN dbo.SoftwareFilePath sfp WITH (NOLOCK)
                ON sfp.SoftwareFilePathID = sf.SoftwareFilePathID
              LEFT JOIN dbo.SoftwareFileName sfn WITH (NOLOCK)
                ON sfn.SoftwareFileNameID = sf.SoftwareFileNameID
              LEFT JOIN dbo.SoftwareVersion  sv WITH (NOLOCK)
                ON sf.SoftwareID = sv.SoftwareID
              LEFT JOIN dbo.f_GetSoftwareTagFiles() IsoTag
                ON sf.SoftwareIsoTagFileID = IsoTag.SoftwareIsoTagFileID) tmp
    WHERE @iType IS NULL
    OR @iType = 0
	OR(@iType = 1 AND tmp.ProductName IS NOT NULL AND LEN(tmp.ProductName) > 0)
	OR(@iType = 2 AND tmp.ProductName IS NOT NULL AND LEN(tmp.ProductName) > 0 AND tmp.SoftwareIsoTagFileID IS NULL)
	OR(@iType = 3 AND tmp.SoftwareIsoTagFileID > 0)
	OR(@iType = 4 AND tmp.SoftwareIsoTagFileID > 0 AND tmp.ProductName IS NOT NULL AND LEN(tmp.ProductName) > 0)
GO

