USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_GetSoftwareFiles' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_GetSoftwareFiles]
GO
-- ==============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 03.05.2019
-- Copyright:   (c) Raynet GmbH
-- Description: Get File Information
-- ==============================================
-- Comment:
-- Created on Base f_sp_GetSoftwareFile for Better Performance (40 fach)
-- ==============================================
-- Changes:
-- ==============================================
-- Parameter @iType
-- - 0 or null   All Entries
-- - 1           Only with ProductName
-- - 2           Only with ProductName and with out Tag Files
-- - 3           Only Tag Files
-- - 4           Only Tag Files and only with ProductName
--
-- Parameter @noUsageData
-- - 0 or null   with Meetering information
-- - 1           no Meetering information
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE PROCEDURE [dbo].[sp_GetSoftwareFiles](@iType TINYINT = 0, @noUsageData BIT = 0)
AS
BEGIN
-- DECLARE @iType TINYINT = 0

  DECLARE @bShowTable   BIT    = 1

  IF OBJECT_ID('tempdb..#sp_GetSoftwareFiles') IS NOT NULL
  BEGIN
    TRUNCATE TABLE #sp_GetSoftwareFiles
    SET @bShowTable = 0
  END
  ELSE
  BEGIN
    CREATE TABLE #sp_GetSoftwareFiles (
      ComputerID           BIGINT                                 NOT NULL,
      SoftwareFileID       BIGINT                                 NULL,
      SoftwareFileNameID   BIGINT                                 NULL,
      SoftwareIsoTagFileID BIGINT                                 NULL,
      FilePath             NVARCHAR(400) COLLATE database_default NULL,
      FileName             NVARCHAR(256) COLLATE database_default NULL,
      FileDescription      NVARCHAR(256) COLLATE database_default NULL,
      FileSize             NVARCHAR(256) COLLATE database_default NULL,
      FileMD5              NVARCHAR(256) COLLATE database_default NULL,
      FileDate             DATETIME                               NULL,
      Publisher            NVARCHAR(256) COLLATE database_default NULL,
      ProductName          NVARCHAR(256) COLLATE database_default NULL,
      ProductVersion       NVARCHAR(256) COLLATE database_default NULL,
      ProductGuid          NVARCHAR(256) COLLATE database_default NULL,
      SwidUniqueId         NVARCHAR(256) COLLATE database_default NULL,
      SwidLicensorID       NVARCHAR(256) COLLATE database_default NULL,
      LastUsage            DATETIME                               NULL,
      SessionPerMonth      BIGINT                                 NULL)
  END

  IF NOT EXISTS (SELECT * 
                   FROM tempdb.sys.indexes 
                   WHERE object_id = OBJECT_ID('tempdb..#sp_GetSoftwareFiles') AND [type] = 2)
  BEGIN
    CREATE NONCLUSTERED INDEX IDX_C_#sp_GetSoftwareFiles_ComputerID ON #sp_GetSoftwareFiles(ComputerID)
    IF (@noUsageData IS NULL OR @noUsageData = 0)
      CREATE NONCLUSTERED INDEX IDX_C_#sp_GetSoftwareFiles_ComputerID_SoftwareFileNameID_FilePath ON #sp_GetSoftwareFiles(ComputerID,SoftwareFileNameID,FilePath)
  END

  INSERT INTO #sp_GetSoftwareFiles (ComputerID, SoftwareFileID, SoftwareFileNameID, SoftwareIsoTagFileID, 
                                    FilePath, [FileName], FileDescription, FileSize, FileMD5, FileDate, Publisher, ProductName, ProductVersion,
                                    ProductGuid, SwidUniqueId, SwidLicensorID)
  SELECT
      sf.ComputerID,
	  sf.SoftwareFileID,
      sf.SoftwareFileNameID,
      sf.SoftwareIsoTagFileID,
      sf.FilePath,
      sf.FileName,
      sf.FileDescription,
      sf.FileSize,
      sf.FileMD5,
      sf.FileDate,
      sf.Publisher,
      sf.ProductName,
      sf.ProductVersion,
      sf.ProductGuid,
      sf.SwidUniqueId,
      sf.SwidLicensorId
    FROM dbo.f_GetSoftwareFiles(@iType) sf

  IF (@noUsageData IS NULL OR @noUsageData = 0) AND @iType <=2
  BEGIN
    IF OBJECT_ID('tempdb..#f_GetSoftwareFileUsage') IS NOT NULL
      DROP TABLE #f_GetSoftwareFileUsage

    CREATE TABLE #f_GetSoftwareFileUsage (
      ComputerID           BIGINT                                 NOT NULL,
      SoftwareFileNameID   BIGINT                                 NOT NULL,
      Publisher            NVARCHAR(256) COLLATE database_default NULL,
      ProductName          NVARCHAR(256) COLLATE database_default NULL,
      ProductVersion       NVARCHAR(256) COLLATE database_default NULL,
      [FileName]           NVARCHAR(256) COLLATE database_default NULL,
      FilePath             NVARCHAR(256) COLLATE database_default NULL,
      FileDescription      NVARCHAR(256) COLLATE database_default NULL,
	  FirstUsage           DATETIME                               NULL,
  	  LastUsage            DATETIME                               NULL,
      SessionPerMonth      BIGINT                                 NULL,
      _NotAdd              BIT                                    NULL)
	
    CREATE NONCLUSTERED INDEX IDX_C_#f_GetSoftwareFileUsage_ComputerID_SoftwareFileNameID_FilePath ON #f_GetSoftwareFileUsage(ComputerID,SoftwareFileNameID,FilePath)

    INSERT INTO #f_GetSoftwareFileUsage (ComputerID, SoftwareFileNameID, Publisher, ProductName, ProductVersion, 
                                         FilePath, [FileName], FileDescription, FirstUsage, LastUsage, SessionPerMonth)
    SELECT
        sfu.ComputerID,
        sfu.SoftwareFileNameID,
        MAX(sfu.Publisher)                                               AS Publisher,
        MAX(sfu.ProductName)                                             AS ProductName,
        MAX(sfu.ProductVersion)                                          AS ProductVerion,
        LEFT(sfu.FileLongName, LEN(sfu.FileLongName) - LEN(sfn.name) -1) AS FilePath,
        sfn.name                                                         AS [FileName],
        MAX(sfu.FileDescription)                                         AS FileDescription,
        MIN(sfu.FirstUsage)                                              AS FirstUsage,
        MAX(sfu.LastUsage)                                               AS LastUsage,
        SUM(sfu.SessionPerMonth)                                         AS SessionPerMonth
      FROM dbo.f_GetSoftwareFileUsage() sfu
        LEFT JOIN dbo.SoftwareFileName sfn WITH (NOLOCK)
          ON sfu.SoftwareFileNameID = sfn.SoftwareFileNameID
      GROUP BY sfu.ComputerID, sfu.SoftwareFileNameID, sfu.FileLongName, sfn.name

    UPDATE sf
        SET LastUsage       = sfu.LastUsage,
            SessionPerMonth = sfu.SessionPerMonth
      FROM #sp_GetSoftwareFiles sf
        INNER JOIN #f_GetSoftwareFileUsage sfu
          ON sf.ComputerID = sfu.ComputerID
            AND sf.SoftwareFileNameID = sfu.SoftwareFileNameID
            AND sf.FilePath = sfu.FilePath

    UPDATE sfu
        SET _NotAdd = 1
      FROM #f_GetSoftwareFileUsage sfu
        INNER JOIN #sp_GetSoftwareFiles sf
          ON sf.ComputerID = sfu.ComputerID
            AND sf.SoftwareFileNameID = sfu.SoftwareFileNameID
            AND sf.FilePath = sfu.FilePath


    INSERT INTO #sp_GetSoftwareFiles (ComputerID, SoftwareFileNameID, FilePath, [FileName], FileDescription, FileDate, 
                                      Publisher, ProductName, ProductVersion, LastUsage, SessionPerMonth)
    SELECT
        sfu.ComputerID,
        sfu.SoftwareFileNameID,
        sfu.FilePath,
        sfu.[FileName],
        sfu.FileDescription,
        sfu.FirstUsage                                                   AS FileDate,
        sfu.Publisher,
        sfu.ProductName,
        sfu.ProductVersion,
        sfu.LastUsage,
        sfu.SessionPerMonth
      FROM #f_GetSoftwareFileUsage sfu
      WHERE sfu._NotAdd IS NULL
        AND LEN(ProductName) > 0

    DROP TABLE #f_GetSoftwareFileUsage
  END

  IF @bShowTable = 1
  BEGIN
    SELECT * FROM #sp_GetSoftwareFiles
    DROP TABLE #sp_GetSoftwareFiles
  END
END

