USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetSoftwareMicrosoftSQL' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetSoftwareMicrosoftSQL]

GO
-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 31.01.2019
-- Copyright:   (c) Raynet GmbH
-- Description:	Get Microsoft SQL Installation information
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_GetSoftwareMicrosoftSQL] ()
  RETURNS TABLE
AS
RETURN
  SELECT DISTINCT
      i.ComputerID,
      i.HardwareObjectID                AS MGSSqlServiceID,
      'Microsoft'                       AS Publisher,
      CASE
	    WHEN dn.Value LIKE '% (%)' THEN LEFT(dn.Value, CHARINDEX(' (', dn.Value) -1)
        ELSE dn.Value
      END                               AS ProductName,
      v.Value                           AS ProductVersion,
      CASE
        WHEN ed.Value IS NOT NULL          THEN ed.Value
        WHEN sku.Value LIKE '%Express%'    THEN 'Express Edition'
        WHEN sku.Value LIKE '%Enterprise%' THEN 'Enterprise Edition'
        WHEN sku.Value LIKE '%Standard%'   THEN 'Standard Edition'
        WHEN sku.Value LIKE '%Developer%'  THEN 'Developer Edition'
      END                               AS ProductEdition,
      CASE
        WHEN v.Value LIKE '8.%'            THEN '2000'
        WHEN v.Value LIKE '9.%'            THEN '2005'
        WHEN v.Value LIKE '10.5%'          THEN '2008 R2'
        WHEN v.Value LIKE '10.25%'         THEN 'Azure'
        WHEN v.Value LIKE '10.%'           THEN '2008'
        WHEN v.Value LIKE '11.%'           THEN '2012'
        WHEN v.Value LIKE '12.%'           THEN '2014'
        WHEN v.Value LIKE '13.%'           THEN '2016'
        WHEN v.Value LIKE '14.%'           THEN '2017'
      END                               AS ProductRelease,
      sp.Value                          AS ServicePack,
      REPLACE(i.Value, 'MSSQL$', '')    AS InstanceName,
      dbo.CIMBoolean(c.Value)           AS [Clustered],
      CAST(NULL AS BIT)                 AS IsIntegratedSecurity,
      dp.Value                          AS DataPath,
      i.Value                           AS ServiceName,
      ss.Value                          AS State,
      CAST(NULL AS NVARCHAR(256))       AS StartMode,
      id.Value                          AS InstanceID,
      sku.Value                         AS Sku,
      dbo.CIMInteger(sst.Value)         AS SqlServiceType,
      ip.Value                          AS InstallLocation
    FROM dbo.f_HardwareProperties('MGS_SqlService') i
      INNER JOIN dbo.f_HardwareValuesByPropertyName('DisplayName') dn
        ON i.HardwareObjectID = dn.HardwareObjectID
          AND dn.Value NOT LIKE '%Integration%' AND dn.Value NOT LIKE '%Full-text%Filter%'
          AND (dn.Value = 'SQL Server' OR dn.Value LIKE 'SQL Server %')
          AND (dn.Value NOT LIKE '%Browser%' AND dn.Value NOT LIKE '%Agent%')
      LEFT JOIN f_HardwareValuesByPropertyName('Version') v
        ON i.HardwareObjectID = v.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('SKUNAME') sku
        ON i.HardwareObjectID = sku.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('Edition') ed
        ON i.HardwareObjectID = ed.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('INSTANCEID') id
        ON i.HardwareObjectID = id.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('INSTALLPATH') ip
        ON i.HardwareObjectID = ip.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('DATAPATH') dp
        ON i.HardwareObjectID = dp.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('SQLServiceType') sst
        ON i.HardwareObjectID = sst.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('CLUSTERED') c
        ON i.HardwareObjectID = c.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('SPLEVEL') sp
        ON i.HardwareObjectID = sp.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('SQLSTATES') ss
        ON i.HardwareObjectID = ss.HardwareObjectID
    WHERE i.Property = 'Name'
GO

