USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetSoftwareMicrosoftExchange' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetSoftwareMicrosoftExchange]

GO
-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 31.01.2019
-- Copyright:   (c) Raynet GmbH
-- Description:	Get Microsoft Exchange Installation information
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================


CREATE FUNCTION [dbo].[f_GetSoftwareMicrosoftExchange]()
  RETURNS TABLE
AS
RETURN
  SELECT
      COALESCE(c.ComputerID, exS.ComputerID) AS ComputerID,
      exS.MGSExchangeServerID,
      exS.ComputerCN,
      exS.ComputerFQDN,
      'Microsoft'                            AS Publisher,
      'Microsoft Exchange'                   AS ProductName,
      exS.Version                            AS ProductVersion,
      exS.Edition                            AS ProductEdition,
      exS.Release                            AS ProductRelease
    FROM (SELECT DISTINCT
              ex.ComputerID,
              ex.HardwareObjectID AS MGSExchangeServerID,
              ex.Value            AS ComputerCN,
              exFQDN.Value        AS ComputerFQDN,
              exEd.Value          AS Edition,
              exV.Value           AS Version,
              CASE 
                WHEN exV.Value LIKE '%15.1 %' THEN '2016'
                WHEN exV.Value LIKE '%15.%'   THEN '2013'
                WHEN exV.Value LIKE '%14.%'   THEN '2010'
                WHEN exV.Value LIKE '%8.%'    THEN '2007'
                WHEN exV.Value LIKE '%6.5 %'  THEN '2003'
                WHEN exV.Value LIKE '%6.0 %'  THEN '2000'
                ELSE NULL
              END                 AS Release
            FROM f_HardwareProperties('MGS_ExchangeServer') ex
              LEFT JOIN f_HardwareValuesByPropertyName('fqdn') exFQDN
                ON ex.HardwareObjectID = exFQDN.HardwareObjectID
              LEFT JOIN f_HardwareValuesByPropertyName('Edition') exEd
                ON ex.HardwareObjectID = exED.HardwareObjectID
              LEFT JOIN f_HardwareValuesByPropertyName('AdminDisplayVersion') exV
                ON ex.HardwareObjectID = exV.HardwareObjectID
            WHERE ex.Property = 'Name') exS
      LEFT JOIN dbo.f_GetDeviceFQDN(1) c
        ON exS.ComputerFQDN = c.FQDN
GO


