USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetSoftwareMicrosoftSharePoint' AND xtype = 'TF'
)
        DROP FUNCTION [dbo].[f_GetSoftwareMicrosoftSharePoint]
GO
-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 26.06.2018
-- Copyright:   (c) Raynet GmbH
-- Description:	Get Microsoft SharePoint Installation
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================


CREATE FUNCTION [dbo].[f_GetSoftwareMicrosoftSharePoint] ()
  RETURNS @returns TABLE (ComputerID      BIGINT,
                          MGSSPFarmID     BIGINT,
                          Publisher       NVARCHAR(10),
                          ProductName     NVARCHAR(125), 
                          ProductEdition  NVARCHAR(32), 
                          ProductRelease  NVARCHAR(32), 
                          ProductVersion  NVARCHAR(32), 
                          FullProductName NVARCHAR(255),
                          SKUID           UNIQUEIDENTIFIER)
AS
BEGIN
  DECLARE @SKUID_List TABLE (
    SKUID           UNIQUEIDENTIFIER,    
    ProductName     NVARCHAR(125),
    ProductEdition  NVARCHAR(32),
    ProductRelease  NVARCHAR(32),
    FullProductName NVARCHAR(255))

  -- 2010
  INSERT INTO @SKUID_LIST VALUES ('84902853-59F6-4B20-BC7C-DE4F419FEFAD', 'Project Server',          'Trial',              '2010', 'Project Server 2010 Trial')
  INSERT INTO @SKUID_LIST VALUES ('ED21638F-97FF-4A65-AD9B-6889B93065E2', 'Project Server',          '',                   '2010', 'Project Server 2010')
  INSERT INTO @SKUID_LIST VALUES ('BC4C1C97-9013-4033-A0DD-9DC9E6D6C887', 'Search Server',           'Trial',              '2010', 'Search Server 2010 Trial')
  INSERT INTO @SKUID_LIST VALUES ('08460AA2-A176-442C-BDCA-26928704D80B', 'Search Server',           '',                   '2010', 'Search Server 2010')
  INSERT INTO @SKUID_LIST VALUES ('1328E89E-7EC8-4F7E-809E-7E945796E511', 'Search Server',           'Express',            '2010', 'Search Server Express 2010')
  INSERT INTO @SKUID_LIST VALUES ('BEED1F75-C398-4447-AEF1-E66E1F0DF91E', 'SharePoint',              'Foundation',         '2010', 'SharePoint Foundation 2010')
  INSERT INTO @SKUID_LIST VALUES ('B2C0B444-3914-4ACB-A0B8-7CF50A8F7AA0', 'SharePoint',              'Standard Trial',     '2010', 'SharePoint Server 2010 Standard Trial') 
  INSERT INTO @SKUID_LIST VALUES ('3FDFBCC8-B3E4-4482-91FA-122C6432805C', 'SharePoint',              'Standard',           '2010', 'SharePoint Server 2010 Standard')
  INSERT INTO @SKUID_LIST VALUES ('88BED06D-8C6B-4E62-AB01-546D6005FE97', 'SharePoint',              'Enterprise Trial',   '2010', 'SharePoint Server 2010 Enterprise Trial')
  INSERT INTO @SKUID_LIST VALUES ('D5595F62-449B-4061-B0B2-0CBAD410BB51', 'SharePoint',              'Enterprise',         '2010', 'SharePoint Server 2010 Enterprise')
  INSERT INTO @SKUID_LIST VALUES ('926E4E17-087B-47D1-8BD7-91A394BC6196', 'Office Web Applications', '',                   '2010', 'Office Web Applications 2010')

  -- 2013
  INSERT INTO @SKUID_LIST VALUES ('35466B1A-B17B-4DFB-A703-F74E2A1F5F5E', 'Project Server',          '',                   '2013', 'Project Server 2013')
  INSERT INTO @SKUID_LIST VALUES ('BC7BAF08-4D97-462C-8411-341052402E71', 'Project Server',          'Preview',            '2013', 'Project Server 2013 Preview')
  INSERT INTO @SKUID_LIST VALUES ('9FF54EBC-8C12-47D7-854F-3865D4BE8118', 'SharePoint',              'Foundation',         '2013', 'SharePoint Foundation 2013')
  INSERT INTO @SKUID_LIST VALUES ('C5D855EE-F32B-4A1C-97A8-F0A28CE02F9C', 'SharePoint',              'Standard',           '2013', 'SharePoint Server 2013')
  INSERT INTO @SKUID_LIST VALUES ('CBF97833-C73A-4BAF-9ED3-D47B3CFF51BE', 'SharePoint',              'Standard Preview',   '2013', 'SharePoint Server 2013 Preview')
  INSERT INTO @SKUID_LIST VALUES ('B7D84C2B-0754-49E4-B7BE-7EE321DCE0A9', 'SharePoint',              'Enterprise',         '2013', 'SharePoint Server 2013 Enterprise')
  INSERT INTO @SKUID_LIST VALUES ('298A586A-E3C1-42F0-AFE0-4BCFDC2E7CD0', 'SharePoint',              'Enterprise Preview', '2013', 'SharePoint Server 2013 Enterprise Preview')
  INSERT INTO @SKUID_LIST VALUES ('D6B57A0D-AE69-4A3E-B031-1F993EE52EDC', 'Office Web Applications', '',                   '2013', 'Microsoft Office Web Apps Server 2013')

  -- 2016
  INSERT INTO @SKUID_LIST VALUES ('435d4d60-f4cf-421d-abc8-129e4b57f7a6', 'SharePoint',              'Foundation',         '2016', 'SharePoint Foundation 2016')
  INSERT INTO @SKUID_LIST VALUES ('5DB351B8-C548-4C3C-BFD1-82308C9A519B', 'SharePoint',              'Trail',              '2016', 'SharePoint Server 2016 Trail')
  INSERT INTO @SKUID_LIST VALUES ('4F593424-7178-467A-B612-D02D85C56940', 'SharePoint',              'Standard',           '2016', 'SharePoint Server 2016 Standard')
  INSERT INTO @SKUID_LIST VALUES ('716578D2-2029-4FF2-8053-637391A7E683', 'SharePoint',              'Enterprise',         '2016', 'SharePoint Server 2016 Enterprise')

  INSERT INTO @returns
  SELECT 
      s.ComputerID, 
      s.HardwareObjectID   AS MGSSPFarmID,
      'Microsoft'          AS Publisher,
      sku.ProductName, 
      sku.ProductEdition, 
      sku.ProductRelease, 
      b.Value              AS ProductVersion, 
      sku.FullProductName,
      sku.SKUID
    FROM f_HardwareProperties('MGS_SPFarm') s
      LEFT JOIN @SKUID_List sku
        ON s.Value LIKE '%' + CAST(sku.SKUID AS NVARCHAR(36)) + '%'
      LEFT JOIN f_HardwareValuesByPropertyName('BuildVersion') b
        ON s.HardwareObjectID = b.HardwareObjectID
    WHERE s.Property = 'Products'
  RETURN
END
