USE [ManageSoft]
GO
/****** Object:  StoredProcedure [dbo].[csp_aspera_connector_device]    Script Date: 18.07.2019 09:40:27 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- =============================================
-- Author:      PA, Raynet GmbH
-- Create date: 30.04.2019
-- Description:	Get Device Information
-- =============================================
-- Connector version:4.0.94.29 
-- =============================================

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'csp_aspera_connector_device' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[csp_aspera_connector_device]
GO

CREATE PROCEDURE [dbo].[csp_aspera_connector_device] (
  @sDataSourceID   NVARCHAR(50) = 'RayVentory',             -- DataSourceID best is RayVentory_{CustomerName}
  @sOrgLevel2ID    NVARCHAR(50) = 'unknown_costcenter',     -- Default OrgLevel 2 ID
  @bImportIDFQDN   BIT          = 1,                        -- 0 ImportID is Hostname, 1 is FQDN (default)
  @bChassisTypeID  BIT          = 0,                        -- 0 Chassis import_device_type_id as Chassis Description, 1 Chassis Type ID as chassis_{ID from WMI}
  @bVMwareRelation BIT          = 1,                        -- 0 no VMware Relation, 1 VMWare Relation (default)
  @bHyperVRelation BIT          = 1,                        -- 0 no HyperV Relation, 1 HyperV Relation (default)
  @bHWPartRelation BIT          = 1,                        -- 0 no Hardware Partition like Zone or LPAR Relation,   1 Hardware Partition Releation  (default)
  @iLastSeenDays   INT          = 90)                       -- What should be the oldest Inventory Date to use.
AS
BEGIN
--   DECLARE @sDataSourceID       NVARCHAR(50)  = 'RayVentory_(Customer)'
--   DECLARE @sOrgLevel2ID        NVARCHAR(50)  = 'Temp'
--   DECLARE @bImportIDFQDN       BIT           = 1
--   DECLARE @bChassisTypeID      INT           = 0
--   DECLARE @bVMWareRelation     BIT           = 0
--   DECLARE @bHyperVRelation     BIT           = 0
--   DECLARE @bHWPartRelation     BIT           = 1
--   DECLARE @iLastSeenDays       INT           = 90

  DECLARE @bDebug              BIT           = 0

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Create Temp Device Type Table'

  IF OBJECT_ID('tempdb..#tmpDevicesTypes') IS NOT NULL
    DROP TABLE #tmpDevicesTypes

  CREATE TABLE #tmpDevicesTypes (
    ComputerID                 INT,
    [Type]                     NVARCHAR(50))

  -- Zone Partitionen
  INSERT INTO #tmpDevicesTypes
  SELECT
      GuestID      AS ComputerID,
	  'zone'       AS [Type]
    FROM dbo.f_GetVirtualZoneLinkGuest() lz

  -- LPAR
  INSERT INTO #tmpDevicesTypes
  SELECT
      GuestID      AS ComputerID,
	  'lpar'       AS [Type]
    FROM dbo.f_GetVirtualLPARLinkGuest() lz
	
  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Create Temp Device Table'

  IF OBJECT_ID('tempdb..#tmpDevices') IS NOT NULL
    DROP TABLE #tmpDevices

  CREATE TABLE #tmpDevices (
    ComputerID                 INT,
    import_id                  NVARCHAR(125)  COLLATE database_default NULL,
    import_data_source_id      NVARCHAR(50)   COLLATE database_default NULL,
    device_key                 NVARCHAR(255)  COLLATE database_default NOT NULL,
    import_org_level_2_id      NVARCHAR(50)   COLLATE database_default NOT NULL,
    device_name                NVARCHAR(255)  COLLATE database_default NULL,
    device_manufacturer        NVARCHAR(255)  COLLATE database_default NULL,
    device_model               NVARCHAR(255)  COLLATE database_default NULL,
    serial_number              NVARCHAR(255)  COLLATE database_default NULL,
    import_operating_system_id NVARCHAR(255)  COLLATE database_default NULL,
    import_device_type_id      NVARCHAR(50)   COLLATE database_default NULL,
    import_device_status_id    NVARCHAR(50)   COLLATE database_default NULL,
    import_domain_id           NVARCHAR(50)   COLLATE database_default NULL,
    import_user_id             NVARCHAR(50)   COLLATE database_default NULL,
    ip_address                 NVARCHAR(255)  COLLATE database_default NULL,
    mac_address                NVARCHAR(255)  COLLATE database_default NULL,
    fqdn                       NVARCHAR(255)  COLLATE database_default NULL,
    inventory_date             DATETIME                                NULL,
    installation_date          DATETIME                                NULL,
    cpu_chip_count             DECIMAL(15, 2)                          NULL,
    cpu_core_count             DECIMAL(15, 2)                          NULL,
    cpu_speed                  INT                                     NULL,
    import_cpu_type_id         NVARCHAR(255)  COLLATE database_default NULL,
    ram                        BIGINT                                  NULL,
    storage                    BIGINT                                  NULL,
    graphics                   NVARCHAR(255)  COLLATE database_default NULL,
    network                    NVARCHAR(255)  COLLATE database_default NULL,
    bios                       NVARCHAR(255)  COLLATE database_default NULL,
    cpu_core_count_limited     DECIMAL(15, 2)                          NULL,
    tmp_CIMProduct             BIGINT NULL,
    tmp_CIMBIOSElement         BIGINT NULL,
	tmp_ModelVirtual           BIT)

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Add Device List'

  INSERT INTO #tmpDevices (ComputerID,
    import_id, import_data_source_id, device_key, import_org_level_2_id,
    device_name, device_manufacturer, device_model, 
    import_device_status_id, import_domain_id,
    fqdn, inventory_date, tmp_CIMProduct, tmp_ModelVirtual)
  SELECT
      c.ComputerID,
      LEFT(IIF(@bImportIDFQDN = 1, c.FQDN, c.ComputerCN), 125) AS import_id,
      @sDataSourceID                                           AS import_data_source_id,
      LEFT(IIF(@bImportIDFQDN = 1, c.FQDN, c.ComputerCN), 255) AS device_key,
      @sOrgLevel2ID                                            AS import_org_level_2_id,
      c.ComputerCN                                             AS device_name,
      m.Manufacturer                                           AS device_manufacturer,
      m.Model                                                  AS device_model,
      'Active'                                                 AS import_device_status_id,
      LEFT(IIF(CHARINDEX('.', c.FQDN) > 0, REPLACE(c.FQDN, c.ComputerCN + '.', ''), NULL), 50)
                                                               AS import_domain_id,
      LEFT(c.FQDN, 255)                                        AS fqdn,
      ir.LastInventoryDate                                     AS inventory_date,
      m.CIMProductID                                           AS tmp_CIMProduct,
      m.Virtual                                                AS tmp_ModelVirtual
    FROM dbo.f_GetDeviceFQDN(1) c
      INNER JOIN dbo.f_GetLastInventoryDate(1) ir
        ON c.ComputerID = ir.ComputerID
          AND (@iLastSeenDays IS NULL OR DATEDIFF(day,  ir.LastInventoryDate, CURRENT_TIMESTAMP) <= @iLastSeenDays)
      LEFT JOIN dbo.f_GetDeviceModel() m
        ON c.ComputerID = m.ComputerID

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Add Serial Number'
  UPDATE dev
      SET serial_number      = s.SerialNumber,
          tmp_CIMBIOSElement = CIMBIOSElementID
    FROM #tmpDevices dev
      INNER JOIN dbo.f_GetDeviceSerialNumber() s
        ON dev.ComputerID = s.ComputerID

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Add BIOS Version'
  UPDATE dev
      SET bios               = bV.Value
    FROM #tmpDevices dev
      INNER JOIN dbo.f_HardwareValuesByPropertyName('Version') bV
        ON dev.tmp_CIMBIOSElement = bV.HardwareObjectID

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Add import_device_type_id'
  UPDATE dev
      SET import_device_type_id = CASE
        WHEN dt.ComputerID > 0              THEN dt.[Type]
        WHEN dev.tmp_ModelVirtual = 1       THEN 'virtual_machine'
		WHEN ch.TypeID IS NULL              THEN IIF(@bChassisTypeID = 1, 'chassis_2', 'Unknown')
        WHEN @bChassisTypeID = 1            THEN 'chassis_' + CAST(ch.TypeID AS VARCHAR(5))
        ELSE ch.[Type]
      END
    FROM #tmpDevices dev
      INNER JOIN dbo.f_GetDeviceChassis(0) ch
        ON dev.ComputerID = ch.ComputerID
      LEFT JOIN #tmpDevicesTypes dt
        ON dev.ComputerID = dt.ComputerID

/* Alternativ zu Add import_device_type_id
  UPDATE dev
      SET import_device_type_id = dt.[Type]
    FROM #tmpDevices dev
      INNER JOIN #tmpDevicesTypes dt
        ON dev.ComputerID = dt.ComputerID

  UPDATE dev
      SET import_device_type_id = 'virtual_machine'
    FROM #tmpDevices dev
    WHERE tmp_ModelVirtual = 1

  UPDATE dev
      SET import_device_type_id = IIF(ch.TypeID >= 0, 
                                      IIF(@bChassisTypeID = 1, 'chassis_' + CAST(ch.TypeID AS VARCHAR(5)), ch.[Type]), 
                                      IIF(@bChassisTypeID = 1, 'chassis_2', 'Unknown'))
    FROM #tmpDevices dev
      INNER JOIN dbo.f_GetDeviceChassis(0) ch
        ON dev.ComputerID = ch.ComputerID
    WHERE import_device_type_id IS NULL


  INSERT INTO #tmpDevices (ComputerID,
    import_id, import_data_source_id, device_key, import_org_level_2_id,
    device_name, device_manufacturer, device_model, serial_number, 
    import_device_type_id, import_device_status_id, import_domain_id,
    fqdn, bios, inventory_date)
  SELECT
      c.ComputerID,
      LEFT(IIF(@bImportIDFQDN = 1, c.FQDN, c.ComputerCN), 125) AS import_id,
      @sDataSourceID                                           AS import_data_source_id,
      LEFT(IIF(@bImportIDFQDN = 1, c.FQDN, c.ComputerCN), 255) AS device_key,
      @sOrgLevel2ID                                            AS import_org_level_2_id,
      c.ComputerCN                                             AS device_name,
      d.Manufacturer                                           AS device_manufacturer,
      d.Model                                                  AS device_model,
      d.SerialNumber                                           AS serial_number,
      CASE
        WHEN dt.ComputerID > 0              THEN dt.[Type]
        WHEN d.Virtual = 1                  THEN 'virtual_machine'
		WHEN d.ChassisTypeID IS NULL        THEN IIF(@bChassisTypeID = 1, 'chassis_2', 'Unknown')
        WHEN @bChassisTypeID = 1            THEN 'chassis_' + CAST(d.ChassisTypeID AS VARCHAR(5))
        ELSE d.ChassisType
      END                                                      AS import_device_type_id,
      'Active'                                                 AS import_device_status_id,
      IIF(CHARINDEX('.', c.FQDN) > 0, REPLACE(c.FQDN, c.ComputerCN + '.', ''), NULL)
                                                               AS import_domain_id,
      c.FQDN                                                   AS fqdn,
      d.BiosVersion                                            AS bios,
      LEFT(CONVERT(VARCHAR,ir.LastInventoryDate,120),10)       AS inventory_date
    FROM dbo.f_GetDeviceFQDN(1) c
      INNER JOIN dbo.f_GetLastInventoryDate(1) ir
        ON c.ComputerID = ir.ComputerID
          AND (@iLastSeenDays IS NULL OR DATEDIFF(day,  ir.LastInventoryDate, CURRENT_TIMESTAMP) <= @iLastSeenDays)
      LEFT JOIN dbo.f_GetDeviceInfo() d
        ON c.ComputerID = d.ComputerID
      LEFT JOIN #tmpDevicesTypes dt
        ON c.ComputerID = dt.ComputerID */

/*		
  UPDATE dev
      SET inventory_date             = LEFT(CONVERT(VARCHAR,ir.LastInventoryDate,120),10)
    FROM #tmpDevices dev
      INNER JOIN dbo.f_GetLastInventoryDate(1) ir
        ON dev.ComputerID = ir.ComputerID
*/	
  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Add OS Information'
  UPDATE dev
      SET import_operating_system_id = os.ProductName,
          import_user_id             = IIF(os.LastLogin LIKE '%SYSTEM', NULL, os.LastLogin),
          installation_date          = os.InstallDate
    FROM #tmpDevices dev
      INNER JOIN dbo.f_GetDeviceOSInfo() os
        ON dev.ComputerID = os.ComputerID

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Add Network Information'
  UPDATE dev
      SET ip_address                 = LEFT(nic.IPAddress, 255),
          mac_address                = LEFT(nic.MACAddress, 255),
          network                    = nic.Model
    FROM #tmpDevices dev
      INNER JOIN dbo.f_GetDeviceNetworkInfo() nic
        ON dev.ComputerID = nic.ComputerID

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Add Processor Information'
  UPDATE dev
      SET cpu_chip_count             = cpu.TotalCpus,
          cpu_core_count             = cpu.TotalCores,
          cpu_speed                  = cpu.ClockSpeed,
          import_cpu_type_id         = cpu.Name
    FROM #tmpDevices dev
      INNER JOIN dbo.f_GetDeviceProcessor() cpu
        ON dev.ComputerID = cpu.ComputerID

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Add Memory Size'
  UPDATE dev
      SET ram                        = mem.Size
    FROM #tmpDevices dev
      INNER JOIN (SELECT ComputerID, SUM(Size) Size FROM dbo.f_GetDeviceMemoryModul() GROUP BY ComputerID) mem
        ON dev.ComputerID = mem.ComputerID

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Add Total Disk capacity'
  UPDATE dev
      SET storage                    = dsk.TotalCapacity
    FROM #tmpDevices dev
      INNER JOIN (SELECT ComputerID, SUM(TotalCapacity) TotalCapacity FROM dbo.f_GetDeviceDiskDrive() GROUP BY ComputerID) dsk
        ON dev.ComputerID = dsk.ComputerID

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Add Video Controller'
  UPDATE dev
      SET  graphics                  = vc.Model
    FROM #tmpDevices dev
      INNER JOIN (SELECT ComputerID, MAX(Model) AS Model FROM dbo.f_GetDeviceVideoController() GROUP BY ComputerID) vc
        ON dev.ComputerID = vc.ComputerID

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Change import_device_type_id if Virtual Host System'
-- VMware ESX Hosts
  UPDATE dev
      SET  import_device_type_id     = 'vmware_esx_host'
    FROM #tmpDevices dev
      INNER JOIN dbo.f_HardwareProperties('VMWARE_ESXComputerSystem') esx
        ON dev.ComputerID = esx.ComputerID
          AND esx.Property = 'ElementName'

-- HyperV Hosts
  UPDATE dev
      SET  import_device_type_id     = 'hyperv_host'
    FROM #tmpDevices dev
      INNER JOIN (SELECT DISTINCT HyperVHostID FROM f_GetVirtualHyperVGuestInfo()) h
        ON dev.ComputerID = h.HyperVHostID
		  
  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Remove double entries'
-- Remove Double Entries
  IF OBJECT_ID('tempdb..#tmpDeviceFix') IS NOT NULL
    DROP TABLE #tmpDeviceFix

  SELECT
      CASE
	    WHEN MIN(o.ComputerID) < n.ComputerID THEN MIN(o.ComputerID)
	    ELSE n.ComputerID
      END               AS toChangeComputerID,
      n.ComputerID      AS inventoryComputerID,
      tmp.import_id
  INTO #tmpDeviceFix
  FROM (SELECT 
            t.import_id, 
            MIN(t.inventory_date) inventory_date_min,
            MAX(t.inventory_date) inventory_date_max
          FROM #tmpDevices t
          GROUP BY t.import_id, t.serial_number, t.import_operating_system_id, t.installation_date
          HAVING COUNT(t.import_id) > 1) tmp
    LEFT JOIN #tmpDevices o
      ON o.import_id = tmp.import_id
        AND o.inventory_date < tmp.inventory_date_max
    LEFT JOIN #tmpDevices n
      ON n.import_id = tmp.import_id
        AND n.inventory_date = tmp.inventory_date_max
  GROUP BY tmp.import_id, n.ComputerID

  UPDATE dev
    SET import_user_id         = inv.import_user_id,
        ip_address             = inv.ip_address, 
        mac_address            = inv.mac_address,
        cpu_chip_count         = inv.cpu_chip_count,
        cpu_core_count         = inv.cpu_core_count,
        cpu_speed              = inv.cpu_speed,
        import_cpu_type_id     = inv.import_cpu_type_id,
        ram                    = inv.ram,
        storage                = inv.storage,
        graphics               = inv.graphics,
        network                = inv.network,
        bios                   = inv.bios,
        import_device_type_id  = inv.import_device_type_id
    FROM #tmpDevices dev
      INNER JOIN #tmpDeviceFix fix
        ON dev.ComputerID = fix.toChangeComputerID
	      AND fix.toChangeComputerID <> fix.inventoryComputerID
      LEFT JOIN #tmpDevices inv
        ON fix.inventoryComputerID = inv.ComputerID

  DELETE FROM #tmpDevices
    WHERE ComputerID IN (SELECT
                              dev.ComputerID
                            FROM #tmpDeviceFix fix
                              INNER JOIN #tmpDevices dev
                                ON fix.import_id = dev.import_id
                                  AND fix.toChangeComputerID <> dev.ComputerID)

  DROP TABLE #tmpDeviceFix 

-- Fix if Machinename is changed
  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Remove old inventories if machinename is changed'
  IF OBJECT_ID('tempdb..#tmpDeviceFixName') IS NOT NULL
    DROP TABLE #tmpDeviceFixName

  SELECT
      o.ComputerID      AS toRemoveComputerID     
  INTO #tmpDeviceFixName
  FROM (SELECT 
            t.serial_number, 
            MIN(t.inventory_date) inventory_date_min,
            MAX(t.inventory_date) inventory_date_max
          FROM #tmpDevices t
          GROUP BY t.import_id, t.serial_number, t.import_operating_system_id, CAST(t.installation_date AS DATE)
          HAVING COUNT(t.serial_number) > 1) tmp
    INNER JOIN #tmpDevices o
      ON o.serial_number = tmp.serial_number
        AND o.inventory_date < tmp.inventory_date_max
    LEFT JOIN #tmpDevices n
      ON n.serial_number = tmp.serial_number
        AND n.inventory_date = tmp.inventory_date_max
  WHERE o.import_operating_system_id = n.import_operating_system_id
    AND o.import_device_type_id = n.import_device_type_id
    AND o.installation_date = n.installation_date
 

  DELETE FROM #tmpDevices
    WHERE ComputerID IN (SELECT
                              fix.toRemoveComputerID
                            FROM #tmpDeviceFixName fix)

  DROP TABLE #tmpDeviceFixName

-- VMWare Cluster Devices
  IF @bVMWareRelation = 1
  BEGIN

    IF OBJECT_ID('tempdb..#f_GetVirtualVMwareLinkVCenter_dev') IS NOT NULL
      DROP TABLE #f_GetVirtualVMwareLinkVCenter_dev

	CREATE TABLE #f_GetVirtualVMwareLinkVCenter_dev(
		vCenterID			  BIGINT NULL,
		vCenterCN			  NVARCHAR(256)    COLLATE database_default NULL,
		vCenterFQDN			  NVARCHAR(256)    COLLATE database_default NULL,
		vCenterURLAddress     NVARCHAR(256)    COLLATE database_default NULL,
		vCenterUUID			  UNIQUEIDENTIFIER)

	CREATE CLUSTERED INDEX IX_vCenterUUID ON #f_GetVirtualVMwareLinkVCenter_dev (vCenterUUID)

	INSERT INTO #f_GetVirtualVMwareLinkVCenter_dev (vCenterID, vCenterCN, vCenterFQDN, vCenterURLAddress, vCenterUUID)
	SELECT vCenterID, vCenterCN, vCenterFQDN, vCenterURLAddress, vCenterUUID FROM dbo.f_GetVirtualVMwareLinkVCenter() 

    IF OBJECT_ID('tempdb..#f_GetVirtualVMwareLinkClusterName_dev') IS NOT NULL
      DROP TABLE #f_GetVirtualVMwareLinkClusterName_dev

    CREATE TABLE #f_GetVirtualVMwareLinkClusterName_dev (
      ESXHostID               BIGINT                                    NULL,
	  vCenterUUID             UNIQUEIDENTIFIER                          NULL,
	  vCenterCN				  NVARCHAR(256)    COLLATE database_default NULL,
	  vCenterFQDN			  NVARCHAR(256)    COLLATE database_default NULL,
      ParentID                NVARCHAR(256)    COLLATE database_default NULL,
      ClusterName             NVARCHAR(256)    COLLATE database_default NULL)

    INSERT INTO #f_GetVirtualVMwareLinkClusterName_dev (ESXHostID, vCenterUUID, vCenterCN, vCenterFQDN, ParentID, ClusterName)
    SELECT ESXHostID, vc.vCenterUUID, vlc.vCenterCN, vlc.vCenterFQDN, ParentID, ClusterName FROM  dbo.f_GetVirtualVMwareLinkClusterName() vc
	INNER JOIN #f_GetVirtualVMwareLinkVCenter_dev vlc
	ON vc.vCenterUUID = vlc.vCenterUUID

	/*
	UPDATE vc
	    SET vCenterCN = vlc.vCenterCN
		FROM #f_GetVirtualVMwareLinkClusterName_dev vc 
		INNER JOIN dbo.f_GetVirtualVMwareLinkVCenter() vlc
		ON vc.vCenterUUID = vlc.vCenterUUID */

    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Add VMWare Cluster Devices'
    INSERT INTO #tmpDevices (import_id, import_data_source_id, device_key, import_org_level_2_id, device_name, serial_number, import_operating_system_id, import_device_type_id, import_device_status_id, inventory_date)
    SELECT
        LEFT(ISNULL(cl.ParentID + '_', '') + cl.ClusterName + ISNULL('_' + RIGHT(CAST(cl.vCenterUUID AS nvarchar(255)),12), cl.vCenterFQDN), 125) AS import_id,
        @sDataSourceID                                            AS import_data_source_id,
        cl.ClusterName + '_' + CONVERT(nvarchar(255),cl.vCenterUUID)                                            AS device_key,
        @sOrgLevel2ID                                             AS import_org_level_2_id,
        cl.ClusterName                                            AS device_name,
        'virtual'                                                 AS serial_number,
        'Cluster'                                                 AS import_operating_system_id,
        'vmware_esx_cluster'                                      AS import_device_type_id,
        'Active'                                                  AS import_device_status_id,
        MIN(ir.LastInventoryDate)                                 AS inventory_date
      FROM #f_GetVirtualVMwareLinkClusterName_dev cl
        LEFT JOIN dbo.f_GetLastInventoryDate(1) ir
          ON cl.ESXHostID = ir.ComputerID
      WHERE (@iLastSeenDays IS NULL OR DATEDIFF(day,  ir.LastInventoryDate, CURRENT_TIMESTAMP) <= @iLastSeenDays)
      GROUP BY cl.vCenterUUID, cl.ParentID, cl.ClusterName, cl.vCenterFQDN
  END

-- HyperV Cluster Devices
  IF @bHyperVRelation = 1
  BEGIN
    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Add HyperV Cluster Devices'
    INSERT INTO #tmpDevices (import_id, import_data_source_id, device_key, import_org_level_2_id, device_name, serial_number, import_operating_system_id, import_device_type_id, import_device_status_id, inventory_date)
    SELECT
        LEFT(cl.ClusterName, 125)                               AS import_id,
        @sDataSourceID                                          AS import_data_source_id,
        cl.ClusterName                                          AS device_key,
        @sOrgLevel2ID                                           AS import_org_level_2_id,
        cl.ClusterName                                          AS device_name,
        'virtual'                                               AS serial_number,
        'Cluster'                                               AS import_operating_system_id,
        'hyperv_cluster'                                        AS import_device_type_id,
        'Active'                                                AS import_device_status_id,
        MIN(ir.LastInventoryDate)                               AS inventory_date
      FROM dbo.f_GetVirtualHyperVLinkClusterName() cl
        LEFT JOIN dbo.f_GetDeviceFQDN(1) c
          ON cl.HyperVHostID = c.ComputerID
        LEFT JOIN dbo.f_GetLastInventoryDate(1) ir
          ON cl.HyperVHostID = ir.ComputerID
      WHERE (@iLastSeenDays IS NULL OR DATEDIFF(day,  ir.LastInventoryDate, CURRENT_TIMESTAMP) <= @iLastSeenDays)
      GROUP BY cl.ClusterName
  END
  
  IF @bHWPartRelation = 1
  BEGIN
    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Add dummy host for zone if not exist inventory of Host'
    INSERT INTO #tmpDevices (import_id, import_data_source_id, device_key, import_org_level_2_id,
                             device_name, serial_number, import_operating_system_id, import_device_type_id, 
                             import_device_status_id, inventory_date, cpu_chip_count,
                             cpu_core_count, cpu_speed, import_cpu_type_id, ram)
    SELECT
        LEFT(vz.ProductHostID, 125)                        AS import_id,
        @sDataSourceID                                     AS import_data_source_id,
        LEFT(vz.ProductHostID, 125)                        AS device_key,
        @sOrgLevel2ID                                      AS import_org_level_2_id,
        LEFT(vz.ProductHostID, 125)                        AS device_name,
        'dummy_host_sun_zone'                              AS serial_number,
        vz.OperatingSystem                                 AS import_operating_system_id,
        'dummy_host'                                       AS import_device_type_id,
        'Active'                                           AS import_device_status_id,
        vz.LastInventoryDate                               AS inventory_date,
        vz.TotalCpus                                       AS cpu_chip_count,
        vz.TotalCores                                      AS cpu_core_count,
        vz.ClockSpeed                                      AS cpu_speed,
        vz.CPUName                                         AS import_cpu_type_id,
        vz.Memory                                          AS ram
      FROM dbo.f_GetVirtualZoneHostInfo() vz
      WHERE vz.ZoneHostID IS NULL
        AND (@iLastSeenDays IS NULL OR DATEDIFF(day,  vz.LastInventoryDate, CURRENT_TIMESTAMP) <= @iLastSeenDays)

    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Add dummy host for lpar if not exist inventory of Host'
    INSERT INTO #tmpDevices (import_id, import_data_source_id, device_key, import_org_level_2_id,
                             device_name, serial_number, import_operating_system_id, import_device_type_id, 
                             import_device_status_id, inventory_date, cpu_chip_count,
                             cpu_core_count, cpu_speed, import_cpu_type_id, ram)
    SELECT
        LEFT(COALESCE(vl.IdentifyingNumber, vl.ProductHostID), 125)
                                                           AS import_id,
        @sDataSourceID                                     AS import_data_source_id,
        LEFT(COALESCE(vl.IdentifyingNumber, vl.ProductHostID), 125)
                                                           AS device_key,
        @sOrgLevel2ID                                      AS import_org_level_2_id,
        LEFT(COALESCE(vl.IdentifyingNumber, vl.ProductHostID), 125)
                                                           AS device_name,
        'dummy_host_lpar'                                  AS serial_number,
        vl.OperatingSystem                                 AS import_operating_system_id,
        'dummy_host'                                       AS import_device_type_id,
        'Active'                                           AS import_device_status_id,
        vl.LastInventoryDate                               AS inventory_date,
        vl.TotalCpus                                       AS cpu_chip_count,
        vl.TotalCores                                      AS cpu_core_count,
        vl.ClockSpeed                                      AS cpu_speed,
        vl.CPUName                                         AS import_cpu_type_id,
        vl.Memory                                          AS ram
      FROM dbo.f_GetVirtualLPARHostInfo() vl
      WHERE vl.LPARHostID IS NULL
        AND (@iLastSeenDays IS NULL OR DATEDIFF(day,  vl.LastInventoryDate, CURRENT_TIMESTAMP) <= @iLastSeenDays)

    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Add lpar pools'
    IF OBJECT_ID('tempdb..#sp_GetVirtualLPARPools') IS NOT NULL
      DROP TABLE #sp_GetVirtualLPARPools

    CREATE TABLE #sp_GetVirtualLPARPools (
      LPARHostID                      BIGINT                                  NULL,
      LPARHostUnitaryComputerSystemID BIGINT                                  NULL,
      ProductHostID                   NVARCHAR(125)  COLLATE database_default NULL,
      IdentifyingNumber               NVARCHAR(125)  COLLATE database_default NULL,
      GuestID                         BIGINT                                  NULL,
      GuestUnitaryComputerSystemID    BIGINT                                  NOT NULL,
      SharedPoolID                    NVARCHAR(125)  COLLATE database_default NULL,
      SharedPoolCapacityInCores       INT                                     NULL,
      Entitlement                     DECIMAL(15,2)                           NULL)

    EXEC sp_GetVirtualLPARPools
    
    INSERT INTO #tmpDevices (import_id, import_data_source_id, device_key, import_org_level_2_id,
                             device_name, import_device_type_id, 
                             import_device_status_id, inventory_date, cpu_core_count)
    SELECT DISTINCT
        LEFT(CASE
               WHEN MAX(lp.ComputerID) > 0 THEN MAX(lp.import_id)
               ELSE COALESCE(p.IdentifyingNumber, p.ProductHostID)
             END, 125) + '_' + p.SharedPoolID              AS import_id,
        @sDataSourceID                                     AS import_data_source_id,
        LEFT(CASE
               WHEN MAX(lp.ComputerID) > 0 THEN MAX(lp.import_id)
               ELSE COALESCE(p.IdentifyingNumber, p.ProductHostID)
             END, 125) + '_' + p.SharedPoolID              AS device_key,
        @sOrgLevel2ID                                      AS import_org_level_2_id,
        LEFT(CASE
               WHEN MAX(lp.ComputerID) > 0 THEN MAX(lp.import_id)
               ELSE COALESCE(p.IdentifyingNumber, p.ProductHostID)
             END, 125) + '_' + p.SharedPoolID              AS device_name,
        'lpar_pool'                                        AS import_device_type_id,
        'Active'                                           AS import_device_status_id,
        MAX(d.inventory_date)                              AS inventory_date,
        MAX(p.SharedPoolCapacityInCores)                   AS cpu_core_count
--        p.Entitlement                                      AS cpu_core_count_limited
      FROM #sp_GetVirtualLPARPools p
        INNER JOIN #tmpDevices d
          ON p.GuestID = d.ComputerID
        LEFT JOIN #tmpDevices lp
          ON p.LPARHostID = lp.ComputerID
      GROUP BY p.IdentifyingNumber, p.ProductHostID, p.SharedPoolID

    UPDATE d
        SET cpu_core_count_limited = p.Entitlement
      FROM #tmpDevices d
        INNER JOIN #sp_GetVirtualLPARPools p
          ON d.ComputerID = p.GuestID

		  
    DROP TABLE #sp_GetVirtualLPARPools
  END

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Update import_device_type_id if virtual_machine'
  -- Update Device Type if Virtual
  UPDATE d
       SET import_device_type_id = 'virtual_machine'
    FROM #tmpDevices d
    WHERE import_operating_system_id NOT LIKE '%ESX%'
      AND ((device_manufacturer = 'VMware Inc.' AND bios IN ('INTEL  - 6040000', 'PhoenixBIOS 4.0 Release 6.0'))
        OR Serial_Number LIKE 'VMware-%')

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Update import_device_type_id if virtual_machine is a ESX OS'
  -- Update Device Type if Virtual
  UPDATE d
       SET import_device_type_id = 'virtual_machine'
    FROM #tmpDevices d
    WHERE import_operating_system_id LIKE '%ESX%'
      AND device_manufacturer = 'VMware Inc.' AND device_model = 'VMware Virtual Platform'

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Change null Date Values to empty string'
  -- Fix InstallDate and InventoryDate in the form for Smarttrack
  UPDATE dev
      SET inventory_date             = ''
    FROM #tmpDevices dev
    WHERE dev.inventory_date IS NULL

  UPDATE dev
      SET installation_date          = ''
    FROM #tmpDevices dev
    WHERE dev.installation_date IS NULL
	
  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '- Show device List'
  IF OBJECT_ID('tempdb..#csp_aspera_connector_device') IS NOT NULL
  BEGIN
    DECLARE @sCPUCoreCountLimited VARCHAR(32) = ''
	IF EXISTS(SELECT * FROM tempdb.INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME = 'cpu_core_count_limited' and TABLE_NAME LIKE '#csp_aspera_connector_device')
      SET @sCPUCoreCountLimited = 'cpu_core_count_limited,'
    DECLARE @sSQL VARCHAR(MAX) = '
      INSERT INTO #csp_aspera_connector_device (import_id, import_data_source_id, device_key, import_org_level_2_id, device_name, device_manufacturer, device_model,
                                                serial_number, import_operating_system_id, import_device_type_id, import_device_status_id, import_domain_id, import_user_id,
                                                ip_address, mac_address, fqdn, inventory_date, installation_date, cpu_chip_count, cpu_core_count, cpu_speed, import_cpu_type_id,
                                                ram, storage, graphics, network, bios, ' + @sCPUCoreCountLimited + 'ComputerID)
      SELECT
          import_id,
          import_data_source_id,
          device_key,
          import_org_level_2_id,
          device_name,
          device_manufacturer,
          device_model,
          serial_number,
          import_operating_system_id,
          import_device_type_id,
          import_device_status_id,
          import_domain_id,
          import_user_id,
          ip_address,
          mac_address,
          fqdn,
          LEFT(CONVERT(VARCHAR,inventory_date,104),10)           AS inventory_date,
          IIF(DATEPART(yyyy, installation_date) = 1900, '',
                LEFT(CONVERT(VARCHAR,installation_date,104),10)) AS installation_date,
          cpu_chip_count,
          cpu_core_count,
          cpu_speed,
          import_cpu_type_id,
          ram     / POWER(2,20) AS ram,
          storage / POWER(2,30) AS storage,
          graphics,
          network,
          bios,
          ' + @sCPUCoreCountLimited + '
          ComputerID                                             AS tmp_computerid
        FROM #tmpDevices'
    IF @bDebug =1 PRINT @sSQL
    EXEC (@sSQL)
  END
  ELSE
    SELECT
        import_id,
        import_data_source_id,
        device_key,
        import_org_level_2_id,
        device_name,
        device_manufacturer,
        device_model,
        serial_number,
        import_operating_system_id,
        import_device_type_id,
        import_device_status_id,
        import_domain_id,
        import_user_id,
        ip_address,
        mac_address,
        fqdn,
        LEFT(CONVERT(VARCHAR,inventory_date,104),10)           AS inventory_date,
        IIF(DATEPART(yyyy, installation_date) = 1900, '',
              LEFT(CONVERT(VARCHAR,installation_date,104),10)) AS installation_date,
        cpu_chip_count,
        cpu_core_count,
        cpu_speed,
        import_cpu_type_id,
        ram     / POWER(2,20) AS ram,
        storage / POWER(2,30) AS storage,
        graphics,
        network,
        bios,
        cpu_core_count_limited,
        ComputerID                                             AS tmp_computerid
      FROM #tmpDevices
	
  DROP TABLE #tmpDevices
END
