USE [ManageSoft]
GO
/****** Object:  StoredProcedure [dbo].[csp_aspera_connector_device_relation]    Script Date: 18.07.2019 09:54:32 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- =============================================
-- Author:      PA, Raynet GmbH
-- Create date: 30.01.2019
-- Description:	Get Device Relation Information
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'csp_aspera_connector_device_relation' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[csp_aspera_connector_device_relation]
GO

CREATE PROCEDURE [dbo].[csp_aspera_connector_device_relation] (
  @sDataSourceID     NVARCHAR(50) = 'RayVentory',             -- DataSourceID best is RayVentory_{CustomerName}
  @bImportIDFQDN     BIT          = 1,                        -- 0 ImportID is Hostname, 1 is FQDN (default)
  @bVMwareRelation   BIT          = 1,                        -- 0 no VMWare Relation, 1 VMWare Relation (default)
  @bHyperVRelation   BIT          = 1,                        -- 0 no HyperV Relation, 1 HyperV Relation (default)
  @bHWPartRelation   BIT          = 1,                        -- 0 no Hardware Partition like Zone or LPAR Relation,   1 Hardware Partition Releation  (default)
  @iLastSeenDays     INT          = 90,                       -- What should be the oldest Inventory Date to use.
  @bHyperVwithSerial BIT          = 0)                        -- HypeV Mappink, link every time by Serialnumber
AS
BEGIN
--  DECLARE @sDataSourceID       NVARCHAR(50)  = 'RayVentory_(Customer)'
--  DECLARE @sOrgLevel2ID        NVARCHAR(50)  = 'Temp'
--  DECLARE @bImportIDFQDN       BIT           = 1
--  DECLARE @bVMWareRelation     BIT           = 0
--  DECLARE @bHyperVRelation     BIT           = 0
--  DECLARE @bHyperVwithSerial   BIT           = 0 -- Link every time by Serialnumber
--  DECLARE @bHWPartRelation     BIT           = 1
--  DECLARE @iLastSeenDays       INT           = 360

  DECLARE @bDebug              BIT           = 1

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Create Temp Device Relation Table'

  IF OBJECT_ID('tempdb..#tmpDeviceRelation') IS NOT NULL
    DROP TABLE #tmpDeviceRelation

  CREATE TABLE #tmpDeviceRelation (
    import_id                       NVARCHAR(125) COLLATE database_default NULL,
    import_data_source_id           NVARCHAR(50)  COLLATE database_default NOT NULL,
    import_device_rel_type_id       NVARCHAR(50)  COLLATE database_default NOT NULL,
    import_from_device_id           NVARCHAR(125) COLLATE database_default NOT NULL,
    import_to_device_id             NVARCHAR(125) COLLATE database_default NOT NULL,
    tmp_hostid                      BIGINT                                 NULL,
	tmp_delete                      BIT                                    NULL,
    tmp_inventory_date_host         DATETIME                               NULL,
    tmp_ipaddress                   NVARCHAR(255) COLLATE database_default NULL,
    tmp_uuid                        NVARCHAR(40)  COLLATE database_default NULL,
    tmp_elementname                 NVARCHAR(125) COLLATE database_default NULL,
    tmp_powerstate                  NVARCHAR(255) COLLATE database_default NULL)

  IF OBJECT_ID('tempdb..#f_GetDeviceFQDN_rel') IS NOT NULL
    DROP TABLE #f_GetDeviceFQDN_rel

  CREATE TABLE #f_GetDeviceFQDN_rel (
    ComputerID               BIGINT                                      NOT NULL,
	ComputerCN               NVARCHAR(256)      COLLATE database_default NOT NULL,
    FQDN                     NVARCHAR(256)      COLLATE database_default NOT NULL)

  INSERT INTO #f_GetDeviceFQDN_rel
  SELECT * FROM dbo.f_GetDeviceFQDN(1)

  IF OBJECT_ID('tempdb..#f_GetInventoryDate_rel') IS NOT NULL
    DROP TABLE #f_GetInventoryDate_rel

  CREATE TABLE #f_GetInventoryDate_rel (
    ComputerID               BIGINT                                      NOT NULL,
	UserID                   BIGINT                                      NOT NULL,
    InventoryDate            DATETIME                                    NULL)

  INSERT INTO #f_GetInventoryDate_rel
  SELECT * FROM f_GetInventoryDate(1)

  IF OBJECT_ID('tempdb..#f_GetLastInventoryDate_rel') IS NOT NULL
    DROP TABLE #f_GetLastInventoryDate_rel

  CREATE TABLE #f_GetLastInventoryDate_rel (
    ComputerID               BIGINT                                      NOT NULL,
	UserID                   BIGINT                                      NOT NULL,
    InventoryDate            DATETIME                                    NULL)

  INSERT INTO #f_GetLastInventoryDate_rel
  SELECT * FROM f_GetLastInventoryDate(1) ir
    WHERE (@iLastSeenDays IS NULL OR DATEDIFF(day,  ir.LastInventoryDate, CURRENT_TIMESTAMP) <= @iLastSeenDays)

  IF @bVMWareRelation = 1
  BEGIN
  IF OBJECT_ID('tempdb..#sp_GetVirtualVMwareLinkGuest') IS NOT NULL
    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Get VMWARE Relations'

    IF OBJECT_ID('tempdb..#sp_GetVirtualVMwareLinkGuest') IS NOT NULL
      DROP TABLE #sp_GetVirtualVMwareLinkGuest

    CREATE TABLE #sp_GetVirtualVMwareLinkGuest (
      ESXHostID                BIGINT                                    NOT NULL,
      GuestID                  BIGINT                                    NULL,
      GuestUUID                UNIQUEIDENTIFIER                          NULL,
      VMWAREVMComputerSystemID BIGINT                                    NOT NULL,
      vCenterURLAddress        NVARCHAR(256)    COLLATE database_default NULL,
      vCenterUUID              UNIQUEIDENTIFIER                          NULL)

    EXEC sp_GetVirtualVMwareLinkGuest

	IF OBJECT_ID('tempdb..#f_GetVirtualVMwareLinkVCenter_rel') IS NOT NULL
      DROP TABLE #f_GetVirtualVMwareLinkVCenter_rel

	CREATE TABLE #f_GetVirtualVMwareLinkVCenter_rel(
		vCenterID			  BIGINT,
		vCenterCN			  NVARCHAR(256)    COLLATE database_default NULL,
		vCenterFQDN			  NVARCHAR(256)    COLLATE database_default NULL,
		vCenterURLAddress     NVARCHAR(256)    COLLATE database_default NULL,
		vCenterUUID			  UNIQUEIDENTIFIER)

	CREATE CLUSTERED INDEX IX_vCenterUUID ON #f_GetVirtualVMwareLinkVCenter_rel (vCenterUUID)

	INSERT INTO #f_GetVirtualVMwareLinkVCenter_rel (vCenterID, vCenterCN, vCenterFQDN, vCenterURLAddress, vCenterUUID)
	SELECT vCenterID, vCenterCN, vCenterFQDN, vCenterURLAddress, vCenterUUID FROM dbo.f_GetVirtualVMwareLinkVCenter() 


    IF OBJECT_ID('tempdb..#f_GetVirtualVMwareLinkClusterName_rel') IS NOT NULL
      DROP TABLE #f_GetVirtualVMwareLinkClusterName_rel

    CREATE TABLE #f_GetVirtualVMwareLinkClusterName_rel (
      ESXHostID               BIGINT                                    NULL,
	  vCenterUUID             UNIQUEIDENTIFIER                          NULL,
	  vCenterFQDN			  NVARCHAR(256)    COLLATE database_default NULL,
	  vCenterCN				  NVARCHAR(256)    COLLATE database_default NULL,
      ParentID                NVARCHAR(256)    COLLATE database_default NULL,
      ClusterName             NVARCHAR(256)    COLLATE database_default NULL)

	  CREATE CLUSTERED INDEX IX_vCenterUUID_cluster ON #f_GetVirtualVMwareLinkClusterName_rel (vCenterUUID)

    INSERT INTO #f_GetVirtualVMwareLinkClusterName_rel  (ESXHostID, vCenterUUID, vCenterFQDN, vCenterCN, ParentID, ClusterName)
    SELECT ESXHostID, vc.vCenterUUID, lvc.vCenterFQDN, lvc.vCenterCN, ParentID, ClusterName FROM  dbo.f_GetVirtualVMwareLinkClusterName() vc
	LEFT JOIN #f_GetVirtualVMwareLinkVCenter_rel lvc
	ON vc.vCenterUUID = lvc.vCenterUUID

	/*UPDATE vc
	    SET vCenterCN = vlc.vCenterCN
		FROM #f_GetVirtualVMwareLinkClusterName_rel vc 
		INNER JOIN dbo.f_GetVirtualVMwareLinkVCenter() vlc
		ON vc.vCenterUUID = vlc.vCenterUUID*/

    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert ESX Host - Cluster relations'

    INSERT INTO #tmpDeviceRelation (import_id, import_data_source_id, import_device_rel_type_id,
                                    import_from_device_id, import_to_device_id)
    SELECT
        LEFT(IIF(@bImportIDFQDN = 0, esx.ComputerCN, esx.fqdn) + '_' + ISNULL(cl.ParentID + '_', '') + cl.ClusterName + '_' + ISNULL(RIGHT(CAST(cl.vCenterUUID AS nvarchar(255)),12), cl.vCenterFQDN), 125)
                                                       AS import_id,
        @sDataSourceID                                 AS import_data_source_id,
        'is_part_of_esx_cluster'                       AS import_device_rel_type_id,
        LEFT(IIF(@bImportIDFQDN = 0, esx.ComputerCN, esx.fqdn), 125)
                                                       AS import_from_device_id,
        LEFT(ISNULL(cl.ParentID + '_', '') + cl.ClusterName + '_' + ISNULL(RIGHT(CAST(cl.vCenterUUID AS nvarchar(255)),12), cl.vCenterFQDN), 125)
                                                       AS import_to_device_id
      FROM #f_GetVirtualVMwareLinkClusterName_rel cl
        INNER JOIN #f_GetLastInventoryDate_rel ir
          ON cl.ESXHostID = ir.ComputerID
        LEFT JOIN #f_GetDeviceFQDN_rel esx
          ON cl.ESXHostID = esx.ComputerID

    IF @bImportIDFQDN = 0
    BEGIN
      IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert Device - Cluster/ESX relations (Hostname)'
      INSERT INTO #tmpDeviceRelation (import_id, import_data_source_id, import_device_rel_type_id,
                                      import_from_device_id, import_to_device_id, tmp_hostid, tmp_ipaddress, tmp_uuid, tmp_elementname, tmp_powerstate)
      SELECT DISTINCT
          LEFT(
            COALESCE(c.ComputerCN, gi.HostName, gi.ElementName) + '_' +                   -- from ID (fqdn, if not vmware, elementname)
            ISNULL(ISNULL(cl.ParentID + '_', '') + cl.ClusterName + '_' + ISNULL(RIGHT(CAST(cl.vCenterUUID AS nvarchar(255)),12), cl.vCenterFQDN), esx.ComputerCN), 125)  -- to   ID (cluster, if not cluster then esx)
                                                                AS import_id,
          @sDataSourceID                                        AS import_data_source_id,
          'is_running_on_esx'                                   AS import_device_rel_type_id,
          LEFT(COALESCE(c.ComputerCN, gi.HostName, gi.ElementName) , 125)    
                                                                AS import_from_device_id,
          LEFT(ISNULL(ISNULL(cl.ParentID + '_', '') + cl.ClusterName + '_' + ISNULL(RIGHT(CAST(cl.vCenterUUID AS nvarchar(255)),12), cl.vCenterFQDN), esx.ComputerCN), 125)
                                                                AS import_to_device_id,
          gst.ESXHostID                                         AS tmp_hostid,
          LEFT(gi.IPAddress , 255)                              AS tmp_ipaddress,
          REPLACE(CAST(gst.GuestUUID AS NVARCHAR(40)), '-', '') AS tmp_uuid,
          LEFT(gi.ElementName, 125)                             AS tmp_elementname,
          gi.PowerState                                         AS tmp_powerstate
        FROM #sp_GetVirtualVMwareLinkGuest gst
          INNER JOIN #f_GetLastInventoryDate_rel ir
            ON gst.ESXHostID = ir.ComputerID
          LEFT JOIN #f_GetVirtualVMwareLinkClusterName_rel cl
            ON gst.ESXHostID = cl.ESXHostID
          LEFT JOIN #f_GetDeviceFQDN_rel c
            ON gst.GuestID = c.ComputerID
          LEFT JOIN #f_GetDeviceFQDN_rel esx
            ON gst.ESXHostID = esx.ComputerID
          LEFT JOIN (SELECT 
                         VMWAREVMComputerSystemID,
                         CASE
                           WHEN CHARINDEX('.' , HostName) > 0    THEN LEFT(HostName, CHARINDEX('.' , HostName) -1)
                           WHEN LEN(Hostname) > 0                THEN Hostname
                           WHEN CHARINDEX('.' , ElementName) > 0 THEN LEFT(ElementName, CHARINDEX('.' , ElementName) -1)
                         END AS Hostname,
                         CASE
                           WHEN CHARINDEX('.' , HostName) > 0    THEN HostName
                           WHEN CHARINDEX('.' , ElementName) > 0 THEN ElementName
                           WHEN LEN(Hostname) > 0                THEN Hostname
                         END AS FQDN,
                         ElementName,
                         IPAddress,
                         PowerState
                       FROM dbo.f_GetVirtualVMwareGuestInfo()) gi
            ON gst.VMWAREVMComputerSystemID = gi.VMWAREVMComputerSystemID
    END
    ELSE
    BEGIN
      IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert Device - Cluster/ESX relations (FQDN)'
      INSERT INTO #tmpDeviceRelation (import_id, import_data_source_id, import_device_rel_type_id,
                                      import_from_device_id, import_to_device_id, tmp_hostid, tmp_ipaddress, tmp_uuid, tmp_elementname, tmp_powerstate)
      SELECT DISTINCT
          LEFT(
            COALESCE(c.fqdn, gi.HostName, gi.ElementName) + '_' +                         -- from ID (fqdn, if not vmware, elementname)
            ISNULL(ISNULL(cl.ParentID + '_', '') + cl.ClusterName + '_' + ISNULL(RIGHT(CAST(cl.vCenterUUID AS nvarchar(255)),12), cl.vCenterFQDN), esx.fqdn), 125)        -- to   ID (cluster, if not cluster then esx)
                                                                AS import_id,
          @sDataSourceID                                        AS import_data_source_id,
          'is_running_on_esx'                                   AS import_device_rel_type_id,
          LEFT(COALESCE(c.fqdn, gi.HostName, gi.ElementName) , 125)    
                                                                AS import_from_device_id,
          LEFT(ISNULL(ISNULL(cl.ParentID + '_', '') + cl.ClusterName + '_' + ISNULL(RIGHT(CAST(cl.vCenterUUID AS nvarchar(255)),12), cl.vCenterFQDN), esx.fqdn), 125)
                                                                AS import_to_device_id,
          gst.ESXHostID                                         AS tmp_hostid,
          LEFT(gi.IPAddress , 255)                              AS tmp_ipaddress,
          REPLACE(CAST(gst.GuestUUID AS NVARCHAR(40)), '-', '') AS tmp_uuid,
          LEFT(gi.ElementName, 125)                             AS tmp_elementname,
          gi.PowerState                                         AS tmp_powerstate
        FROM #sp_GetVirtualVMwareLinkGuest gst
          INNER JOIN #f_GetLastInventoryDate_rel ir
            ON gst.ESXHostID = ir.ComputerID
          LEFT JOIN #f_GetVirtualVMwareLinkClusterName_rel cl
            ON gst.ESXHostID = cl.ESXHostID
          LEFT JOIN dbo.f_GetDeviceFQDN(1) c
            ON gst.GuestID = c.ComputerID
          LEFT JOIN dbo.f_GetDeviceFQDN(1) esx
            ON gst.ESXHostID = esx.ComputerID
          LEFT JOIN dbo.f_GetVirtualVMwareGuestInfo() gi
            ON gst.VMWAREVMComputerSystemID = gi.VMWAREVMComputerSystemID
    END
    DROP TABLE #sp_GetVirtualVMwareLinkGuest
    DROP TABLE #f_GetVirtualVMwareLinkClusterName_rel
  END
  
  IF @bHyperVRelation = 1
  BEGIN
    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert HyperV Host Cluster relations'

    IF OBJECT_ID('tempdb..#f_GetVirtualHyperVLinkClusterName_rel') IS NOT NULL
      DROP TABLE #f_GetVirtualHyperVLinkClusterName_rel

    CREATE TABLE #f_GetVirtualHyperVLinkClusterName_rel (
        HyperVHostID   BIGINT                                    NOT NULL,
        ClusterName    NVARCHAR(256)    COLLATE database_default NULL,
        IsFailover     BIT                                       NULL)

    INSERT INTO #f_GetVirtualHyperVLinkClusterName_rel
    SELECT * FROM dbo.f_GetVirtualHyperVLinkClusterName() cl

    INSERT INTO #tmpDeviceRelation (import_id, import_data_source_id, import_device_rel_type_id,
                                    import_from_device_id, import_to_device_id)
    SELECT
        LEFT(IIF(@bImportIDFQDN = 0, hv.ComputerCN, hv.FQDN) + 
             '_'  + cl.ClusterName, 125)                            AS import_id,
        @sDataSourceID                                              AS import_data_source_id,
        'is_part_of_hyperv_cluster'                                 AS import_device_rel_type_id,
        LEFT(IIF (@bImportIDFQDN = 0, hv.ComputerCN, hv.fqdn), 125) AS import_from_device_id,
        LEFT(cl.ClusterName, 125)                                   AS import_to_device_id
      FROM #f_GetVirtualHyperVLinkClusterName_rel cl
        INNER JOIN #f_GetLastInventoryDate_rel ir
          ON cl.HyperVHostID = ir.ComputerID
        LEFT JOIN #f_GetDeviceFQDN_rel hv
          ON cl.HyperVHostID = hv.ComputerID

    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert Device - Cluster/HyperV Host relations'

    IF OBJECT_ID('tempdb..#sp_GetVirtualHyperVLinkGuest') IS NOT NULL
      DROP TABLE #sp_GetVirtualHyperVLinkGuest

    CREATE TABLE #sp_GetVirtualHyperVLinkGuest (
        HyperVHostID   BIGINT                                    NOT NULL,
        HyperVHostCN   NVARCHAR(256)    COLLATE database_default NULL,
        HyperVHostFQDN NVARCHAR(256)    COLLATE database_default NULL,
        GuestID        BIGINT                                    NULL,
        GuestCN        NVARCHAR(256)    COLLATE database_default NULL,
        GuestFQDN      NVARCHAR(256)    COLLATE database_default NULL,
        PowerState     NVARCHAR(30)     COLLATE database_default NULL)

    EXEC sp_GetVirtualHyperVLinkGuest @bHyperVwithSerial

    IF OBJECT_ID('tempdb..#tmpHyperVLink') IS NOT NULL
      DROP TABLE #tmpHyperVLink

    -- The 2 Steps are need to Performance, if i made this in one step iot takes 40min, in this way 10min
    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '   - Create Temp List - Cluster/HyperV Host relations'
    SELECT 
        h.HyperVHostID,
        LEFT(IIF(@bImportIDFQDN = 0, h.GuestCN, h.GuestFQDN) + '_'  + 
             COALESCE(cl.ClusterName, 
                      IIF(@bImportIDFQDN = 0, h.HyperVHostCN, h.HyperVHostFQDN)), 125)
                                                                   AS import_id,
        @sDataSourceID                                             AS import_data_source_id,
        'is_running_on_hyperv'                                     AS import_device_rel_type_id,
    	LEFT(IIF(@bImportIDFQDN = 0, h.GuestCN, h.GuestFQDN), 125) AS import_from_device_id,
        LEFT(COALESCE(cl.ClusterName, IIF(@bImportIDFQDN = 0, h.HyperVHostCN, h.HyperVHostFQDN)), 125) 
                                                                   AS import_to_device_id,
        h.HyperVHostID                                             AS tmp_hostid,
        h.PowerState                                               AS tmp_powerstate
      INTO #tmpHyperVLink
      FROM #sp_GetVirtualHyperVLinkGuest h
        LEFT JOIN #f_GetVirtualHyperVLinkClusterName_rel cl
          ON h.HyperVHostID = cl.HyperVHostID
      WHERE LEN(h.GuestFQDN) > 0
    PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Show List Filtered on date'  

    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + '   - Add to relation list filtered on Inventory Date'
    INSERT INTO #tmpDeviceRelation (import_id, import_data_source_id, import_device_rel_type_id,
                                    import_from_device_id, import_to_device_id, tmp_hostid, tmp_powerstate)
    SELECT
        import_id,
        import_data_source_id,
        import_device_rel_type_id,
        import_from_device_id,
        import_to_device_id,
        tmp_hostid,
        tmp_powerstate
      FROM #tmpHyperVLink h
        INNER JOIN #f_GetLastInventoryDate_rel ir
          ON h.HyperVHostID = ir.ComputerID

    DROP TABLE #tmpHyperVLink
    DROP TABLE #sp_GetVirtualHyperVLinkGuest
    DROP TABLE #f_GetVirtualHyperVLinkClusterName_rel
  END

  IF @bHWPartRelation = 1
  BEGIN
  -- Sun/Oracle Solaris Zone
    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert Device - Sun Solaris Zone Host relations (Hostname)'
    INSERT INTO #tmpDeviceRelation (import_id, import_data_source_id, import_device_rel_type_id,
                                    import_from_device_id, import_to_device_id)
    SELECT
        LEFT(IIF(@bImportIDFQDN = 0, vzG.ComputerCN, vzG.FQDN) + '_' +
             CASE
               WHEN @bImportIDFQDN = 0 AND vz.ZoneHostID > 0 THEN vzH.ComputerCN
               WHEN @bImportIDFQDN = 1 AND vz.ZoneHostID > 0 THEN vzH.FQDN
               ELSE vz.ProductHostID
             END, 125)                                               AS import_id,
        @sDataSourceID                                               AS import_data_source_id,
        'is_running_on_zone'                                         AS import_device_rel_type_id,
	    LEFT(IIF(@bImportIDFQDN = 0, vzG.ComputerCN, vzG.FQDN), 125) AS import_from_device_id,
        LEFT(CASE
               WHEN @bImportIDFQDN = 0 AND vz.ZoneHostID > 0 THEN vzH.ComputerCN
               WHEN @bImportIDFQDN = 1 AND vz.ZoneHostID > 0 THEN vzH.FQDN
               ELSE vz.ProductHostID
             END, 125)                                               AS import_to_device_id
      FROM f_GetVirtualZoneLinkGuest() vz
        INNER JOIN #f_GetLastInventoryDate_rel ir
          ON vz.GuestID = ir.ComputerID
        LEFT JOIN #f_GetDeviceFQDN_rel vzH
          ON vz.ZoneHostID = vzH.ComputerID
        LEFT JOIN #f_GetDeviceFQDN_rel vzG
          ON vz.GuestID = vzG.ComputerID

  -- AIX LPAR
    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert Device - AIX LPAR Host relations (Hostname)'
    INSERT INTO #tmpDeviceRelation (import_id, import_data_source_id, import_device_rel_type_id,
                                    import_from_device_id, import_to_device_id)
    SELECT
        LEFT(IIF(@bImportIDFQDN = 0, vlG.ComputerCN, vlG.FQDN) + '_' +
             CASE
               WHEN @bImportIDFQDN = 0 AND vl.LPARHostID > 0 THEN vlH.ComputerCN
               WHEN @bImportIDFQDN = 1 AND vl.LPARHostID > 0 THEN vlH.FQDN
               ELSE COALESCE(vl.IdentifyingNumber, vl.ProductHostID)
             END, 125)                                               AS import_id,
        @sDataSourceID                                               AS import_data_source_id,
        'is_running_on_lpar'                                         AS import_device_rel_type_id,
	    LEFT(IIF(@bImportIDFQDN = 0, vlG.ComputerCN, vlG.FQDN), 125) AS import_from_device_id,
        LEFT(CASE
               WHEN @bImportIDFQDN = 0 AND vl.LPARHostID > 0 THEN vlH.ComputerCN
               WHEN @bImportIDFQDN = 1 AND vl.LPARHostID > 0 THEN vlH.FQDN
               ELSE COALESCE(vl.IdentifyingNumber, vl.ProductHostID)
             END, 125)                                               AS import_to_device_id
      FROM f_GetVirtualLPARLinkGuest() vl
        INNER JOIN #f_GetLastInventoryDate_rel ir
          ON vl.GuestID = ir.ComputerID
        LEFT JOIN #f_GetDeviceFQDN_rel vlH
          ON vl.LPARHostID = vlH.ComputerID
        LEFT JOIN #f_GetDeviceFQDN_rel vlG
          ON vl.GuestID = vlG.ComputerID

    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert Device - AIX LPAR Pool relations'

    CREATE TABLE #sp_GetVirtualLPARPools (
      LPARHostID                      BIGINT                                  NULL,
      LPARHostUnitaryComputerSystemID BIGINT                                  NULL,
      ProductHostID                   NVARCHAR(125)  COLLATE database_default NULL,
      IdentifyingNumber               NVARCHAR(125)  COLLATE database_default NULL,
      GuestID                         BIGINT                                  NULL,
      GuestUnitaryComputerSystemID    BIGINT                                  NOT NULL,
      SharedPoolID                    NVARCHAR(125)  COLLATE database_default NULL,
      SharedPoolCapacityInCores       INT                                     NULL,
      Entitlement                     DECIMAL(15,2)                           NULL)

    EXEC sp_GetVirtualLPARPools

    INSERT INTO #tmpDeviceRelation (import_id, import_data_source_id, import_device_rel_type_id,
                                    import_from_device_id, import_to_device_id)
    SELECT
        LEFT(IIF(@bImportIDFQDN = 0, vlG.ComputerCN, vlG.FQDN) + '_' +
             CASE
               WHEN @bImportIDFQDN = 0 AND vl.LPARHostID > 0 THEN vlH.ComputerCN
               WHEN @bImportIDFQDN = 1 AND vl.LPARHostID > 0 THEN vlH.FQDN
               ELSE COALESCE(vl.IdentifyingNumber, vl.ProductHostID)
             END + '_' + vl.SharedPoolID, 125)                       AS import_id,
        @sDataSourceID                                               AS import_data_source_id,
        'is_running_on_lpar_pool'                                    AS import_device_rel_type_id,
	    LEFT(IIF(@bImportIDFQDN = 0, vlG.ComputerCN, vlG.FQDN), 125) AS import_from_device_id,
        LEFT(CASE
               WHEN @bImportIDFQDN = 0 AND vl.LPARHostID > 0 THEN vlH.ComputerCN
               WHEN @bImportIDFQDN = 1 AND vl.LPARHostID > 0 THEN vlH.FQDN
               ELSE COALESCE(vl.IdentifyingNumber, vl.ProductHostID)
             END + '_' + vl.SharedPoolID, 125)                       AS import_to_device_id
      FROM #sp_GetVirtualLPARPools vl
        INNER JOIN #f_GetLastInventoryDate_rel ir
          ON vl.GuestID = ir.ComputerID
        LEFT JOIN #f_GetDeviceFQDN_rel vlH
          ON vl.LPARHostID = vlH.ComputerID
        LEFT JOIN #f_GetDeviceFQDN_rel vlG
          ON vl.GuestID = vlG.ComputerID
  END

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Delete older Relations if Relations of 2 ESX Server exists'
  UPDATE d
      SET tmp_inventory_date_host = ir.InventoryDate
    FROM #tmpDeviceRelation d
      INNER JOIN #f_GetInventoryDate_rel ir
        ON d.tmp_hostid = ir.ComputerID
    WHERE d.tmp_hostid > 0

  IF OBJECT_ID('tempdb..#tmpDeviceRelationFix') IS NOT NULL
    DROP TABLE #tmpDeviceRelationFix

  SELECT
      CASE
	    WHEN o.tmp_hostid < n.tmp_hostid THEN o.tmp_hostid 
	    ELSE n.tmp_hostid
      END AS toRemoveHostID,
      tmp.import_from_device_id
  INTO #tmpDeviceRelationFix
  FROM (SELECT 
            t.import_from_device_id, 
            MIN(t.tmp_inventory_date_host) inventory_date_min,
            MAX(t.tmp_inventory_date_host) inventory_date_max
          FROM #tmpDeviceRelation t
          GROUP BY import_from_device_id
          HAVING COUNT(t.import_from_device_id) > 1) tmp
    LEFT JOIN #tmpDeviceRelation o
      ON o.import_from_device_id = tmp.import_from_device_id
        AND o.tmp_inventory_date_host = tmp.inventory_date_min
    LEFT JOIN #tmpDeviceRelation n
      ON n.import_from_device_id = tmp.import_from_device_id
        AND n.tmp_inventory_date_host = tmp.inventory_date_max
  UPDATE r
      SET tmp_delete = 1
    FROM #tmpDeviceRelation r
      INNER JOIN #tmpDeviceRelationFix f
        ON r.import_from_device_id = f.import_from_device_id
          AND r.tmp_hostid = f.toRemoveHostID

  DELETE FROM #tmpDeviceRelation
    WHERE tmp_delete = 1

  DROP TABLE #tmpDeviceRelationFix

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Show List of Device Relation'
  SELECT
      import_id,
      import_data_source_id,
      import_device_rel_type_id,
      import_from_device_id,
      import_to_device_id,
      tmp_ipaddress,
      tmp_uuid,
      tmp_elementname,
      tmp_powerstate
  FROM #tmpDeviceRelation
  WHERE import_id IS NOT NULL AND LEN(import_from_device_id) > 0 AND LEN(import_to_device_id) > 0

  DROP TABLE #tmpDeviceRelation
  DROP TABLE #f_GetDeviceFQDN_rel
  DROP TABLE #f_GetInventoryDate_rel
  DROP TABLE #f_GetLastInventoryDate_rel

END



