USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'csp_aspera_connector_device_relation_types' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[csp_aspera_connector_device_relation_types]

GO
-- =============================================
-- Author:      PA, Raynet GmbH
-- Create date: 28.01.2019
-- Description:	Get Device Information
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE PROCEDURE [dbo].[csp_aspera_connector_device_relation_types] (
  @sDataSourceID   NVARCHAR(50) = 'RayVentory')             -- DataSourceID best is RayVentory_{CustomerName}
AS
BEGIN
  IF OBJECT_ID('tempdb..#tmpDdeviceTypes') IS NOT NULL
    DROP TABLE #tmpDeviceReleationTypes
  CREATE TABLE #tmpDeviceReleationTypes(
    import_id             NVARCHAR(50)   COLLATE database_default NULL,
    import_data_source_id NVARCHAR(50)   COLLATE database_default NULL,
    [name]                NVARCHAR(255)  COLLATE database_default NOT NULL,
    virtualization_type   NVARCHAR(4000) COLLATE database_default NULL)

  -- Types for LPAR and Zone
  INSERT INTO #tmpDeviceReleationTypes VALUES ('is_running_on_zone',        @sDataSourceID, 'Is running on Zone',        'hw_partition')
  INSERT INTO #tmpDeviceReleationTypes VALUES ('is_running_on_lpar',        @sDataSourceID, 'Is running on LPAR',        'hw_partition')
  INSERT INTO #tmpDeviceReleationTypes VALUES ('is_running_on_lpar_pool',   @sDataSourceID, 'Is running on LPAR Pool',   'part_of_pool')

  -- Types for VMware
  INSERT INTO #tmpDeviceReleationTypes VALUES ('is_part_of_esx_cluster',    @sDataSourceID, 'Is part of ESX Cluster',    'part_of_cluster')
  INSERT INTO #tmpDeviceReleationTypes VALUES ('is_running_on_esx',         @sDataSourceID, 'Is running on ESX',         'sw_partition')

  -- Types for HyperV
  INSERT INTO #tmpDeviceReleationTypes VALUES ('is_part_of_hyperv_cluster', @sDataSourceID, 'Is part of HyperV Cluster', 'part_of_cluster')
  INSERT INTO #tmpDeviceReleationTypes VALUES ('is_running_on_hyperv',      @sDataSourceID, 'Is running on HyperV',      'sw_partition')

  SELECT DISTINCT
      import_id,
      import_data_source_id,
      [name],
      virtualization_type
    FROM #tmpDeviceReleationTypes
  DROP TABLE #tmpDeviceReleationTypes
END

