USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'csp_aspera_connector_device_types' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[csp_aspera_connector_device_types]

GO
-- =============================================
-- Author:      PA, Raynet GmbH
-- Create date: 28.01.2019
-- Description:	Get Device Information
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE PROCEDURE [dbo].[csp_aspera_connector_device_types] (
  @sDataSourceID   NVARCHAR(50) = 'RayVentory',             -- DataSourceID best is RayVentory_{CustomerName}
  @bChassisTypeID  BIT          = 0)                        -- 0 Chassis import_device_type_id as Chassis Description, 1 Chassis Type ID as chassis_{ID from WMI}
AS
BEGIN
  IF OBJECT_ID('tempdb..#tmpDdeviceTypes') IS NOT NULL
    DROP TABLE #tmpDdeviceTypes
  CREATE TABLE #tmpDdeviceTypes(
    import_id             NVARCHAR(50)   COLLATE database_default NULL,
    import_data_source_id NVARCHAR(50)   COLLATE database_default NULL,
    [name]                NVARCHAR(255)  COLLATE database_default NOT NULL,
    remarks               NVARCHAR(4000) COLLATE database_default NULL,
    virtualization_type   NVARCHAR(4000) COLLATE database_default NULL)

  INSERT INTO #tmpDdeviceTypes
  SELECT DISTINCT
      CASE
        WHEN @bChassisTypeID = 1 THEN 'chassis_' + CAST(c.TypeID AS NVARCHAR(3))
        ELSE c.Type
      END             AS import_id,
      @sDataSourceID  AS import_data_source_id,
      c.Type          AS [name],
      NULL            AS remarks,
      NULL            AS virtualization_type
    FROM dbo.f_GetDeviceChassis(1) c

  -- Types for LPAR and Zone
  INSERT INTO #tmpDdeviceTypes VALUES ('dummy_host',             @sDataSourceID, 'Dummy Host',         NULL, NULL)
  INSERT INTO #tmpDdeviceTypes VALUES ('zone',                   @sDataSourceID, 'Zone',               NULL, 'partition')
  INSERT INTO #tmpDdeviceTypes VALUES ('lpar',                   @sDataSourceID, 'LPAR',               NULL, 'partition')
  INSERT INTO #tmpDdeviceTypes VALUES ('lpar_pool',              @sDataSourceID, 'LPAR Pool',          NULL, 'pool')

  -- Types for VMware
  INSERT INTO #tmpDdeviceTypes VALUES ('vmware_esx_cluster',     @sDataSourceID, 'VMware ESX Cluster', NULL, 'cluster')
  INSERT INTO #tmpDdeviceTypes VALUES ('vmware_esx_host',        @sDataSourceID, 'VMware ESX Host',    NULL, NULL)

  -- Types for HyperV
  INSERT INTO #tmpDdeviceTypes VALUES ('hyperv_cluster',         @sDataSourceID, 'HyperV Cluster',     NULL, 'cluster')
  INSERT INTO #tmpDdeviceTypes VALUES ('hyperv_host',            @sDataSourceID, 'HyperV Host',        NULL, NULL)

  -- Type for Virtual Machine VMWare and HyperV
  INSERT INTO #tmpDdeviceTypes VALUES ('virtual_machine',        @sDataSourceID, 'Virtual Machine',    NULL, 'partition')

  INSERT INTO #tmpDdeviceTypes VALUES (IIF(@bChassisTypeID = 1, 'chassis_2', 'Unknown'),
                                                                 @sDataSourceID, 'Unknown',            NULL, NULL)

  SELECT DISTINCT
      import_id,
      import_data_source_id,
      [name],
      remarks,
      virtualization_type
    FROM #tmpDdeviceTypes
END

