USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'csp_aspera_connector_software_arp' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[csp_aspera_connector_software_arp]

GO
-- =============================================
-- Author:      PA, Raynet GmbH
-- Create date: 31.01.2019
-- Description:	Get Software from Devices
-- ===============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE PROCEDURE [dbo].[csp_aspera_connector_software_arp] (
  @sDataSourceID     NVARCHAR(50) = 'RayVentory',             -- DataSourceID best is RayVentory_{CustomerName}
  @bImportIDFQDN     BIT          = 1,                        -- 0 ImportID is Hostname, 1 is FQDN (default)
  @bNoUpdate         BIT          = 1,                        -- 0 All package information, 1 No Update Packages
  @iLastSeenDays     INT          = 90)                       -- What should be the oldest Inventory Date to use.
AS
BEGIN  
  DECLARE @bDebug              BIT           = 0

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Get SoftwareInformation filtered on ARP'
  IF OBJECT_ID('tempdb..#sp_GetSoftwareInstallInformation') IS NOT NULL
    DROP TABLE #sp_GetSoftwareInstallInformation

  CREATE TABLE #sp_GetSoftwareInstallInformation (
      ComputerID           BIGINT                                 NOT NULL,
      SoftwareOccurrenceID BIGINT                                 NOT NULL,
      SoftwareID           BIGINT                                 NULL,
      SoftwareDetailsID    BIGINT                                 NULL,
      Evidence             VARCHAR(32)   COLLATE database_default NULL,
      Publisher            NVARCHAR(256) COLLATE database_default NULL,
      ProductName          NVARCHAR(256) COLLATE database_default NULL,
      ProductVersion       NVARCHAR(256) COLLATE database_default NULL,
      ProductCode          NVARCHAR(256) COLLATE database_default NULL,
      Summary              NVARCHAR(256) COLLATE database_default NULL,
      InstallationDate     DATETIME                               NULL)

  CREATE NONCLUSTERED INDEX IDX_C_#sp_GetSoftwareInstallInformation_ComputerID           ON #sp_GetSoftwareInstallInformation(ComputerID)
--  CREATE NONCLUSTERED INDEX IDX_C_#sp_GetSoftwareInstallInformation_SoftwareOccurrenceID ON #sp_GetSoftwareInstallInformation(SoftwareOccurrenceID)

  EXEC sp_GetSoftwareInstallInformation 1, NULL, @bNoUpdate, @iLastSeenDays

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Get Import ComputerID'
  IF OBJECT_ID('tempdb..#tmpImportID') IS NOT NULL
    DROP TABLE #tmpImportID

  CREATE TABLE #tmpImportID(
    ComputerID             BIGINT                                 NOT NULL,
    ImportID               NVARCHAR(256) COLLATE database_default NOT NULL
  )

  CREATE NONCLUSTERED INDEX IDX_C_#tmpImportID ON #tmpImportID(ComputerID)

  INSERT INTO #tmpImportID
  SELECT 
      c.ComputerID,
      LEFT(IIF(@bImportIDFQDN = 0, c.ComputerCN, c.FQDN), 125)  ImportID
    FROM dbo.f_GetDeviceFQDN(1) c

-- ARP Information
  IF OBJECT_ID('tempdb..#csp_aspera_connector_software_arp') IS NOT NULL
  BEGIN
    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert Information to Return Table'
    INSERT INTO #csp_aspera_connector_software_arp
    SELECT
        c.ImportID + '_' + CAST(so.SoftwareOccurrenceID AS NVARCHAR(16)) AS import_id,
        @sDataSourceID                                                   AS import_data_source_id,  -- Import ID Vergeben ber ReportingConfiguration
        LEFT(ISNULL(so.Publisher, ''), 255)                              AS publisher,
        LEFT(ISNULL(so.ProductName, ''), 255)                            AS product,
        LEFT(ISNULL(so.ProductVersion, ''), 255)                         AS product_version,
--      so.ProductCode                                                   AS guid,
        c.ImportID                                                       AS import_device_id,
        ISNULL(CONVERT(NVARCHAR(10), so.InstallationDate, 104),'')       AS installation_date
      FROM #sp_GetSoftwareInstallInformation so
        INNER JOIN #tmpImportID c
          ON so.ComputerID = c.ComputerID
  END
  ELSE
  BEGIN
    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Show Information'
    SELECT
        c.ImportID + '_' + CAST(so.SoftwareOccurrenceID AS NVARCHAR(16)) AS import_id,
        @sDataSourceID                                                   AS import_data_source_id,  -- Import ID Vergeben ber ReportingConfiguration
        LEFT(ISNULL(so.Publisher, ''), 255)                              AS publisher,
        LEFT(ISNULL(so.ProductName, ''), 255)                            AS product,
        LEFT(ISNULL(so.ProductVersion, ''), 255)                         AS product_version,
--      so.ProductCode                                                   AS guid,
        c.ImportID                                                       AS import_device_id,
        ISNULL(CONVERT(NVARCHAR(10), so.InstallationDate, 104),'')       AS installation_date
      FROM #sp_GetSoftwareInstallInformation so
        INNER JOIN #tmpImportID c
          ON so.ComputerID = c.ComputerID
  END
  -- OPTION (MAXDOP 1)

  DROP TABLE #tmpImportID
  DROP TABLE #sp_GetSoftwareInstallInformation
  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Finish'

END
GO

