USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'csp_aspera_connector_software_file' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[csp_aspera_connector_software_file]

GO
-- =============================================
-- Author:      PA, Raynet GmbH
-- Create date: 31.01.2019
-- Last update: 03.05.2019
-- Description:	Get Fileinformation from Devices
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE PROCEDURE [dbo].[csp_aspera_connector_software_file] (
  @sDataSourceID     NVARCHAR(50) = 'RayVentory',             -- DataSourceID best is RayVentory_{CustomerName}
  @bImportIDFQDN     BIT          = 1,                        -- 0 ImportID is Hostname, 1 is FQDN (default)
  @noUsageData       BIT          = 0,                        -- 0 include Meetering Information, 1 with out Meetering Information
  @iLastSeenDays     INT          = 90)                       -- What should be the oldest Inventory Date to use.
AS
BEGIN  
  IF OBJECT_ID('tempdb..#tmpImportID') IS NOT NULL
    DROP TABLE #tmpImportID

  CREATE TABLE #tmpImportID(
    ComputerID             BIGINT                                  NOT NULL,
    ImportID               NVARCHAR(256)  COLLATE database_default NOT NULL
  )

  CREATE NONCLUSTERED INDEX IDX_C_#tmpImportID ON #tmpImportID(ComputerID)

  INSERT INTO #tmpImportID
  SELECT 
      c.ComputerID,
      LEFT(IIF(@bImportIDFQDN = 0, c.ComputerCN, c.FQDN), 125)  ImportID
    FROM dbo.f_GetDeviceFQDN(1) c
      INNER JOIN dbo.f_GetLastInventoryDate(1) ir
        ON c.ComputerID = ir.ComputerID
          AND (@iLastSeenDays IS NULL OR DATEDIFF(day,  ir.LastInventoryDate, CURRENT_TIMESTAMP) <= @iLastSeenDays)

  IF OBJECT_ID('tempdb..#sp_GetSoftwareFiles') IS NOT NULL
    DROP TABLE #sp_GetSoftwareFiles

  CREATE TABLE #sp_GetSoftwareFiles (
    ComputerID           BIGINT                                 NOT NULL,
    SoftwareFileID       BIGINT                                 NULL,
    SoftwareFileNameID   BIGINT                                 NULL,
    SoftwareIsoTagFileID BIGINT                                 NULL,
    FilePath             NVARCHAR(400) COLLATE database_default NULL,
    FileName             NVARCHAR(256) COLLATE database_default NULL,
    FileDescription      NVARCHAR(256) COLLATE database_default NULL,
    FileSize             NVARCHAR(256) COLLATE database_default NULL,
    FileMD5              NVARCHAR(256) COLLATE database_default NULL,
    FileDate             DATETIME                               NULL,
    Publisher            NVARCHAR(256) COLLATE database_default NULL,
    ProductName          NVARCHAR(256) COLLATE database_default NULL,
    ProductVersion       NVARCHAR(256) COLLATE database_default NULL,
    ProductGuid          NVARCHAR(256) COLLATE database_default NULL,
    SwidUniqueId         NVARCHAR(256) COLLATE database_default NULL,
    SwidLicensorID       NVARCHAR(256) COLLATE database_default NULL,
    LastUsage            DATETIME                               NULL,
    SessionPerMonth      BIGINT                                 NULL,
    IMPORT_ID            NVARCHAR(400) COLLATE database_default NULL)

  EXEC sp_GetSoftwareFiles @iType = 1, @noUsageData = @noUsageData

  UPDATE sf
      SET IMPORT_ID = LEFT(c.ImportID + '_' + CONVERT(VARCHAR(32), HashBytes('MD5', ISNULL(CAST(SoftwareFileID AS NVARCHAR(255)), [FileName] + '_' + [FilePath])), 2), 255)
    FROM #sp_GetSoftwareFiles sf
      INNER JOIN #tmpImportID c
        ON sf.ComputerID = c.ComputerID

-- File Information
  IF OBJECT_ID('tempdb..#csp_aspera_connector_software_file') IS NOT NULL
    INSERT INTO #csp_aspera_connector_software_file
    SELECT
        sf.IMPORT_ID                                                       AS import_id,
        @sDataSourceID                                                     AS import_data_source_id,  -- Import ID Vergeben ber ReportingConfiguration
        LEFT(sf.FileName, 255)                                             AS [file_name],
        LEFT(sf.FileDescription, 255)                                      AS description,
        LEFT(sf.FileSize, 255)                                             AS file_size,
        LEFT(sf.FilePath, 255)                                             AS file_path,
        LEFT(ISNULL(sf.Publisher, ''), 255)                                AS publisher,
        LEFT(ISNULL(sf.ProductName, ''), 255)                              AS product,
        LEFT(ISNULL(sf.ProductVersion, ''), 255)                           AS product_version,
        c.ImportID                                                         AS import_device_id,
        ISNULL(CONVERT(NVARCHAR(10), sf.FileDate, 104),'')                 AS installation_date,
        CONVERT(NVARCHAR(10), sf.LastUsage, 104)                           AS last_usage,
        CAST(sf.SessionPerMonth AS NVARCHAR(100)) COLLATE database_default AS usage_frequency_month
      FROM #sp_GetSoftwareFiles sf
        INNER JOIN #tmpImportID c
          ON sf.ComputerID = c.ComputerID
  ELSE
    SELECT
        sf.IMPORT_ID                                                       AS import_id,
        @sDataSourceID                                                     AS import_data_source_id,  -- Import ID Vergeben ber ReportingConfiguration
        LEFT(sf.FileName, 255)                                             AS [file_name],
        LEFT(sf.FileDescription, 255)                                      AS description,
        LEFT(sf.FileSize, 255)                                             AS file_size,
        LEFT(sf.FilePath, 255)                                             AS file_path,
        LEFT(ISNULL(sf.Publisher, ''), 255)                                AS publisher,
        LEFT(ISNULL(sf.ProductName, ''), 255)                              AS product,
        LEFT(ISNULL(sf.ProductVersion, ''), 255)                           AS product_version,
        c.ImportID                                                         AS import_device_id,
        ISNULL(CONVERT(NVARCHAR(10), sf.FileDate, 104),'')                 AS installation_date,
        CONVERT(NVARCHAR(10), sf.LastUsage, 104)                           AS last_usage,
        CAST(sf.SessionPerMonth AS NVARCHAR(100)) COLLATE database_default AS usage_frequency_month
      FROM #sp_GetSoftwareFiles sf
        INNER JOIN #tmpImportID c
          ON sf.ComputerID = c.ComputerID

  DROP TABLE #sp_GetSoftwareFiles
  DROP TABLE #tmpImportID
  -- OPTION (MAXDOP 1)

END
GO

