USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'csp_aspera_connector_software_generic_microsoft' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[csp_aspera_connector_software_generic_microsoft]

GO
-- =============================================
-- Author:      PA, Raynet GmbH
-- Create date: 31.01.2019
-- Last update: 
-- Description:	Get Software from Devices
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================


CREATE PROCEDURE [dbo].[csp_aspera_connector_software_generic_microsoft] (
  @sDataSourceID     NVARCHAR(50) = 'RayVentory',             -- DataSourceID best is RayVentory_{CustomerName}
  @bImportIDFQDN     BIT          = 1,                        -- 0 ImportID is Hostname, 1 is FQDN (default)
  @iLastSeenDays     INT          = 90)                       -- What should be the oldest Inventory Date to use.
AS
BEGIN  
  IF OBJECT_ID('tempdb..#tmpImportID') IS NOT NULL
    DROP TABLE #tmpImportID

  CREATE TABLE #tmpImportID(
    ComputerID             BIGINT                                 NOT NULL,
    ImportID               NVARCHAR(256) COLLATE database_default NOT NULL
  )

  CREATE CLUSTERED INDEX IDX_C_#tmpImportID ON #tmpImportID(ComputerID)

  INSERT INTO #tmpImportID
  SELECT 
      c.ComputerID,
      LEFT(IIF(@bImportIDFQDN = 0, c.ComputerCN, c.FQDN), 125)  ImportID
    FROM dbo.f_GetDeviceFQDN(1) c
      INNER JOIN dbo.f_GetLastInventoryDate(1) ir
        ON c.ComputerID = ir.ComputerID
          AND (@iLastSeenDays IS NULL OR DATEDIFF(day,  ir.LastInventoryDate, CURRENT_TIMESTAMP) <= @iLastSeenDays)

  IF OBJECT_ID('tempdb..#tmpSoftware_gen_microsoft') IS NOT NULL
    DROP TABLE #tmpSoftware_gen_microsoft

  CREATE TABLE #tmpSoftware_gen_microsoft (
    ComputerID              BIGINT,
    import_id               NVARCHAR(125) COLLATE database_default,
    import_data_source_id   NVARCHAR(125) COLLATE database_default,
    data_type               NVARCHAR(32)  COLLATE database_default,
    generic_key             NVARCHAR(255) COLLATE database_default NULL, 
    description             NVARCHAR(255) COLLATE database_default NULL,
    publisher               NVARCHAR(255) COLLATE database_default,
    product                 NVARCHAR(255) COLLATE database_default,
    product_version         NVARCHAR(255) COLLATE database_default NULL,
    product_guid            NVARCHAR(255) COLLATE database_default NULL,
    quantity                NVARCHAR(3)   COLLATE database_default NULL,
    import_device_id        NVARCHAR(125) COLLATE database_default, 
    import_account_id       NVARCHAR(125) COLLATE database_default NULL,
    installation_date       DATETIME      NULL,
    instance_name           NVARCHAR(255) COLLATE database_default NULL)


-- Microsoft SQL
  INSERT INTO #tmpSoftware_gen_microsoft (ComputerID, import_id, import_data_source_id, data_type, generic_key, 
                                          [description], publisher, product, product_version,
                                          quantity, import_device_id, installation_date)
  SELECT
      msql.ComputerID,
      LEFT(c.ImportID + '_' + msql.ProductName + '_' + msql.ProductVersion + 
             ISNULL('_' + msql.ProductEdition, ''), 255)                      AS import_id,
       @sDataSourceID                                                         AS import_data_source_id,  -- Import ID Vergeben ber ReportingConfiguration
      'GEN_MSSQL'                                                             AS data_type, 
      LEFT(msql.ProductName + ISNULL(' ' + msql.ProductEdition, ''), 255)     AS generic_key, 
      LEFT(msql.ProductVersion + ISNULL(' ' + msql.ProductEdition, ''), 255)  AS [description],
      msql.Publisher                                                          AS publisher,
      LEFT(msql.ProductName + ISNULL(' ' + msql.ProductEdition, ''), 255)     AS product,
      LEFT(msql.ProductVersion, 255)                                          AS product_version,
      '1'                                                                     AS quantity,
      c.ImportID                                                              AS import_device_id,
      CAST(NULL AS DATETIME)                                                  AS installation_date
    FROM dbo.f_GetSoftwareMicrosoftSQL() msql
      INNER JOIN #tmpImportID c
        ON msql.ComputerID = c.ComputerID

-- Microsoft Exchange
  INSERT INTO #tmpSoftware_gen_microsoft (ComputerID, import_id, import_data_source_id, data_type, generic_key, 
                                          [description], publisher, product, product_version,
                                          quantity, import_device_id, installation_date)
  SELECT
      ex.ComputerID,
      LEFT(c.ImportID + '_' + 
             ex.ProductName + '_' + ex.ProductRelease + '_' + ex.ProductEdition, 255)  AS import_id,
       @sDataSourceID                                                                  AS import_data_source_id,  -- Import ID Vergeben ber ReportingConfiguration
      'GEN_Exchange'                                                                   AS data_type, 
      LEFT(ex.ProductName + '_' + ex.ProductRelease + '_' + ex.ProductEdition, 255)    AS generic_key, 
      ''                                                                               AS [description],
      ex.Publisher                                                                     AS publisher,
      LEFT(ex.ProductName + '_' + ex.ProductRelease + '_' + ex.ProductEdition, 255)    AS product,
      LEFT(ex.ProductVersion, 255)                                                     AS product_version,
      '1'                                                                              AS quantity,
      c.ImportID                                                                       AS import_device_id,
      CAST(NULL AS DATETIME)                                                           AS installation_date
    FROM f_GetSoftwareMicrosoftExchange() ex
      INNER JOIN #tmpImportID c
        ON ex.ComputerID = c.ComputerID

  -- Microsoft SharePoint
  INSERT INTO #tmpSoftware_gen_microsoft (ComputerID, import_id, import_data_source_id, data_type, generic_key, 
                                          [description], publisher, product, product_version,
                                          quantity, import_device_id, installation_date)
  SELECT
      sp.ComputerID,
      LEFT(c.ImportID + '_' + sp.FullProductName, 255)            AS import_id,
       @sDataSourceID                                             AS import_data_source_id,  -- Import ID Vergeben ber ReportingConfiguration
      'GEN_SharePoint'                                            AS data_type, 
      LEFT(sp.FullProductName, 255)                               AS generic_key, 
      CAST(sp.SKUID AS NVARCHAR(38))                              AS [description],
      sp.Publisher                                                AS publisher,
      LEFT(sp.FullProductName, 255)                               AS product,
      LEFT(sp.ProductVersion, 255)                                AS product_version,
      '1'                                                         AS quantity,
      c.ImportID                                                  AS import_device_id,
      CAST(NULL AS DATETIME)                                      AS installation_date
    FROM f_GetSoftwareMicrosoftSharePoint() sp
      INNER JOIN #tmpImportID c
        ON sp.ComputerID = c.ComputerID

  IF OBJECT_ID('tempdb..#csp_aspera_connector_software_generic_microsoft') IS NOT NULL
    INSERT INTO #csp_aspera_connector_software_generic_microsoft
    SELECT DISTINCT
        gen.import_id,
        gen.import_data_source_id,
        gen.generic_key,
        gen.description,
        gen.publisher,
        gen.product,
        gen.product_version,
        gen.quantity,
        gen.import_device_id,
        COALESCE(gen.import_account_id, '')                             AS import_account_id,
        ISNULL(CONVERT(NVARCHAR(10), gen.installation_date, 104), '')   AS installation_date,
        gen.instance_name
      FROM #tmpSoftware_gen_microsoft gen
        WHERE gen.import_id IS NOT NULL AND gen.product IS NOT NULL
  ELSE
    SELECT DISTINCT
        gen.import_id,
        gen.import_data_source_id,
        gen.generic_key,
        gen.description,
        gen.publisher,
        gen.product,
        gen.product_version,
        gen.quantity,
        gen.import_device_id,
        COALESCE(gen.import_account_id, '')                             AS import_account_id,
        ISNULL(CONVERT(NVARCHAR(10), gen.installation_date, 104), '')   AS installation_date,
        gen.instance_name
      FROM #tmpSoftware_gen_microsoft gen
        WHERE gen.import_id IS NOT NULL AND gen.product IS NOT NULL

  DROP TABLE #tmpSoftware_gen_microsoft
  DROP TABLE #tmpImportID
END
GO

