USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'csp_aspera_connector_software_generic_oracle' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[csp_aspera_connector_software_generic_oracle]

GO
-- =============================================
-- Author:      PA, Raynet GmbH
-- Create date: 31.01.2019
-- Description:	Get Software from Devices
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE PROCEDURE [dbo].[csp_aspera_connector_software_generic_oracle] (
  @sDataSourceID     NVARCHAR(50) = 'RayVentory',             -- DataSourceID best is RayVentory_{CustomerName}
  @bImportIDFQDN     BIT          = 1,                        -- 0 ImportID is Hostname, 1 is FQDN (default)
  @iLastSeenDays     INT          = 90)                       -- What should be the oldest Inventory Date to use.
AS
BEGIN  
--  DECLARE @sDataSourceID     NVARCHAR(50) = 'RayVentory'             -- DataSourceID best is RayVentory_{CustomerName}
--  DECLARE @bImportIDFQDN     BIT          = 1                        -- 0 ImportID is Hostname, 1 is FQDN (default)
--  DECLARE @iLastSeenDays     INT          = 900                       -- What should be the oldest Inventory Date to use.

  DECLARE @bDebug BIT = 1

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + ' - Start csp_aspera_connector_software_generic_oracle'
  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Get Import IDs'
  IF OBJECT_ID('tempdb..#tmpImportID') IS NOT NULL
    DROP TABLE #tmpImportID

  CREATE TABLE #tmpImportID(
    ComputerID             BIGINT                                 NOT NULL,
    ImportID               NVARCHAR(256) COLLATE database_default NOT NULL
  )

  CREATE CLUSTERED INDEX IDX_C_#tmpImportID ON #tmpImportID(ComputerID)

  INSERT INTO #tmpImportID
  SELECT 
      c.ComputerID,
      LEFT(IIF(@bImportIDFQDN = 0, c.ComputerCN, c.FQDN), 125)  ImportID
    FROM dbo.f_GetDeviceFQDN(1) c
      INNER JOIN dbo.f_GetLastInventoryDate(1) ir
        ON c.ComputerID = ir.ComputerID
          AND (@iLastSeenDays IS NULL OR DATEDIFF(day,  ir.LastInventoryDate, CURRENT_TIMESTAMP) <= @iLastSeenDays)

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Get Oracle Instance'
  IF OBJECT_ID('tempdb..#tmpOracleInstance') IS NOT NULL
    DROP TABLE #tmpOracleInstance

  CREATE TABLE #tmpOracleInstance(
    ClusterDBName          NVARCHAR(256) COLLATE database_default NULL,
    ComputerID             INT                                    NULL,
    ComputerCN             NVARCHAR(256) COLLATE database_default NULL,
    ComputerFQDN           NVARCHAR(256) COLLATE database_default NULL,
    InstanceID             INT                                    NOT NULL,
    Instance               NVARCHAR(256) COLLATE database_default NOT NULL,
    SystemDateTime         DATETIME                               NULL,
    SystemDisplayName      NVARCHAR(256) COLLATE database_default NULL,
    SystemDisplayVersion   NVARCHAR(256) COLLATE database_default NULL,
    AppName                NVARCHAR(256) COLLATE database_default NULL,
    AppVersion             NVARCHAR(256) COLLATE database_default NULL,
    AppEdition             NVARCHAR(256) COLLATE database_default NULL,
    DatabaseRole           NVARCHAR(256) COLLATE database_default NULL,
    DatabaseName           NVARCHAR(256) COLLATE database_default NULL,
    DatabaseNameUnique     NVARCHAR(256) COLLATE database_default NULL,
    DatabaseCreationDate   DATETIME                               NULL,
    OperatingSystem        NVARCHAR(256) COLLATE database_default NULL
  )

  CREATE CLUSTERED INDEX IDX_C_#tmpOracleInstance_ComputerID ON #tmpOracleInstance(ComputerID, InstanceID)

  INSERT INTO #tmpOracleInstance
  SELECT * FROM dbo.f_OracleInstances(null) oi
		  
  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Add Oracle DB Information'
  IF OBJECT_ID('tempdb..#tmpSoftware_gen_oracle') IS NOT NULL
    DROP TABLE #tmpSoftware_gen_oracle

  CREATE TABLE #tmpSoftware_gen_oracle (
    ComputerID              BIGINT                                 NULL,
    import_id               NVARCHAR(125) COLLATE database_default,
    import_data_source_id   NVARCHAR(125) COLLATE database_default,
    data_type               NVARCHAR(32)  COLLATE database_default,
    generic_key             NVARCHAR(255) COLLATE database_default NULL, 
    description             NVARCHAR(255) COLLATE database_default NULL,
    publisher               NVARCHAR(255) COLLATE database_default,
    product                 NVARCHAR(255) COLLATE database_default,
    product_version         NVARCHAR(255) COLLATE database_default NULL,
    product_guid            NVARCHAR(255) COLLATE database_default NULL,
    quantity                NVARCHAR(3)   COLLATE database_default NULL,
    import_device_id        NVARCHAR(125) COLLATE database_default, 
    import_account_id       NVARCHAR(125) COLLATE database_default NULL,
    installation_date       DATETIME      NULL,
    instance_name           NVARCHAR(255) COLLATE database_default NULL)

  -- Oracle

-- Oracle DB's
  INSERT INTO #tmpSoftware_gen_oracle (ComputerID, import_id, import_data_source_id, data_type, generic_key, 
                                       description, publisher, product, product_version, quantity, import_device_id, 
                                       installation_date, instance_name)
  SELECT DISTINCT
      i.ComputerID,
      LEFT(c.importID + '_' + i.Instance, 255)                    AS import_id,
      @sDataSourceID                                              AS import_data_source_id,  -- Import ID Vergeben ber ReportingConfiguration
      'GEN_ORACLEDB'                                              AS data_type, 
      LEFT(i.SystemDisplayName, 255)                              AS generic_key,
      ''                                                          AS description,
      'Oracle'                                                    AS publisher,
      LEFT(i.AppName, 255)                                        AS product,
      i.SystemDisplayVersion                                      AS product_version,
      '1'                                                         AS quantity,
      c.ImportID                                                  AS import_device_id,
      i.DatabaseCreationDate                                      AS installation_date,
      i.Instance                                                  AS instance_name
    FROM #tmpOracleInstance i
/*      LEFT JOIN (SELECT InstanceID, COUNT(*) AS UserCount 
                     FROM f_OracleUsers (null) 
                     WHERE AccountStatus = 'OPEN'
                     GROUP BY InstanceID) uc
        ON i.InstanceID = uc.InstanceID */
      INNER JOIN #tmpImportID c
        ON i.ComputerID = c.ComputerID

-- Oracle Feature
  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Get Oracle Option List'
  IF OBJECT_ID('tempdb..#sp_rp_OracleOptionList') IS NOT NULL
    DROP TABLE #sp_rp_OracleOptionList

  CREATE TABLE #sp_rp_OracleOptionList (
    DBObjectID        BIGINT        NOT NULL,
    Product           NVARCHAR(256) COLLATE database_default NOT NULL,
    FeatureName       NVARCHAR(256) COLLATE database_default NULL,
    Installed         INT           NULL,
	RaynetUsed        INT           NULL,
	OracleUsed        INT           NULL,
	TechnicalLicense  BIT           NULL,
	RaynetComment     NVARCHAR(256) COLLATE database_default NULL,
	OracleComment     NVARCHAR(256) COLLATE database_default NULL,
	CustomLicense     INT           NULL,
	CustomComment     NVARCHAR(256) COLLATE database_default NULL,
	LicenseResult     INT           NULL,
    [Version]         NVARCHAR(256) COLLATE database_default NULL)

  CREATE CLUSTERED INDEX IDX_C_#sp_rp_OracleOptionList_DBObjectID ON #sp_rp_OracleOptionList(DBObjectID)

  EXEC sp_rp_OracleOptionList @DBObjectID = NULL, @ComputerID = NULL, @OnlyProduct = 1

  UPDATE #sp_rp_OracleOptionList
      SET Product   = LEFT(Product, LEN(Product) - CHARINDEX(' ', REVERSE(Product))),
          [Version] = RIGHT(Product, CHARINDEX(' ', REVERSE(Product))-1)
    WHERE Product LIKE '%[1-9]%.[0-9]%'

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Add Oracle Option'
  INSERT INTO #tmpSoftware_gen_oracle (ComputerID, import_id, import_data_source_id, data_type, generic_key,
                                       description, publisher, product, product_version, quantity, import_device_id, 
                                       installation_date, instance_name) 
  SELECT
      oi.ComputerID,
      LEFT(c.ImportID + '_' + oi.Instance + '_' + i.Product, 255) AS import_id,
      @sDataSourceID                                              AS import_data_source_id,  -- Import ID Vergeben ber ReportingConfiguration
      'GEN_ORACLEDB'                                              AS data_type, 
      LEFT('Oracle ' + i.Product + ' ' + oi.Instance, 255)        AS generic_key,
      'RayVentory'                                                AS description,
      'Oracle'                                                    AS publisher,
      LEFT('Oracle '+i.Product, 255)                              AS product,
      MAX(oi.SystemDisplayVersion)                                AS product_version,
      '1'                                                         AS quantity,
      c.ImportID                                                  AS import_device_id,
      oi.DatabaseCreationDate                                     AS installation_date,
      oi.Instance                                                 AS instance_name
    FROM #sp_rp_OracleOptionList i
      INNER JOIN #tmpOracleInstance oi
        ON oi.InstanceID  = i.DBObjectID
      INNER JOIN #tmpImportID c
        ON oi.ComputerID = c.ComputerID
    WHERE i.LicenseResult > 0
    GROUP BY c.ImportID, oi.ComputerID, oi.Instance, i.Product, oi.DatabaseCreationDate /* using the FQDN as import_id */
    HAVING COUNT(i.Product) > 0

    DROP TABLE #sp_rp_OracleOptionList
	/*
    DROP TABLE #sp_OracleRaynetOptionList
    DROP TABLE #tmpRayOracleOptionOld
    DROP TABLE #sp_OracleScriptOptionList */
  -- OPTION (MAXDOP 1)
  
  IF OBJECT_ID('tempdb..#csp_aspera_connector_software_generic_oracle') IS NOT NULL
  BEGIN
    IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Add Oracle Info to Return Table'
    INSERT INTO #csp_aspera_connector_software_generic_oracle
    SELECT
        gen.import_id,
        gen.import_data_source_id,
        gen.generic_key,
        gen.description,
        gen.publisher,
        gen.product,
        gen.product_version,
        gen.quantity,
        gen.import_device_id,
        COALESCE(gen.import_account_id, '')                             AS import_account_id,
        ISNULL(CONVERT(NVARCHAR(10), gen.installation_date, 104), '')   AS installation_date,
        gen.instance_name
      FROM #tmpSoftware_gen_oracle gen
        WHERE gen.import_id IS NOT NULL AND gen.product IS NOT NULL
  END
  ELSE
  BEGIN
    IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Show Oracle Info'
    SELECT
        gen.import_id,
        gen.import_data_source_id,
        gen.generic_key,
        gen.description,
        gen.publisher,
        gen.product,
        gen.product_version,
        gen.quantity,
        gen.import_device_id,
        COALESCE(gen.import_account_id, '')                             AS import_account_id,
        ISNULL(CONVERT(NVARCHAR(10), gen.installation_date, 104), '')   AS installation_date,
        gen.instance_name
      FROM #tmpSoftware_gen_oracle gen
        WHERE gen.import_id IS NOT NULL AND gen.product IS NOT NULL
  END

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '- Finish csp_aspera_connector_software_generic_oracle'

  DROP TABLE #tmpSoftware_gen_oracle
  DROP TABLE #tmpImportID
END
GO

