USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'csp_aspera_connector_software_generic_os' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[csp_aspera_connector_software_generic_os]

GO
-- =============================================
-- Author:      PA, Raynet GmbH
-- Create date: 31.01.2019
-- Description:	Get Software from Devices
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================
CREATE PROCEDURE [dbo].[csp_aspera_connector_software_generic_os] (
  @sDataSourceID     NVARCHAR(50) = 'RayVentory',             -- DataSourceID best is RayVentory_{CustomerName}
  @bImportIDFQDN     BIT          = 1,                        -- 0 ImportID is Hostname, 1 is FQDN (default)
  @iLastSeenDays     INT          = 90)                       -- What should be the oldest Inventory Date to use.
AS
BEGIN  
  IF OBJECT_ID('tempdb..#tmpImportID') IS NOT NULL
    DROP TABLE #tmpImportID

  CREATE TABLE #tmpImportID(
    ComputerID             BIGINT                                 NOT NULL,
    ImportID               NVARCHAR(256) COLLATE database_default NOT NULL
  )

  CREATE CLUSTERED INDEX IDX_C_#tmpImportID ON #tmpImportID(ComputerID)

  INSERT INTO #tmpImportID
  SELECT 
      c.ComputerID,
      LEFT(IIF(@bImportIDFQDN = 0, c.ComputerCN, c.FQDN), 125)  ImportID
    FROM dbo.f_GetDeviceFQDN(1) c
      INNER JOIN dbo.f_GetLastInventoryDate(1) ir
        ON c.ComputerID = ir.ComputerID
          AND (@iLastSeenDays IS NULL OR DATEDIFF(day,  ir.LastInventoryDate, CURRENT_TIMESTAMP) <= @iLastSeenDays)


-- Operating System

  IF OBJECT_ID('tempdb..#csp_aspera_connector_software_generic_os') IS NOT NULL
    INSERT INTO #csp_aspera_connector_software_generic_os
    SELECT DISTINCT
        LEFT(c.ImportID + '_' + os.ProductName ,255)                AS import_id,
        @sDataSourceID                                              AS import_data_source_id,  -- Import ID Vergeben ber ReportingConfiguration
        LEFT(os.ProductName, 255)                                   AS generic_key, 
        CAST('' AS NVARCHAR(255))                                   AS description,
        LEFT(os.Manufacturer, 255)                                  AS publisher,
        LEFT(os.ProductName, 255)                                   AS product,
        LEFT(os.Version, 255)                                       AS product_version,
        '1'                                                         AS quantity,
        c.ImportID                                                  AS import_device_id,
        CAST('' AS NVARCHAR(125))                                   AS import_account_id,
        ISNULL(CONVERT(NVARCHAR(10), os.InstallDate, 104),'')       AS installation_date,
        CAST(NULL AS NVARCHAR(255))                                 AS instance_name
      FROM dbo.f_GetDeviceOSInfo() os
        INNER JOIN #tmpImportID c
          ON os.ComputerID = c.ComputerID
        WHERE c.ImportID IS NOT NULL AND LEN(os.ProductName) > 0
  ELSE
    SELECT DISTINCT
        LEFT(c.ImportID + '_' + os.ProductName ,255)                AS import_id,
        @sDataSourceID                                              AS import_data_source_id,  -- Import ID Vergeben ber ReportingConfiguration
        LEFT(os.ProductName, 255)                                   AS generic_key, 
        CAST('' AS NVARCHAR(255))                                   AS description,
        LEFT(os.Manufacturer, 255)                                  AS publisher,
        LEFT(os.ProductName, 255)                                   AS product,
        LEFT(os.Version, 255)                                       AS product_version,
        '1'                                                         AS quantity,
        c.ImportID                                                  AS import_device_id,
        CAST('' AS NVARCHAR(125))                                   AS import_account_id,
        ISNULL(CONVERT(NVARCHAR(10), os.InstallDate, 104),'')       AS installation_date,
        CAST(NULL AS NVARCHAR(255))                                 AS instance_name
      FROM dbo.f_GetDeviceOSInfo() os
        INNER JOIN #tmpImportID c
          ON os.ComputerID = c.ComputerID
        WHERE c.ImportID IS NOT NULL AND LEN(os.ProductName) > 0

  DROP TABLE #tmpImportID
END
GO

