USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'csp_aspera_connector_software_generic_othersw' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[csp_aspera_connector_software_generic_othersw]

GO
-- =============================================
-- Author:      PA, Raynet GmbH
-- Create date: 31.01.2019
-- Description:	Get Software from Devices
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================
CREATE PROCEDURE [dbo].[csp_aspera_connector_software_generic_othersw] (
  @sDataSourceID     NVARCHAR(50) = 'RayVentory',             -- DataSourceID best is RayVentory_{CustomerName}
  @bImportIDFQDN     BIT          = 1,                        -- 0 ImportID is Hostname, 1 is FQDN (default)
  @bNoUpdate         BIT          = 1,                        -- 0 All package information, 1 No Update Packages
  @iLastSeenDays     INT          = 90)                       -- What should be the oldest Inventory Date to use.
AS
BEGIN  
  DECLARE @bDebug BIT = 0

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + ' - Start'


  IF OBJECT_ID('tempdb..#sp_GetSoftwareInstallInformation') IS NOT NULL
    DROP TABLE #sp_GetSoftwareInstallInformation

  CREATE TABLE #sp_GetSoftwareInstallInformation (
      ComputerID           BIGINT                                 NOT NULL,
      SoftwareOccurrenceID BIGINT                                 NOT NULL,
      SoftwareID           BIGINT                                 NULL,
      SoftwareDetailsID    BIGINT                                 NULL,
      Evidence             VARCHAR(32)   COLLATE database_default NULL,
      Publisher            NVARCHAR(256) COLLATE database_default NULL,
      ProductName          NVARCHAR(256) COLLATE database_default NULL,
      ProductVersion       NVARCHAR(256) COLLATE database_default NULL,
      ProductCode          NVARCHAR(256) COLLATE database_default NULL,
      Summary              NVARCHAR(256) COLLATE database_default NULL,
      InstallationDate     DATETIME                               NULL)

  CREATE NONCLUSTERED INDEX IDX_C_#sp_GetSoftwareInstallInformation_ComputerID           ON #sp_GetSoftwareInstallInformation(ComputerID)

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + ' - GetSoftwareInstallInformation'
  EXEC sp_GetSoftwareInstallInformation 3, NULL, @bNoUpdate, @iLastSeenDays

  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Get Import ComputerID'
  IF OBJECT_ID('tempdb..#tmpImportID') IS NOT NULL
    DROP TABLE #tmpImportID

  CREATE TABLE #tmpImportID(
    ComputerID             BIGINT                                  NOT NULL,
    ImportID               NVARCHAR(256)  COLLATE database_default NOT NULL
  )

  CREATE NONCLUSTERED INDEX IDX_C_#tmpImportID ON #tmpImportID(ComputerID)

  INSERT INTO #tmpImportID
  SELECT 
      c.ComputerID,
      LEFT(IIF(@bImportIDFQDN = 0, c.ComputerCN, c.FQDN), 125)  ImportID
    FROM dbo.f_GetDeviceFQDN(1) c
  
-- Other Software like Linux and other installer then MSI and ARP
  IF OBJECT_ID('tempdb..#csp_aspera_connector_software_generic_othersw') IS NOT NULL
  BEGIN
    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Insert Information to Return Table'
    INSERT INTO #csp_aspera_connector_software_generic_othersw
    SELECT
        c.ImportID + '_' + CAST(so.SoftwareOccurrenceID AS NVARCHAR(16)) AS import_id,
        @sDataSourceID                                                   AS import_data_source_id,  -- Import ID Vergeben über ReportingConfiguration
        LEFT(ISNULL(so.ProductVersion, ''), 255)                         AS generic_key,
        LEFT(ISNULL(so.Summary, ''), 255)                                AS [description],
        LEFT(ISNULL(so.Publisher, ''), 255)                              AS publisher,
        LEFT(so.ProductName, 255)                                        AS product,
        LEFT(ISNULL(so.ProductVersion, ''), 255)                         AS product_version,
--        so.ProductCode                                                   AS product_guid,
        '1'                                                              AS quantity,
        c.ImportID                                                       AS import_device_id,
        CAST('' AS NVARCHAR(125))                                        AS import_account_id,
        ISNULL(CONVERT(NVARCHAR(10), so.InstallationDate, 104),'')       AS installation_date,
        CAST(NULL AS NVARCHAR(255))                                      AS instance_name
      FROM #sp_GetSoftwareInstallInformation so
        INNER JOIN #tmpImportID c
          ON so.ComputerID = c.ComputerID
      WHERE LEN(so.ProductName) > 0
  END
  ELSE
  BEGIN
    IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Show Information'
    SELECT
        c.ImportID + '_' + CAST(so.SoftwareOccurrenceID AS NVARCHAR(16)) AS import_id,
        @sDataSourceID                                                   AS import_data_source_id,  -- Import ID Vergeben über ReportingConfiguration
        LEFT(ISNULL(so.ProductVersion, ''), 255)                         AS generic_key,
        LEFT(ISNULL(so.Summary, ''), 255)                                AS [description],
        LEFT(ISNULL(so.Publisher, ''), 255)                              AS publisher,
        LEFT(so.ProductName, 255)                                        AS product,
        LEFT(ISNULL(so.ProductVersion, ''), 255)                         AS product_version,
--        so.ProductCode                                                   AS product_guid,
        '1'                                                              AS quantity,
        c.ImportID                                                       AS import_device_id,
        CAST('' AS NVARCHAR(125))                                        AS import_account_id,
        ISNULL(CONVERT(NVARCHAR(10), so.InstallationDate, 104),'')       AS installation_date,
        CAST(NULL AS NVARCHAR(255))                                      AS instance_name
      FROM #sp_GetSoftwareInstallInformation so
        INNER JOIN #tmpImportID c
          ON so.ComputerID = c.ComputerID
      WHERE LEN(so.ProductName) > 0
  END
  -- OPTION (MAXDOP 1)
  IF @bDebug =1 PRINT CAST(CURRENT_TIMESTAMP AS VARCHAR(30)) + ' - Finish'
  
   
/*  IF @Debug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + ' - Create Datatable for Aspera'
  INSERT INTO #tmpSoftware_gen_othersw (ComputerID, import_id, import_data_source_id, data_type, generic_key, 
					                    [description], publisher, product, product_version, product_guid,
                                        quantity, import_device_id, installation_date)
  SELECT
      so.ComputerID,
      LEFT(IIF(@bImportIDFQDN = 0, c.ComputerCN, c.FQDN) + '_' + 
             CAST(so.SoftwareOccurrenceID AS NVARCHAR(16)) ,255)  AS import_id,
       @sDataSourceID                                             AS import_data_source_id,  -- Import ID Vergeben ?ber ReportingConfiguration
      'GEN_SW_OTHER'                                              AS data_type, 
	  LEFT(ISNULL(so.ProductName, ''),255)                        AS generic_key,
      ''                                                          AS [description],
      LEFT(ISNULL(so.Publisher, ''), 255)                         AS publisher,
      LEFT(ISNULL(so.ProductName, ''), 255)                       AS product,
      LEFT(ISNULL(so.ProductVersion, ''), 255)                    AS product_version,
      so.ProductCode                                              AS product_guid,
      '1'                                                         AS quantity,
      LEFT(IIF(@bImportIDFQDN = 0, c.ComputerCN, c.FQDN), 125)    AS import_device_id,
      so.InstallationDate                                         AS installation_date
    FROM #sp_GetSoftwareInstallInformation so
      INNER JOIN dbo.f_GetDeviceFQDN(1) c
        ON so.ComputerID = c.ComputerID

  DROP TABLE #sp_GetSoftwareInstallInformation

  IF @Debug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + ' - Show Table for Aspera'
  SELECT
      gen.import_id,
      gen.import_data_source_id,
      gen.generic_key,
      gen.description,
      gen.publisher,
      gen.product,
      gen.product_version,
      gen.quantity,
      gen.import_device_id,
      COALESCE(gen.import_account_id, '')                             AS import_account_id,
      ISNULL(CONVERT(NVARCHAR(10), gen.installation_date, 104), '')   AS installation_date,
      gen.instance_name
    FROM #tmpSoftware_gen_othersw gen
    WHERE gen.import_id IS NOT NULL AND gen.product != ''

  DROP TABLE #tmpSoftware_gen_othersw */
  DROP TABLE #sp_GetSoftwareInstallInformation
  DROP TABLE #tmpImportID
END
GO
