USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'csp_aspera_connector_software_tag' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[csp_aspera_connector_software_tag]

GO
-- =============================================
-- Author:      PA, Raynet GmbH
-- Create date: 31.01.2019
-- Description:	Get Fileinformation from Devices
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE PROCEDURE [dbo].[csp_aspera_connector_software_tag] (
  @sDataSourceID     NVARCHAR(50) = 'RayVentory',             -- DataSourceID best is RayVentory_{CustomerName}
  @bImportIDFQDN     BIT          = 1,                        -- 0 ImportID is Hostname, 1 is FQDN (default)
  @iLastSeenDays     INT          = 90)                       -- What should be the oldest Inventory Date to use.
AS
BEGIN  
  IF OBJECT_ID('tempdb..#tmpImportID') IS NOT NULL
    DROP TABLE #tmpImportID

  CREATE TABLE #tmpImportID(
    ComputerID             BIGINT                                  NOT NULL,
    ImportID               NVARCHAR(256)  COLLATE database_default NOT NULL
  )

  CREATE CLUSTERED INDEX IDX_C_#tmpImportID ON #tmpImportID(ComputerID)

  INSERT INTO #tmpImportID
  SELECT 
      c.ComputerID,
      LEFT(IIF(@bImportIDFQDN = 0, c.ComputerCN, c.FQDN), 125)  ImportID
    FROM dbo.f_GetDeviceFQDN(1) c
      INNER JOIN dbo.f_GetLastInventoryDate(1) ir
        ON c.ComputerID = ir.ComputerID
          AND (@iLastSeenDays IS NULL OR DATEDIFF(day,  ir.LastInventoryDate, CURRENT_TIMESTAMP) <= @iLastSeenDays)

  -- File Information
  IF OBJECT_ID('tempdb..#csp_aspera_connector_software_tag') IS NOT NULL
  BEGIN
    DECLARE @SoftwareUniqueID VARCHAR(32) = ''
	IF EXISTS(SELECT * FROM tempdb.INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME = 'software_unique_id' and TABLE_NAME LIKE '#csp_aspera_connector_software_tag%')
      SET @SoftwareUniqueID = ',st.SoftwareUniqueID AS software_unique_id'
    DECLARE @sSQL VARCHAR(MAX) = '
    INSERT INTO #csp_aspera_connector_software_tag
    SELECT
        c.ImportID + ''_'' + CAST(sf.SoftwareFileID AS NVARCHAR(16)) AS import_id,
        ''' + @sDataSourceID + '''                                             AS import_data_source_id,  -- Import ID Vergeben ber ReportingConfiguration
        LEFT(sf.ProductName, 255)                                  AS product_title,
        LEFT(ISNULL(st.ProductVersion, ''''), 255)                   AS product_ver_name,
        LEFT(ISNULL(st.ProductVersionMajor, ''''), 255)              AS product_ver_major,
        LEFT(ISNULL(st.ProductVersionMinor, ''''), 255)              AS product_ver_minor,
        LEFT(ISNULL(st.Publisher, ''''), 255)                        AS manufacturer_name,
        LEFT(ISNULL(st.SwidLicensorId, ''''), 255)                   AS manufacturer_guid,
        c.ImportID                                                 AS import_device_id,
        ISNULL(CONVERT(NVARCHAR(10), sf.FileDate, 104),'''')         AS installation_date'
        + @SoftwareUniqueID + '
      FROM dbo.f_GetSoftwareTagFiles() st
	    INNER JOIN dbo.f_GetSoftwareFiles(3) sf
          ON st.SoftwareIsoTagFileID = sf.SoftwareIsoTagFileID
        INNER JOIN #tmpImportID c
          ON sf.ComputerID = c.ComputerID'
    EXEC (@sSQL)
  END
  ELSE
    SELECT
        c.ImportID + '_' + CAST(sf.SoftwareFileID AS NVARCHAR(16)) AS import_id,
        @sDataSourceID                                             AS import_data_source_id,  -- Import ID Vergeben ber ReportingConfiguration
        LEFT(sf.ProductName, 255)                                  AS product_title,
        LEFT(ISNULL(st.ProductVersion, ''), 255)                   AS product_ver_name,
        LEFT(ISNULL(st.ProductVersionMajor, ''), 255)              AS product_ver_major,
        LEFT(ISNULL(st.ProductVersionMinor, ''), 255)              AS product_ver_minor,
        LEFT(ISNULL(st.Publisher, ''), 255)                        AS manufacturer_name,
        LEFT(ISNULL(st.SwidLicensorId, ''), 255)                   AS manufacturer_guid,
        c.ImportID                                                 AS import_device_id,
        ISNULL(CONVERT(NVARCHAR(10), sf.FileDate, 104),'')         AS installation_date,
        st.SoftwareUniqueID                                        AS software_unique_id
      FROM dbo.f_GetSoftwareTagFiles() st
	    INNER JOIN dbo.f_GetSoftwareFiles(3) sf
          ON st.SoftwareIsoTagFileID = sf.SoftwareIsoTagFileID
        INNER JOIN #tmpImportID c
          ON sf.ComputerID = c.ComputerID

  DROP TABLE #tmpImportID		  
  -- OPTION (MAXDOP 1)

END
GO

