USE [ManageSoft]
GO


IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetDeviceFQDN' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetDeviceFQDN]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 14.01.2019
-- Copyright:   (c) Raynet GmbH
-- Description:	Get FQDN from all System
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_GetDeviceFQDN] (@bOnlyComputer BIT)
  RETURNS TABLE
AS
RETURN
  SELECT 
      c.ComputerID,
      LOWER(COALESCE(opCSName.Value, c.ComputerCN)) AS ComputerCN,
      LOWER(CASE
              WHEN baseDom.Value IS NOT NULL
                AND baseDom.Value != 'WORKGROUP'
                  THEN COALESCE(opCSName.Value, c.ComputerCN) + '.' + baseDom.Value
              WHEN CHARINDEX('.', c.ComputerCN) > 0 THEN c.ComputerCN
              WHEN  ISNULL(d.DomainType, '') != 'MGS' AND CHARINDEX(',', d.DN) > 1
                  THEN c.ComputerCN + '.' + REPLACE(REPLACE(d.DN, 'DC=', ''), ',', '.')
              WHEN LEN(nd.DNSFullName) > 0 AND nd.DNSFullName NOT LIKE '%.managesoft'
                  THEN nd.DNSFullName
              WHEN dns.DNSSuffix IS NOT NULL THEN c.ComputerCN + '.' + dns.DNSSuffix
              ELSE COALESCE(opCSName.Value, c.ComputerCN)
            END) FQDN
    FROM dbo.Computer c WITH (NOLOCK)
      LEFT JOIN dbo.Organization o WITH (NOLOCK)
        ON c.ComputerOUID = o.OrganizationID
      LEFT JOIN dbo.Domain d WITH (NOLOCK)
        ON o.DomainID = d.OrganizationID
      LEFT JOIN (SELECT ComputerID, MAX(DNSFullName) AS DNSFullName 
                   FROM dbo.NetworkDevice nd WITH (NOLOCK) GROUP BY ComputerID) nd
        ON c.ComputerID = nd.ComputerID
      LEFT JOIN dbo.f_HardwareProperties('CIM_UnitaryComputerSystem') baseDom
        ON c.ComputerID = baseDom.ComputerID
          AND Property = 'Domain' AND CHARINDEX(' ', baseDom.Value) = 0
      LEFT JOIN (SELECT
                     nic.ComputerID,
                     MAX(dns.Value) AS DNSSuffix
                   FROM dbo.f_HardwareProperties('Win32_NetworkAdapterConfiguration') nic
                     LEFT JOIN dbo.f_HardwareValuesByPropertyName('DNSDomain') dns
                       ON nic.HardwareObjectID = dns.HardwareObjectID
                   WHERE nic.Property = 'IPEnabled' AND nic.Value = 'TRUE' AND dns.Value IS NOT NULL
                   GROUP BY nic.ComputerID) dns
        ON c.ComputerID = dns.ComputerID
      LEFT JOIN (SELECT 
                     ComputerID, 
                     CASE
                       WHEN (CHARINDEX('.', Value) > 0) THEN LEFT(Value, CHARINDEX('.', Value)-1)
                       WHEN (CHARINDEX(' ', Value) > 0) THEN NULL
                       ELSE Value
                     END AS Value
                   FROM dbo.f_HardwareProperties('CIM_OperatingSystem')
                   WHERE Property = 'CSNAME') opCSName
        ON c.ComputerID = opCSName.ComputerID
      LEFT JOIN (SELECT DISTINCT ComputerID FROM dbo.f_HardwareProperties('CIM_%')) i
        ON c.ComputerID = i.ComputerID
    WHERE @bOnlyComputer = 0
      OR @bOnlyComputer IS NULL
      OR (@bOnlyComputer = 1 AND i.ComputerID > 0)


GO

