USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetDeviceNetworkInterface' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetDeviceNetworkInterface]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 16.01.2019
-- Copyright:   (c) Raynet GmbH
-- Description:	Get All Network Interfaces
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_GetDeviceNetworkInterface]()
  RETURNS TABLE
AS
RETURN
  SELECT
      n.ComputerID,
      COALESCE(nn.Value, n.Caption) AS InterfaceName,
      CASE
        WHEN n.Model LIKE '%Microsoft%'  THEN 'Microsoft'
        WHEN n.Model LIKE '%Hyper-V%'    THEN 'Microsoft'
        WHEN n.Model LIKE '%Intel%'      THEN 'Intel'
        WHEN n.Model LIKE '%AMD%'        THEN 'AMD'
        WHEN n.Model LIKE '%Broadcom%'   THEN 'Broadcom'
        WHEN n.Model LIKE '%Realtec%'    THEN 'Realtec'
        WHEN n.Model LIKE '%Cisco%'      THEN 'Cisco'
        WHEN n.Model LIKE '%Juniper%'    THEN 'Juniper'
        WHEN n.Model LIKE '%TeamViewer%' THEN 'TeamViewer'
        WHEN n.Model LIKE '%HP %'        THEN 'HP'
        WHEN n.Model LIKE '%vmxnet%'     THEN 'VMware'
      END                           AS Manufacturer,
      IIF(n.Model LIKE '[ev][tim][hrk][0-9b]%', NULL, n.Model)
                                    AS Model,
      mac.Value                     AS MACAddress,
      dbo.CIMBoolean(nie.Value)     AS IPEnabled,
      dbo.CIMBoolean(nde.Value)     AS DHCPEnabled,
      ni4.Value                     AS IPAddress,
      ni6.Value                     AS IPAddressV6,
      ns.Value                      AS Subnetmask,
      nb.Value                      AS Broadcast,
      ng.Value                      AS DefaultGateway,
      nds.Value                     AS DNSServer,
      nd.Value                      AS DNSDomain,
      nhn.Value                     AS DNSHostname,
      dbo.CIMBigInteger(nsp.Value)  AS Speed
    FROM (SELECT 
              nt.ComputerID,
              nt.HardwareObjectID,
              nc.Value                    AS Caption,
              d.Value                     AS [Description],
              COALESCE(d.Value, nc.Value) AS Model
	        FROM (SELECT DISTINCT ComputerID, HardwareObjectID 
                    FROM dbo.f_HardwareProperties('CIM_MGS_NetworkAdapterConfiguration')) nt
                      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Caption') nc
                        ON nt.HardwareObjectID = nc.HardwareObjectID
                      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Description') d
                        ON nt.HardwareObjectID = d.HardwareObjectID) n
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Name') nn
        ON n.HardwareObjectID = nn.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('MACAddress') mac
        ON n.HardwareObjectID = mac.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('IPEnabled') nie
        ON n.HardwareObjectID = nie.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('DHCPEnabled') nde
        ON n.HardwareObjectID = nde.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('IPAddress') ni4
        ON n.HardwareObjectID = ni4.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('IP6Address') ni6
        ON n.HardwareObjectID = ni6.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('DefaultIPGateway') ng
        ON n.HardwareObjectID = ng.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('IPBroadcastAddress') nb
        ON n.HardwareObjectID = nb.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('IPSubnet') ns
        ON n.HardwareObjectID = ns.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('DNSDomain') nd
        ON n.HardwareObjectID = nd.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('DNSHostname') nhn
        ON n.HardwareObjectID = nhn.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('DNSServerSearchOrder') nds
        ON n.HardwareObjectID = nds.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Speed') nsp
        ON n.HardwareObjectID = nsp.HardwareObjectID
GO




