USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetDeviceNetworkInfo' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetDeviceNetworkInfo]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 25.01.2019
-- Copyright:   (c) Raynet GmbH
-- Description:	Get All IP Addresses and MAC Addresses
--              of the Systems
--              Model and Manufacturer from one NIC
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_GetDeviceNetworkInfo]()
  RETURNS TABLE
AS
RETURN
  WITH nic
    AS (SELECT ComputerID, IPAddress, MACAddress, Manufacturer, Model
          FROM dbo.f_GetDeviceNetworkInterface()
          WHERE IPEnabled = 1)
  SELECT
      na.ComputerID,
      na.Manufacturer,
      na.Model,
      (SELECT LEFT(LTRIM((SELECT ' '+nic.IPAddress
                            FROM nic
                            WHERE nic.IPAddress NOT IN ('0.0.0.0', '127.0.0.1')
                              AND nic.ComputerID = na.ComputerID
                            FOR XML PATH(''))), 4000)) AS IPAddress,
      (SELECT LEFT(LTRIM((SELECT ' '+nic.MACAddress
                            FROM nic
                            WHERE nic.ComputerID = na.ComputerID
                            FOR XML PATH(''))), 4000)) AS MACAddress
    FROM (SELECT DISTINCT
              ComputerID, 
              MAX(Manufacturer) AS Manufacturer, 
              MAX(Model)        AS Model
            FROM nic
            GROUP BY ComputerID) na
GO

