USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetDeviceModel' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetDeviceModel]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 21.01.2019
-- Copyright:   (c) Raynet GmbH
-- Description:	Get Model and Manufacture Information
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================
CREATE FUNCTION [dbo].[f_GetDeviceModel]()
  RETURNS TABLE
AS
RETURN
  SELECT
      c.ComputerID                                          AS ComputerID,
      basePro.HardwareObjectID                              AS CIMProductID,
      COALESCE(base.Manufacturer, basePro.Manufacturer)     AS Manufacturer,
      COALESCE(basePro.Model, base.Model)                   AS Model,
      IIF(COALESCE(basePro.Model, base.Model) IN ('Virtual Machine', 'VMware Virtual Platform', 'VMware7,1'), 1, 0)
                                                            AS Virtual
    FROM dbo.Computer c WITH (NOLOCK)
      LEFT JOIN (SELECT
                     man.HardwareObjectID                                     AS HardwareObjectID,
                     man.ComputerID                                           AS ComputerID,
                     man.Value                                                AS Manufacturer,
                     model.Value                                              AS Model
                   FROM (SELECT ComputerID, HardwareObjectID, Value
                           FROM dbo.f_HardwareProperties('CIM_UnitaryComputerSystem')
                           WHERE Property = 'Manufacturer') man
                     LEFT JOIN dbo.f_HardwareValuesByPropertyName('Model') model
                       ON man.HardwareObjectID = model.HardwareObjectID) base
        ON c.ComputerID = base.ComputerID
      LEFT JOIN (SELECT
                     hwO.HardwareObjectID                                     AS HardwareObjectID,
                     hwO.ComputerID                                           AS ComputerID,
                     man.Value                                                AS Manufacturer,
                     CASE
                       WHEN LEN(prdVer.Value) < 5          THEN prdName.Value                           -- Some IBM Server has in the Version realy a Version and then its the Name
                       WHEN man.Value IN ('LENOVO', 'IBM') THEN COALESCE(prdVer.Value, prdName.Value)   -- For IBM take this
                       WHEN prdName.Value = 'HVM domU'     THEN prdName.Value + ' ' + prdVer.Value      -- 02.03.2018 Add Version Information if Amazon Cloud
                       ELSE prdName.Value                                                               -- For the rest like Fujitsu
                     END                                                      AS Model
                   FROM (SELECT DISTINCT ComputerID, HardwareObjectID 
                           FROM dbo.f_HardwareProperties('CIM_Product')) hwO
                     LEFT JOIN dbo.f_HardwareValuesByPropertyName('Vendor') man
                       ON hwO.HardwareObjectID = man.HardwareObjectID
                     LEFT JOIN dbo.f_HardwareValuesByPropertyName('Name') prdName
                       ON hwO.HardwareObjectID = prdName.HardwareObjectID
                     LEFT JOIN dbo.f_HardwareValuesByPropertyName('Version') prdVer
                       ON hwO.HardwareObjectID = prdVer.HardwareObjectID) basePro
        ON c.ComputerID = basePro.ComputerID
    WHERE base.ComputerID > 0 OR basePro.ComputerID > 0
GO

