USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetVirtualVMwareGuestUUID' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetVirtualVMwareGuestUUID]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 22.01.2019
-- Copyright:   (c) Raynet GmbH
-- Description:	Get VMWare Guest UUID from ESX Inventory
-- ===============================================
-- Used for Matching the Machines
-- Sample:
--   SELECT
--       esx.ComputerID    AS ESXServerID,
--       guest.ComputerID  AS GuestID 
--     FROM f_GetVMwareGuestUUID() esx
--       LEFT JOIN f_GetDeviceVMWareUUID() guest
--         ON esx.UUID = guest.UUID
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_GetVirtualVMwareGuestUUID]()
  RETURNS TABLE
AS
RETURN
  SELECT
      vh.ComputerID                           AS ESXHostID, 
      vh.HardwareObjectID                     AS VMWAREVMComputerSystemID,
      TRY_CAST(vh.Value AS UNIQUEIDENTIFIER)  AS UUID 
     FROM dbo.f_HardwareProperties('VMWARE_VMComputerSystem') vh
     WHERE vh.Property = 'UUID'
       AND LEN(vh.Value) IN (36,38)
       AND vh.Value NOT LIKE '%[^-0-9a-fA-F]%'
GO

