USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetVirtualHyperVGuestInfo' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetVirtualHyperVGuestInfo]
GO
-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 30.01.2019
-- Copyright:   (c) Raynet GmbH
-- Description:	Get all Guest Information from all HyperV Hosts
--              ndtrack and RWI
--              Based on f_Virtual_HyperV_HostInfo
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================
-- Used Functions:
-- - for All
--   - f_HardwareProperties(<ClassName>)
--   - f_HardwareValuesByPropertyName(<PropertyName>)


CREATE FUNCTION [dbo].[f_GetVirtualHyperVGuestInfo]()
  RETURNS TABLE
AS
RETURN
-- HyperV from RWI Data
  SELECT
      gN.ComputerID                          AS HyperVHostID,
      TRY_CAST(gN.Value AS UNIQUEIDENTIFIER) AS SystemGuid,
      gE.Value                               AS ElementName,
      gD.Value                               AS [Description],
      CAST(NULL AS NVARCHAR(255))            AS FQDN,
      dbo.f_SQLDateTime(gDa.Value)           AS InstallDate,
      CASE gs.Value
        WHEN '0' THEN 'Unknown'
        WHEN '2' THEN 'Running'
        WHEN '3' THEN 'Off'
        WHEN '6' THEN 'Saved'
        WHEN '32768' THEN 'Paused'
        WHEN '32769' THEN 'Suspended'
        WHEN '32770' THEN 'Starting'
        WHEN '32771' THEN 'Snapshotting'
        WHEN '32773' THEN 'Saving'
        WHEN '32774' THEN 'Stopping'
        WHEN '32775' THEN 'Pausing'
        WHEN '32775' THEN 'Resuming'
        ELSE 'Unknown'
      END                                    AS PowerState,
      dbo.CIMInteger(gs.Value)               AS PowerStateID,
      TRY_CAST(bG.Value AS UNIQUEIDENTIFIER) AS BIOSGuid,
      bs.Value                               AS BIOSSerialNumber,
      mVQ.Value                              AS Memory,
      mMin.Value                             AS MemoryMin,
      mMax.Value                             AS MemoryMax,
      pVQ.Value                              AS CPUs
    FROM (SELECT ComputerID, HardwareObjectID, Value FROM dbo.f_HardwareProperties('Msvm_ComputerSystem') WHERE Property = 'Name') gN
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('ElementName') gE
        ON gN.HardwareObjectID = gE.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Description') gD
        ON gN.HardwareObjectID = gD.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('InstallDate') gDa
        ON gN.HardwareObjectID = gDa.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('EnabledState') gS
        ON gN.HardwareObjectID = gS.HardwareObjectID
      LEFT JOIN dbo.f_HardwareProperties('Msvm_VirtualSystemSettingData') bId
        ON gN.ComputerID = bId.ComputerID
          AND bId.Property = 'INSTANCEID'
          AND bId.Value = 'Microsoft:' + gN.Value
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('BIOSGUID') bG
        ON bId.HardwareObjectID = bG.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('BIOSSerialNumber') bS
        ON bId.HardwareObjectID = bS.HardwareObjectID
      LEFT JOIN dbo.f_HardwareProperties('Msvm_MemorySettingData') mId
        ON gN.ComputerID = mId.ComputerID
          AND mId.Property = 'INSTANCEID'
          AND mId.Value LIKE 'Microsoft:' + gN.Value + '%'
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('VirtualQuantity') mVQ
        ON mId.HardwareObjectID = mVQ.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Limit') mMax
        ON mId.HardwareObjectID = mMax.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Reservation') mMin
        ON mId.HardwareObjectID = mMin.HardwareObjectID
      LEFT JOIN dbo.f_HardwareProperties('Msvm_ProcessorSettingData') pId
        ON gN.ComputerID = pId.ComputerID
          AND pId.Property = 'InstanceID'
          AND pId.Value LIKE 'Microsoft:' + gN.Value + '%'
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('VirtualQuantity') pVQ
        ON pId.HardwareObjectID = pVQ.HardwareObjectID
 
    WHERE gD.Value != 'Microsoft Hosting Computer System'
-- HyperV from ndtrack PlugIN
  UNION SELECT
      gN.ComputerID                          AS HyperVHostID,
      TRY_CAST(gN.Value AS UNIQUEIDENTIFIER) AS SystemGuid,
      gE.Value                               AS ElementName,
	  gD.Value                               AS Discription,
	  dFQDN.Value                            AS FQDN,
      dbo.f_SQLDateTime(gDa.Value)           AS InstallDate,
      gs.Value                               AS PowerState,
      dbo.CIMInteger(gsi.Value)              AS PowerStateID,
      TRY_CAST(bG.Value AS UNIQUEIDENTIFIER) AS BIOSGuid,
      bs.Value                               AS BIOSSerialNumber,
	  mVQ.Value                              AS Memory,
      mMin.Value                             AS MemoryMin,
      mMax.Value                             AS MemoryMax,
      pVQ.Value                              AS CPUs
    FROM (SELECT ComputerID, HardwareObjectId, Value FROM dbo.f_HardwareProperties('HyperV_Guest') WHERE Property = 'SystemGuid') gN
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('ElementName') gE
        ON gN.HardwareObjectID = gE.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Description') gD
        ON gN.HardwareObjectID = gD.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('InstallDate') gDa
        ON gN.HardwareObjectID = gDa.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('State') gS
        ON gN.HardwareObjectID = gS.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('StateInt') gSi
        ON gN.HardwareObjectID = gSi.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('BIOSGuid') bG
        ON gN.HardwareObjectID = bG.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('BIOSSerialNumber') bS
        ON gN.HardwareObjectID = bS.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Memory') mVQ
        ON gN.HardwareObjectID = mVQ.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('MemoryMax') mMax
        ON gN.HardwareObjectID = mMax.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('MemoryMin') mMin
        ON gN.HardwareObjectID = mMin.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('ProcessorCount') pVQ
        ON gN.HardwareObjectID = pVQ.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('DNSNameGuest') dFQDN
        ON gN.HardwareObjectID = dFQDN.HardwareObjectID
GO
