USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_Virtual_HyperV' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_Virtual_HyperV]
GO

-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================
 
CREATE PROCEDURE [dbo].[sp_Virtual_HyperV] 
AS
 
 BEGIN
 SELECT * INTO #HyperV_Guest		FROM f_HardwareProperties('HyperV_Guest')
 SELECT * INTO #HyperV_ClusterName	FROM f_HardwareProperties('HyperV_ClusterName')
 SELECT * INTO #IsFailover			FROM f_HardwareProperties('IsFailover')
 SELECT * INTO #System_Base			FROM f_System_Base()
 SELECT * INTO #HyperV_Host			FROM f_HardwareProperties('HyperV_Host')
 SELECT * INTO #DNSNameGuest				FROM f_HardwareValuesByPropertyName('DNSNameGuest')
 SELECT * INTO #CIM_UnitaryComputerSystem	FROM f_HardwareProperties('CIM_UnitaryComputerSystem')
 SELECT * INTO #Win32_NetworkAdapterConfiguration	FROM f_HardwareProperties('Win32_NetworkAdapterConfiguration')
 SELECT * INTO #DNSDomain							FROM f_HardwareValuesByPropertyName('DNSDomain')

   SELECT * INTO #Virtual_HyperV FROM 
   (
  -- Host Server
    SELECT 
        c.ComputerID                                       AS ComputerID,
        c.ComputerCN                                       AS ComputerName,
        base.ComputerFQDN                                  AS ComputerFQDN,
        c.ComputerID                                       AS HostID,
        c.ComputerCN                                       AS HostName,
        CAST(NULL AS BIGINT)                               AS GuestID,
        CAST(NULL AS NVARCHAR(255))                        AS GuestName,
        vCluster.Value                                     AS ClusterName,
        vFailover.Value                                    AS [Failover],
        'Host'                                             AS [Type],
        base.Manufacturer                                  AS Manufacturer,
        base.Model                                         AS Model,
        base.SerialNumber                                  AS SerialNumber,
        Base.BiosVersion                                   AS BiosVersion,
        base.OS                                            AS OS,
        base.CPUManufacturer                               AS CPUManufacturer,
        base.CPUModel                                      AS CPUModel,
        base.CPUs                                          AS CPUs,
        base.Cores                                         AS Cores,
        base.HyperThreating                                AS HyperThreating,
        base.CPUSpeed                                      AS CPUSpeed,
        base.Memory                                        AS Memory,
        base.DiskSize                                      AS DiskSize,
        base.LastLogin                                     AS LastLogin,
        base.InventoryDate                                 AS InventoryDate
      FROM Computer c WITH (NOLOCK)
        INNER JOIN (SELECT DISTINCT ComputerID FROM #HyperV_Guest WHERE Property = 'DNSNameGuest') host
          ON c.ComputerID = host.ComputerID
        LEFT JOIN (SELECT ComputerID, HardwareObjectID, Value FROM #HyperV_ClusterName WHERE Property = 'ClusterName') vCluster
          ON c.ComputerID = vCluster.ComputerID
        LEFT JOIN #IsFailover vFailover
          ON vCluster.HardwareObjectID = vFailover.HardwareObjectID
  -- System Information
        LEFT JOIN #System_Base base
          ON c.ComputerID = base.ComputerID
  UNION
  -- Guest Systems
    SELECT 
        c.ComputerID                                       AS ComputerID,
        c.ComputerCN                                       AS ComputerName,
        base.ComputerFQDN                                  AS ComputerFQDN,
        Host.ComputerID                                    AS HostID,
        Host.ComputerCN                                    AS HostName,
        c.ComputerID                                       AS GuestID,
        c.ComputerCN                                       AS GuestName,
        vCluster.Value                                     AS ClusterName,
        NULL                                               AS ClusterFailover,
        'Guest'                                            AS Type,
        base.Manufacturer                                  AS Manufacturer,
        base.Model                                         AS Model,
        base.SerialNumber                                  AS SerialNumber,
        Base.BiosVersion                                   AS BiosVersion,
        base.OS                                            AS OS,
        base.CPUManufacturer                               AS CPUManufacturer,
        base.CPUModel                                      AS CPUModel,
        base.CPUs                                          AS CPUs,
        base.Cores                                         AS Cores,
        base.HyperThreating                                AS HyperThreating,
        base.CPUSpeed                                      AS CPUSpeed,
        base.Memory                                        AS Memory,
        base.DiskSize                                      AS DiskSize,
        base.LastLogin                                     AS LastLogin,
        base.InventoryDate                                 AS InventoryDate
      FROM Computer c WITH (NOLOCK)
  -- System Information
        LEFT JOIN #System_Base base
          ON c.ComputerID = base.ComputerID
  -- Host Relation (Tmp for Relation on DNSNameHost
        LEFT JOIN (SELECT ComputerId, Value  FROM #HyperV_Host WHERE Property = 'DNSNameHost') HostTmp
          ON c.ComputerID = HostTmp.ComputerID
  -- Host Relation 
        INNER JOIN (SELECT 
                        tmp.ComputerID, 
                        fqdn.ComputerCN,
                        fqdn.FQDN AS ComputerFQDN,
                        tmp.HardwareObjectID,
                        tmp.Value        [BiosSerialNumber],
                        dng.Value        [DNSNameGuest]
                     FROM #HyperV_Guest tmp
                        LEFT JOIN  #DNSNameGuest dng
                          ON tmp.HardwareObjectID = dng.HardwareObjectID
                  -- DNS
                        LEFT JOIN (SELECT
                                       c.ComputerID,
                                       c.ComputerCN,
                                       CASE
                                         WHEN CHARINDEX('.', c.ComputerCN) > 0 THEN c.ComputerCN
                                         WHEN CHARINDEX(',', d.DN) > 1 THEN c.ComputerCN + REPLACE(ISNULL('.' + REPLACE(REPLACE(dbo.CIMString(o.RDN), 'CN=', ''), 'OU=', ''), '') + '.' + REPLACE(d.DN, 'DC=', ''), ',', '.')
                                         WHEN dbo.CIMString(nd.DNSFullName) IS NOT NULL THEN nd.DNSFullName
                                         WHEN dns.DNSSuffix IS NOT NULL THEN c.ComputerCN + '.' + dns.DNSSuffix
                 --                        WHEN CHARINDEX('.', baseDom.Value) > 0 THEN baseDom.Value
                                         ELSE c.ComputerCN
                                       END FQDN
                                    FROM Computer c WITH (NOLOCK)
                                      LEFT JOIN Organization o WITH (NOLOCK)
                                        ON c.ComputerOUID = o.OrganizationID
                                      LEFT JOIN Domain d WITH (NOLOCK)
                                        ON o.DomainID = d.OrganizationID
                                      LEFT JOIN NetworkDevice nd WITH (NOLOCK)
-- 01.12.2017 add filter on DupplicateID IS NULL, only the last one is from intresting
-- 01.12.2017 add filter on computerCN to DeviceName, some times it give entrys with other devicename what
--            is mapped to the ComputerDevice, this are not from intresst
                                        ON nd.DuplicateID IS NULL AND c.ComputerID = nd.ComputerID AND c.ComputerCN = nd.DeviceName 
                                      LEFT JOIN  #CIM_UnitaryComputerSystem baseDom
                                        ON c.ComputerID = baseDom.ComputerID
                                          AND Property = 'Domain'
                                      LEFT JOIN (SELECT
                                                      nic.ComputerID,
                                                      MAX(dns.Value) AS DNSSuffix
                                                    FROM  #Win32_NetworkAdapterConfiguration nic
                                                      LEFT JOIN #DNSDomain dns
                                                        ON nic.HardwareObjectID = dns.HardwareObjectID
                                                    WHERE nic.Property = 'IPEnabled' AND nic.Value = 'TRUE' AND dns.Value IS NOT NULL
                                                    GROUP BY nic.ComputerID) dns
                                        ON c.ComputerID = dns.ComputerID
                                  ) fqdn
                          ON tmp.ComputerID = fqdn.ComputerID
                      WHERE tmp.Property = 'BIOSSerialNumber'
                   ) Host
          ON base.SerialNumber = Host.BiosSerialNumber
            AND (Host.DNSNameGuest LIKE base.ComputerFQDN 
              OR (HostTmp.Value IS NOT NULL AND HostTmp.Value LIKE Host.ComputerFQDN))
-- ClusterName
        LEFT JOIN (SELECT ComputerID, HardwareObjectID, Value FROM  #HyperV_ClusterName WHERE Property = 'ClusterName') vCluster
          ON Host.ComputerID = vCluster.ComputerID
      WHERE Host.ComputerID IS NOT NULL
	  ) Virtual_HyperV

	 DROP TABLE #HyperV_Guest	 
	 DROP TABLE #HyperV_ClusterName	 
	 DROP TABLE #IsFailover		 
	 DROP TABLE #System_Base		 
	 DROP TABLE #HyperV_Host			 
	 DROP TABLE #DNSNameGuest			 
	 DROP TABLE #CIM_UnitaryComputerSystem	 
	 DROP TABLE #Win32_NetworkAdapterConfiguration	
	 DROP TABLE #DNSDomain	
	 
	 						
END
