USE [ManageSoft]
GO


IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_Virtual_VMware' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_Virtual_VMware]
GO

-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================
 
CREATE PROCEDURE [dbo].[sp_Virtual_VMware] (@AllClients BIT)
 AS
 BEGIN
  
  
	SELECT  ComputerID, REPLACE(REPLACE(REPLACE(LEFT(Value,100),' ',''),'-',''),'VMware','') CIMValue
	INTO #CIM_BIOSElement
		FROM f_HardwareProperties('CIM_BIOSElement')
	WHERE Property = 'SerialNumber'
	
	SELECT ComputerID, HardwareObjectID, REPLACE(REPLACE(Value,' ',''),'-','') VMWAREValue
	INTO #VMWARE_VMComputerSystem
		FROM f_HardwareProperties('VMWARE_VMComputerSystem')  WHERE Property IN ('UUID')
	AND ComputerID IS NOT NULL

  INSERT INTO #Virtual_VMware
  SELECT *
  FROM
  (
      SELECT 
        c.ComputerID                                       AS ComputerID,
        c.ComputerCN                                       AS ComputerName,
        CASE
          WHEN CHARINDEX('.', esxFQDN.Value) > 0 AND esxFQDN.Value NOT LIKE '[1234567890.]%' THEN esxFQDN.Value
          ELSE base.ComputerFQDN
        END                                                AS ComputerFQDN,
        vCenterSrv.vCenterID                               AS vCenterID,
        vCenterSrv.vCenterCN                               AS vCenterName,
        c.ComputerID                                       AS ESXID,
        c.ComputerCN                                       AS ESXName,
        NULL                                               AS GuestID,
        NULL                                               AS GuestName,
        vCluster.Value                                     AS ClusterName,
        'Host'                                             AS Type,
        base.Manufacturer                                  AS Manufacturer,
        base.Model                                         AS Model,
        base.SerialNumber                                  AS SerialNumber,
        Base.BiosVersion                                   AS BiosVersion,
        base.OS                                            AS OS,
        base.CPUManufacturer                               AS CPUManufacturer,
        base.CPUModel                                      AS CPUModel,
        base.CPUs                                          AS CPUs,
        base.Cores                                         AS Cores,
        base.HyperThreating                                AS HyperThreating,
        base.CPUSpeed                                      AS CPUSpeed,
        base.Memory                                        AS Memory,
        base.DiskSize                                      AS DiskSize,
        base.LastLogin                                     AS LastLogin,
        CAST(NULL AS BIT)                                  AS PowerState,
        base.InventoryDate                                 AS InventoryDate
      FROM Computer c WITH (NOLOCK)
  -- Manufacturer and Model
        INNER JOIN (SELECT 
                        ComputerID, HardwareObjectID
                      FROM f_HardwareProperties('VMWARE_ESXComputerSystem')
                      WHERE Property = 'Manufacturer') baseESX
          ON c.ComputerID = baseESX.ComputerID
  -- FQDN
        LEFT JOIN f_HardwareValuesByPropertyName('ElementName') esxFQDN
          ON baseESX.HardwareObjectID = esxFQDN.HardwareObjectID
  -- vCenter
        LEFT JOIN (SELECT
                       esx.ComputerID                            AS ESXID,
                       vCenter.ComputerID                        AS vCenterID,
                       ISNULL(vCenter.ComputerCN, esx.vCenterCN) AS vCenterCN
                     FROM (SELECT
                               ComputerID,
                               CASE
                                 WHEN esxTmp.vCenter LIKE '[1234567890!.]%' THEN esxTmp.vCenter
                                 WHEN CHARINDEX('.', esxTmp.vCenter) > 0   THEN LEFT(esxTmp.vCenter, CHARINDEX('.', esxTmp.vCenter)-1)
                                 ELSE esxTmp.vCenter
                               END              AS vCenterCN,
                               esxTmp.vCenter   AS vCenterFQDN,
                            /* CASE
                                 WHEN esxTmp.vCenter LIKE '[1234567890.]%' THEN 1
                                 ELSE 0
                               END              AS IsIP, */
                               CASE
                                 WHEN esxTmp.vCenter NOT LIKE '[1234567890.]%' AND CHARINDEX('.', esxTmp.vCenter) = 0 THEN 1
                                 ELSE 0
                               END              AS IsNoFQDN
                             FROM (SELECT
                                       ComputerID, 
                                       Replace(Replace(Replace(Replace(Value, 'http://', ''), 'https://', ''), '/sdk', ''), '/', '') vCenter
                                     FROM f_HardwareProperties('VMWARE_InfrastructureSDK')
                                     WHERE Property = 'ServerURL') esxTmp
                          ) esx
                       LEFT JOIN (SELECT c.ComputerID, c.ComputerCN, Replace(Replace(d.DN, 'dc=', '.'), ',', '') domain
                                    FROM Computer c WITH (NOLOCK)
                                      LEFT JOIN Organization o WITH (NOLOCK)
                                        ON c.ComputerOUID = o.OrganizationID
                                      LEFT JOIN Domain d WITH (NOLOCK)
                                        ON o.DomainID = d.OrganizationID) vCenter
                         ON (esx.IsNoFQDN = 0 AND esx.vCenterFQDN LIKE vCenter.ComputerCN + vCenter.domain) 
                           OR (esx.IsNoFQDN = 1 AND esx.vCenterCN = vCenter.ComputerCN)) vCenterSrv
          ON c.ComputerID = vCenterSrv.ESXID
  -- ClusterName
        LEFT JOIN (SELECT ComputerID, Value FROM f_HardwareProperties('MGS_Cluster') WHERE Property = 'ElementName') vCluster
          ON c.ComputerID = vCluster.ComputerID  
  -- System Information
        LEFT JOIN f_System_Base() base
          ON c.ComputerID = base.ComputerID
  UNION
  -- Guest Systems
    SELECT 
	    c.ComputerID                                       AS ComputerID,
        c.ComputerCN                                       AS ComputerName,
        base.ComputerFQDN                                  AS ComputerFQDN,
        vCenterSrv.vCenterID                               AS vCenterID,
        vCenterSrv.vCenterCN                               AS vCenterName,
        esx.ESXComputerID                                  AS ESXID,
        esx.ESXComputerCN                                  AS ESXName,
        c.ComputerID                                       AS GuestID,
        c.ComputerCN                                       AS GuestName,
        vCluster.Value                                     AS ClusterName,
        'Guest'                                            AS Type,
        base.Manufacturer                                  AS Manufacturer,
        base.Model                                         AS Model,
        base.SerialNumber                                  AS SerialNumber,
        Base.BiosVersion                                   AS BiosVersion,
        base.OS                                            AS OS,
        base.CPUManufacturer                               AS CPUManufacturer,
        base.CPUModel                                      AS CPUModel,
        base.CPUs                                          AS CPUs,
        base.Cores                                         AS Cores,
        base.HyperThreating                                AS HyperThreating,
        base.CPUSpeed                                      AS CPUSpeed,
        base.Memory                                        AS Memory,
        base.DiskSize                                      AS DiskSize,
        base.LastLogin                                     AS LastLogin,
        esx.PowerState                                     AS PowerState,
        base.InventoryDate                                 AS InventoryDate
      FROM Computer c WITH (NOLOCK)
  -- ESX Server
        INNER JOIN (SELECT DISTINCT
                        esx.ComputerID                             ESXComputerID,
                        esx.ComputerCN                             ESXComputerCN,
                        link.GuestID                               GuestComputerID,
                        CASE
                          WHEN pw.Value = 'poweredOn'  THEN 1
                          WHEN pw.Value = 'poweredOff' THEN 0
                          ELSE NULL
                        END                                        PowerState
                      FROM Computer esx WITH (NOLOCK)
                        LEFT JOIN (

						SELECT 
							 CIM.ComputerID           GuestID, 
							 VMWARE.ComputerID        HostID,
							 VMWARE.HardwareObjectID  VMWareGuestHWID
							 FROM #CIM_BIOSElement AS CIM
							 LEFT JOIN #VMWARE_VMComputerSystem AS VMWARE  ON CIM.CIMValue = VMWARE.VMWAREValue  AND CIM.ComputerID <> VMWARE.ComputerID 
							 
							    UNION SELECT
                                       tmpC.ComputerID           GuestID,
                                       VMHName.ComputerID        HostID,
                                       VMHName.HardwareObjectID  VMWareGuestHWID
                                     FROM f_HardwareProperties('VMWARE_VMComputerSystem') VMHName
									                     LEFT JOIN (SELECT c.ComputerID,
                                                      CASE
                                                        WHEN CHARINDEX('.', ComputerCN) > 0 THEN ComputerCN
                                                        ELSE c.ComputerCN + REPLACE(REPLACE(d.DN, 'DC=', '.'), ',', '')
                                                      END AS FQDN
                                                    FROM Computer c WITH (NOLOCK)
                                                      LEFT JOIN Organization o WITH (NOLOCK)
                                                        ON c.ComputerOUID = o.OrganizationID
                                                      LEFT JOIN Domain d WITH (NOLOCK)
                                                        ON o.DomainID = d.OrganizationID) AS tmpC
                                         ON VMHName.Value = tmpC.FQDN
                                     WHERE VMHName.Property = 'HostName' AND tmpC.ComputerID <> VMHName.ComputerID
                                   UNION SELECT
                                       tmpC.ComputerID           GuestID,
                                       VMHName.ComputerID        HostID,
                                       VMHName.HardwareObjectID  VMWareGuestHWID
                                     FROM f_HardwareProperties('VMWARE_VMComputerSystem') VMHName
									                     LEFT JOIN Computer tmpC WITH (NOLOCK)
                                         ON VMHName.Value = tmpC.ComputerCN
                                       LEFT JOIN f_HardwareValuesByPropertyName('IPAddress') vmIP
                                         ON vmIP.HardwareObjectID = VMHName.HardwareObjectID
                                       INNER JOIN f_HardwareProperties('CIM_MGS_NetworkAdapterConfiguration') nicIP
                                         ON tmpc.ComputerID = nicIP.ComputerID
                                           AND nicIP.Property = 'IPAddress'
                                           AND vmIP.Value = nicIP.Value
                                     WHERE VMHName.Property = 'Elementname' AND tmpC.ComputerID <> VMHName.ComputerID
                                  ) AS link
                           ON esx.ComputerID = link.HostID
                         LEFT JOIN f_HardwareValuesByPropertyName('PowerState') pw
                           ON link.VMWareGuestHWID = pw.HardwareObjectID
                      WHERE link.GuestID > 0
                   ) esx
          ON c.ComputerID = esx.GuestComputerID
  -- System Information
        LEFT JOIN f_System_Base() base
          ON c.ComputerID = base.ComputerID
  -- vCenter
        LEFT JOIN (SELECT
                       esx.ComputerID                            AS ESXID,
                       vCenter.ComputerID                        AS vCenterID,
                       ISNULL(vCenter.ComputerCN, esx.vCenterCN) AS vCenterCN
                     FROM (SELECT
                               ComputerID,
                               CASE
                                 WHEN esxTmp.vCenter LIKE '[1234567890!.]%' THEN esxTmp.vCenter
                                 WHEN CHARINDEX('.', esxTmp.vCenter) > 0   THEN LEFT(esxTmp.vCenter, CHARINDEX('.', esxTmp.vCenter)-1)
                                 ELSE esxTmp.vCenter
                               END              AS vCenterCN,
                               esxTmp.vCenter   AS vCenterFQDN,
                            /* CASE
                                 WHEN esxTmp.vCenter LIKE '[1234567890.]%' THEN 1
                                 ELSE 0
                               END              AS IsIP, */
                               CASE
                                 WHEN esxTmp.vCenter NOT LIKE '[1234567890.]%' AND CHARINDEX('.', esxTmp.vCenter) = 0 THEN 1
                                 ELSE 0
                               END              AS IsNoFQDN
                             FROM (SELECT
                                       ComputerID, 
                                       Replace(Replace(Replace(Replace(Value, 'http://', ''), 'https://', ''), '/sdk', ''), '/', '') vCenter
                                     FROM f_HardwareProperties('VMWARE_InfrastructureSDK')
                                     WHERE Property = 'ServerURL') esxTmp
                          ) esx
                       LEFT JOIN (SELECT c.ComputerID, c.ComputerCN, Replace(Replace(d.DN, 'dc=', '.'), ',', '') domain
                                    FROM Computer c WITH (NOLOCK)
                                      LEFT JOIN Organization o WITH (NOLOCK)
                                        ON c.ComputerOUID = o.OrganizationID
                                      LEFT JOIN Domain d WITH (NOLOCK)
                                        ON o.DomainID = d.OrganizationID) vCenter
                         ON (esx.IsNoFQDN = 0 AND esx.vCenterFQDN LIKE vCenter.ComputerCN + vCenter.domain) 
                           OR (esx.IsNoFQDN = 1 AND esx.vCenterCN = vCenter.ComputerCN)) vCenterSrv
          ON esx.ESXComputerID = vCenterSrv.ESXID
  -- ClusterName
        LEFT JOIN (SELECT ComputerID, Value FROM f_HardwareProperties('MGS_Cluster') WHERE Property = 'ElementName') vCluster
          ON esx.ESXComputerID = vCluster.ComputerID  
) VM
 

DROP TABLE #CIM_BIOSElement
DROP TABLE #VMWARE_VMComputerSystem


END
