USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_OracleRAC' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_OracleRAC]
GO
-- ==============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 22.05.2017
-- Copyright:   (c) Raynet GmbH
-- Description: Get Oracle RAC Nodes
-- ==============================================
-- -- Used Functions:
--   - f_HardwareProperties
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_OracleRAC] ()
      RETURNS TABLE
AS

RETURN
SELECT
    dbn.ComputerID AS DBObjectID,
    dbn.DBName     AS DBName,
    iid.NodeInstanceID,
    fqdn.ComputerID   AS NodeComputerID,
    fqdn.ComputerCN   AS NodeComputerCN,
    nhn.NodeHostname,
    iname.NodeInstanceName,
    irole.NodeInstanceRole
  FROM (SELECT ComputerID, Property, Value AS DBName FROM dbo.f_HardwareProperties('RVSORACLE_CLUSTER') WHERE HardwareName = 'DB_NAME' AND Property NOT IN ('Type', 'Length')) dbn
    INNER JOIN (SELECT ComputerID, Property, CAST(Value AS INT) AS NodeInstanceID FROM dbo.f_HardwareProperties('RVSORACLE_CLUSTER') WHERE HardwareName = 'THREAD#' AND Property NOT IN ('Type', 'Length') AND Value NOT IN ('NUMBER')) iid
      ON dbn.ComputerID = iid.ComputerID
        AND dbn.Property = iid.Property
    INNER JOIN (SELECT DISTINCT df.ComputerID 
                  FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') df
                    INNER JOIN OracleMetric om
                      ON df.Value = om.FeatureName
                        AND om.Product IN ('Real Application Clusters', 'Real Application Clusters One Node')
                    INNER JOIN (SELECT ComputerID, Property, dbo.CIMInteger(Value) AS Usage FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'DETECTED_USAGES' AND Property NOT IN ('TYPE', 'Length')) du
                      ON df.ComputerID = du.ComputerID
                        AND df.Property = du.Property
                        AND Usage > 0
                  WHERE HardwareName = 'NAME' AND df.Property NOT IN ('TYPE', 'Length')) dfus
      ON dbn.ComputerID = dfus.ComputerID
    LEFT JOIN (SELECT ComputerID, Property, Value AS NodeInstanceName FROM dbo.f_HardwareProperties('RVSORACLE_CLUSTER') WHERE HardwareName = 'INSTANCE_NAME' AND Property NOT IN ('Type', 'Length')) iname
      ON dbn.ComputerID = iname.ComputerID
        AND dbn.Property = iname.Property
    LEFT JOIN (SELECT ComputerID, Property, Value AS NodeHostname FROM dbo.f_HardwareProperties('RVSORACLE_CLUSTER') WHERE HardwareName = 'NODE_HOSTNAME' AND Property NOT IN ('Type', 'Length')) nhn
      ON dbn.ComputerID = nhn.ComputerID
        AND dbn.Property = nhn.Property
    LEFT JOIN (SELECT ComputerID, Property, Value AS NodeInstanceRole FROM dbo.f_HardwareProperties('RVSORACLE_CLUSTER') WHERE HardwareName = 'INSTANCE_ROLE' AND Property NOT IN ('Type', 'Length')) irole
      ON dbn.ComputerID = irole.ComputerID
        AND dbn.Property = irole.Property
    LEFT JOIN dbo.f_GetDeviceFQDN(1) fqdn
      ON nhn.NodeHostname = fqdn.FQDN
        OR (CHARINDEX('.', nhn.NodeHostname) = 0 AND nhn.NodeHostname = fqdn.ComputerCN)
    WHERE iid.Property = iid.NodeInstanceID
      OR (iid.Property != iid.NodeInstanceID
        AND dbn.ComputerID NOT IN (SELECT
                                       tmpdbn.ComputerID
                                     FROM (SELECT ComputerID, Property, Value AS DBName FROM dbo.f_HardwareProperties('RVSORACLE_CLUSTER') WHERE HardwareName = 'DB_NAME' AND Property NOT IN ('Type', 'Length')) tmpdbn
                                       INNER JOIN (SELECT ComputerID, Property, Value AS NodeInstanceID FROM dbo.f_HardwareProperties('RVSORACLE_CLUSTER') WHERE HardwareName = 'THREAD#' AND Property NOT IN ('Type', 'Length')) tmpiid
                                         ON tmpdbn.ComputerID = tmpiid.ComputerID
                                           AND tmpdbn.Property = tmpiid.Property
                                       LEFT JOIN (SELECT ComputerID, Property, Value AS NodeHostname FROM dbo.f_HardwareProperties('RVSORACLE_CLUSTER') WHERE HardwareName = 'NODE_HOSTNAME' AND Property NOT IN ('Type', 'Length')) tmpnhn
                                         ON tmpdbn.ComputerID = tmpnhn.ComputerID
                                           AND tmpdbn.Property = tmpnhn.Property
                                     WHERE nhn.NodeHostname = tmpnhn.NodeHostname
-- Added, to fix if only second instance is online
                                       AND iid.Property != tmpiid.Property))


GO

