USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_OracleNodes' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_OracleNodes]
GO
-- ==============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 17.04.2018
-- Copyright:   (c) Raynet GmbH
-- Description: Get Oracle Instance Nodes incl. RAC
-- ==============================================
-- Used Functions:
--   - f_HardwareProperties
--   - f_OracleRAC
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_OracleNodes] ()
      RETURNS TABLE
AS
RETURN
SELECT
    ih.ComputerID               AS DBObjectID,
    CAST(NULL AS nvarchar(255)) AS ClusterDBName,
	idbn.Value                  AS DBName,
    CAST(NULL AS BIGINT)        AS NodeInstanceID,
    fqdn.ComputerID             AS NodeComputerID,
    CASE
      WHEN CHARINDEX('.', ih.Value) > 0 THEN LEFT(ih.Value, CHARINDEX('.', ih.Value)-1)
      ELSE ih.Value
    END                         AS NodeComputerCN,
    ih.Value                    AS NodeHostname,
	i.Value                     AS NodeInstanceName
  FROM dbo.f_HardwareProperties('RVSORACLE_INSTNAME_HOSTNAME') ih
    LEFT JOIN (SELECT ComputerID, Value FROM f_HardwareProperties('RVSORACLE_INSTNAME_HOSTNAME') WHERE HardwareName = 'INSTANCE_NAME' AND Property = '1') i
      ON ih.ComputerID = i.ComputerID
    LEFT JOIN (SELECT
                   c.ComputerID,
                   c.ComputerCN,
                   CASE
                     WHEN CHARINDEX('.', c.ComputerCN) > 0 THEN c.ComputerCN
                     WHEN  ISNULL(d.DomainType, '') != 'MGS' AND CHARINDEX(',', d.DN) > 1
                       THEN c.ComputerCN + '.' + REPLACE(REPLACE(d.DN, 'DC=', ''), ',', '.')
                     WHEN LEN(nd.DNSFullName) > 0 AND nd.DNSFullName NOT LIKE '%.managesoft'
                       THEN nd.DNSFullName
                     WHEN dns.DNSSuffix IS NOT NULL THEN c.ComputerCN + '.' + dns.DNSSuffix
                     ELSE c.ComputerCN
                   END FQDN
                 FROM Computer c WITH (NOLOCK)
                   LEFT JOIN Organization o WITH (NOLOCK)
                     ON c.ComputerOUID = o.OrganizationID
                   LEFT JOIN Domain d WITH (NOLOCK)
                     ON o.DomainID = d.OrganizationID
                   LEFT JOIN (SELECT ComputerID, MAX(DNSFullName) AS DNSFullName FROM NetworkDevice nd WITH (NOLOCK) GROUP BY ComputerID) nd
                     ON c.ComputerID = nd.ComputerID
                   LEFT JOIN dbo.f_HardwareProperties('CIM_UnitaryComputerSystem') baseDom
                     ON c.ComputerID = baseDom.ComputerID
                       AND Property = 'Domain'
                   LEFT JOIN (SELECT
                                  nic.ComputerID,
                                  MAX(dns.Value) AS DNSSuffix
                                FROM dbo.f_HardwareProperties('Win32_NetworkAdapterConfiguration') nic
                                  LEFT JOIN dbo.f_HardwareValuesByPropertyName('DNSDomain') dns
                                    ON nic.HardwareObjectID = dns.HardwareObjectID
                                WHERE nic.Property = 'IPEnabled' AND nic.Value = 'TRUE' AND dns.Value IS NOT NULL
                                GROUP BY nic.ComputerID) dns
                     ON c.ComputerID = dns.ComputerID
              ) fqdn
      ON ih.Value = fqdn.FQDN
        OR (CHARINDEX('.', ih.Value) = 0 AND ih.Value = fqdn.ComputerCN)
    LEFT JOIN (SELECT ComputerID, Value FROM dbo.f_HardwareProperties('RVSORACLE_DBNAME') WHERE HardwareName = 'NAME' AND Property = '1') idbn
      ON ih.ComputerID = idbn.ComputerID
    LEFT JOIN dbo.f_OracleRAC() rac
      ON ih.ComputerID = rac.DBObjectID
  WHERE ih.HardwareName = 'HOST_NAME' AND ih.Property = '1'
    AND rac.DBObjectID IS NULL
UNION SELECT
    DBObjectID,
    DBName AS ClusterDBName,
    DBName,
    NodeInstanceID,
    NodeComputerID,
    NodeComputerCN,
    NodeHostname,
    NodeInstanceName
  FROM dbo.f_OracleRAC()
GO

