USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_OracleNodes' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_OracleNodes]
GO

-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================


CREATE PROCEDURE [dbo].[sp_OracleNodes]
	 
AS
BEGIN
  DECLARE @bShowTable BIT = 0

  IF OBJECT_ID('tempdb..#OracleNodes') IS NOT NULL
  BEGIN
    TRUNCATE TABLE #OracleNodes
    SET @bShowTable = 0
  END
  ELSE 
  BEGIN
    SET @bShowTable = 1
    CREATE TABLE #OracleNodes (
	  [DBObjectID] [int] NOT NULL,
      [ClusterDBName] [nvarchar](256) NULL,
      [DBName] [nvarchar](256) NULL,
      [NodeInstanceID] [bigint] NULL,
      [NodeComputerID] [int] NULL,
      [NodeComputerCN] [nvarchar](256) NULL,
      [NodeHostname] [nvarchar](256) NULL,
      [NodeInstanceName] [nvarchar](256) NULL
    )
  END

  SELECT * 
  INTO #rac 
  FROM dbo.f_OracleRAC()

  SELECT 
      ComputerID,
      Property,
      Value,
      CASE
        WHEN CHARINDEX('.', Value) = 0 THEN 1
        ELSE 0
      END  AS bNoFQDN
    INTO #RVSORACLE_INSTNAME_HOSTNAME 
    FROM dbo.f_HardwareProperties('RVSORACLE_INSTNAME_HOSTNAME') 
    WHERE HardwareName = 'HOST_NAME' AND Property = '1'  
      AND ComputerID NOT IN (SELECT DBObjectID FROM #rac)

  SELECT ComputerID, Value INTO #i FROM f_HardwareProperties('RVSORACLE_INSTNAME_HOSTNAME') WHERE HardwareName = 'INSTANCE_NAME' AND Property = '1'
  SELECT ComputerID, Value INTO #idbn FROM dbo.f_HardwareProperties('RVSORACLE_DBNAME') WHERE HardwareName = 'NAME' AND Property = '1'
  SELECT c.ComputerID, c.ComputerCN,
                   CASE  WHEN baseDom.Value IS NOT NULL AND baseDom.Value != 'WORKGROUP' THEN c.ComputerCN + '.' + baseDom.Value
                         WHEN CHARINDEX('.', c.ComputerCN) > 0 THEN c.ComputerCN 
                         WHEN ISNULL(d.DomainType, '') != 'MGS' AND CHARINDEX(',', d.DN) > 1   THEN c.ComputerCN + '.' + REPLACE(REPLACE(d.DN, 'DC=', ''), ',', '.')
                         WHEN LEN(nd.DNSFullName) > 0 AND nd.DNSFullName NOT LIKE '%.managesoft' THEN nd.DNSFullName WHEN dns.DNSSuffix IS NOT NULL THEN c.ComputerCN + '.' + dns.DNSSuffix ELSE c.ComputerCN
                   END FQDN
       INTO #fqdn
	             FROM Computer c WITH (NOLOCK)  LEFT JOIN Organization o WITH (NOLOCK) ON c.ComputerOUID = o.OrganizationID
                   LEFT JOIN Domain d WITH (NOLOCK) ON o.DomainID = d.OrganizationID
                   LEFT JOIN (SELECT ComputerID, MAX(DNSFullName) AS DNSFullName FROM NetworkDevice nd WITH (NOLOCK) GROUP BY ComputerID) nd ON c.ComputerID = nd.ComputerID
                   LEFT JOIN dbo.f_HardwareProperties('CIM_UnitaryComputerSystem') baseDom ON c.ComputerID = baseDom.ComputerID AND Property = 'Domain'
                   LEFT JOIN (SELECT nic.ComputerID, MAX(dns.Value) AS DNSSuffix FROM dbo.f_HardwareProperties('Win32_NetworkAdapterConfiguration') nic
                                  LEFT JOIN dbo.f_HardwareValuesByPropertyName('DNSDomain') dns ON nic.HardwareObjectID = dns.HardwareObjectID
                                WHERE nic.Property = 'IPEnabled' AND nic.Value = 'TRUE' AND dns.Value IS NOT NULL
                                GROUP BY nic.ComputerID) dns
                     ON c.ComputerID = dns.ComputerID
					 
  INSERT #OracleNodes
  SELECT
      ih.ComputerID               AS DBObjectID,
      CAST(NULL AS nvarchar(255)) AS ClusterDBName,
      #idbn.Value                 AS DBName,
      CAST(NULL AS BIGINT)        AS NodeInstanceID,
      #fqdn.ComputerID             AS NodeComputerID,
      CASE
        WHEN ih.bNoFQDN=0 THEN LEFT(ih.Value, CHARINDEX('.', ih.Value)-1)
        ELSE ih.Value
      END                         AS NodeComputerCN,
      ih.Value                    AS NodeHostname,
	  #i.Value                     AS NodeInstanceName
    FROM #RVSORACLE_INSTNAME_HOSTNAME ih
      LEFT JOIN #i      ON ih.ComputerID = #i.ComputerID
      LEFT JOIN #fqdn   ON ih.Value = #fqdn.FQDN OR (ih.bNoFQDN = 1 AND ih.Value = #fqdn.ComputerCN)
      LEFT JOIN #idbn   ON ih.ComputerID = #idbn.ComputerID


  INSERT #OracleNodes
  SELECT
      DBObjectID,
      DBName AS ClusterDBName,
      DBName,
      NodeInstanceID,
      NodeComputerID,
      NodeComputerCN,
      NodeHostname,
      NodeInstanceName
    FROM #rac
  
  IF @bShowTable = 1
  BEGIN
    SELECT * FROM #OracleNodes
    DROP TABLE #OracleNodes
  END

  DROP TABLE #RVSORACLE_INSTNAME_HOSTNAME  
  DROP TABLE #rac  
  DROP TABLE #i  
  DROP TABLE #idbn    
  DROP TABLE #fqdn
END


