USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_OracleInstances' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_OracleInstances]
GO

-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE PROCEDURE [dbo].[sp_OracleInstances]
	  @InstanceID INT
AS
BEGIN
 
 CREATE TABLE #OracleNodes (
	[DBObjectID] [int] NOT NULL,
	[ClusterDBName] [nvarchar](256) NULL,
	[DBName] [nvarchar](256) NULL,
	[NodeInstanceID] [bigint] NULL,
	[NodeComputerID] [int] NULL,
	[NodeComputerCN] [nvarchar](256) NULL,
	[NodeHostname] [nvarchar](256) NULL,
	[NodeInstanceName] [nvarchar](256) NULL
) 
EXEC [sp_OracleNodes]
 INSERT #OracleInstances
   SELECT
        ih.ClusterDBName    AS ClusterDBName,
        ih.NodeComputerID   AS ComputerID,
        ih.NodeComputerCN   AS ComputerCN,
        ih.NodeHostname     AS ComputerFQDN,
        ih.DBObjectID       AS InstanceID,
        ih.NodeInstanceName AS Instance,
        dbo.f_SQLDateTime(ids.Value) AS SystemDateTime,
        iv.SystemDisplayName,
        iv.SystemDisplayVersion,
        iv.AppName,
        iv.AppVersion,
        iv.AppEdition,
        ir.Value            AS DatabaseRole,
        ih.DBName           AS DatabaseName,
        idbun.Value         AS DatabaseNameUnique,
        dbo.f_SQLDateTime(idc.Value) AS DatabaseCreationDate,
        plat.Value      AS OperatingSystem

	 

      FROM #OracleNodes ih
       LEFT JOIN (SELECT * FROM dbo.f_HardwareProperties('RVSORACLE_SYSDATE') WHERE HardwareName = 'SYSDATE' AND Property = '1') ids
         ON ih.DBObjectID = ids.ComputerID
       LEFT JOIN (SELECT * FROM dbo.f_HardwareProperties('RVSORACLE_DBNAME') WHERE HardwareName = 'CreationDate' AND Property = '1') idc
         ON ih.DBObjectID = idc.ComputerID
       LEFT JOIN (SELECT * FROM dbo.f_HardwareProperties('RVSORACLE_ROLE') WHERE HardwareName = 'DATABASE_ROLE' AND Property = '1') ir
         ON ih.DBObjectID = ir.ComputerID
       LEFT JOIN (SELECT * FROM dbo.f_HardwareProperties('RVSORACLE_UNIQUENAME') WHERE HardwareName = 'DB_UNIQUE_NAME' AND Property = '1') idbun
         ON ih.DBObjectID = idbun.ComputerID
       LEFT JOIN (SELECT
                      ib.ComputerID,
                      ib.Value              AS SystemDisplayName,
                      iv.Value              AS SystemDisplayVersion,
                      mo.ApplicationName    AS AppName,
                      mo.ApplicationEdition AS AppEdition,
                      mo.ApplicationVersion AS AppVersion
                    FROM (SELECT ComputerID, Value FROM dbo.f_HardwareProperties('RVSORACLE_BANNER') WHERE HardwareName = 'BANNER' AND Property NOT IN ('Type', 'Length')) ib 
                      LEFT JOIN (SELECT ComputerID, Value FROM dbo.f_HardwareProperties('RVSORACLE_VERSION') WHERE HardwareName = 'VERSION' AND Property = '1') iv
                        ON ib.ComputerID = iv.ComputerID
                      INNER JOIN ReportingMappingOracle mo
                        ON PATINDEX(mo.DisplayName, ib.Value) > 0 AND (mo.[Version] IS NULL OR PATINDEX(mo.[Version], iv.Value) > 0)) iv
         ON ih.DBObjectID = iv.ComputerID
       LEFT JOIN (SELECT ComputerID, Value FROM dbo.f_HardwareProperties('RVSORACLE_PLATFORM') WHERE Property = '1') plat
         ON ih.DBObjectID = plat.ComputerID
     WHERE (@InstanceID IS NULL OR ih.DBObjectID = @InstanceID)

	 DROP TABLE #OracleNodes
 
 END


