USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_OracleDFUS' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_OracleDFUS]
GO
-- ==============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 18.04.2018
-- Copyright:   (c) Raynet GmbH
-- Description: Get Oracle DFUS Table
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================
-- Parameter:
--   DBObjectID    Instance bzw. DBObjectID
-- ==============================================
-- Used Functions:
--   - f_HardwareProperties
-- ==============================================



CREATE FUNCTION [dbo].[f_OracleDFUS] (@DBObjectID INT)
      RETURNS TABLE
AS
RETURN
SELECT
    n.ComputerID                 AS DBObjectID,
    n.Value                      AS FeatureName,
    v.Value                      AS Version,
    CASE
      WHEN cu.Value = 'TRUE' THEN 1
      WHEN cu.Value = '1'    THEN 1
	  ELSE 0
    END                          AS CurrentlyUsed,
    CAST(du.Value AS INT)        AS DetectedUsages,
    dbo.f_SQLDateTime(fud.Value) AS FirstUsageDate,
    dbo.f_SQLDateTime(lud.Value) AS LastUsageDate,
    dbo.f_SQLDateTime(lsd.Value) AS LastSampleDate,
    CAST(si.Value AS BIGINT)     AS SampleInterval,
    CAST(ts.Value AS INT)        AS TotalSamples,
    CASE
      WHEN fi.Value = 'null' THEN NULL
      ELSE fi.Value
    END                          AS FeatureInfo
  FROM (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'NAME' AND Property NOT IN ('TYPE', 'Length')) n
    LEFT JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'VERSION' AND Property NOT IN ('TYPE', 'Length')) v
      ON n.ComputerID = v.ComputerID
        AND n.Property = v.Property
    LEFT JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'CURRENTLY_USED' AND Property NOT IN ('TYPE', 'Length')) cu
      ON n.ComputerID = cu.ComputerID
        AND n.Property = cu.Property
    LEFT JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'DETECTED_USAGES' AND Property NOT IN ('TYPE', 'Length')) du
      ON n.ComputerID = du.ComputerID
        AND n.Property = du.Property
    LEFT JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'FIRST_USAGE_DATE' AND Property NOT IN ('TYPE', 'Length')) fud
      ON n.ComputerID = fud.ComputerID
        AND n.Property = fud.Property
    LEFT JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'LAST_USAGE_DATE' AND Property NOT IN ('TYPE', 'Length')) lud
      ON n.ComputerID = lud.ComputerID
        AND n.Property = lud.Property
    LEFT JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'LAST_SAMPLE_DATE' AND Property NOT IN ('TYPE', 'Length')) lsd
      ON n.ComputerID = lsd.ComputerID
        AND n.Property = lsd.Property
    LEFT JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'SAMPLE_INTERVAL' AND Property NOT IN ('TYPE', 'Length')) si
      ON n.ComputerID = si.ComputerID
        AND n.Property = si.Property
    LEFT JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'TOTAL_SAMPLES' AND Property NOT IN ('TYPE', 'Length')) ts
      ON n.ComputerID = ts.ComputerID
        AND n.Property = ts.Property
    LEFT JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'FEATURE_INFO' AND Property NOT IN ('TYPE', 'Length')) fi
      ON n.ComputerID = fi.ComputerID
        AND n.Property = fi.Property
  WHERE @DBObjectID IS NULL OR n.ComputerID = @DBObjectID 

GO
