USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_OracleOptions' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_OracleOptions]
GO 
-- ==============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Copyright:   (c) Raynet GmbH
-- Description: Get Oracle Options and Features
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================-- ==============================================
-- Used Functions:
--   - f_HardwareProperties
-- ==============================================


CREATE FUNCTION [dbo].[f_OracleOptions] (@InstanceID INT, @ClassName NVARCHAR(256))
      RETURNS TABLE
AS

RETURN
	SELECT
		c.ComputerID,
		c.ComputerCN,
		i.ComputerID           AS InstanceID,
		i.Value                AS Instance,
		opn.Class              AS OptionClassName,
		opn.Value              AS OptionName,
		CASE WHEN opi.Value = '1' THEN 1 ELSE 0 END AS OptionInstalled,
		CASE WHEN opu.Value = '1' THEN 1 ELSE 0 END AS OptionUsed
	  FROM (SELECT ComputerID, Class, Value FROM dbo.f_HardwareProperties('RVSORACLE_INSTNAME_HOSTNAME') 
	          WHERE HardwareName = 'HOST_NAME' AND Property = '1'
                AND (@InstanceID IS NULL OR ComputerID = @InstanceID)) ih
		LEFT JOIN (SELECT ComputerID, Class, Value FROM dbo.f_HardwareProperties('RVSORACLE_INSTNAME_HOSTNAME') WHERE HardwareName = 'INSTANCE_NAME' AND Property = '1') i
		  ON ih.ComputerID = i.ComputerID
		LEFT JOIN Computer c WITH (NOLOCK)
		  ON ih.Value LIKE c.ComputerCN + '.%'
		LEFT JOIN (SELECT ComputerID, Class, Value FROM dbo.f_HardwareProperties(@CLassName) WHERE HardwareName = 'OPTION' AND Property = '1') opn
		  ON ih.ComputerID = opn.ComputerID
		LEFT JOIN (SELECT ComputerID, Class, Value FROM dbo.f_HardwareProperties(@CLassName) WHERE HardwareName = 'USED' AND Property = '1') opu
		  ON ih.ComputerID = opu.ComputerID
			AND opn.Class = opu.Class
		LEFT JOIN (SELECT ComputerID, Class, Value FROM dbo.f_HardwareProperties(@CLassName) WHERE HardwareName = 'INSTALLED' AND Property = '1') opi
		  ON ih.ComputerID = opi.ComputerID
			AND opn.Class = opi.Class

