USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_OracleUserCounts' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_OracleUserCounts]
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 07.05.2019
-- Copyright:   (c) Raynet GmbH
-- Description:	Get Counts of Users in OracleDB
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE FUNCTION [dbo].[f_OracleUserCounts] (@IncludeSystemUser BIT)
      RETURNS TABLE
AS

RETURN
  SELECT
      ComputerID AS DBObjectID,
      SUM(AllUsers) AS AllUsers,
      SUM(Expired)  AS Expired,
      SUM(Locked)   AS Locked,
      SUM([Open])   AS [Open]
  FROM (SELECT
            uca.ComputerID,
            CAST(uca.Value AS INT) AS AllUsers,
            CAST(uce.Value AS INT) AS Expired,
            CAST(ucl.Value AS INT) AS Locked,
            CAST(uco.Value AS INT) AS [Open]
          FROM (SELECT ComputerID, Value FROM f_HardwareProperties('RVSORACLE_USERS_COUNT') WHERE HardwareName = 'ALLUSERS' AND Property = '1') uca
            LEFT JOIN (SELECT ComputerID, Value FROM f_HardwareProperties('RVSORACLE_USERS_COUNT') WHERE HardwareName = 'EXPIRED' AND Property = '1') uce
              ON uca.ComputerID = uce.ComputerID
            LEFT JOIN (SELECT ComputerID, Value FROM f_HardwareProperties('RVSORACLE_USERS_COUNT') WHERE HardwareName = 'LOCKED' AND Property = '1') ucl
              ON uca.ComputerID = ucl.ComputerID
            LEFT JOIN (SELECT ComputerID, Value FROM f_HardwareProperties('RVSORACLE_USERS_COUNT') WHERE HardwareName = 'OPEN' AND Property = '1') uco
              ON uca.ComputerID = uco.ComputerID
        UNION SELECT
            uca.ComputerID,
            CAST(uca.Value AS INT) AllUsers,
            CAST(uce.Value AS INT) Expired,
            CAST(ucl.Value AS INT) Locked,
            CAST(uco.Value AS INT) AS [Open]
          FROM (SELECT ComputerID, Value FROM f_HardwareProperties('RVSORACLE_USERS_COUNT_INTERNAL') WHERE HardwareName = 'ALLINTERNAL' AND Property = '1' AND @IncludeSystemUser = 1) uca
            LEFT JOIN (SELECT ComputerID, Value FROM f_HardwareProperties('RVSORACLE_USERS_COUNT_INTERNAL') WHERE HardwareName = 'EXPIRED' AND Property = '1') uce
              ON uca.ComputerID = uce.ComputerID
            LEFT JOIN (SELECT ComputerID, Value FROM f_HardwareProperties('RVSORACLE_USERS_COUNT_INTERNAL') WHERE HardwareName = 'LOCKED' AND Property = '1') ucl
              ON uca.ComputerID = ucl.ComputerID
            LEFT JOIN (SELECT ComputerID, Value FROM f_HardwareProperties('RVSORACLE_USERS_COUNT_INTERNAL') WHERE HardwareName = 'OPEN' AND Property = '1') uco
              ON uca.ComputerID = uco.ComputerID) tmp
    GROUP BY ComputerID
GO
