USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_OracleAccessCheck' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_OracleAccessCheck]
GO
 
-- ==============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 11.04.2018
-- Copyright:   (c) Raynet GmbH
-- Description: Check if UserRights enough for full inventory
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================
-- Used Functions:
--   - f_HardwareProperties
-- Used Tables:
--   - OracleUserRights
-- ==============================================
-- Todo:
CREATE FUNCTION [dbo].[f_OracleAccessCheck] (@All BIT)
      RETURNS TABLE
AS

RETURN
SELECT
    daT.ComputerID AS DBObjectID,
    daT.TableName,
    MAX(CASE
      WHEN aaT.ComputerID IS NULL THEN 0
      ELSE aat.ReadRight
    END) AS RightStatus
  FROM (SELECT ComputerID, Property, Value AS TableName FROM dbo.f_HardwareProperties('RVSORACLE_CHECK_USER') WHERE HardwareName = 'DBA_TABLE_NAME' AND Property NOT IN ('Type', 'Length')) daT
    LEFT JOIN (SELECT ComputerID, Property, Value AS ReadRight FROM dbo.f_HardwareProperties('RVSORACLE_CHECK_USER') WHERE HardwareName = 'READRIGHT' AND Property NOT IN ('Type', 'Length')) aaT
      ON daT.ComputerID = aaT.ComputerID
        AND daT.Property = aaT.Property
    INNER JOIN OracleUserRights our WITH (NOLOCK)
      ON daT.TableName = our.TableName
  WHERE (@All IS NULL OR @All = 1 OR aaT.ComputerID IS NULL)
  GROUP BY daT.ComputerID, daT.TableName
GO

