USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_OracleFailures' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_OracleFailures]
GO
-- ==============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 18.04.2018
-- Copyright:   (c) Raynet GmbH
-- Description: Get Oracle Inventory Failures
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================


CREATE FUNCTION [dbo].[f_OracleFailures] ()
      RETURNS TABLE
AS
RETURN
SELECT 
    co.ComputerID,
    CASE
      WHEN failCount.cntFailure IS NOT NULL THEN failCount.Instance
      ELSE oi.Instance
    END AS Instance,
    oi.InstanceID,
    CASE 
      WHEN failCount.cntFailure IS NOT NULL   THEN 1
	  WHEN nOC.OptionCount < 10               THEN 2
      WHEN newScr.DBObjectID IS NULL
        AND (oV.Major >= 1 AND ov.Minor >= 5) THEN 2
      WHEN nR.DBObjectID IS NOT NULL          THEN 2
      WHEN newScr.DBObjectID IS NULL          THEN 3
      WHEN oV.Major < 1 AND ov.Minor < 5      THEN 3
      WHEN failCount.cntFailure IS NULL       THEN 1000
      ELSE 0
    END AS StatusID,
    CASE 
      WHEN failCount.cntFailure IS NOT NULL   THEN 'Failed connection'
	  WHEN nOC.OptionCount < 10               THEN 'not enough rights'
      WHEN newScr.DBObjectID IS NULL
        AND (oV.Major >= 1 AND ov.Minor >= 5) THEN 'no rights on DBA_TABLES, DBA_OBJECTS, DBA_VIEWS'
      WHEN nR.DBObjectID IS NOT NULL          THEN 'please use new right script'
      WHEN newScr.DBObjectID IS NULL          THEN 'old oratrack.xml.enc'
      WHEN oV.Major < 1 AND ov.Minor < 5      THEN 'old oratrack.xml.enc'
      WHEN failCount.cntFailure IS NULL       THEN 'Connection ok'
      ELSE 'Missing Inventory'
    END AS [Status],
    COALESCE(oV.Version, 'unknown (old)') AS [Version]
  FROM Computer co WITH (NOLOCK)
    LEFT JOIN (SELECT ComputerID, Value AS Instance, COUNT(*) AS cntFailure 
                 FROM dbo.f_HardwareProperties('ORATRACK_ConnectionStatus') 
                 WHERE HardwareName = 'FAILURE' AND Property = 'Instance' 
                 GROUP BY ComputerID, Value) failCount 
      ON co.ComputerID = failCount.ComputerID
    LEFT JOIN dbo.f_OracleInstances(NULL) oi
      ON co.ComputerID = oi.ComputerID
        AND (failCount.Instance IS NULL OR (failCount.Instance != oi.Instance))
    LEFT JOIN (SELECT DISTINCT DBObjectID FROM dbo.f_OracleAccessCheck(1)) newScr
      ON oi.InstanceID = newScr.DBObjectID
-- Check for new rights script
    LEFT JOIN (SELECT DBObjectID FROM dbo.f_OracleAccessCheck(0) GROUP BY DBObjectID HAVING Count(*) > 0) nR
      ON oi.InstanceID = nR.DBObjectID
-- Get the Count of Oracle Options can be used < 10 not enough rights
    LEFT JOIN (SELECT ComputerID, Count(*) AS OptionCount FROM dbo.f_HardwareProperties('RVSORACLE_OPTIONS_%') WHERE Property = '1' AND HardwareName = 'INSTALLED' GROUP BY ComputerID) nOC
      ON oi.InstanceID = nOC.ComputerID
    LEFT JOIN (SELECT
                   ComputerID, 
                   Value AS Version, 
                   CAST(LEFT(Value, CHARINDEX('.', Value)-1) AS INT) AS Major, 
                   CAST(SUBSTRING(Value, CHARINDEX('.', Value) +1, LEN(Value) - CHARINDEX('.', REVERSE(Value)) -CHARINDEX('.', Value)) AS INT) AS Minor,
                   CAST(RIGHT(Value, CHARINDEX('.', REVERSE(Value))-1) AS INT) AS Patch
                 FROM dbo.f_HardwareProperties('RVSORACLE_QUERYXML_VERSION') WHERE Property = '1' AND HardwareName = 'VERSION') oV
      ON oi.InstanceID = oV.ComputerID
  WHERE oi.ComputerID IS NOT NULL OR failCount.ComputerID IS NOT NULL
UNION SELECT
    NULL,
    i.Instance,
    i.InstanceID,
	4,
	'No Hardware Inventory',
    COALESCE(oV.Version, 'unknown (old)') AS [Version]
FROM dbo.f_OracleInstances(null) i
  LEFT JOIN (SELECT
                 ComputerID, 
                 Value AS Version, 
                 CAST(LEFT(Value, CHARINDEX('.', Value)-1) AS INT) AS Major, 
                 CAST(SUBSTRING(Value, CHARINDEX('.', Value) +1, LEN(Value) - CHARINDEX('.', REVERSE(Value)) -CHARINDEX('.', Value)) AS INT) AS Minor,
                 CAST(RIGHT(Value, CHARINDEX('.', REVERSE(Value))-1) AS INT) AS Patch
               FROM dbo.f_HardwareProperties('RVSORACLE_QUERYXML_VERSION') WHERE Property = '1' AND HardwareName = 'VERSION') oV
    ON i.InstanceID = oV.ComputerID
 WHERE i.ComputerID IS NULL
GO

