USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'v_OracleReport' AND xtype = 'V'
)
        DROP VIEW [dbo].[v_OracleReport]
GO
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO
-- ==============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Copyright:   (c) Raynet GmbH
-- Description: Get Oracle Inventory Status
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================
-- Used Functions:
-- - for All
--   - f_HardwareProperties()
--   - f_OracleInstances()
--   - f_OracleUsers()
--   - f_OracleFailures()
-- ==============================================
-- Todo:
CREATE VIEW [dbo].[v_OracleReport] AS
SELECT DISTINCT
    i.ClusterDBName,
    i.ComputerID,
    i.ComputerCN,
	i.InstanceID,
    stat.Instance,
    i.SystemDateTime,
    i.SystemDisplayName,
    i.SystemDisplayVersion,
    i.AppName,
    i.AppVersion,
    i.AppEdition,
    i.DatabaseRole,
    i.DatabaseName,
    i.DatabaseNameUnique,
    i.DatabaseCreationDate,
    i.OperatingSystem,
    uc.UserCount,
    LTRIM(ISNULL(ucs_manufacturer.Value,'') + ' ' + 
          ISNULL(CASE
                   WHEN ucs_manufacturer.Value IN ('IBM', 'LENOVO')
                     THEN CASE 
                            WHEN ch_chassis.Value IN ('9', '10') /* Laptop or Notebook */ 
                              THEN LEFT(ISNULL(ISNULL(prd_version.Value, prd_name.Value), ucs_model.Value), 128)
                            ELSE LEFT(ISNULL(prd_name.Value, ucs_model.Value), 128)
                          END
                   ELSE LEFT(ucs_model.Value, 128)
                 END,'')) AS MakeModel,
    CASE
      WHEN p_cs.MAXClockSpeed > 99999999 THEN CONVERT(FLOAT, p_cs.MAXClockSpeed / 1000 /1000)
      WHEN p_cs.MAXClockSpeed > 9999 THEN CONVERT(FLOAT, p_cs.MAXClockSpeed / 1000)
      ELSE p_cs.MAXClockSpeed
    END AS MaxClockSpeed,
    ISNULL(CASE ISNUMERIC(CONVERT(varchar(512),p_nop.Value)) WHEN 1 THEN p_nop.Value ELSE 1 END, 1) AS NumberOfProcessors,
    ISNULL(CASE ISNUMERIC(CONVERT(varchar(512),p_nolp.Value)) WHEN 1 THEN p_nolp.Value ELSE NULL END, NULL) AS NumberOfLogicalProcessors,
    stat.Status,
    stat.Version AS XMLVersion
  FROM (SELECT
            stat.ComputerID,
            MAX(stat.InstanceID) AS InstanceID,
            stat.Instance,
            stat.StatusID,
            stat.Status,
            MAX(stat.Version)    AS Version
          FROM dbo.f_OracleFailures() stat
            LEFT JOIN InventoryReport ir WITH (NOLOCK)
              ON stat.InstanceID = ir.ComputerID
            LEFT JOIN (SELECT oi.ComputerID, oi.Instance, MAX(ir.HWDate) HWDate
                            FROM dbo.f_OracleInstances(null) oi
                              LEFT JOIN InventoryReport ir WITH (NOLOCK)
                                ON oi.InstanceID = ir.ComputerID
                            GROUP BY oi.ComputerID, oi.Instance) ni
              ON stat.ComputerID = ni.ComputerID
                AND stat.Instance = ni.Instance
          WHERE ir.HWDate IS NULL OR ni.HWDate IS NULL OR ir.HWDate = ni.HWDate
          GROUP BY stat.ComputerID, stat.Instance, stat.StatusID, stat.Status) stat
    LEFT JOIN dbo.f_OracleInstances(null) i
      ON stat.InstanceID = i.InstanceID
        AND stat.Instance = i.Instance		
--        AND (status.InstanceID IS NULL OR i.InstanceID = status.InstanceID)
    LEFT JOIN (SELECT InstanceID, COUNT(*) AS UserCount 
                       FROM dbo.f_OracleUsers (null) 
                       WHERE AccountStatus = 'OPEN'
                       GROUP BY InstanceID) uc
      ON i.InstanceID = uc.InstanceID
    LEFT JOIN (SELECT * FROM dbo.f_HardwareProperties('CIM_UnitaryComputerSystem') WHERE Property = 'Model') ucs_model
      ON i.ComputerID = ucs_model.ComputerID
    LEFT JOIN (SELECT * FROM dbo.f_HardwareProperties('CIM_UnitaryComputerSystem') WHERE Property = 'Manufacturer')  ucs_manufacturer
      ON i.ComputerID = ucs_manufacturer.ComputerID
    LEFT JOIN (SELECT * FROM dbo.f_HardwareProperties('Win32_ComputerSystemProduct') WHERE Property = 'Name')  prd_name
      ON i.ComputerID = prd_name.ComputerID
    LEFT JOIN (SELECT * FROM dbo.f_HardwareProperties('Win32_ComputerSystemProduct') WHERE Property = 'Version')  prd_version
      ON i.ComputerID = prd_version.ComputerID
    LEFT JOIN (SELECT * FROM dbo.f_HardwareProperties('CIM_Chassis') WHERE Property = 'ChassisTypes')  ch_chassis
      ON i.ComputerID = ch_chassis.ComputerID
    LEFT JOIN (SELECT 
                   ComputerID,
                   MAX(CONVERT(FLOAT,
                                 CASE ISNUMERIC(CONVERT(varchar(512),Value))
                                   WHEN 1 THEN CONVERT(varchar(512),Value)
                                   ELSE CASE ISNUMERIC(SUBSTRING(Value, 0, CASE CHARINDEX(' ', Value) WHEN 0 THEN 0 ELSE CHARINDEX(' ', Value) END))
                                          WHEN 1 THEN
                                            CONVERT(varchar(512), SUBSTRING(Value, 0,
                                                      CASE CHARINDEX(' ', Value)
                                                        WHEN 0 THEN CONVERT(varchar(512),'0')
                                                        ELSE CHARINDEX(' ', Value)
                                                      END)
                                                   )
                                          ELSE CONVERT(varchar(512),'0')
                                        END
                                 END)) AS MAXClockSpeed
                           FROM dbo.f_HardwareProperties('CIM_Processor')
                           WHERE Property = 'MAXClockSpeed'
                           GROUP BY ComputerID) p_cs
      ON i.ComputerID = p_cs.ComputerID
    LEFT JOIN (SELECT * FROM dbo.f_HardwareProperties('CIM_UnitaryComputerSystem') WHERE Property = 'NumberOfProcessors' AND Value IS NOT NULL)  p_nop
      ON i.ComputerID = p_nop.ComputerID
    LEFT JOIN (SELECT * FROM dbo.f_HardwareProperties('CIM_UnitaryComputerSystem') WHERE Property = 'NumberOfLogicalProcessors' AND Value IS NOT NULL)  p_nolp
      ON i.ComputerID = p_nolp.ComputerID
 
GO

