USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_OracleDFUS' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_OracleDFUS]
GO
-- ==============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 01.04.2019
-- Copyright:   (c) Raynet GmbH
-- Description: Get Raynet DFUS Table
-- ==============================================
-- Comment:
-- Created on Base f_OracleDFUS for Better Performance (40 fach)
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================-- Parameter:
--   DBObjectID    Instance bzw. DBObjectID
-- ==============================================
-- Used Functions:
--   - f_HardwareProperties
-- Used By:
--   - sp_OracleRaynetOptionList
-- ==============================================
-- Todo:
CREATE PROCEDURE [dbo].[sp_OracleDFUS] 
(
  @DBObjectID  BIGINT   = null
)

AS
BEGIN
  DECLARE @bShowTable BIT    = 1

  IF OBJECT_ID('tempdb..#sp_OracleDFUS') IS NOT NULL
  BEGIN
    TRUNCATE TABLE #sp_OracleDFUS
    SET @bShowTable = 0
  END
  ELSE
    CREATE TABLE #sp_OracleDFUS (
      DBObjectID        BIGINT                                 NOT NULL,
      RowID             NVARCHAR(256) COLLATE database_default NULL,
      FeatureName       NVARCHAR(256) COLLATE database_default NULL,
      Version           NVARCHAR(256) COLLATE database_default NULL,       -- Added 16.10.2018 for Fix with Version
      CurrentlyUsed     BIT                                    NULL,
      DetectedUsages    INT                                    NULL,
      FirstUsageDate    DATETIME                               NULL,
      LastUsageDate     DATETIME                               NULL,
      LastSampleDate    DATETIME                               NULL,
      SampleInterval    BIGINT                                 NULL,
      TotalSamples      INT                                    NULL,
      FeatureInfo       NVARCHAR(256) COLLATE database_default NULL)

  INSERT INTO #sp_OracleDFUS (DBObjectID, RowID, FeatureName)
  SELECT
      ComputerID AS DBObjectID, 
      Property   AS RowID,
      Value      AS FeatureName
    FROM dbo.f_HardwareProperties('RVSORACLE_DFUS')
    WHERE HardwareName = 'NAME' AND Property NOT IN ('TYPE', 'Length')

  UPDATE df
      SET Version = v.Value
    FROM #sp_OracleDFUS df
      INNER JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'VERSION' AND Property NOT IN ('TYPE', 'Length')) v
        ON df.DBObjectID = v.ComputerID AND df.RowID = v.Property

  UPDATE df
      SET CurrentlyUsed =  CASE
                             WHEN cu.Value = 'TRUE' THEN 1
                             WHEN cu.Value = '1'    THEN 1
                             ELSE 0
                           END
    FROM #sp_OracleDFUS df
      INNER JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'CURRENTLY_USED' AND Property NOT IN ('TYPE', 'Length')) cu
        ON df.DBObjectID = cu.ComputerID AND df.RowID = cu.Property

  UPDATE df
      SET DetectedUsages = CAST(du.Value AS INT)
    FROM #sp_OracleDFUS df
      INNER JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'DETECTED_USAGES' AND Property NOT IN ('TYPE', 'Length')) du
        ON df.DBObjectID = du.ComputerID AND df.RowID = du.Property

  UPDATE df
      SET FirstUsageDate = dbo.f_SQLDateTime(fud.Value)
    FROM #sp_OracleDFUS df
      INNER JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'FIRST_USAGE_DATE' AND Property NOT IN ('TYPE', 'Length')) fud
        ON df.DBObjectID = fud.ComputerID AND df.RowID = fud.Property

  UPDATE df
      SET LastUsageDate = dbo.f_SQLDateTime(lud.Value)
    FROM #sp_OracleDFUS df
      INNER JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'LAST_USAGE_DATE' AND Property NOT IN ('TYPE', 'Length')) lud
        ON df.DBObjectID = lud.ComputerID AND df.RowID = lud.Property

  UPDATE df
      SET LastSampleDate = dbo.f_SQLDateTime(lsd.Value)
    FROM #sp_OracleDFUS df
      INNER JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'LAST_SAMPLE_DATE' AND Property NOT IN ('TYPE', 'Length')) lsd
        ON df.DBObjectID = lsd.ComputerID AND df.RowID = lsd.Property

  UPDATE df
      SET SampleInterval = CAST(si.Value AS BIGINT)
    FROM #sp_OracleDFUS df
      INNER JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'SAMPLE_INTERVAL' AND Property NOT IN ('TYPE', 'Length')) si
        ON df.DBObjectID = si.ComputerID AND df.RowID = si.Property

  UPDATE df
      SET TotalSamples = CAST(ts.Value AS BIGINT)
    FROM #sp_OracleDFUS df
      INNER JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'TOTAL_SAMPLES' AND Property NOT IN ('TYPE', 'Length')) ts
        ON df.DBObjectID = ts.ComputerID AND df.RowID = ts.Property

  UPDATE df
      SET FeatureInfo =  CASE
                           WHEN fi.Value = 'null' THEN NULL
                           ELSE fi.Value
                         END
    FROM #sp_OracleDFUS df
      INNER JOIN (SELECT ComputerID, Property, Value FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'FEATURE_INFO' AND Property NOT IN ('TYPE', 'Length')) fi
        ON df.DBObjectID = fi.ComputerID AND df.RowID = fi.Property

  IF @bShowTable = 1
  BEGIN
    SELECT * FROM #sp_OracleDFUS
    DROP TABLE #sp_OracleDFUS
  END
END
