USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_OracleOption' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_OracleOption]
GO

-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE PROCEDURE [dbo].[sp_OracleOption] 
(
  @DBObjectID  BIGINT        = null,
  @ClassName   NVARCHAR(256) = 'RVSORACLE_OPTIONS_%'
)

AS
BEGIN
  DECLARE @bShowTable BIT = 1

  IF OBJECT_ID('tempdb..#sp_OracleOption') IS NOT NULL
  BEGIN
    TRUNCATE TABLE #sp_OracleOption
    SET @bShowTable = 0
  END
  ELSE 
    CREATE TABLE #sp_OracleOption (
      NodeComputerID    BIGINT                                 NULL,
      NodeComputerCN    NVARCHAR(256) COLLATE database_default NULL,
      DBObjectID        BIGINT                                 NOT NULL,
      NodeInstanceName  NVARCHAR(256) COLLATE database_default NULL,
      OptionClassName   NVARCHAR(256) COLLATE database_default NULL,
      OptionName        NVARCHAR(256) COLLATE database_default NULL,
      OptionInstall     BIT                                    NULL,
      OptionUsed        BIT                                    NULL)

  INSERT INTO #sp_OracleOption (DBObjectID, OptionClassName, OptionName)
  SELECT
      opn.ComputerID AS DBObjectID,
      opn.Class      AS OptionClassName,
      opn.Value      AS OptionName 
    FROM dbo.f_HardwareProperties(@ClassName) opn
    WHERE opn.HardwareName = 'OPTION'
      AND opn.Property = '1'
      AND (@DBObjectID IS NULL OR opn.ComputerID = @DBObjectID)

  CREATE TABLE #OracleNodes (
	[DBObjectID] [int]    NOT NULL,
	[ClusterDBName] [nvarchar](256)   COLLATE database_default NULL,
	[DBName] [nvarchar](256)		  COLLATE database_default NULL,
	[NodeInstanceID] [bigint] NULL,
	[NodeComputerID] [int] NULL,
	[NodeComputerCN] [nvarchar](256)    COLLATE database_default NULL,
	[NodeHostname] [nvarchar](256)      COLLATE database_default NULL,
	[NodeInstanceName] [nvarchar](256)  COLLATE database_default NULL
  ) 
  EXEC [sp_OracleNodes]

  UPDATE do
      SET NodeComputerID   = n.NodeComputerID, 
          NodeComputerCN   = n.NodeComputerID, 
          NodeInstanceName = n.NodeInstanceName
    FROM #sp_OracleOption do
      INNER JOIN #OracleNodes n
        ON do.DBObjectID = n.DBObjectID

  UPDATE do
      SET OptionUsed      = CASE WHEN opu.Value = '1' THEN 1 ELSE 0 END
    FROM #sp_OracleOption do
      INNER JOIN (SELECT * FROM dbo.f_HardwareProperties(@ClassName) WHERE HardwareName = 'Used' AND Property = '1') opu
        ON do.DBObjectID = opu.ComputerID
          AND do.OptionClassName = opu.Class

  UPDATE do
      SET OptionInstall   = CASE WHEN opi.Value = '1' THEN 1 ELSE 0 END
    FROM #sp_OracleOption do
      INNER JOIN (SELECT * FROM dbo.f_HardwareProperties(@ClassName) WHERE HardwareName = 'INSTALLED' AND Property = '1') opi
        ON do.DBObjectID = opi.ComputerID
          AND do.OptionClassName = opi.Class

  IF @bShowTable = 1
  BEGIN
    SELECT * FROM #sp_OracleOption
    DROP TABLE #sp_OracleOption
  END
  DROP TABLE #OracleNodes
END

