USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_OracleFailures' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_OracleFailures]
GO

-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE PROCEDURE [dbo].[sp_OracleFailures] (
  @ComputerCN NVARCHAR(256) = NULL)
AS
BEGIN
--  DECLARE @ComputerCN NVARCHAR(256) = NULL

  DECLARE @bShowTable BIT           = 0

  IF OBJECT_ID('tempdb..#tmpFailure_of') IS NOT NULL
    DROP TABLE #tmpFailure_of

  CREATE TABLE #tmpFailure_of(
    MessageID              INT                                    NULL,
    StatusID               INT                                    NOT NULL,
    Status                 NVARCHAR(256) COLLATE database_default NOT NULL)

  INSERT INTO #tmpFailure_of VALUES (NULL,    0, 'Missing Inventory')
  INSERT INTO #tmpFailure_of VALUES (   1,    1, 'Failed connection')
  INSERT INTO #tmpFailure_of VALUES (   2,    2, 'not enough rights')
  INSERT INTO #tmpFailure_of VALUES (   3,    2, 'no rights on DBA_TABLES, DBA_OBJECTS, DBA_VIEWS')
  INSERT INTO #tmpFailure_of VALUES (   4,    2, 'please use new right script')
  INSERT INTO #tmpFailure_of VALUES (   5,    3, 'old oratrack.xml.enc')
  INSERT INTO #tmpFailure_of VALUES (   6,    3, 'old oratrack.xml.enc')
  INSERT INTO #tmpFailure_of VALUES (   7,    4, 'No Hardware Inventory')
  INSERT INTO #tmpFailure_of VALUES (1000, 1000, 'Connection ok')

  IF OBJECT_ID('tempdb..#OracleInstances') IS NOT NULL
    DROP TABLE #OracleInstances

  CREATE TABLE #OracleInstances(
    ClusterDBName          NVARCHAR(256) COLLATE database_default NULL,
    ComputerID             INT                                    NULL,
    ComputerCN             NVARCHAR(256) COLLATE database_default NULL,
    ComputerFQDN           NVARCHAR(256) COLLATE database_default NULL,
    InstanceID             INT                                    NOT NULL,
    Instance               NVARCHAR(256) COLLATE database_default NOT NULL,
    SystemDateTime         DATETIME                               NULL,
    SystemDisplayName      NVARCHAR(256) COLLATE database_default NULL,
    SystemDisplayVersion   NVARCHAR(256) COLLATE database_default NULL,
    AppName                NVARCHAR(256) COLLATE database_default NULL,
    AppVersion             NVARCHAR(256) COLLATE database_default NULL,
    AppEdition             NVARCHAR(256) COLLATE database_default NULL,
    DatabaseRole           NVARCHAR(256) COLLATE database_default NULL,
    DatabaseName           NVARCHAR(256) COLLATE database_default NULL,
    DatabaseNameUnique     NVARCHAR(256) COLLATE database_default NULL,
    DatabaseCreationDate   DATETIME                               NULL,
    OperatingSystem        NVARCHAR(256) COLLATE database_default NULL
  )

  CREATE NONCLUSTERED INDEX IDX_C_#OracleInstances_ComputerID ON #OracleInstances(ComputerID)
  CREATE NONCLUSTERED INDEX IDX_C_#OracleInstances_InstanceID ON #OracleInstances(InstanceID)


  EXEC sp_OracleInstances NULL 
  
  DELETE FROM #OracleInstances
  WHERE @ComputerCN IS NOT NULL AND LEN(@ComputerCN) <> 0 AND ComputerCN NOT LIKE @ComputerCN


  IF OBJECT_ID('tempdb..#tmpOracleVersion_of') IS NOT NULL
    DROP TABLE #tmpOracleVersion_of

  CREATE TABLE #tmpOracleVersion_of(
    InstanceID             BIGINT                                 NOT NULL,
    [Version]              NVARCHAR(256) COLLATE database_default NULL,
	Major                  INT                                    NULL,
	Minor                  INT                                    NULL
  )

  INSERT INTO #tmpOracleVersion_of
  SELECT
      ComputerID AS InstanceID, 
      Value      AS [Version], 
      CAST(LEFT(Value, CHARINDEX('.', Value)-1) AS INT) AS Major, 
      CAST(SUBSTRING(Value, CHARINDEX('.', Value) +1, LEN(Value) - CHARINDEX('.', REVERSE(Value)) -CHARINDEX('.', Value)) AS INT) AS Minor
    FROM dbo.f_HardwareProperties('RVSORACLE_QUERYXML_VERSION') WHERE Property = '1' AND HardwareName = 'VERSION'

  IF OBJECT_ID('tempdb..#sp_OracleFailures') IS NOT NULL
  BEGIN
    TRUNCATE TABLE #sp_OracleFailures
    SET @bShowTable = 0
  END
  ELSE
    CREATE TABLE #sp_OracleFailures(
      ComputerID    INT                                    NULL,
      Instance      NVARCHAR(256) COLLATE database_default NULL,
      InstanceID    INT                                    NULL,
      StatusID      INT                                    NULL,
      [Status]      NVARCHAR(256) COLLATE database_default NULL,
      [Version]     NVARCHAR(256) COLLATE database_default NULL,
      tmp_MessageID INT                                    NULL,
      tmp_VerMajor  INT                                    NULL,
      tmp_VerMinor  INT                                    NULL)
	  
  INSERT INTO #sp_OracleFailures(ComputerID, Instance, InstanceID, [Version], tmp_MessageID, tmp_VerMajor, tmp_VerMinor)
  SELECT
      c.ComputerID,
      ISNULL(st.Instance, i.Instance)      AS Instance,
      TRY_CAST(i.InstanceID AS INT),
      v.Version,
      IIF(i.InstanceID > 0, NULL, 1)       AS tmp_MessageID,
      v.Major                              AS tmp_VerMajor,
      v.Minor                              AS tmp_VerMinor
    FROM Computer c WITH (NOLOCK)
      LEFT JOIN (SELECT DISTINCT ComputerID, Value AS Instance
                   FROM dbo.f_HardwareProperties('ORATRACK_ConnectionStatus') WHERE HardwareName = 'FAILURE' AND Property = 'Instance') st
        ON c.ComputerID = st.ComputerID
      LEFT JOIN #OracleInstances i
        ON c.ComputerID = i.ComputerID
          AND (st.Instance IS NULL OR (st.Instance != i.Instance))
      LEFT JOIN #tmpOracleVersion_of v
        ON i.InstanceID = v.InstanceID
    WHERE st.ComputerID >0 OR i.ComputerID >0

  UPDATE f
      SET StatusID   = 2
    FROM #sp_OracleFailures f
      INNER JOIN (SELECT ComputerID, Count(*) AS OptionCount FROM dbo.f_HardwareProperties('RVSORACLE_OPTIONS_%') WHERE Property = '1' AND HardwareName = 'INSTALLED' GROUP BY ComputerID) nOC
        ON f.InstanceID = nOC.ComputerID
          AND OptionCount < 10

  UPDATE f
      SET StatusID   = 3
    FROM #sp_OracleFailures f
      INNER JOIN (SELECT DBObjectID FROM dbo.f_OracleAccessCheck(0) GROUP BY DBObjectID HAVING Count(*) > 0) nR
        ON f.InstanceID = nR.DBObjectID

  UPDATE f
      SET tmp_MessageID   = 5
    FROM #sp_OracleFailures f
      LEFT JOIN (SELECT DISTINCT DBObjectID FROM dbo.f_OracleAccessCheck(1)) newScr
        ON f.InstanceID = newScr.DBObjectID
    WHERE newScr.DBObjectID IS NULL

  UPDATE f
      SET tmp_MessageID   = 4
    FROM #sp_OracleFailures f
    WHERE f.tmp_MessageID = 5 AND (f.tmp_VerMajor >=1 OR (f.tmp_VerMajor = 1 AND f.tmp_VerMinor >= 5))

  UPDATE f
      SET tmp_MessageID   = 6
    FROM #sp_OracleFailures f
    WHERE f.tmp_VerMajor <1 OR (f.tmp_VerMajor = 1 AND f.tmp_VerMinor < 5)

  UPDATE f
      SET tmp_MessageID   = 1000
    FROM #sp_OracleFailures f
    WHERE f.tmp_MessageID IS NULL AND f.InstanceID > 0

  INSERT INTO #sp_OracleFailures(ComputerID, Instance, InstanceID, [Version], tmp_MessageID)
  SELECT
      NULL,
      i.Instance,
      i.InstanceID,
      COALESCE(oV.Version, 'unknown (old)') AS [Version],
	  7
    FROM #OracleInstances i
      LEFT JOIN (SELECT
                     ComputerID, 
                     Value AS Version
                   FROM dbo.f_HardwareProperties('RVSORACLE_QUERYXML_VERSION') WHERE Property = '1' AND HardwareName = 'VERSION') oV
        ON i.InstanceID = oV.ComputerID
    WHERE i.ComputerID IS NULL

  UPDATE f
      SET StatusID   = m.StatusID,
          [Status]   = m.Status
    FROM #sp_OracleFailures f
      INNER JOIN #tmpFailure_of m
        ON f.tmp_MessageID = m.MessageID

  IF @bShowTable = 1
  BEGIN
    SELECT * from #sp_OracleFailures
    DROP TABLE #sp_OracleFailures
  END

  DROP TABLE #OracleInstances
  DROP TABLE #tmpFailure_of
END

