USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_OracleRaynetOptionList' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_OracleRaynetOptionList]
GO

-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE PROCEDURE [dbo].[sp_OracleRaynetOptionList] 
(
  @DBObjectID  BIGINT   = null
)

AS
BEGIN
--  DECLARE @DBObjectID BIGINT = null
  DECLARE @bDebug BIT = 0

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + ' - Start sp_OracleRaynetOptionList'
  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Get Oracle Instance'

  IF OBJECT_ID('tempdb..#OracleInstances') IS NOT NULL
    DROP TABLE #OracleInstances

  CREATE TABLE #OracleInstances(
    ClusterDBName          NVARCHAR(256) COLLATE database_default NULL,
    ComputerID             BIGINT                                 NULL,
    ComputerCN             NVARCHAR(256) COLLATE database_default NULL,
    ComputerFQDN           NVARCHAR(256) COLLATE database_default NULL,
    InstanceID             BIGINT                                 NOT NULL,
    Instance               NVARCHAR(256) COLLATE database_default NOT NULL,
    SystemDateTime         DATETIME                               NULL,
    SystemDisplayName      NVARCHAR(256) COLLATE database_default NULL,
    SystemDisplayVersion   NVARCHAR(256) COLLATE database_default NULL,
    AppName                NVARCHAR(256) COLLATE database_default NULL,
    AppVersion             NVARCHAR(256) COLLATE database_default NULL,
    AppEdition             NVARCHAR(256) COLLATE database_default NULL,
    DatabaseRole           NVARCHAR(256) COLLATE database_default NULL,
    DatabaseName           NVARCHAR(256) COLLATE database_default NULL,
    DatabaseNameUnique     NVARCHAR(256) COLLATE database_default NULL,
    DatabaseCreationDate   DATETIME                               NULL,
    OperatingSystem        NVARCHAR(256) COLLATE database_default NULL
  )

  CREATE NONCLUSTERED INDEX IDX_C_#OracleInstances_InstanceID ON #OracleInstances(InstanceID)

  EXEC sp_OracleInstances @DBObjectID 

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Get Oracle DFUS'
  IF OBJECT_ID('tempdb..#sp_OracleDFUS') IS NOT NULL
    DROP TABLE #sp_OracleDFUS

  CREATE TABLE #sp_OracleDFUS (
    DBObjectID        BIGINT                                 NOT NULL,
    RowID             NVARCHAR(256) COLLATE database_default NULL,
    FeatureName       NVARCHAR(256) COLLATE database_default NULL,
    Version           NVARCHAR(256) COLLATE database_default NULL, 
    CurrentlyUsed     BIT                                    NULL,
    DetectedUsages    INT                                    NULL,
    FirstUsageDate    DATETIME                               NULL,
    LastUsageDate     DATETIME                               NULL,
    LastSampleDate    DATETIME                               NULL,
    SampleInterval    BIGINT                                 NULL,
    TotalSamples      INT                                    NULL,
    FeatureInfo       NVARCHAR(256) COLLATE database_default NULL)

  CREATE NONCLUSTERED INDEX IDX_C_#sp_OracleDFUS_DBObjectID ON #sp_OracleDFUS(DBObjectID)

  EXEC sp_OracleDFUS @DBObjectID

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Get Oracle Options'
  CREATE TABLE #sp_OracleOption (
    NodeComputerID    BIGINT                                 NULL,
    NodeComputerCN    NVARCHAR(256) COLLATE database_default NULL,
    DBObjectID        BIGINT                                 NOT NULL,
    NodeInstanceName  NVARCHAR(256) COLLATE database_default NULL,
    OptionClassName   NVARCHAR(256) COLLATE database_default NULL,
    OptionName        NVARCHAR(256) COLLATE database_default NULL,
    OptionInstall     BIT                                    NULL,
    OptionUsed        BIT                                    NULL)

  CREATE NONCLUSTERED INDEX IDX_C_#sp_OracleOption_DBObjectID ON #sp_OracleOption(DBObjectID)

  EXEC sp_OracleOption @DBObjectID

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Create Tmp Oracle Options from DFUS'
  IF OBJECT_ID('tempdb..#tmpOracleOption') IS NOT NULL
    DROP TABLE #tmpOracleOption

  CREATE TABLE #tmpOracleOption (
    DBObjectID        BIGINT                                 NOT NULL,
    Product           NVARCHAR(256) COLLATE database_default NOT NULL,
    ProductType       NVARCHAR(256) COLLATE database_default NULL,
    FeatureName       NVARCHAR(256) COLLATE database_default NULL,
    Version           NVARCHAR(256) COLLATE database_default NULL, 
    FirstUsageDate    DATETIME                               NULL,
    LastUsageDate     DATETIME                               NULL,
    Used              BIT                                    NOT NULL,
    Active            BIT                                    NULL,
    NeedLicense       BIT                                    NULL,
    DependFeatureName NVARCHAR(256) COLLATE database_default NULL,
    Comment           NVARCHAR(256) COLLATE database_default NULL)

  CREATE NONCLUSTERED INDEX IDX_C_#tmpOracleOption ON #tmpOracleOption(DBObjectID)
	
-- Fix 16.10.2018 DFUS (FeatureInfoFilter and Product Query to Null and '')
  INSERT INTO #tmpOracleOption
  SELECT
      dfus.DBObjectID,
      om.Product,
      om.ProductType,
      dfus.FeatureName,
      dfus.Version,                                -- Added 16.10.2018 for Fix with Version
      dfus.FirstUsageDate,
      dfus.LastUsageDate,
      CASE
        WHEN dfus.LastUsageDate IS NOT NULL THEN 1
        ELSE dfus.CurrentlyUsed
      END                    AS Used,
      om.Active,
      om.NeedLicense,
      om.DependFeatureName,
      om.Comment
    FROM #sp_OracleDFUS dfus
      INNER JOIN #OracleInstances i
        ON dfus.DBObjectID = i.InstanceID
--          AND dfus.Version = i.SystemDisplayVersion    -- Filter on DB Version, this is wrong
      INNER JOIN dbo.OracleMetric om WITH (NOLOCK)
        ON dfus.FeatureName = om.FeatureName
          AND (Ignore = 0 OR Ignore IS NULL)
          AND (dfus.FeatureInfo LIKE om.FeatureInfoFilter
            OR om.FeatureInfoFilter = '' OR om.FeatureInfoFilter IS NULL)
          AND dbo.f_CheckVersion(dfus.Version, om.VersionFrom, om.VersionTo) = 1
    WHERE om.Product != '' AND om.Product IS NOT NULL

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Create Tmp Oracle Options from Option'
  INSERT INTO #tmpOracleOption
  SELECT
      o.DBObjectID,
      om.Product,
      om.ProductType,
	  o.OptionName + ' (' + o.OptionClassName + ')' AS FeatureName,
      i.SystemDisplayVersion                        AS Version,           -- Added 16.10.2018 for Fix with Version
      NULL                                          AS FirstUsageDate,
      NULL                                          AS LastUsageDate,
      o.OptionUsed                                  AS Used,
      om.Active,
      om.NeedLicense,
      om.DependFeatureName,
      NULL                                          AS Comment
    FROM #sp_OracleOption o
      INNER JOIN dbo.OracleMetric om WITH (NOLOCK)
        ON o.OptionClassName = om.FeatureName
          AND om.Ignore = 0
      LEFT JOIN #OracleInstances i
        ON o.DBObjectID = i.InstanceID

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Create Tmp Oracle Options Dependencies'
  INSERT INTO #tmpOracleOption
  SELECT DISTINCT
      dfus.DBObjectID,
      om.Product,
      om.ProductType,
      CASE
        WHEN dfus.DependFeatureName = om.FeatureName THEN om.FeatureName
        ELSE NULL
      END AS FeatureName,
      dfus.Version,                           -- Added 16.10.2018 for Fix with Version
      dfus.FirstUsageDate,
      dfus.LastUsageDate,
      dfus.Used,
      CASE
        WHEN dfus.DependFeatureName = om.FeatureName THEN om.Active
        ELSE 1
      END AS Active,
      om.NeedLicense,
      om.DependFeatureName,
      'Depend from ' + dfus.Product
    FROM #tmpOracleOption dfus
      INNER JOIN dbo.OracleMetric om WITH (NOLOCK)
        ON dfus.Used =1 AND om.Ignore = 0
          AND (dfus.DependFeatureName = om.Product
            OR dfus.DependFeatureName = om.FeatureName)
          AND (om.NeedLicense = 1 OR om.NeedLicense IS NULL)

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Create Tmp Oracle Options Dependencies 2nd time'
  INSERT INTO #tmpOracleOption
  SELECT DISTINCT
      dfus.DBObjectID,
      om.Product,
      om.ProductType,
      CASE
        WHEN dfus.DependFeatureName = om.FeatureName THEN om.FeatureName
        ELSE NULL
      END AS FeatureName,
      dfus.Version,                           -- Added 16.10.2018 for Fix with Version
      dfus.FirstUsageDate,
      dfus.LastUsageDate,
      dfus.Used,
      CASE
        WHEN dfus.DependFeatureName = om.FeatureName THEN om.Active
        ELSE 1
      END AS Active,
      om.NeedLicense,
      om.DependFeatureName,
      'Depend from ' + dfus.Product
    FROM #tmpOracleOption dfus
      INNER JOIN dbo.OracleMetric om WITH (NOLOCK)
        ON dfus.Used =1 AND om.Ignore = 0
          AND dfus.Comment LIKE 'Depend from%'
          AND (dfus.DependFeatureName = om.Product
            OR dfus.DependFeatureName = om.FeatureName)
          AND (om.NeedLicense = 1 OR om.NeedLicense IS NULL)

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Fix Versions'
-- Fix 16.10.2018 Add Product Version of DFUS if not the same as System
  UPDATE o
      SET Product = o.Product + ' ' + o.Version
    FROM #tmpOracleOption o
      LEFT JOIN #OracleInstances i
        ON o.DBObjectID = i.InstanceID
          AND o.Version = i.SystemDisplayVersion
    WHERE i.InstanceID IS NULL AND o.Version IS NOT NULL

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Return Option List'
  IF OBJECT_ID('tempdb..#sp_OracleRaynetOptionList') IS NOT NULL
    INSERT INTO #sp_OracleRaynetOptionList
    SELECT
      o.DBObjectID,
	  o.Product,
      o.ProductType,
      o.FeatureName,
      o.FirstUsageDate,
      o.LastUsageDate,
      o.Used,
      o.Active,
      o.NeedLicense,
      o.Comment
    FROM #tmpOracleOption o    
  ELSE
    SELECT
      o.DBObjectID,
      o.Product,
      o.ProductType,
      o.FeatureName,
      o.FirstUsageDate,
      o.LastUsageDate,
      o.Used,
      o.Active,
      o.NeedLicense,
      o.Comment
    FROM #tmpOracleOption o    

  DROP TABLE #OracleInstances
  DROP TABLE #sp_OracleDFUS
  DROP TABLE #sp_OracleOption
  DROP TABLE #tmpOracleOption

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '- Finish sp_OracleRaynetOptionList'
END

