USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_OracleScriptOptionList' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_OracleScriptOptionList]
GO

-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE PROCEDURE [dbo].[sp_OracleScriptOptionList] 
(
  @DBObjectID  BIGINT   = null
)

AS
BEGIN

  IF OBJECT_ID('tempdb..#tmpOracleOption') IS NOT NULL
    DROP TABLE #tmpOracleOption

  CREATE TABLE #tmpOracleOption (
    DBObjectID        BIGINT                                 NOT NULL,
    ContainerName     NVARCHAR(256) COLLATE database_default NULL,
    Product           NVARCHAR(256) COLLATE database_default NULL,
    FeatureName       NVARCHAR(256) COLLATE database_default NULL,
    Version           NVARCHAR(256) COLLATE database_default NULL,
    Usage             NVARCHAR(32)  COLLATE database_default NULL,
    CurrentlyUsed     BIT                                    NULL,
    DetectedUsages    INTEGER                                NULL,
    FirstUsageDate    DATETIME                               NULL,
    LastUsageDate     DATETIME                               NULL,
    LastSampleDate    DATETIME                               NULL,
    TotalSamples      INTEGER                                NULL,
    FeatureInfo       NVARCHAR(max) COLLATE database_default NULL,
    Used              BIT                                    NULL,
    NeedLicense       BIT                                    NULL,
    DependFeatureName NVARCHAR(256) COLLATE database_default NULL,
    Comment           NVARCHAR(256) COLLATE database_default NULL)

  INSERT INTO #tmpOracleOption
  SELECT
      dP.ComputerID                 AS DBObjectID,
      dC.Value                      AS ContainerName,
      dP.Value                      AS Product,
      dF.Value                      AS FeatureName,
      dV.Value                      AS Version,
      dU.Value                      AS Usage,
      CASE
        WHEN dCU.Value = 'TRUE' THEN 1
        WHEN dCU.Value = '1'    THEN 1
        ELSE 0
      END                           AS CurrentlyUsed,
      CAST(dDU.Value AS INT)        AS DetectedUsages,
      dbo.f_SQLDateTime(dFUD.Value) AS FirstUsageDate,
      dbo.f_SQLDateTime(dLUD.Value) AS LastUsageDate,
      dbo.f_SQLDateTime(dL.Value)   AS LastSampleDate,
      CAST(dTS.Value AS BIGINT)     AS TotalSamples,
      CASE
        WHEN dEFI.Value = ''     THEN NULL
        WHEN dEFI.Value = 'null' THEN NULL
        ELSE dEFI.Value
      END                          AS FeatureInfo,
      CASE
        WHEN CAST(dDU.Value AS INT) > 0 THEN 1
        WHEN dCU.Value IN ('TRUE', '1') THEN 1
        ELSE 0
      END                          AS Used,
      CASE
-- 20180904 fix NO_CURRENT_USAGE
        WHEN dU.Value = 'NO_CURRENT_USAGE'                                      THEN 0
        WHEN dU.Value != 'SUPPRESSED_DUE_TO_BUG' AND CAST(dDU.Value AS INT) > 0 THEN 1
        WHEN dU.Value != 'SUPPRESSED_DUE_TO_BUG' AND dCU.Value IN ('TRUE', '1') THEN 1
        ELSE 0
      END                          AS NeedLicense,
      NULL                         AS DependFeatureName,
      dU.Value                     AS Comment
    FROM dbo.f_HardwareProperties('RMS_ORACLEDFUS1') dP
  -- Get Last Table Base Number
      INNER JOIN (SELECT ComputerID, MAX(TableNumber) AS TableNumber
                    FROM (SELECT DISTINCT ComputerID, LEFT(HardwareName, CHARINDEX(':', HardwareName) - 1) AS TableNumber 
                            FROM dbo.f_HardwareProperties('RMS_ORACLEDFUS1') 
                            WHERE HardwareName LIKE '%:%') x
                    GROUP BY ComputerID) lT
        ON dP.ComputerID = lT.ComputerID
          AND dp.HardwareName LIKE lt.TableNumber + ':[0123456789]%'
          AND (@DBObjectID IS NULL OR dp.ComputerID = @DBObjectID)
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('CON_NAME') dC
        ON dP.HardwareObjectID = dC.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('FEATURE_BEING_USED') dF
        ON dP.HardwareObjectID = dF.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('USAGE') dU
        ON dP.HardwareObjectID = dU.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('LAST_SAMPLE_DATE') dL
        ON dP.HardwareObjectID = dL.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('VERSION') dV
        ON dP.HardwareObjectID = dV.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('DETECTED_USAGES') dDU
        ON dP.HardwareObjectID = dDU.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('TOTAL_SAMPLES') dTS
        ON dP.HardwareObjectID = dTS.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('CURRENTLY_USED') dCU
        ON dP.HardwareObjectID = dCU.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('FIRST_USAGE_DATE') dFUD
        ON dP.HardwareObjectID = dFUD.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('LAST_USAGE_DATE') dLUD
        ON dP.HardwareObjectID = dLUD.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('EXTRA_FEATURE_INFO') dEFI
        ON dP.HardwareObjectID = dEFI.HardwareObjectID
    WHERE Property = 'PRODUCT'

  UPDATE o
      SET DependFeatureName = om.DependFeatureName
    FROM #tmpOracleOption o
      INNER JOIN dbo.OracleMetric om
        ON o.FeatureName = om.FeatureName
          AND o.Product = om.Product
		  AND om.DependFeatureName IS NOT NULL

  INSERT INTO #tmpOracleOption (DBObjectID, Product, FeatureName, Version, FirstUsageDate, LastUsageDate, Used, NeedLicense, DependFeatureName, Comment)
  SELECT DISTINCT
      o.DBObjectID,
      om.Product,
      o.Version,
      o.FeatureName,
      o.FirstUsageDate,
      o.LastUsageDate,
      o.Used,
      om.NeedLicense,
      om.DependFeatureName,
      'Depend from ' + o.Product AS Comment
    FROM #tmpOracleOption o
      INNER JOIN dbo.OracleMetric om
        ON o.Used =1 AND om.Ignore = 0
          AND (o.DependFeatureName = om.Product
            OR o.DependFeatureName = om.FeatureName)
          AND (om.NeedLicense = 1 OR om.NeedLicense IS NULL)

  INSERT INTO #tmpOracleOption (DBObjectID, Product, FeatureName, Version, FirstUsageDate, LastUsageDate, Used, NeedLicense, DependFeatureName, Comment)
  SELECT DISTINCT
      o.DBObjectID,
      om.Product,
      o.FeatureName,
      o.Version,
      o.FirstUsageDate,
      o.LastUsageDate,
      o.Used,
      om.NeedLicense,
      om.DependFeatureName,
      'Depend from ' + o.Product AS Comment
    FROM #tmpOracleOption o
      INNER JOIN dbo.OracleMetric om
        ON o.Used =1 AND om.Ignore = 0
          AND o.Comment LIKE 'Depend from%'
          AND (o.DependFeatureName = om.Product
            OR o.DependFeatureName = om.FeatureName)
          AND (om.NeedLicense = 1 OR om.NeedLicense IS NULL)

-- Fix 16.10.2018 Add Product Version of DFUS if not teh same as System
  
  
  
CREATE TABLE #OracleInstances(
	[ClusterDBName] [nvarchar](256) COLLATE database_default NULL,
	[ComputerID] [int] NULL,
	[ComputerCN] [nvarchar](256) COLLATE database_default NULL,
	[ComputerFQDN] [nvarchar](256) COLLATE database_default NULL,
	[InstanceID] [int] NOT NULL,
	[Instance] [nvarchar](256) COLLATE database_default NULL,
	[SystemDateTime] [datetime] NULL,
	[SystemDisplayName] [nvarchar](256) COLLATE database_default NULL,
	[SystemDisplayVersion] [nvarchar](256) COLLATE database_default NULL,
	[AppName] [nvarchar](255) NULL,
	[AppVersion] [nvarchar](255) NULL,
	[AppEdition] [nvarchar](255) NULL,
	[DatabaseRole] [nvarchar](256) COLLATE database_default NULL,
	[DatabaseName] [nvarchar](256) COLLATE database_default NULL,
	[DatabaseNameUnique] [nvarchar](256) COLLATE database_default NULL,
	[DatabaseCreationDate] [datetime] NULL,
	[OperatingSystem] [nvarchar](256) COLLATE database_default NULL
) 
EXEC sp_OracleInstances @DBObjectID
 
  
  UPDATE o
      SET Product = Product + ' ' + Version
    FROM #tmpOracleOption o
      LEFT JOIN #OracleInstances i
        ON o.DBObjectID = i.InstanceID
          AND o.Version = i.SystemDisplayVersion
    WHERE i.InstanceID IS NULL

  IF OBJECT_ID('tempdb..#sp_OracleScriptOptionList') IS NOT NULL
    INSERT INTO #sp_OracleScriptOptionList
    SELECT
        o.DBObjectID,
        o.ContainerName,
        o.Product,
        o.FeatureName,
        o.Version,
        o.Usage,
        o.CurrentlyUsed,
        o.DetectedUsages,
        o.FirstUsageDate,
        o.LastUsageDate,
        o.LastSampleDate,
        o.TotalSamples,
        o.FeatureInfo,
        o.Used,
        o.NeedLicense,
        o.Comment
      FROM #tmpOracleOption o
  ELSE
    SELECT
        o.DBObjectID,
        o.ContainerName,
        o.Product,
        o.FeatureName,
        o.Version,
        o.Usage,
        o.CurrentlyUsed,
        o.DetectedUsages,
        o.FirstUsageDate,
        o.LastUsageDate,
        o.LastSampleDate,
        o.TotalSamples,
        o.FeatureInfo,
        o.Used,
        o.NeedLicense,
        o.Comment
      FROM #tmpOracleOption o
  
  DROP TABLE #tmpOracleOption
  DROP TABLE #OracleInstances
END

