USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_rp_OracleOptionList' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_rp_OracleOptionList]
GO
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE PROCEDURE [dbo].[sp_rp_OracleOptionList] 
(
  @DBObjectID  BIGINT   = NULL,
  @ComputerID  BIGINT   = NULL,
  @OnlyProduct BIT      = NULL
)

AS
BEGIN
--  DECLARE @DBObjectID  BIGINT   = 570
--  DECLARE @ComputerID  BIGINT   = 67
--  DECLARE @OnlyProduct BIT      = 0


  DECLARE @DefaultMetric      NVARCHAR(50) = (SELECT Value FROM ReportingConfiguration WHERE Property = 'OracleLicenseMetric')

  DECLARE @bShowTable         BIT          = 0
  DECLARE @bDebug             BIT          = 0

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + ' - Start sp_rp_OracleOptionList'

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Get List of Oracle Instance'
  IF OBJECT_ID('tempdb..#tmpInstanceTable_ool') IS NOT NULL
    DROP TABLE #tmpInstanceTable_ool

  CREATE TABLE #tmpInstanceTable_ool(
    DBObjectID             BIGINT                                 NOT NULL)

  
  CREATE TABLE #OracleInstances(
	[ClusterDBName] [nvarchar](256) COLLATE database_default NULL,
	[ComputerID] [int] NULL,
	[ComputerCN] [nvarchar](256) COLLATE database_default NULL,
	[ComputerFQDN] [nvarchar](256) COLLATE database_default NULL,
	[InstanceID] [int] NOT NULL,
	[Instance] [nvarchar](256) COLLATE database_default NULL,
	[SystemDateTime] [datetime] NULL,
	[SystemDisplayName] [nvarchar](256) COLLATE database_default NULL,
	[SystemDisplayVersion] [nvarchar](256) COLLATE database_default NULL,
	[AppName] [nvarchar](255) NULL,
	[AppVersion] [nvarchar](255) NULL,
	[AppEdition] [nvarchar](255) NULL,
	[DatabaseRole] [nvarchar](256) COLLATE database_default NULL,
	[DatabaseName] [nvarchar](256) COLLATE database_default NULL,
	[DatabaseNameUnique] [nvarchar](256) COLLATE database_default NULL,
	[DatabaseCreationDate] [datetime] NULL,
	[OperatingSystem] [nvarchar](256) COLLATE database_default NULL
) 
EXEC sp_OracleInstances  @DBObjectID
  
  
  INSERT INTO #tmpInstanceTable_ool
  SELECT DISTINCT
      InstanceID AS DBObjectID
    FROM #OracleInstances i
    WHERE @ComputerID IS NULL OR ComputerID = @ComputerID
	
  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Get List of Feature_Info for Raynet Inventory'

  IF OBJECT_ID('tempdb..#tmpNewRaynetInventory') IS NOT NULL
    DROP TABLE #tmpNewRaynetInventory

  CREATE TABLE #tmpNewRaynetInventory(
    DBObjectID BIGINT NOT NULL)

  INSERT INTO #tmpNewRaynetInventory
  SELECT DISTINCT ComputerID AS DBObjectID
    FROM dbo.f_HardwareProperties('RVSORACLE_DFUS') WHERE HardwareName = 'FEATURE_INFO' AND Property IS NOT NULL

  IF OBJECT_ID('tempdb..#sp_OracleRaynetOptionList') IS NOT NULL
    DROP TABLE #sp_OracleRaynetOptionList

  IF OBJECT_ID('tempdb..#sp_OracleScriptOptionList') IS NOT NULL
    DROP TABLE #sp_OracleScriptOptionList

  IF OBJECT_ID('tempdb..#tmpRayOracleOptionOld') IS NOT NULL
    DROP TABLE #tmpRayOracleOptionOld

  IF OBJECT_ID('tempdb..#tmpScrOracleOption') IS NOT NULL
    DROP TABLE #tmpScrOracleOption

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Get Oracle Raynet Option Old List'
  CREATE TABLE #tmpRayOracleOptionOld (
    DBObjectID        BIGINT                                 NOT NULL,
    Product           NVARCHAR(256) COLLATE database_default NOT NULL,
    ProductType       NVARCHAR(256) COLLATE database_default NULL,
    FeatureName       NVARCHAR(256) COLLATE database_default NULL,
    Used              BIT                                    NOT NULL)

  CREATE NONCLUSTERED INDEX IDX_C_#tmpRayOracleOptionOld_DBObjectID ON #tmpRayOracleOptionOld(DBObjectID,Product,FeatureName)

  
    CREATE TABLE #sp_OracleOption (
      NodeComputerID    BIGINT                                 NULL,
      NodeComputerCN    NVARCHAR(256) COLLATE database_default NULL,
      DBObjectID        BIGINT                                 NOT NULL,
      NodeInstanceName  NVARCHAR(256) COLLATE database_default NULL,
      OptionClassName   NVARCHAR(256) COLLATE database_default NULL,
      OptionName        NVARCHAR(256) COLLATE database_default NULL,
      OptionInstall     BIT                                    NULL,
      OptionUsed        BIT                                    NULL)
	  EXEC sp_OracleOption @DBObjectID, '%'


  INSERT INTO #tmpRayOracleOptionOld
  SELECT
      o.DBObjectID                AS DBObjectID,
      o.OptionName                AS Product,
      CAST (NULL AS NVARCHAR(32)) AS ProductType,
      o.OptionClassName           AS FeatureName,
      o.OptionUsed                AS Used
    FROM #sp_OracleOption o
    WHERE o.DBObjectID NOT IN (SELECT DBObjectID FROM #tmpNewRaynetInventory)

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Get Oracle Raynet Option List'
  CREATE TABLE #sp_OracleRaynetOptionList (
    DBObjectID        BIGINT                                 NOT NULL,
    Product           NVARCHAR(256) COLLATE database_default NOT NULL,
    ProductType       NVARCHAR(256) COLLATE database_default NULL,
    FeatureName       NVARCHAR(256) COLLATE database_default NULL,
    FirstUsageDate    DATETIME                               NULL,
    LastUsageDate     DATETIME                               NULL,
    Used              BIT                                    NOT NULL,
    Active            BIT                                    NULL,
    NeedLicense       BIT                                    NULL,
    Comment           NVARCHAR(256) COLLATE database_default NULL)

  CREATE NONCLUSTERED INDEX IDX_C_#sp_OracleRaynetOptionList_DBObjectID ON #sp_OracleRaynetOptionList(DBObjectID,Product,FeatureName)

--  INSERT INTO #tmpRayOracleOption
  EXEC sp_OracleRaynetOptionList @DBObjectID

  DELETE FROM #sp_OracleRaynetOptionList WHERE DBObjectID IN (SELECT DBObjectID FROM #tmpRayOracleOptionOld)
  
  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Get Oracle Script Option List'
  CREATE TABLE #sp_OracleScriptOptionList (
    DBObjectID        BIGINT                                 NOT NULL,
    ContainerName     NVARCHAR(256) COLLATE database_default NULL,
    Product           NVARCHAR(256) COLLATE database_default NULL,
    FeatureName       NVARCHAR(256) COLLATE database_default NULL,
    Version           NVARCHAR(256) COLLATE database_default NULL,
    Usage             NVARCHAR(32)  COLLATE database_default NULL,
    CurrentlyUsed     BIT                                    NULL,
    DetectedUsages    INTEGER                                NULL,
    FirstUsageDate    DATETIME                               NULL,
    LastUsageDate     DATETIME                               NULL,
    LastSampleDate    DATETIME                               NULL,
    TotalSamples      INTEGER                                NULL,
    FeatureInfo       NVARCHAR(max) COLLATE database_default NULL,
    Used              BIT                                    NOT NULL,
    NeedLicense       BIT                                    NULL,
    Comment           NVARCHAR(256) COLLATE database_default NULL
  )

  CREATE NONCLUSTERED INDEX IDX_C_#sp_OracleScriptOptionList_DBObjectID ON #sp_OracleScriptOptionList(DBObjectID,Product,FeatureName)
  
--  INSERT INTO #tmpScrOracleOption
  EXEC sp_OracleScriptOptionList @DBObjectID

  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Get List of Oracle Inventory'
  IF OBJECT_ID('tempdb..#tmpOracleInventory_ool') IS NOT NULL
    DROP TABLE #tmpOracleInventory_ool

  CREATE TABLE #tmpOracleInventory_ool(
    DBObjectID             BIGINT                                 NOT NULL)

  INSERT INTO #tmpOracleInventory_ool
  SELECT DISTINCT DBObjectID
    FROM #sp_OracleScriptOptionList

  IF OBJECT_ID('tempdb..#sp_rp_OracleOptionList') IS NOT NULL
  BEGIN
    TRUNCATE TABLE #sp_rp_OracleOptionList
    SET @bShowTable = 0
  END
  ELSE
  BEGIN
    CREATE TABLE #sp_rp_OracleOptionList (
      DBObjectID       BIGINT        NULL,
      Product          NVARCHAR(256) NULL,
      FeatureName      NVARCHAR(256) NULL,
      Installed        INT           NULL,
      RaynetUsed       INT           NULL,
      OracleUsed       INT           NULL,
      TechnicalLicense INT           NULL,
      RaynetComment    NVARCHAR(256) NULL,
      OracleComment    NVARCHAR(256) NULL,
      CustomLicense    INT           NULL,
      CustomComment    NVARCHAR(256) NULL,
      LicenseResult    INT           NULL)
    SET @bShowTable = 1
  END
  
  IF @OnlyProduct = 1
  BEGIN
    IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Add Only Product List to #sp_rp_OracleOptionList Table'
    INSERT INTO #sp_rp_OracleOptionList (DBObjectID, Product, FeatureName, Installed, RaynetUsed, OracleUsed, 
                                         TechnicalLicense, RaynetComment, OracleComment, CustomLicense, CustomComment, LicenseResult)
    SELECT DISTINCT
        DBObjectID,
        Product,
        CAST(NULL AS nvarchar(256))     AS FeatureName,
        CAST (1 AS INT)                 AS Installed,
        MAX(CAST(RaynetUsed AS INT))    AS RaynetUsed,
        MAX(CAST(OracleUsed AS INT))    AS OracleUsed,
        MAX(TechnicalLicense)           AS TechnicalLicense,
        MAX(RaynetComment)              AS RaynetComment,
        MAX(OracleComment)              AS OracleComment,
        MAX(CAST(CustomLicense AS INT)) AS CustomLicense,
        MAX(CustomComment)              AS CustomComment,
        MAX(CAST(LicenseResult AS INT)) AS LicenseResult
      FROM(SELECT
               COALESCE(tOO.DBObjectID, oOS.DBObjectID, old.DBObjectID)   AS DBObjectID,
               COALESCE(tOO.Product, oOS.Product, old.Product)            AS Product,
               CAST(CASE
                      WHEN tOO.Used = 1 THEN 1
                      WHEN old.Used = 1 THEN 1
                      ELSE 0
                    END AS BIT)                           AS RaynetUsed,
               oOS.NeedLicense                            AS OracleUsed,
               CASE
                 WHEN tOO.Used = 1 AND (oOS.NeedLicense = 1 OR ot.DBObjectID IS NULL) THEN 2
                 WHEN tOO.Used = 1 OR  oOS.NeedLicense = 1                            THEN 1
                 WHEN old.Used = 1                                                    THEN 2
                 ELSE 0
               END                                        AS TechnicalLicense,
               LEFT(tOO.Comment, 20)                      AS RaynetComment,
               oOS.Usage                                  AS OracleComment,
               CAST(NULL AS BIT)                          AS CustomLicense,
               CAST(NULL AS NVARCHAR(256))                AS CustomComment,
               CAST(CASE
                       WHEN (@DefaultMetric IS NULL OR ot.DBObjectID IS NULL) 
                         AND (too.Used = 1 OR oOS.NeedLicense = 1)                           THEN 1
                       WHEN @DefaultMetric = 'Oracle'                                        THEN ISNULL(oOS.NeedLicense, 0)
                       WHEN @DefaultMetric = 'Raynet' AND too.Used = 1                       THEN 1
                       WHEN (SELECT TOP 1 ComputerID
                               FROM f_HardwareProperties('RMS_ORACLEDFUS1') 
                               WHERE ComputerID = tOO.DBObjectID) IS NULL AND too.Used = 1   THEN 1
                       WHEN old.Used = 1                                                     THEN 1
                       ELSE 0
                     END AS BIT)                          AS  LicenseResult
             FROM #sp_OracleRaynetOptionList tOO
               FULL OUTER JOIN #sp_OracleScriptOptionList oOS
                 ON tOO.DBObjectID = oOS.DBObjectID
                   AND tOO.Product = oOS.Product
                   AND tOO.FeatureName = oOS.FeatureName
               FULL OUTER JOIN #tmpRayOracleOptionOld old
                 ON tOO.DBObjectID = old.DBObjectID
                   AND tOO.Product = old.Product
                   AND tOO.FeatureName = old.FeatureName
               LEFT JOIN #tmpOracleInventory_ool ot
                 ON COALESCE(tOO.DBObjectID, oOS.DBObjectID) = ot.DBObjectID
-- 20180904 fix tOO.Active
             WHERE (tOO.DBObjectID IS NOT NULL AND ((tOO.Used = 1 AND tOO.Active = 1) OR (tOO.Used = 1 AND tOO.Active IS NULL)))
               OR (oOS.DBObjectID IS NOT NULL)
               OR (old.DBObjectID IS NOT NULL)) x
      WHERE DBOBjectID IN (SELECT DBObjectID FROM #tmpInstanceTable_ool)
        AND TechnicalLicense IN (1,2)
      GROUP BY DBObjectID, Product
  END
  ELSE
  BEGIN
    IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Add All Product List to #sp_rp_OracleOptionList Table'
    INSERT INTO #sp_rp_OracleOptionList (DBObjectID, Product, FeatureName, Installed, RaynetUsed, OracleUsed, 
                                         TechnicalLicense, RaynetComment, OracleComment, CustomLicense, CustomComment, LicenseResult)
    SELECT DISTINCT
        DBObjectID, Product, FeatureName, Installed, RaynetUsed, OracleUsed, TechnicalLicense, RaynetComment, 
        OracleComment, CustomLicense, CustomComment, LicenseResult
      FROM(SELECT
               COALESCE(tOO.DBObjectID, oOS.DBObjectID, old.DBObjectID)    AS DBObjectID,
               COALESCE(tOO.Product, oOS.Product, old.Product)             AS Product,
               COALESCE(tOO.FeatureName, oOS.FeatureName, old.FeatureName) AS FeatureName,
               CAST (1 AS BIT)                                             AS Installed,
               CAST(CASE
                      WHEN tOO.Used = 1 THEN 1
                      WHEN old.Used = 1 THEN 1
                      ELSE 0
                    END AS BIT)                                            AS RaynetUsed,
               oOS.NeedLicense                                             AS OracleUsed,
               CASE
                 WHEN tOO.Used = 1 AND (oOS.NeedLicense = 1 OR ot.DBObjectID IS NULL) THEN 2
                 WHEN tOO.Used = 1 OR  oOS.NeedLicense = 1 THEN 1
                 WHEN old.Used = 1                                                    THEN 2
                 ELSE 0
               END                                                         AS TechnicalLicense,
               LEFT(tOO.Comment, 20)                                       AS RaynetComment,
               oOS.Usage                                                   AS OracleComment,
               CAST(NULL AS BIT)                                           AS CustomLicense,
               CAST(NULL AS NVARCHAR(256))                                 AS CustomComment,
               too.Used, oOS.NeedLicense,
               CAST(CASE
                       WHEN (@DefaultMetric IS NULL OR ot.DBObjectID IS NULL) 
                         AND (too.Used = 1 OR oOS.NeedLicense = 1)                           THEN 1
                       WHEN @DefaultMetric = 'Oracle'                                        THEN oOS.NeedLicense
                       WHEN @DefaultMetric = 'Raynet' AND too.Used = 1                       THEN 1
                       WHEN (SELECT TOP 1 ComputerID
                               FROM f_HardwareProperties('RMS_ORACLEDFUS1') 
                               WHERE ComputerID = tOO.DBObjectID) IS NULL AND too.Used = 1   THEN 1
                       WHEN old.Used = 1                                                     THEN 1
                       ELSE 0
                     END AS BIT)                                           AS  LicenseResult
             FROM #sp_OracleRaynetOptionList tOO
               FULL OUTER JOIN #sp_OracleScriptOptionList oOS ON tOO.DBObjectID = oOS.DBObjectID AND tOO.Product = oOS.Product AND tOO.FeatureName = oOS.FeatureName
               FULL OUTER JOIN #tmpRayOracleOptionOld old ON tOO.DBObjectID = old.DBObjectID  AND tOO.Product = old.Product AND tOO.FeatureName = old.FeatureName
               LEFT JOIN #tmpOracleInventory_ool ot  ON COALESCE(tOO.DBObjectID, oOS.DBObjectID) = ot.DBObjectID
-- 20180904 fix tOO.Active
             WHERE (tOO.DBObjectID IS NOT NULL AND ((tOO.Used = 1 AND tOO.Active = 1) OR (tOO.Used = 1 AND tOO.Active IS NULL)))
               OR (oOS.DBObjectID IS NOT NULL)
               OR (old.DBObjectID IS NOT NULL)) x
      WHERE DBOBjectID IN (SELECT DBObjectID FROM #tmpInstanceTable_ool)
      ORDER BY DBObjectID, Product, FeatureName
  END
  
  IF @bShowTable = 1
  BEGIN
    IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Show All Product List'
    SELECT * FROM #sp_rp_OracleOptionList
    DROP TABLE #sp_rp_OracleOptionList
  END
  
  DROP TABLE #tmpNewRaynetInventory
  DROP TABLE #sp_OracleRaynetOptionList
  DROP TABLE #tmpRayOracleOptionOld
  DROP TABLE #sp_OracleScriptOptionList
  DROP TABLE #tmpInstanceTable_ool
  DROP TABLE #tmpOracleInventory_ool
  DROP TABLE #sp_OracleOption
  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + ' - Finish sp_rp_OracleOptionList'
END


