USE [ManageSoft]
GO
IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_rp_OracleVirtualInfrastructure' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_rp_OracleVirtualInfrastructure]
GO
-- ==============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 07.05.2018
-- Copyright:   (c) Raynet GmbH
-- Description: Get Virtual Infratsructure for Oracle Server
-- ==============================================
-- Used by Report:
--   - OracleGuestOverview
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================
-- Parameter:
--   ComputerID  ComputerID of Guest System
CREATE PROCEDURE [dbo].[sp_rp_OracleVirtualInfrastructure] 
(
  @ComputerID        BIGINT   = null,
  @HyperVWithSerial  BIT      = 0
)

AS
BEGIN
--  DECLARE @ComputerID        BIGINT   = 1187
--  DECLARE @HyperVWithSerial  BIT      = 0


  IF OBJECT_ID('tempdb..#f_GetDeviceFQDN_ovi') IS NOT NULL
    DROP TABLE #f_GetDeviceFQDN_ovi

  CREATE TABLE #f_GetDeviceFQDN_ovi (
    ComputerID               BIGINT                                      NOT NULL,
	ComputerCN               NVARCHAR(256)      COLLATE database_default NOT NULL,
    FQDN                     NVARCHAR(256)      COLLATE database_default NULL)

  CREATE NONCLUSTERED INDEX IDX_C_#f_GetDeviceFQDN_ovi ON #f_GetDeviceFQDN_ovi(ComputerID)

  INSERT INTO #f_GetDeviceFQDN_ovi
  SELECT * FROM dbo.f_GetDeviceFQDN(1) f

  IF OBJECT_ID('tempdb..#f_GetDeviceFQDN_clt_ovi') IS NOT NULL
    DROP TABLE #f_GetDeviceFQDN_clt_ovi

  CREATE TABLE #f_GetDeviceFQDN_clt_ovi (
    ComputerID               BIGINT                                      NOT NULL,
	ComputerCN               NVARCHAR(256)      COLLATE database_default NOT NULL,
    FQDN                     NVARCHAR(256)      COLLATE database_default NULL)

  CREATE NONCLUSTERED INDEX IDX_C_#f_GetDeviceFQDN_clt_ovi ON #f_GetDeviceFQDN_clt_ovi(ComputerID)

  INSERT INTO #f_GetDeviceFQDN_clt_ovi
  SELECT * FROM #f_GetDeviceFQDN_ovi f
    WHERE @ComputerID IS NULL OR @ComputerID = f.ComputerID
	
  IF OBJECT_ID('tempdb..#f_GetDeviceVirtualVMwareUUID_ovi') IS NOT NULL
    DROP TABLE #f_GetDeviceVirtualVMwareUUID_ovi

  CREATE TABLE #f_GetDeviceVirtualVMwareUUID_ovi (
    GuestID                  INT                                         NOT NULL,
    UUID                     UNIQUEIDENTIFIER                            NOT NULL)

  CREATE NONCLUSTERED INDEX IDX_C_#f_GetDeviceVirtualVMwareUUID_ovi_ComputerID ON #f_GetDeviceVirtualVMwareUUID_ovi(GuestID)
  CREATE NONCLUSTERED INDEX IDX_C_#f_GetDeviceVirtualVMwareUUID_ovi_UUID ON #f_GetDeviceVirtualVMwareUUID_ovi(UUID)

  INSERT INTO #f_GetDeviceVirtualVMwareUUID_ovi
  SELECT GuestID, UUID FROM dbo.f_GetDeviceVirtualVMwareUUID() v
    INNER JOIN #f_GetDeviceFQDN_clt_ovi f
      ON v.GuestID = f.ComputerID
    WHERE UUID IS NOT NULL

  IF OBJECT_ID('tempdb..#f_GetVirtualVMwareGuestUUID_ovi') IS NOT NULL
    DROP TABLE #f_GetVirtualVMwareGuestUUID_ovi

  CREATE TABLE #f_GetVirtualVMwareGuestUUID_ovi (
    ESXHostID                INT                                         NOT NULL,
    UUID                     UNIQUEIDENTIFIER                            NOT NULL)

  CREATE NONCLUSTERED INDEX IDX_C_#f_GetVirtualVMwareGuestUUID_ovi_ComputerID ON #f_GetVirtualVMwareGuestUUID_ovi(ESXHostID)
  CREATE NONCLUSTERED INDEX IDX_C_#f_f_GetVirtualVMwareGuestUUID_ovi_UUID ON #f_GetVirtualVMwareGuestUUID_ovi(UUID)

  INSERT INTO #f_GetVirtualVMwareGuestUUID_ovi
  SELECT ESXHostID, UUID FROM dbo.f_GetVirtualVMwareGuestUUID()
    WHERE UUID IS NOT NULL

  IF OBJECT_ID('tempdb..#tmpESX') IS NOT NULL
    DROP TABLE #tmpESX

  CREATE TABLE #tmpESX (
    HostID            INT,
    HostCN            NVARCHAR(256) COLLATE database_default,
    ClusterName       NVARCHAR(256) COLLATE database_default,
    GuestID           INT,
    GuestCN           NVARCHAR(256) COLLATE database_default,
    ProcHostID        INT,
    ProcHostCN        NVARCHAR(256) COLLATE database_default,
    ProcManufacturer  NVARCHAR(256) COLLATE database_default,
    ProcName          NVARCHAR(256) COLLATE database_default,
    Speed             INT,
    TotalCPUs         INT,
    TotalCores        INT,
    TotalLogical      INT)

  CREATE NONCLUSTERED INDEX IDX_C_#tmpESX_ovi_HostID ON #tmpESX(HostID)
  CREATE NONCLUSTERED INDEX IDX_C_#tmpESX_ovi_ProcHostID ON #tmpESX(ProcHostID)

  INSERT INTO #tmpESX (HostID, GuestID, ProcHostID)
  SELECT
      esx.ESXHostID         AS HostID,
      guest.GuestID         AS GuestID,
      esx.ESXHostID         AS ProcHostID
    FROM #f_GetVirtualVMwareGuestUUID_ovi esx
      INNER JOIN #f_GetDeviceVirtualVMwareUUID_ovi guest
        ON esx.UUID = guest.UUID

  INSERT INTO #tmpESX (HostID, GuestID, ProcHostID)
  SELECT
      esx.ComputerID        AS HostID,
      fqdn.ComputerID       AS GuestID,
      esx.ComputerID        AS ProcHostID
    FROM dbo.f_HardwareProperties('VMWARE_VMComputerSystem') esx
      INNER JOIN #f_GetDeviceFQDN_clt_ovi fqdn
        ON esx.Value = fqdn.FQDN
    WHERE fqdn.ComputerID NOT IN (SELECT GuestID FROM #tmpESX)
	  AND esx.Property = 'HostName' AND fqdn.ComputerID <> esx.ComputerID

  INSERT INTO #tmpESX (HostID, GuestID, ProcHostID)
  SELECT
      esx.ComputerID        AS ESXHostID,
      c.ComputerID          AS GuestID,
      esx.HardwareObjectID  AS VMWAREVMComputerSystemID 
    FROM dbo.f_HardwareProperties('VMWARE_VMComputerSystem') esx
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('IPAddress') vmIP
        ON vmIP.HardwareObjectID = esx.HardwareObjectID
      INNER JOIN #f_GetDeviceFQDN_clt_ovi c 
        ON esx.Value = c.ComputerCN
      INNER JOIN dbo.f_GetDeviceNetworkInterface() nic
        ON nic.IPEnabled = 1
          AND c.ComputerID = nic.ComputerID
          AND vmIP.Value = nic.IPAddress
    WHERE c.ComputerID NOT IN (SELECT GuestID FROM #tmpESX)
      AND esx.Property = 'Elementname' AND c.ComputerID <> esx.ComputerID


  UPDATE e
      SET ClusterName = cl.ClusterName
    FROM #tmpESX e
      INNER JOIN dbo.f_GetVirtualVMwareLinkClusterName() cl
        ON e.HostID = cl.ESXHostID

  INSERT INTO #tmpESX (HostID, ClusterName, GuestID, ProcHostID)
  SELECT
      e.HostID,
      cl.ClusterName,
      e.GuestID,
      cl.ESXHostID    AS ProcHostID
    FROM dbo.f_GetVirtualVMwareLinkClusterName() cl
      INNER JOIN #tmpESX e
        ON cl.ClusterName = e.ClusterName
          AND cl.ESXHostID != e.HostID

  UPDATE e
      SET HostCN       = c.ComputerCN
    FROM #tmpESX e
      INNER JOIN #f_GetDeviceFQDN_ovi c
        ON e.HostID = c.ComputerID

  UPDATE e
      SET GuestCN       = c.ComputerCN
    FROM #tmpESX e
      INNER JOIN #f_GetDeviceFQDN_ovi c
        ON e.GuestID = c.ComputerID

  UPDATE e
      SET ProcHostCN       = c.ComputerCN
    FROM #tmpESX e
      INNER JOIN #f_GetDeviceFQDN_ovi c
        ON e.ProcHostID = c.ComputerID

  UPDATE e
      SET ProcManufacturer = pr.Manufacturer,
          ProcName         = pr.Name,
          Speed            = pr.ClockSpeed,
          TotalCPUs        = pr.TotalCPUs,
          TotalCores       = pr.TotalCores,
          TotalLogical     = pr.TotalLogical
    FROM #tmpESX e
      INNER JOIN dbo.f_GetDeviceProcessor() pr
        ON e.ProcHostID = pr.ComputerID

  IF OBJECT_ID('tempdb..#f_GetVirtualHyperVGuestInfo_ovi') IS NOT NULL
    DROP TABLE #f_GetVirtualHyperVGuestInfo_ovi

  CREATE TABLE #f_GetVirtualHyperVGuestInfo_ovi (
    HostID                   BIGINT                                      NOT NULL,
	FQDN                     NVARCHAR(256)      COLLATE database_default NULL,
	PowerState               NVARCHAR(30)       COLLATE database_default NULL,
    PowerStateID             INT                                         NOT NULL)

  CREATE NONCLUSTERED INDEX IDX_C_#f_GetVirtualHyperVGuestInfo_ovi_HostID ON #f_GetVirtualHyperVGuestInfo_ovi(HostID)

  INSERT INTO #f_GetVirtualHyperVGuestInfo_ovi
  SELECT
     g.HyperVHostID,
     g.FQDN,
     g.PowerState,
     g.PowerStateID 
   FROM dbo.f_GetVirtualHyperVGuestInfo() g

  IF OBJECT_ID('tempdb..#tmpSerial') IS NOT NULL
    DROP TABLE #tmpSerial

  CREATE TABLE #tmpSerial (
    HostID                   BIGINT                                      NOT NULL,
    GuestID                  BIGINT                                      NOT NULL,
    GuestCN                  NVARCHAR(256)      COLLATE database_default NOT NULL,
	SerialNumber             NVARCHAR(256)      COLLATE database_default NOT NULL,
    PowerStateID             INT                                         NOT NULL)

  INSERT INTO #tmpSerial
  SELECT g.HyperVHostID AS HostID, ser.ComputerID AS GuestID, c.ComputerCN AS GuestCN, ser.SerialNumber, g.PowerStateID
    FROM dbo.f_GetVirtualHyperVGuestInfo() g
      INNER JOIN dbo.f_GetDeviceSerialNumber() ser
        ON g.BIOSSerialNumber = ser.SerialNumber
      INNER JOIN #f_GetDeviceFQDN_clt_ovi c
        ON ser.ComputerID = c.ComputerID

  IF OBJECT_ID('tempdb..#tmpSerialON') IS NOT NULL
    DROP TABLE #tmpSerialON

  CREATE TABLE #tmpSerialON (
    HostID                   BIGINT                                      NOT NULL,
    GuestID                  BIGINT                                      NOT NULL,
    GuestCN                  NVARCHAR(256)      COLLATE database_default NOT NULL,
	SerialNumber             NVARCHAR(256)      COLLATE database_default NOT NULL,
    PowerStateID             INT                                         NOT NULL)

  INSERT INTO #tmpSerialON
  SELECT MAX(HostID) AS HostID, MAX(GuestID) AS GuestID, MAX(GuestCN) AS GuestCN, SerialNumber, PowerStateID
    FROM #tmpSerial
    WHERE PowerStateID = 2
    GROUP BY SerialNumber, PowerStateID
    HAVING COUNT(*) = 1

  IF OBJECT_ID('tempdb..#tmpSerialOFF') IS NOT NULL
    DROP TABLE #tmpSerialOFF

  CREATE TABLE #tmpSerialOFF (
    HostID                   BIGINT                                      NOT NULL,
    GuestID                  BIGINT                                      NOT NULL,
    GuestCN                  NVARCHAR(256)      COLLATE database_default NOT NULL,
	SerialNumber             NVARCHAR(256)      COLLATE database_default NOT NULL,
    PowerStateID             INT                                         NOT NULL)

  INSERT INTO #tmpSerialON
  SELECT MAX(HostID) AS HostID, MAX(GuestID) AS GuestID, MAX(GuestCN) AS GuestCN, SerialNumber, PowerStateID
    FROM #tmpSerial
    WHERE PowerStateID = 3
      AND SerialNumber NOT IN (SELECT SerialNumber FROM #tmpSerialON)
    GROUP BY SerialNumber, PowerStateID
    HAVING COUNT(*) = 1 

  IF OBJECT_ID('tempdb..#tmpHyperV') IS NOT NULL
    DROP TABLE #tmpHyperV

  CREATE TABLE #tmpHyperV (
    HostID            INT,
    HostCN            NVARCHAR(256) COLLATE database_default,
    ClusterName       NVARCHAR(256) COLLATE database_default,
    GuestID           INT,
    GuestCN           NVARCHAR(256) COLLATE database_default,
    ProcHostID        INT,
    ProcHostCN        NVARCHAR(256) COLLATE database_default,
    ProcManufacturer  NVARCHAR(256) COLLATE database_default,
    ProcName          NVARCHAR(256) COLLATE database_default,
    Speed             INT,
    TotalCPUs         INT,
    TotalCores        INT,
    TotalLogical      INT)

  CREATE NONCLUSTERED INDEX IDX_C_#tmpHyperV_ovi_HostID ON #tmpHyperV(HostID)
  CREATE NONCLUSTERED INDEX IDX_C_#tmpHyperV_ovi_ProcHostID ON #tmpHyperV(ProcHostID)

  IF @HyperVWithSerial = 1
    INSERT INTO #tmpHyperV(HostID, GuestID, GuestCN, ProcHostID)
    SELECT HostId, GuestID, GuestCN, HostID AS ProcHostID FROM #tmpSerial
  ELSE
  BEGIN
    INSERT INTO #tmpHyperV(HostID, GuestID, GuestCN, ProcHostID)
    SELECT HostId, GuestID, GuestCN, HostID AS ProcHostID FROM #tmpSerialON

    INSERT INTO #tmpHyperV(HostID, GuestID, GuestCN, ProcHostID)
    SELECT HostId, GuestID, GuestCN, HostID AS ProcHostID FROM #tmpSerialOFF
  END

  INSERT INTO #tmpHyperV(HostID, GuestID, GuestCN, ProcHostID)
  SELECT
      h.HostID,
      f.ComputerID                AS GuestID,
      f.ComputerCN                AS GuestCN,
      h.HostID                    AS ProcHostID
    FROM (SELECT HostID, FQDN, PowerStateID FROM #f_GetVirtualHyperVGuestInfo_ovi WHERE LEN(FQDN) > 0) h
      INNER JOIN #f_GetDeviceFQDN_clt_ovi f
        ON h.FQDN = f.FQDN
    WHERE f.ComputerID NOT IN (SELECT GuestID FROM #tmpHyperV)

  INSERT INTO #tmpHyperV(HostID, GuestID, GuestCN, ProcHostID)
  SELECT
      h.ComputerID                AS HostID,
      g.HyperVGuestID             AS GuestID,
      c.ComputerCN                AS GuestCN,
      h.ComputerID                AS ProcHostID
    FROM dbo.f_GetVirtualHyperVHostInfo() g
      INNER JOIN #f_GetDeviceFQDN_clt_ovi c
        ON g.HyperVGuestID = c.ComputerID
      INNER JOIN #f_GetDeviceFQDN_ovi h
        ON g.FQDN = h.FQDN
      LEFT JOIN #f_GetVirtualHyperVGuestInfo_ovi hi
        ON h.ComputerID = hi.HostID
    WHERE g.HyperVGuestID NOT IN (SELECT GuestID FROM #tmpHyperV)

  UPDATE h
      SET HostCN = c.ComputerCN
    FROM #tmpHyperV h
      INNER JOIN #f_GetDeviceFQDN_ovi c
        ON h.HostID = c.ComputerID

  UPDATE h
      SET ClusterName = cl.ClusterName
    FROM #tmpHyperV h
      INNER JOIN dbo.f_GetVirtualHyperVLinkClusterName() cl
        ON h.HostID = cl.HyperVHostID

  INSERT INTO #tmpHyperV (HostID, HostCN, GuestID, GuestCN, ProcHostID, ClusterName)
  SELECT
      h.HostID,
      h.HostCN,     
      h.GuestID,
      h.GuestCN,
      cl.HyperVHostID AS ProcHostID,
      cl.ClusterName
    FROM dbo.f_GetVirtualHyperVLinkClusterName() cl
      INNER JOIN #tmpHyperV h
        ON cl.ClusterName = h.ClusterName
          AND cl.HyperVHostID != h.HostID

  UPDATE h
      SET ProcHostCN       = c.ComputerCN
    FROM #tmpHyperV h
      INNER JOIN #f_GetDeviceFQDN_ovi c
        ON h.ProcHostID = c.ComputerID

  UPDATE h
      SET ProcManufacturer = pr.Manufacturer,
          ProcName         = pr.Name,
          Speed            = pr.ClockSpeed,
          TotalCPUs        = pr.TotalCPUs,
          TotalCores       = pr.TotalCores,
          TotalLogical     = pr.TotalLogical
    FROM #tmpHyperV h
      INNER JOIN dbo.f_GetDeviceProcessor() pr
        ON h.HostID = pr.ComputerID

  SELECT
      'VMware'         AS Virtualization,
      e.HostID,
      e.HostCN,
      e.ClusterName,
      e.GuestID,
      e.GuestCN,
      e.ProcHostID,
      e.ProcHostCN,
      e.ProcName,
      e.TotalCores     AS ProcCore
    FROM #tmpESX e
  UNION SELECT
      'HyperV'         AS Virtualization,
      h.HostID,
      h.HostCN,
      h.ClusterName,
      h.GuestID,
      h.GuestCN,
      h.ProcHostID,
      h.ProcHostCN,
      h.ProcName,
      h.TotalCores     AS ProcCore
    FROM #tmpHyperV h

  DROP TABLE #tmpSerial
  DROP TABLE #tmpSerialOFF
  DROP TABLE #tmpSerialON
  DROP TABLE #tmpESX
  DROP TABLE #tmpHyperV
  DROP TABLE #f_GetDeviceFQDN_ovi
  DROP TABLE #f_GetDeviceFQDN_clt_ovi
  DROP TABLE #f_GetDeviceVirtualVMwareUUID_ovi
  DROP TABLE #f_GetVirtualHyperVGuestInfo_ovi
  DROP TABLE #f_GetVirtualVMwareGuestUUID_ovi
END
GO

