USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_rp_OracleReport' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_rp_OracleReport]
GO

-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================

CREATE PROCEDURE [dbo].[sp_rp_OracleReport] (
  @ComputerCN   NVARCHAR(256) = NULL,
  @InstanceName NVARCHAR(256) = NULL
)
AS
BEGIN
  SET FMTONLY OFF
--  DECLARE @ComputerCN   NVARCHAR(256) = NULL
--  DECLARE @InstanceName NVARCHAR(256) = NULL

  DECLARE @bDebug BIT = 1
  IF @bDebug = 1 PRINT CAST(CURRENT_TIMESTAMP AS NVARCHAR(20)) + '   - Get Oracle Instance'
  IF OBJECT_ID('tempdb..#tmpOracleInstance') IS NOT NULL
    DROP TABLE #tmpOracleInstance
 

CREATE TABLE #OracleInstances(
	[ClusterDBName][nvarchar](256) COLLATE database_default NULL,
	[ComputerID] [int] NULL,
	[ComputerCN][nvarchar](256) COLLATE database_default NULL,
	[ComputerFQDN][nvarchar](256) COLLATE database_default NULL,
	[InstanceID] [int] NOT NULL,
	[Instance][nvarchar](256) COLLATE database_default NULL,
	[SystemDateTime] [datetime] NULL,
	[SystemDisplayName][nvarchar](256) COLLATE database_default NULL,
	[SystemDisplayVersion][nvarchar](256) COLLATE database_default NULL,
	[AppName] [nvarchar](255) NULL,
	[AppVersion] [nvarchar](255) NULL,
	[AppEdition] [nvarchar](255) NULL,
	[DatabaseRole][nvarchar](256) COLLATE database_default NULL,
	[DatabaseName][nvarchar](256) COLLATE database_default NULL,
	[DatabaseNameUnique][nvarchar](256) COLLATE database_default NULL,
	[DatabaseCreationDate] [datetime] NULL,
	[OperatingSystem] [nvarchar](256)  COLLATE database_default  NULL
) 
EXEC sp_OracleInstances null
 


  CREATE TABLE #tmpOracleInstance(
    ClusterDBName          NVARCHAR(256) COLLATE database_default NULL,
    ComputerID             BIGINT                                 NULL,
    ComputerCN             NVARCHAR(256) COLLATE database_default NULL,
    ComputerFQDN           NVARCHAR(256) COLLATE database_default NULL,
    InstanceID             BIGINT                                 NOT NULL,
    Instance               NVARCHAR(256) COLLATE database_default NOT NULL,
    SystemDateTime         DATETIME                               NULL,
    SystemDisplayName      NVARCHAR(256) COLLATE database_default NULL,
    SystemDisplayVersion   NVARCHAR(256) COLLATE database_default NULL,
    AppName                NVARCHAR(256) COLLATE database_default NULL,
    AppVersion             NVARCHAR(256) COLLATE database_default NULL,
    AppEdition             NVARCHAR(256) COLLATE database_default NULL,
    DatabaseRole           NVARCHAR(256) COLLATE database_default NULL,
    DatabaseName           NVARCHAR(256) COLLATE database_default NULL,
    DatabaseNameUnique     NVARCHAR(256) COLLATE database_default NULL,
    DatabaseCreationDate   DATETIME                               NULL,
    OperatingSystem        NVARCHAR(256) COLLATE database_default NULL,
    HWDate                 DATETIME                               NULL
  )

  CREATE NONCLUSTERED INDEX IDX_C_#tmpOracleInstance_ComputerID ON #tmpOracleInstance(ComputerID)
  CREATE NONCLUSTERED INDEX IDX_C_#tmpOracleInstance_ComputerID_InstanceID ON #tmpOracleInstance(ComputerID, InstanceID)

  INSERT INTO #tmpOracleInstance
  SELECT oi.*, ir.HWDate
    FROM #OracleInstances oi
      LEFT JOIN InventoryReport ir WITH (NOLOCK)
        ON oi.InstanceID = ir.ComputerID
    WHERE @ComputerCN IS NULL OR LEN(@ComputerCN) = 0 OR oi.ComputerCN LIKE @ComputerCN
      AND @InstanceName IS NULL OR LEN(@InstanceName) = 0 OR oi.Instance LIKE @InstanceName

  IF OBJECT_ID('tempdb..#sp_OracleFailures') IS NOT NULL
    DROP TABLE #sp_OracleFailures

  CREATE TABLE #sp_OracleFailures(
    ComputerID    INT                                    NULL,
    Instance      NVARCHAR(256) COLLATE database_default NULL,
    InstanceID    INT                                    NULL,
    StatusID      INT                                    NULL,
    [Status]      NVARCHAR(256) COLLATE database_default NULL,
    [Version]     NVARCHAR(256) COLLATE database_default NULL,
    tmp_MessageID INT                                    NULL,
    tmp_VerMajor  INT                                    NULL,
    tmp_VerMinor  INT                                    NULL)

  CREATE NONCLUSTERED INDEX IDX_C_#sp_OracleFailures_ComputerID ON #sp_OracleFailures(ComputerID, InstanceID)

  EXEC sp_OracleFailures @ComputerCN

  IF OBJECT_ID('tempdb..#sp_rp_OracleOptionList') IS NOT NULL
    DROP TABLE #sp_rp_OracleOptionList

  CREATE TABLE #sp_rp_OracleOptionList (
    DBObjectID       BIGINT        NULL,
    Product          NVARCHAR(256) NULL,
    FeatureName      NVARCHAR(256) NULL,
    Installed        INT           NULL,
    RaynetUsed       INT           NULL,
    OracleUsed       INT           NULL,
    TechnicalLicense INT           NULL,
    RaynetComment    NVARCHAR(256) NULL,
    OracleComment    NVARCHAR(256) NULL,
    CustomLicense    INT           NULL,
    CustomComment    NVARCHAR(256) NULL,
    LicenseResult    INT           NULL)

  EXEC sp_rp_OracleOptionList @OnlyProduct=1

  IF OBJECT_ID('tempdb..#tmpOracleOption') IS NOT NULL
    DROP TABLE #tmpOracleOption

  CREATE TABLE #tmpOracleOption(
    DBObjectID BIGINT,
    [Option]   NVARCHAR(max))

  CREATE NONCLUSTERED INDEX IDX_C_#tmpOracleOption_DBObjectID ON #tmpOracleOption(DBObjectID)

  INSERT INTO #tmpOracleOption
  SELECT 
      DBObjectID,
      STUFF((SELECT ';' + Product + ',' + CAST(TechnicalLicense AS NVARCHAR(1))
               FROM #sp_rp_OracleOptionList pt
               WHERE pt.DBObjectID = ool.DBObjectID
               ORDER BY Product
               FOR XML PATH('')),1 ,1, '') [option]
    FROM #sp_rp_OracleOptionList ool
      INNER JOIN #tmpOracleInstance i
        ON ool.DBObjectID = i.InstanceID
    GROUP BY DBObjectID


  IF OBJECT_ID('tempdb..#tmpOraProcessor') IS NOT NULL
    DROP TABLE #tmpOraProcessor

  CREATE TABLE #tmpOraProcessor(
    ComputerID    BIGINT,
    ClockSpeed    BIGINT,
	TotalCPUs     INT,
	TotalLogical  INT)

  CREATE NONCLUSTERED INDEX IDX_C_#tmpOraProcessor_ComputerID ON #tmpOraProcessor(ComputerID)

  INSERT INTO #tmpOraProcessor
  SELECT p.ComputerID, p.ClockSpeed, p.TotalCPUs, p.TotalLogical FROM f_GetDeviceProcessor() p
    INNER JOIN #tmpOracleInstance i
      ON p.ComputerID = i.ComputerID
	
  IF OBJECT_ID('tempdb..#tmpOraModel') IS NOT NULL
    DROP TABLE #tmpOraModel

  CREATE TABLE #tmpOraModel(
    ComputerID BIGINT,
    Manufacturer NVARCHAR(256),
    Model        NVARCHAR(256),
    Vrtual       BIT)

  CREATE NONCLUSTERED INDEX IDX_C_#tmpOraModel_ComputerID ON #tmpOraModel(ComputerID)
	
  INSERT INTO #tmpOraModel
  SELECT m.ComputerID, m.Manufacturer, m.Model, m.Virtual FROM f_GetDeviceModel() m
    INNER JOIN #tmpOracleInstance i 
      ON m.ComputerID = i.ComputerID
	
  SELECT DISTINCT
      i.ClusterDBName,
      i.ComputerID,
      i.ComputerCN,
	  i.InstanceID,
      stat.Instance,
      i.SystemDateTime,
      i.SystemDisplayName,
      i.SystemDisplayVersion,
      i.AppName,
      i.AppVersion,
      i.AppEdition,
      i.DatabaseRole,
      i.DatabaseName,
      i.DatabaseNameUnique,
      i.DatabaseCreationDate,
      i.OperatingSystem,
      uc.[Open]       AS UserCount,
      ucs_model.Model AS MakeModel,
      cp.ClockSpeed   AS MaxClockSpeed,
      cp.TotalCPUs    AS NumberOfProcessors,
      cp.TotalLogical AS NumberOfLogicalProcessors,
      stat.Status,
      stat.Version AS XMLVersion,
      o.[Option]   AS OracleOption
    FROM (SELECT
              stat.ComputerID,
              MAX(stat.InstanceID) AS InstanceID,
              stat.Instance,
              stat.StatusID,
              stat.Status,
              MAX(stat.Version)    AS Version
            FROM #sp_OracleFailures stat
              LEFT JOIN InventoryReport ir WITH (NOLOCK)
                ON stat.InstanceID = ir.ComputerID
              LEFT JOIN (SELECT oi.ComputerID, oi.Instance, MAX(ir.HWDate) HWDate
                           FROM #tmpOracleInstance oi
                             LEFT JOIN InventoryReport ir WITH (NOLOCK)
                               ON oi.InstanceID = ir.ComputerID
                           GROUP BY oi.ComputerID, oi.Instance) ni
                ON stat.ComputerID = ni.ComputerID
                  AND stat.Instance = ni.Instance
            WHERE ir.HWDate IS NULL OR ni.HWDate IS NULL OR ir.HWDate = ni.HWDate
            GROUP BY stat.ComputerID, stat.Instance, stat.StatusID, stat.Status) stat
      LEFT JOIN #tmpOracleInstance i
        ON stat.InstanceID = i.InstanceID
          AND stat.Instance = i.Instance		
--        AND (status.InstanceID IS NULL OR i.InstanceID = status.InstanceID)
      LEFT JOIN dbo.f_OracleUserCounts(0) uc
        ON i.InstanceID = uc.DBObjectID
      LEFT JOIN #tmpOraModel ucs_model
        ON i.ComputerID = ucs_model.ComputerID
      LEFT JOIN (SELECT * FROM dbo.f_HardwareProperties('CIM_Chassis') WHERE Property = 'ChassisTypes')  ch_chassis
        ON i.ComputerID = ch_chassis.ComputerID
      LEFT JOIN #tmpOraProcessor cp
        ON i.ComputerID = cp.ComputerID
      LEFT JOIN #tmpOracleOption o
        ON i.InstanceID = o.DBObjectID

  DROP TABLE #sp_OracleFailures
  DROP TABLE #tmpOracleInstance
  DROP TABLE #sp_rp_OracleOptionList
  DROP TABLE #tmpOracleOption
  DROP TABLE #tmpOraProcessor
  DROP TABLE #tmpOraModel
 
END 

