USE [ManageSoft]
GO


IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'sp_rp_Infrastructure' AND xtype = 'P'
)
        DROP PROCEDURE [dbo].[sp_rp_Infrastructure]
GO

-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================
 
CREATE PROCEDURE [dbo].[sp_rp_Infrastructure] 
(
  @pGuestSearch              NVARCHAR(255) = null,
  @pHostSearch               NVARCHAR(255) = null,
  @pClusterSearch            NVARCHAR(255) = null,
  @pSystemType               NVARCHAR(MAX) = null,
  @pViewAll                  BIT           = 0,
  @pMaxAge                   INT           = null,
  @pGetSystemTypes           BIT           = 0
)

AS
BEGIN

  DECLARE  @GuestSearch              NVARCHAR(255) = @pGuestSearch
  DECLARE  @HostSearch               NVARCHAR(255) = @pHostSearch
  DECLARE  @ClusterSearch            NVARCHAR(255) = @pClusterSearch
  DECLARE  @SystemType               NVARCHAR(MAX) = @pSystemType
  DECLARE  @ViewAll                  BIT           = @pViewAll
  DECLARE  @MaxAge                   INT           = @pMaxAge
  DECLARE  @GetSystemTypes           BIT           = @pGetSystemTypes

  IF @GetSystemTypes = 1 BEGIN
    SELECT 'Physical' AS SystemType, 'Physical' AS Name
    UNION SELECT 'VMware', 'VMware'
    UNION SELECT 'HyperV', 'Hyper-V'
    UNION SELECT 'LPAR', 'LPAR'
    RETURN
  END
  
  IF @GuestSearch = '%' SET @GuestSearch = NULL
  IF @HostSearch  = '%' SET @HostSearch  = NULL

  IF OBJECT_ID('tempdb..#Devices') IS NOT NULL
    DROP TABLE #Devices

PRINT '*** Start ('+CONVERT(nvarchar(50),GETDATE(), 113)+')***'

  select * INTO #Virtual_LPAR from f_Virtual_LPAR() l
  PRINT '*** #Virtual_LPAR ('+CONVERT(nvarchar(50),GETDATE(), 113)+')***'

  select * INTO #GetDeviceNetworkInfo from f_GetDeviceNetworkInfo()
  PRINT '*** #GetDeviceNetworkInfo ('+CONVERT(nvarchar(50),GETDATE(), 113)+')***'

  select * INTO #System_Base from f_System_Base()
  PRINT '*** #System_Base ('+CONVERT(nvarchar(50),GETDATE(), 113)+')***'

  select * INTO #VMWARE_VMComputerSystem from f_HardwareProperties('VMWARE_VMComputerSystem')
  PRINT '*** #VMWARE_VMComputerSystem ('+CONVERT(nvarchar(50),GETDATE(), 113)+')***'
 
  CREATE TABLE #Devices (
    ComputerID         BIGINT,
    SystemType         NVARCHAR(255),
    SystemTypeShort    NVARCHAR(255),
    vCenter            NVARCHAR(255)  NULL,
    vCenterID          BIGINT         NULL,
    Cluster            NVARCHAR(255)  NULL,
    Host               NVARCHAR(255)  NULL,
    HostID             BIGINT         NULL,
    Client             NVARCHAR(255)  NULL,
    ClientID           BIGINT         NULL,
    DNSName            NVARCHAR(255)  NULL,
    Role               NVARCHAR(255)  NULL,
    ScanDate           DATETIME       NULL,
    Manufacturer       NVARCHAR(255)  NULL,
    Model              NVARCHAR(255)  NULL,
    Serialnumber       NVARCHAR(255)  NULL,
    Memory             REAL           NULL,
    DiskSize           REAL           NULL,
    ProcManufacturer   NVARCHAR(255)  NULL,
    ProcModel          NVARCHAR(255)  NULL,
    ProcCPUs           NVARCHAR(255)  NULL,
    ProcCores          NVARCHAR(255)  NULL,
    ProcSpeed          NVARCHAR(255)  NULL,
    ProcHyperThreating NVARCHAR(255)  NULL,
    ProcEntitlement    NVARCHAR(255)  NULL,
    OS                 NVARCHAR(255)  NULL,
    LastLogin          NVARCHAR(255)  NULL,
    IPAddress          NVARCHAR(max)  NULL,
    MACAddress         NVARCHAR(max)  NULL,
    ESXLicenseKey      NVARCHAR(255)  NULL,
    ESXLicenseTotal    NVARCHAR(255)  NULL,
    ESXLicenseUsed     NVARCHAR(255)  NULL,
    [Failover]         NVARCHAR(10)   NULL,
    PowerState         BIT            NULL)

  INSERT INTO #Devices (
      ComputerID,
      SystemType,
      SystemTypeShort,
      Host,
      HostID,
      Client,
      ClientID,
      DNSName,
      Role,
      ScanDate,
      Manufacturer,
      Model,
      Serialnumber,
      Memory,
      DiskSize,
      ProcManufacturer,
      ProcModel,
      ProcCPUs,
      ProcCores,
      ProcSpeed,
      ProcHyperThreating,
      ProcEntitlement,
      OS,
      LastLogin,
	  IPAddress,
	  MACAddress )

    SELECT
      l.ComputerID        AS ComputerID,
      'LPAR ' + CASE
                  WHEN l.Type = 'Host' THEN l.LPARType
                  ELSE l.Type
                END       AS SystemType,
      'LPAR'              AS SystemTypeShort,
      l.HostName          AS Host,
      l.HostID            AS HostID,
      l.GuestName         AS Client,
      l.GuestID           AS ClientID,
      l.ComputerFQDN      AS DNSName,
      CASE
        WHEN l.Type = 'Host' THEN 'Server'
        ELSE dr.Description
      END                  AS Role,
      l.InventoryDate      AS ScanDate,
      l.Manufacturer       AS Manufacturer,
      l.Model              AS Model,
      l.Serialnumber       AS Serialnumber,
      l.Memory             AS Memory,
      l.DiskSize           AS DiskSize,
      l.CPUManufacturer    AS ProcManufacturer,
      l.CPUModel           AS ProcModel,
      l.CPUs               AS ProcCPUs,
      l.Cores              AS ProcCores,
      l.CPUSpeed           AS ProcSpeed,
      null                 AS ProcHyperThreating,
      l.Entitlement        AS ProcEntitlement,
      l.OS                 AS OS,
      l.LastLogin          AS LastLogin,
	  devInfo.IPAddress    AS IPAddress,
	  devInfo.MACAddress   AS MACAddress
    FROM #Virtual_LPAR l
      LEFT JOIN NetworkDevice nd WITH (NOLOCK)
-- 01.12.2017 add filter on DupplicateID IS NULL, only the last one is from intresting
-- 01.12.2017 add filter on computerCN to DeviceName, some times it give entrys with other devicename what
--            is mapped to the ComputerDevice, this are not from intresst
        ON nd.DuplicateID IS NULL AND l.ComputerID = nd.ComputerID AND l.ComputerName = nd.DeviceName 
      LEFT JOIN DeviceRole dr WITH (NOLOCK)
        ON nd.DeviceRole = dr.DeviceRoleID
			LEFT JOIN #GetDeviceNetworkInfo devInfo ON devInfo.ComputerID = l.ComputerID
      WHERE @GuestSearch IS NULL OR l.ComputerName LIKE @GuestSearch  
	
   PRINT '*** Insert LPAR ('+CONVERT(nvarchar(50),GETDATE(), 113)+')***'
	  	  
CREATE TABLE #Virtual_VMware(
	[ComputerID] [int] NOT NULL,
	[ComputerName] [nvarchar](64) NOT NULL,
	[ComputerFQDN] [nvarchar](4000) NULL,
	[vCenterID] [int] NULL,
	[vCenterName] [nvarchar](64) NULL,
	[ESXID] [int] NOT NULL,
	[ESXName] [nvarchar](64) NOT NULL,
	[GuestID] [int] NULL,
	[GuestName] [nvarchar](64) NULL,
	[ClusterName] [nvarchar](256) NULL,
	[Type] [varchar](5) NOT NULL,
	[Manufacturer] [nvarchar](256) NULL,
	[Model] [nvarchar](513) NULL,
	[SerialNumber] [nvarchar](256) NULL,
	[BiosVersion] [nvarchar](256) NULL,
	[OS] [nvarchar](513) NULL,
	[CPUManufacturer] [nvarchar](256) NULL,
	[CPUModel] [nvarchar](256) NULL,
	[CPUs] [int] NULL,
	[Cores] [int] NULL,
	[HyperThreating] [int] NULL,
	[CPUSpeed] [float] NULL,
	[Memory] [bigint] NULL,
	[DiskSize] [bigint] NULL,
	[LastLogin] [nvarchar](256) NULL,
	[PowerState] [int] NULL,
	[InventoryDate] [datetime] NULL
) 
 exec [sp_Virtual_VMware] 0

 PRINT '*** [sp_Virtual_VMware] ('+CONVERT(nvarchar(50),GETDATE(), 113)+')***'
 

CREATE TABLE [dbo].[#Virtual_HyperV](
	[ComputerID] [int] NOT NULL,
	[ComputerName] [nvarchar](64) NOT NULL,
	[ComputerFQDN] [nvarchar](4000) NULL,
	[HostID] [int] NOT NULL,
	[HostName] [nvarchar](64) NULL,
	[GuestID] [bigint] NULL,
	[GuestName] [nvarchar](255) NULL,
	[ClusterName] [nvarchar](256) NULL,
	[Failover] [nvarchar](256) NULL,
	[Type] [varchar](5) NOT NULL,
	[Manufacturer] [nvarchar](256) NULL,
	[Model] [nvarchar](513) NULL,
	[SerialNumber] [nvarchar](256) NULL,
	[BiosVersion] [nvarchar](256) NULL,
	[OS] [nvarchar](513) NULL,
	[CPUManufacturer] [nvarchar](256) NULL,
	[CPUModel] [nvarchar](256) NULL,
	[CPUs] [int] NULL,
	[Cores] [int] NULL,
	[HyperThreating] [int] NULL,
	[CPUSpeed] [float] NULL,
	[Memory] [bigint] NULL,
	[DiskSize] [bigint] NULL,
	[LastLogin] [nvarchar](256) NULL,
	[InventoryDate] [datetime] NULL
) ON [PRIMARY]
 
 exec sp_Virtual_HyperV

   PRINT '*** sp_Virtual_HyperV ('+CONVERT(nvarchar(50),GETDATE(), 113)+')***'

   INSERT INTO #Devices (
      ComputerID,
      SystemType,
      SystemTypeShort,
      vCenter,
      vCenterID,
      Cluster,
      Host,
      HostID,
      Client,
      ClientID,
      DNSName,
      Role,
      ScanDate,
      Manufacturer,
      Model,
      Serialnumber,
      Memory,
      DiskSize,
      ProcManufacturer,
      ProcModel,
      ProcCPUs,
      ProcCores,
      ProcSpeed,
      ProcHyperThreating,
      OS,
      LastLogin,
      ESXLicenseKey,
      ESXLicenseTotal,
      ESXLicenseUsed,
      PowerState,
	  IPAddress,
	  MACAddress)

    SELECT
      v.ComputerID         AS ComputerID,
      'VMware ' + v.Type   AS SystemType,
      'VMware'             AS SystemTypeShort,
      v.vCenterName        AS vCenter,
      v.vCenterID          AS vCenterID,
      v.ClusterName        AS Cluster,
      v.ESXName            AS Host,
      v.ESXID              AS HostID,
      v.GuestName          AS Client,
      v.GuestID            AS ClientID,
      v.ComputerFQDN       AS DNSName,
      CASE
        WHEN v.Type = 'Host' THEN 'Server'
        ELSE dr.Description
      END                  AS Role,
      v.InventoryDate      AS ScanDate,
      v.Manufacturer       AS Manufacturer,
      v.Model              AS Model,
      v.Serialnumber       AS Serialnumber,
      v.Memory             AS Memory,
      v.DiskSize           AS DiskSize,
      v.CPUManufacturer    AS ProcManufacturer,
      v.CPUModel           AS ProcModel,
      v.CPUs               AS ProcCPUs,
      v.Cores              AS ProcCores,
      v.CPUSpeed           AS ProcSpeed,
      v.HyperThreating     AS ProcHyperThreating,
      v.OS                 AS OS,
      v.LastLogin          AS LastLogin,
      lic.LicenseKey       AS ESXLicenseKey,
      lic.LicenseTotal     AS ESXLicenseTotal,
      lic.LicenseUsed      AS ESXLicenseUsed,
      v.PowerState         AS PowerState,
	  devInfo.IPAddress    AS IPAddress,
	  devInfo.MACAddress   AS MACAddress
    FROM #Virtual_VMware v
      LEFT JOIN NetworkDevice nd WITH (NOLOCK)
-- 01.12.2017 add filter on DupplicateID IS NULL, only the last one is from intresting
-- 01.12.2017 add filter on computerCN to DeviceName, some times it give entrys with other devicename what
--            is mapped to the ComputerDevice, this are not from intresst
        ON nd.DuplicateID IS NULL AND v.ComputerID = nd.ComputerID AND v.ComputerName = nd.DeviceName 
      LEFT JOIN DeviceRole dr WITH (NOLOCK)
        ON nd.DeviceRole = dr.DeviceRoleID
      LEFT JOIN f_VMWare_License() lic
        ON v.ComputerID = lic.ComputerID
		LEFT JOIN #GetDeviceNetworkInfo devInfo ON devInfo.ComputerID = v.ComputerID
    WHERE @GuestSearch IS NULL OR v.ComputerName LIKE @GuestSearch  

   PRINT '*** Insert VMWare ('+CONVERT(nvarchar(50),GETDATE(), 113)+')***'
   
-- HyperV
  INSERT INTO #Devices (
      ComputerID,
      SystemType,
      SystemTypeShort,
      Cluster,
      Host,
      HostID,
      Client,
      ClientID,
      DNSName,
      Role,
      ScanDate,
      Manufacturer,
      Model,
      Serialnumber,
      Memory,
      DiskSize,
      ProcManufacturer,
      ProcModel,
      ProcCPUs,
      ProcCores,
      ProcSpeed,
      ProcHyperThreating,
      OS,
      LastLogin,
      [Failover],
	  IPAddress,
	  MACAddress)
    SELECT
      v.ComputerID         AS ComputerID,
      'HyperV ' + v.Type   AS SystemType,
      'HyperV'             AS SystemTypeShort,
      v.ClusterName        AS Cluster,
      v.HostName           AS Host,
      v.HostID             AS HostID,
      v.GuestName          AS Client,
      v.GuestID            AS ClientID,
      v.ComputerFQDN       AS DNSName,
      CASE
        WHEN dr.Description IS NULL AND v.Type = 'Host' THEN 'Server'
        ELSE dr.Description
      END                  AS Role,
      v.InventoryDate      AS ScanDate,
      v.Manufacturer       AS Manufacturer,
      v.Model              AS Model,
      v.Serialnumber       AS Serialnumber,
      v.Memory             AS Memory,
      v.DiskSize           AS DiskSize,
      v.CPUManufacturer    AS ProcManufacturer,
      v.CPUModel           AS ProcModel,
      v.CPUs               AS ProcCPUs,
      v.Cores              AS ProcCores,
      v.CPUSpeed           AS ProcSpeed,
      v.HyperThreating     AS ProcHyperThreating,
      v.OS                 AS OS,
      v.LastLogin          AS LastLogin,
      v.[Failover]           AS [Failover],
	  devInfo.IPAddress    AS IPAddress,
	  devInfo.MACAddress   AS MACAddress
    FROM #Virtual_HyperV v
      LEFT JOIN NetworkDevice nd WITH (NOLOCK)
-- 01.12.2017 add filter on DupplicateID IS NULL, only the last one is from intresting
-- 01.12.2017 add filter on computerCN to DeviceName, some times it give entrys with other devicename what
--            is mapped to the ComputerDevice, this are not from intresst
        ON nd.DuplicateID IS NULL AND v.ComputerID = nd.ComputerID AND v.ComputerName = nd.DeviceName 
      LEFT JOIN DeviceRole dr WITH (NOLOCK)
        ON nd.DeviceRole = dr.DeviceRoleID
			LEFT JOIN #GetDeviceNetworkInfo devInfo ON devInfo.ComputerID = v.ComputerID
     WHERE @GuestSearch IS NULL OR v.ComputerName LIKE @GuestSearch  

  PRINT '*** Insert HyperV ('+CONVERT(nvarchar(50),GETDATE(), 113)+')***'

  INSERT INTO #Devices (
      ComputerID,
      SystemType,
      SystemTypeShort,
      Client,
      ClientID,
      DNSName,
      Role,
      ScanDate,
      Manufacturer,
      Model,
      Serialnumber,
      Memory,
      DiskSize,
      ProcManufacturer,
      ProcModel,
      ProcCPUs,
      ProcCores,
      ProcSpeed,
      ProcHyperThreating,
      OS,
      LastLogin,
	  IPAddress,
	  MACAddress)
    SELECT
      s.ComputerID         AS ComputerID,
      'Physical'           AS SystemType,
      'Physical'           AS SystemTypeShort,
      s.ComputerName       AS Client,
      s.ComputerID         AS ClientID,
      s.ComputerFQDN       AS DNSName,
      dr.Description       AS Role,
      s.InventoryDate      AS ScanDate,
      s.Manufacturer       AS Manufacturer,
      s.Model              AS Model,
      s.Serialnumber       AS Serialnumber,
      s.Memory             AS Memory,
      s.DiskSize           AS DiskSize,
      s.CPUManufacturer    AS ProcManufacturer,
      s.CPUModel           AS ProcModel,
      s.CPUs               AS ProcCPUs,
      s.Cores              AS ProcCores,
      s.CPUSpeed           AS ProcSpeed,
      s.HyperThreating     AS ProcHyperThreating,
      s.OS                 AS OS,
      s.LastLogin          AS LastLogin,
	  devInfo.IPAddress    AS IPAddress,
	  devInfo.MACAddress   AS MACAddress
    FROM #System_Base s
      LEFT JOIN NetworkDevice nd WITH (NOLOCK)
-- 01.12.2017 add filter on DupplicateID IS NULL, only the last one is from intresting
-- 01.12.2017 add filter on computerCN to DeviceName, some times it give entrys with other devicename what
--            is mapped to the ComputerDevice, this are not from intresst
        ON nd.DuplicateID IS NULL AND s.ComputerID = nd.ComputerID AND s.ComputerName = nd.DeviceName 
      LEFT JOIN DeviceRole dr WITH (NOLOCK)
        ON nd.DeviceRole = dr.DeviceRoleID
			LEFT JOIN #GetDeviceNetworkInfo devInfo ON devInfo.ComputerID = s.ComputerID
 
    WHERE s.InventoryDate IS NOT NULL
      AND s.ComputerID NOT IN (SELECT ComputerID FROM #Devices WHERE ComputerID IS NOT NULL)
      AND (@GuestSearch IS NULL OR s.ComputerName LIKE @GuestSearch)
-- 26.01.2017 fix, remove Devices with OS and no DeviceRoleID on Physical Devices, why this 99,9% SNMP Devices
      AND ((dr.DeviceRoleID IS NULL AND s.OS IS NOT NULL) OR dr.DeviceRoleID < 3)

  PRINT '*** Insert SystemBase ('+CONVERT(nvarchar(50),GETDATE(), 113)+')***'
 
 
  UPDATE #Devices
    SET
      Memory   = memory.Value,
      DiskSize = diskdrives.Size
    FROM #Devices d
      LEFT JOIN (SELECT ComputerID, ROUND(CAST(Value AS REAL)/1024/1024/1024,2) AS Value 
                   FROM f_HardwareProperties('CIM_UnitaryComputerSystem')
                   WHERE Property = 'TotalPhysicalMemory' AND Value NOT LIKE '%[^0-9]%') memory ON d.ComputerID = memory.ComputerID
  -- Disk Size
      LEFT JOIN (SELECT ComputerID,  ROUND(SUM(CAST(Value AS REAL)/1024/1024/1024),2) AS Size 
                   FROM (SELECT ComputerID, CAST(Value AS BIGINT) Value FROM f_HardwareProperties('CIM_DiskDrive')
                           WHERE Property = 'Size' AND Value NOT LIKE '%[^0-9]%') tmp
                   GROUP BY ComputerID) diskdrives  ON d.ComputerID = diskdrives.ComputerID
    WHERE memory.Value IS NOT NULL OR diskdrives.Size IS NOT NULL
      AND (d.Memory = 0 OR d.DiskSize = 0)

  PRINT '*** Update Memory  ('+CONVERT(nvarchar(50),GETDATE(), 113)+')***'
 
 

  IF @ViewAll = 1
  BEGIN
    -- VMWare
    INSERT INTO #Devices (
        SystemType,
        SystemTypeShort,
        vCenter,
        vCenterID,
        Cluster,
        Host,
        HostID,
        Client,
        DNSName,
        ScanDate,
        Manufacturer,
        Model,
        Memory,
        ProcManufacturer,
        ProcModel,
        ProcCPUs,
        ProcSpeed,
        OS,
        IPAddress,
        PowerState)
      SELECT
        'VMware Guest'           AS SystemType,
        'VMware'                 AS SystemTypeShort,
        vCenterSrv.vCenter       AS vCenter,
        vCenterSrv.vCenterID     AS vCenterID,
        vCluster.Value           AS Cluster,
        esx.Host                 AS Host,
        esx.HostID               AS HostID,
        esx.Client               AS Client,
        esx.ClientFQDN           AS DNSName,
        ir.HWDate                AS ScanDate, --
        'VMware'                 AS Manufacturer,
        'Virtual Machine'        AS Model,
        CAST(vmMem.Value AS INT) AS Memory,
        pro.Manufacturer         AS ProcManufacturer,
        pro.Name                 AS ProcModel,
        CAST(vmCPU.Value AS INT) AS CPUs,
        pro.Speed                AS ProcSpeed,
        tmpOS.Value              AS OS,
        tmpIP.Value              AS IPAddress,
        esx.PowerState           AS PowerState
      FROM (SELECT 
                CASE
                  WHEN CHARINDEX('.',cClient.Value) > 0 THEN LEFT(cClient.Value, CHARINDEX('.',cClient.Value)-1)
                  WHEN cClient.Value IS NOT NULL THEN cClient.Value
                  WHEN CHARINDEX('.',cElement.Value) > 0 THEN LEFT(cElement.Value, CHARINDEX('.',cElement.Value)-1)
                  ELSE cElement.Value
                END                      AS Client,
                CASE
                  WHEN CHARINDEX('.',cClient.Value) > 0 THEN cClient.Value
                  WHEN CHARINDEX('.',cElement.Value) > 0 THEN cElement.Value
                END                      AS ClientFQDN,
                cClient.HardwareObjectID AS ClientObjectID,
                cHost.ComputerID         AS HostID,
                cHost.ComputerCN         AS Host,
                CASE
                  WHEN pw.Value = 'poweredOn'  THEN 1
                  WHEN pw.Value = 'poweredOff' THEN 0
                  ELSE NULL
                END                      AS PowerState
              FROM (SELECT ComputerID, HardwareObjectID, Value 
                      FROM #VMWARE_VMComputerSystem
                      WHERE Property = 'Hostname') cClient
                FULL OUTER JOIN (SELECT ComputerID, HardwareObjectID, Value 
                                   FROM #VMWARE_VMComputerSystem
                                   WHERE Property = 'Elementname') cElement
                  ON cClient.HardwareObjectID = cElement.HardwareObjectID
                LEFT JOIN Computer cHost
                  ON cClient.ComputerID = cHost.ComputerID
                    OR cElement.ComputerID = cHost.ComputerID
                LEFT JOIN f_HardwareValuesByPropertyName('PowerState') pw
                  ON cClient.HardwareObjectID = pw.HardwareObjectID
              ) esx
        LEFT JOIN (SELECT HardwareObjectID, Value FROM f_HardwareValuesByPropertyName('guestFullName')) tmpOS
          ON esx.ClientObjectID = tmpOS.HardwareObjectID
        LEFT JOIN (SELECT HardwareObjectID, Value FROM f_HardwareValuesByPropertyName('UUID')) tmpUUID
          ON esx.ClientObjectID = tmpUUID.HardwareObjectID
        LEFT JOIN (SELECT HardwareObjectID, Value FROM f_HardwareValuesByPropertyName('IPAddress')) tmpIP
          ON esx.ClientObjectID = tmpIP.HardwareObjectID
    -- vCenter
        LEFT JOIN (SELECT
                         esx.ComputerID                            AS HostID,
                         vCenter.ComputerID                        AS vCenterID,
                         ISNULL(vCenter.ComputerCN, esx.vCenterCN) AS vCenter
                       FROM (SELECT
                                 ComputerID,
                                 CASE
                                   WHEN esxTmp.vCenter LIKE '[1234567890!.]%' THEN esxTmp.vCenter
                                   WHEN CHARINDEX('.', esxTmp.vCenter) > 0   THEN LEFT(esxTmp.vCenter, CHARINDEX('.', esxTmp.vCenter)-1)
                                   ELSE esxTmp.vCenter
                                 END              AS vCenterCN,
                                 esxTmp.vCenter   AS vCenterFQDN,
                              /* CASE
                                   WHEN esxTmp.vCenter LIKE '[1234567890.]%' THEN 1
                                   ELSE 0
                                 END              AS IsIP, */
                                 CASE
                                   WHEN esxTmp.vCenter NOT LIKE '[1234567890.]%' AND CHARINDEX('.', esxTmp.vCenter) = 0 THEN 1
                                   ELSE 0
                                 END              AS IsNoFQDN
                               FROM (SELECT
                                         ComputerID, 
                                         Replace(Replace(Replace(Replace(Value, 'http://', ''), 'https://', ''), '/sdk', ''), '/', '') vCenter
                                       FROM f_HardwareProperties('VMWARE_InfrastructureSDK')
                                       WHERE Property = 'ServerURL') esxTmp
                            ) esx
                         LEFT JOIN (SELECT c.ComputerID, c.ComputerCN, Replace(Replace(d.DN, 'dc=', '.'), ',', '') domain
                                      FROM Computer c WITH (NOLOCK)
                                        LEFT JOIN Organization o WITH (NOLOCK)
                                          ON c.ComputerOUID = o.OrganizationID
                                        LEFT JOIN Domain d WITH (NOLOCK)
                                          ON o.DomainID = d.OrganizationID) vCenter
                           ON (esx.IsNoFQDN = 0 AND esx.vCenterFQDN LIKE vCenter.ComputerCN + vCenter.domain) 
                             OR (esx.IsNoFQDN = 1 AND esx.vCenterCN = vCenter.ComputerCN)) vCenterSrv
          ON esx.HostID = vCenterSrv.HostID
    -- ClusterName
        LEFT JOIN (SELECT ComputerID, Value FROM f_HardwareProperties('MGS_Cluster') WHERE Property = 'ElementName') vCluster
          ON esx.HostID = vCluster.ComputerID
    -- Invenory Date
        LEFT JOIN InventoryReport ir
          ON esx.HostID = ir.ComputerID
    -- Hardware Settings
        LEFT JOIN f_HardwareValuesByPropertyName('numCPU') vmCPU
          ON esx.ClientObjectID = vmCPU.HardwareObjectID
        LEFT JOIN f_HardwareValuesByPropertyName('MaxMemoryUsage') vmMem
          ON esx.ClientObjectID = vmMem.HardwareObjectID
    -- Processor
        LEFT JOIN f_System_Processor() pro
          ON esx.HostID = pro.ComputerID
        LEFT JOIN #Devices d
          ON esx.HostID = d.HostID
            AND ((esx.ClientFQDN IS NULL AND esx.Client = d.Client)
              OR esx.ClientFQDN = d.DNSName
              OR REPLACE(REPLACE(REPLACE(LEFT(d.Serialnumber,100),' ',''),'-',''),'VMware','') = REPLACE(REPLACE(tmpUUID.Value,' ',''),'-',''))
      WHERE d.ClientID IS NULL
        AND (@GuestSearch IS NULL OR esx.Client like @GuestSearch)

INSERT INTO #Devices (
      SystemType,
      SystemTypeShort,
      Cluster,
      Host,
      HostID,
      Client,
      DNSName,
      ScanDate,
      Manufacturer,
      Model,
      Memory,
      ProcManufacturer,
      ProcModel,
      ProcCPUs,
      ProcSpeed)
    SELECT
      'HyperV Guest'                       AS SystemType,
      'HyperV'                             AS SystemTypeShort,
      vCluster.Value                       AS Cluster,
      Host.HostCN                          AS Host,
      Host.HostID                          AS HostID,
      Host.GuestCN                         AS Client,
      Host.GuestFQDN                       AS DNSName,
      ir.HWDate                            AS ScanDate,
      'Microsoft'                          AS Manufacturer,
      'Virtual Machine'                    AS Model,
      ROUND(CAST(mem.Value AS INT)/1024,2) AS Memory,
      pro.Manufacturer                     AS ProcManufacturer,
      pro.Name                             AS ProcModel,
      CAST(prc.Value AS INT)               AS CPUs,
      pro.Speed                            AS ProcSpeed
    FROM (SELECT 
              tmp.ComputerID        AS HostID, 
              host.ComputerCN       AS HostCN,
              CASE
                WHEN CHARINDEX(dng.Value, '.') > 0 THEN LEFT(dng.Value, CHARINDEX(dng.Value, '.') -1)
                ELSE dng.Value
              END                   AS GuestCN,
              dng.Value             AS GuestFQDN,
              tmp.Value             AS BiosSerialNumber,
              tmp.HardwareObjectID
            FROM f_HardwareProperties('HyperV_Guest') tmp
              LEFT JOIN f_HardwareValuesByPropertyName('DNSNameGuest') dng
                ON tmp.HardwareObjectID = dng.HardwareObjectID
              LEFT JOIN Computer host
                ON tmp.ComputerID = host.ComputerID
            WHERE tmp.Property = 'BIOSSerialNumber') Host
  -- Memory
      LEFT JOIN f_HardwareValuesByPropertyName('Memory') mem
        ON Host.HardwareObjectID = mem.HardwareObjectID
  -- ProcessorCount
      LEFT JOIN f_HardwareValuesByPropertyName('ProcessorCount') prc
        ON Host.HardwareObjectID = mem.HardwareObjectID
  -- ClusterName
      LEFT JOIN (SELECT ComputerID, HardwareObjectID Value FROM f_HardwareProperties('HyperV_ClusterName') WHERE Property = 'ClusterName') vCluster
        ON Host.HostID = vCluster.ComputerID
  -- Invenory Date
      LEFT JOIN InventoryReport ir
        ON Host.HostID = ir.ComputerID
  -- Processor
      LEFT JOIN f_System_Processor() pro
        ON Host.HostID = pro.ComputerID
      LEFT JOIN #Devices d
        ON Host.HostID = d.HostID
          AND d.SerialNumber = Host.BiosSerialNumber
          AND ((Host.GuestFQDN IS NULL AND Host.GuestCN = d.Client)
            OR Host.GuestFQDN = d.DNSName)
    WHERE d.ClientID IS NULL
      AND (@GuestSearch IS NULL OR Host.GuestCN like @GuestSearch)
  END

  SELECT * FROM #Devices d
    WHERE (@SystemType IS NULL OR @SystemType LIKE '%' + d.SystemTypeShort + '%')
      AND (@HostSearch IS NULL OR d.vCenter LIKE @HostSearch OR d.Host LIKE @HostSearch)
      AND (@MaxAge IS NULL OR DATEDIFF(day,  d.ScanDate , CURRENT_TIMESTAMP) <= @MaxAge)
    ORDER BY d.vCenter, d.Cluster, d.Host, d.Client

  DROP TABLE #Devices
  DROP TABLE #Virtual_VMware
  DROP TABLE #Virtual_HyperV
  DROP TABLE #GetDeviceNetworkInfo
  DROP TABLE #Virtual_LPAR  
  DROP TABLE #System_Base
 

END

