USE [ManageSoft]
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_GetDeviceProcessor' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_GetDeviceProcessor]

GO
-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 31.01.2019
-- Copyright:   (c) Raynet GmbH
-- Description:	Get All Processor Information
-- ==============================================
-- Connector version:4.0.94.29 
-- ==============================================


CREATE FUNCTION [dbo].[f_GetDeviceProcessor] ()
  RETURNS TABLE
AS

RETURN
SELECT
      p.ComputerID                                 AS ComputerID,
      MAX(COALESCE(pn.Value, pc.Value, pd.Value))  AS Name,
      CASE
        WHEN MAX(m.Value) LIKE '%Intel%'     THEN 'Intel'
        WHEN MAX(m.Value) LIKE '%AMD%'       THEN 'AMD'
        WHEN MAX(pd.Value) LIKE '%Intel%'     THEN 'Intel'
        WHEN MAX(pd.Value) LIKE '%AMD%'       THEN 'AMD'
        WHEN  MAX(COALESCE(pn.Value, pc.Value, pd.Value))   LIKE '%PowerPC%'   THEN 'IBM'
        WHEN  MAX(COALESCE(pn.Value, pc.Value, pd.Value))   LIKE '%SPARC%'     THEN 'SUN/Oracle'
        ELSE MAX(m.Value)
      END                                           AS Manufacturer,
      dbo.CIMInteger(MAX(IIF(PATINDEX('%[.,]%', sp.Value) > 0,
                             LEFT(sp.Value, PATINDEX('%[.,]%', sp.Value)-1),
                             sp.Value)))            AS ClockSpeed,
      dbo.CIMInteger(MAX(REPLACE(IIF(cs.Value = '0', NULL, cs.Value), 'KB', ''))) 
                                                    AS L2CacheSize,
      CASE
        WHEN PATINDEX('%[,.]%', MAX(cv.Value)) > 0 THEN dbo.CIMReal(REPLACE(MAX(cv.Value), ',', '.'))
        WHEN MAX(cv.Value) IS NULL                 THEN dbo.CIMReal(NULL)
        ELSE dbo.CIMReal(MAX(cv.Value)) / 10
      END                                           AS CurrentVoltage, 
      CASE
        WHEN MAX(lVMC.Value) IS NOT NULL AND  MAX(COALESCE(pn.Value, pc.Value, pd.Value))   LIKE '%SPARC%' THEN NULL
        WHEN MAX(cU.Value) IS NOT NULL AND  MAX(COALESCE(pn.Value, pc.Value, pd.Value))   LIKE '%PowerPC%' THEN NULL
        ELSE COUNT(*)
      END                                           AS CPUs,
      CASE
        WHEN MAX(lVMCC.Value) IS NOT NULL AND  MAX(COALESCE(pn.Value, pc.Value, pd.Value))   LIKE '%SPARC%' THEN NULL
        WHEN MAX(cU.Value) IS NOT NULL AND  MAX(COALESCE(pn.Value, pc.Value, pd.Value))   LIKE '%PowerPC%'  THEN NULL
        ELSE dbo.CIMInteger(MAX(c.Value))
      END                                           AS Cores,
      CASE
        WHEN MAX(lVMCC.Value) IS NOT NULL AND  MAX(COALESCE(pn.Value, pc.Value, pd.Value))   LIKE '%SPARC%' THEN NULL
        WHEN MAX(lcU.Value) IS NOT NULL AND  MAX(COALESCE(pn.Value, pc.Value, pd.Value))   LIKE '%PowerPC%' THEN NULL
        ELSE dbo.CIMInteger(MAX(lc.Value))
      END                                           AS Logical, 
      CASE
        WHEN MAX(lVMC.Value) IS NOT NULL AND  MAX(COALESCE(pn.Value, pc.Value, pd.Value))   LIKE '%SPARC%' THEN dbo.CIMInteger(ROUND(MAX(lVMC.Value),0))
        WHEN MAX(cU.Value) IS NOT NULL AND  MAX(COALESCE(pn.Value, pc.Value, pd.Value))   LIKE '%PowerPC%' THEN dbo.CIMInteger(ROUND(MAX(cU.Value),0))
        ELSE COUNT(*)
      END                                           AS TotalCPUs,
      CASE
        WHEN MAX(lVMCC.Value) IS NOT NULL AND  MAX(COALESCE(pn.Value, pc.Value, pd.Value))   LIKE '%SPARC%' THEN dbo.CIMInteger(ROUND(MAX(lVMCC.Value),0))
        WHEN MAX(cU.Value) IS NOT NULL AND  MAX(COALESCE(pn.Value, pc.Value, pd.Value))   LIKE '%PowerPC%'  THEN NULL
        ELSE SUM(dbo.CIMInteger(c.Value))
      END                                           AS TotalCores,
      CASE
        WHEN MAX(lVMCC.Value) IS NOT NULL AND  MAX(COALESCE(pn.Value, pc.Value, pd.Value))   LIKE '%SPARC%' THEN NULL
        WHEN MAX(lcU.Value) IS NOT NULL AND  MAX(COALESCE(pn.Value, pc.Value, pd.Value))   LIKE '%PowerPC%' THEN dbo.CIMInteger(ROUND(MAX(lCU.Value),0))
        ELSE SUM(dbo.CIMInteger(lc.Value))
      END                                           AS TotalLogical, 
      CASE
        WHEN MAX(m.Value) LIKE '%Intel%' AND dbo.CIMInteger(MAX(lc.Value)) > dbo.CIMInteger(MAX(c.Value)) THEN 1
        WHEN MAX(m.Value) LIKE '%AMD%'   AND dbo.CIMInteger(MAX(lc.Value)) > dbo.CIMInteger(MAX(c.Value)) THEN 1
        WHEN MAX(pd.Value) LIKE '%Intel%' AND dbo.CIMInteger(MAX(lc.Value)) > dbo.CIMInteger(MAX(c.Value)) THEN 1
        WHEN MAX(pd.Value) LIKE '%AMD%'   AND dbo.CIMInteger(MAX(lc.Value)) > dbo.CIMInteger(MAX(c.Value)) THEN 1
        ELSE 0
      END                                           AS HyperThreading
    FROM (SELECT DISTINCT ComputerID, HardwareObjectID  FROM dbo.f_HardwareProperties('CIM_Processor') WHERE Property = 'Name' OR Property = 'Caption' OR Property = 'Description'  ) p
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Manufacturer') m
        ON p.HardwareObjectID = m.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('Description') pd
        ON p.HardwareObjectID = pd.HardwareObjectID
		LEFT JOIN dbo.f_HardwareValuesByPropertyName('Name') pn
        ON p.HardwareObjectID = pn.HardwareObjectID
		LEFT JOIN dbo.f_HardwareValuesByPropertyName('Caption') pc
        ON p.HardwareObjectID = pc.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('MaxClockSpeed') sp
        ON p.HardwareObjectID = sp.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('L2CacheSize') cs
        ON p.HardwareObjectID = cs.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('CurrentVoltage') cv
        ON p.HardwareObjectID = cv.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('NumberOfCores') c
        ON p.HardwareObjectID = c.HardwareObjectID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('NumberOfLogicalProcessors') lc
        ON p.HardwareObjectID = lc.HardwareObjectID
      LEFT JOIN (SELECT ComputerID, HardwareObjectID, Value 
                   FROM dbo.f_HardwareProperties('CIM_UnitaryComputerSystem') 
                   WHERE Property = 'VMCapacity') lVMC
        ON p.ComputerID = lVMC.ComputerID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('VMCapacityInCores') lVMCC
        ON lVMC.HardwareObjectID = lVMCC.HardwareObjectID
      LEFT JOIN (SELECT ComputerID, HardwareObjectID, Value 
                   FROM dbo.f_HardwareProperties('CIM_UnitaryComputerSystem') 
                   WHERE Property = 'NumberOfProcessors') cU
        ON p.ComputerID = cU.ComputerID
      LEFT JOIN dbo.f_HardwareValuesByPropertyName('NumberOfLogicalProcessors') lcU
        ON cU.HardwareObjectID = lcU.HardwareObjectID
    GROUP BY p.ComputerID
GO

